"""--------------------------------------------------------------------
 * $Id: aerosol_options.py 2784 2012-08-26 11:42:19Z svn-kylling $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""


from option_definition import *

class aerosol_mixed_options(option):
	"""
	Logic for haze, season, vulcan and visibility
	"""
	def __init__(self, *args, **kwargs):
		option.__init__(self, *args, **kwargs)
		self.dependencies.extend(['aerosol_haze', 'aerosol_vulcan',
				     'aerosol_season', 'aerosol_visibility'])
	def isMandatory(self, is_set, get_value):
		cond = [is_set(opt) for opt in 'aerosol_haze',
			'aerosol_vulcan', 'aerosol_season',
			'aerosol_visibility']
		if all(cond):
			return False
		elif any(cond):
			return True
		else:
			return False
		

def setup_aerosol_group():
	aerosol_angstrom = continious_option(
		name='aerosol_angstrom',
	        group='aerosol',  
		helpstr='Scale the aerosol optical depth using the {\AA}ngstr{\"o}m formula.', 
		documentation= aerosol_documentation['aerosol_angstrom'], 
		tokens=[addToken('Input.aer.alpha', float, default=0.0 ),
		addToken('Input.aer.beta', float, default='NOT_DEFINED_FLOAT', valid_range=[0, 1e6] )],
		settings=addSetting('Input.aer.spec', 1, default=0),
		parents=['aerosol_default'], 
	)
		
	aerosol_default = continious_option(
		'aerosol_default',	#name
		'aerosol',	#group
		'',	#supgroup
		'Set up a default aerosol',	#helpstr
		aerosol_documentation['aerosol_default'],	#documentation	
		None,
		[addSetting('Input.aer.haze', 1, default=0),
		addSetting('Input.aer.vulcan', 1, default=0),
		addSetting('Input.aer.seasn', 1, default=0),
		addSetting('Input.aer.visibility', 50.0, default='NOT_DEFINED_FLOAT')],	#settings
		'',	#parents
		'',	#non_parents
		childs=['aerosol_angstrom']
	)
	
	aerosol_files = option(
		'aerosol_files',	#name
		'aerosol',	#group
		'specification',	#supgroup
		'Enter filename of aerosol_files',	#helpstr
		aerosol_documentation['aerosol_files'],	#documentation	
		addToken('Input.aer.filename[FN_AER_FILES]', file), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
		'',	#parents
		'',	#non_parents
		''	#childs
	)
	
	aerosol_gg_file = option(
		'aerosol_gg_file',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Location of aerosol asymmetry parameter file',	#helpstr
		aerosol_documentation['aerosol_gg_file'],     #documentation
		addToken('Input.aer.filename[FN_AER_GG]', file), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_haze = aerosol_mixed_options(
		'aerosol_haze',	#name
	        'aerosol',      #group
	        '',	#supgroup
		'Specify the aerosol type in the lower 2 km of the atmosphere',	#helpstr
		aerosol_documentation['aerosol_haze'], #documentation
		addToken( 'Input.aer.haze', int, default=0, valid_range=[1,6] ),     #token
		None, #settings
		[],#Please review! ['aerosol_vulcan', 'aerosol_season', 'aerosol_visibility'],     #parents
		'',     #non_parents
		['aerosol_option_specification']      #childs
	)
	
	aerosol_moments_file = option(
		'aerosol_moments_file',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Location of phase function moments file',	#helpstr
		aerosol_documentation['aerosol_moments_file'], #documentation
		addToken('Input.aer.filename[FN_AER_MOM]', file), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_no_scattering = option(
		'aerosol_no_scattering',	#name
	        'aerosol',      #group
	        '',	#supgroup
		'Switch off scattering by aerosols',	#helpstr
		aerosol_documentation['aerosol_no_scattering'], #documentation
		None, #token
		addSetting('Input.aer.no_scattering', True, default=False),     #setting
		'',     #parents
		'',     #non_parents
		''      #childs
	)

	aerosol_profile_modtran = option(
		'aerosol_profile_modtran', #name
		'aerosol', #group
		'', #subgroup
		'Squeeze aerosol profile', #helpstr
		aerosol_documentation['aerosol_profile_modtran'], #documentation
		None, #token
		addSetting('Input.aer.profile_modtran', True), #setting
		'',     #parents
		'',     #non_parents
		''      #childs
	)

	aerosol_refrac_file = option(
		'aerosol_refrac_file',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Location of file containing the wavelength-dependent refractive index of the aerosol',	#helpstr
		aerosol_documentation['aerosol_refrac_file'], #documentation
		addToken('Input.aer.filename[FN_AER_REF]', file), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        ['aerosol_sizedist_file'],     #parents
	        '',     #non_parents 
	        ''      #childs
	)
	
	aerosol_refrac_index = option(
		'aerosol_refrac_index',	#name
	        'aerosol',      #group
	        'specification',	#supgroup
		'Scale the aerosol optical depth using the {\AA}ngstr{\"o}m formula.',	#helpstr
		aerosol_documentation['aerosol_refrac_index'], #documentation
		[	addToken('Input.aer.re', float, valid_range=[0,1e6]),
			addToken('Input.aer.im', float, valid_range=[0,1e6])
		], #token
		addSetting('Input.aer.spec', 1, default=0), #settings
		['aerosol_sizedist_file'],     #parents
		'',     #non_parents 
		''      #childs
	)
	
	aerosol_scale_ssa = option(
		'aerosol_scale_ssa',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Scale the aerosol single scattering albedo',	#helpstr
		aerosol_documentation['aerosol_scale_ssa'], #documentation
		addToken('Input.aer.scale_ssa', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6]), #token #TODO:default value in uvspec_lex: not_defined_float
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_scale_tau = option(
		'aerosol_scale_tau',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Scale the aerosol extinction',	#helpstr
		aerosol_documentation['aerosol_scale_tau'], #documentation
		addToken('Input.aer.scale_tau', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6]), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_set_gg = option(
		'aerosol_set_gg',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Set the aerosol asymmetry parameter',	#helpstr
		aerosol_documentation['aerosol_set_gg'], #documentation
		addToken('Input.aer.set_gg', float, default='NOT_DEFINED_FLOAT', valid_range=[-1,1]), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_set_ssa = option(
		'aerosol_set_ssa',		#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Set the aerosol single scattering albedo',	#helpstr
		aerosol_documentation['aerosol_set_ssa'], #documentation
		addToken('Input.aer.set_ssa', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1]), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_set_tau = option(
		'aerosol_set_tau',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Set the aerosol optical thickness',	#helpstr
		aerosol_documentation['aerosol_set_tau'], #documentation
		addToken('Input.aer.set_tau', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6]), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_set_tau550 = option(
		'aerosol_set_tau550',	#name
	        'aerosol',      #group
		'',      #supgroup
		'Set the aerosol optical thickness at 550nm',	#helpstr
		aerosol_documentation['aerosol_set_tau550'], #documentation
		addToken('Input.aer.tau550', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6]), #token
		None, #setting
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)

	aerosol_set_tau_at_wvl = option(
		'aerosol_set_tau_at_wvl',	#name
	        'aerosol',      #group
		'',      #supgroup
		'Set the aerosol optical thickness at lambda (nm)',	#helpstr
		aerosol_documentation['aerosol_set_tau_at_wvl'], #documentation
		[addToken('Input.aer.tau_wvl_lambda', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6]),
		 addToken('Input.aer.tau_wvl_tau', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6])],#tokens
		None, #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_season = aerosol_mixed_options(
		'aerosol_season',	#name
	        'aerosol',      #group
		'standard',      #supgroup
		'Specify season',	#helpstr
		aerosol_documentation['aerosol_season'], #documentation
		addToken('Input.aer.seasn', int, default=0, valid_range=[1,2]), #token
		addSetting('Input.aer.spec', 1, default=0), #setting
	        [],#['aerosol_haze', 'aerosol_vulcan', 'aerosol_visibility'],     #parents
	        '',     #non_parents
	        ['aerosol_option_specification']      #childs
	)
	
	aerosol_sizedist_file = option(
		'aerosol_sizedist_file',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Location of aerosol size distribution file',	#helpstr
		aerosol_documentation['aerosol_sizedist_file'], #documentation
		addToken('Input.aer.filename[FN_AER_SIZ]', file), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        ['aerosol_refrac_file', 'aerosol_refrac_index'],     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_species_file = option(	#TODO: undefined number of optional arguments are allowed
		'aerosol_species_file',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Specify mass density profiles of a mixture of aerosol types',	#helpstr
		aerosol_documentation['aerosol_species_file'], #documentation
		addToken('Input.aer.mixture_name', str, valid_range = ['continental_clean', 'continental_average', 'continental_polluted', 
					'urban', 'maritime_clean', 'maritime_polluted', 'maritime_tropical', 'desert', 'antarctic'] ), #tokens
		[addSetting('Input.aer.n_species', -1, default='NOT_DEFINED_INTEGER'),
		addSetting('Input.aer.spec', 1, default=0)], #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_species_library = option(
		'aerosol_species_library',	#name
	        'aerosol',      #group
		'',      #supgroup
		'Location of optical property files',	#helpstr
		aerosol_documentation['aerosol_species_library'], #documentation
		addToken('Input.aer.filename[FN_AER_SPECIES_LIB]', file), #token
		None, #setting
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_ssa_file = option(
		'aerosol_ssa_file',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Location of aerosol single scattering albedo file',	#helpstr
		aerosol_documentation['aerosol_ssa_file'], #documentation
		addToken('Input.aer.filename[FN_AER_SSA]', file), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_tau_file = option(
		'aerosol_tau_file',	#name
	        'aerosol',      #group
		'specification',      #supgroup
		'Location of aerosol optical depth file',	#helpstr
		aerosol_documentation['aerosol_tau_file'], #documentation
		addToken('Input.aer.filename[FN_AER_TAU]', file), #token
		addSetting('Input.aer.spec', 1, default=0), #settings
	        '',     #parents
	        '',     #non_parents
	        ''      #childs
	)
	
	aerosol_visibility = aerosol_mixed_options(
		'aerosol_visibility',	#name
	        'aerosol',      #group
		'',      #supgroup
		'Horizontal visibility in km',	#helpstr
		aerosol_documentation['aerosol_visibility'], #documentation
		addToken('Input.aer.visibility', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6]), #token
		None, #setting
	        [],#['aerosol_haze', 'aerosol_season', 'aerosol_vulcan'],     #parents
	        '',     #non_parents
	        ['aerosol_option_specification']      #childs
	)
	
	aerosol_vulcan = aerosol_mixed_options(
		'aerosol_vulcan',	#name
	        'aerosol',      #group
		'',     #supgroup
		'Aerosol situation above 2 km',	#helpstr
		aerosol_documentation['aerosol_vulcan'], #documentation
		addToken('Input.aer.vulcan', int, default=0, valid_range=[1,2,3,4]), #token
		None, #setting
	        [],#['aerosol_haze', 'aerosol_season', 'aerosol_visibility'],     #parents
	        '',     #non_parents
	        ['aerosol_option_specification']      #childs
	)

	return[ aerosol_default, aerosol_haze, aerosol_season, aerosol_vulcan, aerosol_visibility,
		aerosol_angstrom, aerosol_files, aerosol_gg_file, aerosol_moments_file, 
		aerosol_refrac_file, aerosol_refrac_index, aerosol_scale_ssa, aerosol_scale_tau,
		aerosol_set_gg, aerosol_set_ssa, aerosol_set_tau, aerosol_sizedist_file,
		aerosol_species_file, aerosol_ssa_file, aerosol_tau_file, aerosol_profile_modtran,
		aerosol_species_library, aerosol_set_tau550, aerosol_set_tau_at_wvl, aerosol_no_scattering]

aerosol_documentation = {
	'aerosol_angstrom'	: r'''
	Scale the aerosol optical depth using the {\AA}ngstr{\"o}m formula:
	\begin{equation}
	\tau = \beta \lambda^{-\alpha}
	\end{equation}
	where $\lambda$ is in units of micrometer \citep{Angstrom1929}. Specify 
	the {\AA}ngstr{\"o}m alpha and beta coefficients by
	\fcode{
	   aerosol_angstrom alpha beta
	}
	The optical thickness defined here  is the integral from the user-defined 
	\code{altitude} to TOA (top of atmosphere). 
	''',
	'aerosol_default'	: r'''
	Set up a default aerosol according to \citet{shettle89}. 
	The default properties are 
	a rural type aerosol in the boundary layer, background aerosol above 2km,
	spring-summer conditions and a visibility of 50km. These settings may
	be modified with \code{aerosol_haze}, \code{aerosol_vulcan}, \code{aerosol_season},
	and \code{aerosol_visibility}.''',
	'aerosol_files'		: r'''
	A way to specify aerosol optical depth, single scattering albedo, 
	and phase function moments for each layer. The file specified by 
	\code{aerosol_files} by
	\fcode{
	aerosol_files file_name
	}
	has two columns where column 1 is the altitude in km. The second
	column is a the name of a file which defines the optical properties of the layer
	starting at the given altitude. The files specified in the second column must have the 
	following format:
	\begin{description}
	\parameter{Column 1: } 
	The wavelength in nm. These wavelengths may be different from those in \code{solar_file}. 
	Optical properties are interpolated to the requested wavelengths.
	\parameter{Column 2:} 
	The extinction coefficient of the layer in units km-1. 
	\parameter{Column 3:} 
	The aerosol single scattering albedo of the layer.
	\parameter{Column 4-(nmom+4):} 
	The moments of the aerosol phase function. 
	\end{description}
	For some simple examples see the files \file{examples/AERO\_*.LAYER}. Note that 
	if using the \code{rte_solver cdisort} it makes good sense to make the 
	number of moments larger than \code{nstr}. For \code{rte_solver fdisort1} and 
	\code{rte_solver polradtran} the number of moments included in the calculations 
	will be \code{nstr}+1. Higher order moments will be ignored for these solvers.
	Please note that the uppermost line of the \code{aerosol_files}
	denotes simply the top altitude of the uppermost layer. The optical
	properties of this line are consequently ignored. There are two
	options for this line: either an optical property file with zero
	optical thickness is specified or "NULL" us used.
	''',
	'aerosol_gg_file'	: r'''
	Location of aerosol asymmetry parameter file specified by
	\fcode{
	   aerosol_gg_file file_name
	}
	The file must have two columns.
	Column 1 is the altitude in km. 
	Column 2 is the asymmetry parameter of each layer. 
	The asymmetry parameter defined with this option is constant with wavelength.
	If you require spectral dependence please use \code{aerosol_files}.
	Comments start with \code{\#}. Empty lines are ignored.
	''',
	'aerosol_haze': r'''
	Specify the aerosol type in the lower 2 km of the atmosphere as
	\fcode{
	   aerosol_haze type
	}
	where \code{type} is an integer identifying the following aerosol types:
	\begin{description}
	\item[1] Rural type aerosols.
	\item[2] Maritime type aerosols.
	\item[5] Urban type aerosols.
	\item[6] Tropospheric type aerosols.
	\end{description}
	For a description of the different aerosol types see \citet{shettle89}.
	''',
	'aerosol_moments_file'	: r'''
	Set the aerosol phase function moments to the values specified in 
	the aerosol moments file 
	\fcode{
	   aerosol_moments_file file_name
	}
	where the file contains one column with arbitrary number of 
	Legendre terms of the phase function. The phase function 
	 $p(\mu)$ is
	\begin{equation}
	   p (\mu) = \sum_{m=0}^{\infty} (2m+1) \cdot k_m \cdot P_m (\mu) 
	\end{equation}
	where $k_m$ is the m'th moment and $P_m (\mu)$ is the m'th Legendre
	polynomial.  If not specified, a Henyey-Greenstein phase function is
	assumed where the asymmetry parameter g is either a default value
	depending on the aerosol type or it may be specified using
	\code{aerosol_set_gg}.  The phase function will be the same for all
	altitudes and wavelengths.  See \code{aerosol_files} if more
	flexibility is wanted. May only be used together with the
	\code{cdisort} or \code{fdisort2} solver in combination with the option
	\code{disort_icm moments}.
	''',
	'aerosol_no_scattering'	: r'''
	Switch off scattering by aerosols.
	''',
	'aerosol_profile_modtran': r'''
	Squeeze aerosol profile up to 6 km when altitude is non-zero as in MODTRAN.
	Per default the aerosol profile is shifted upwards
	and remains unchanged. 
	''',
	'aerosol_refrac_file'	: r'''
	The command line
	\fcode{
	   aerosol_refrac_file file_name
	}
	specifies the file containing the wavelength-dependent refractive index of the
	aerosol. Three columns are expected: wavelength [nm] and the 
	real and imaginary parts of the refractive index.
	Together with \code{aerosol_sizedist_file} this forms the 
	input to Mie calculations of the aerosol optical properties. Please note that 
	only the single-scattering albedo, the scattering phase function, and the
	wavelength-dependence of the extinction coefficient are affected by the Mie calculation 
	while the absolute value of the extinction coefficient is taken from other sources;
	generally, the extinction coefficient at the first \emph{internal} wavelength is
	taken from whatever is available (either default \citet{shettle89} or user-defined);
	the extinction at all other wavelengths is scaled according to the Mie calculation. 
	For this reason, the absolute numbers are not relevant - only the shape of the 
	size distribution matters. In detail:
	If the aerosol properties are defined using the refractive index and the 
	size distribution, the wavelength dependence of the optical properties is 
	determined by Mie theory. At present there are at least three ways to define the 
	absolute value of the optical thickness: (1) \code{visibility} defines the 
	profile at the first \emph{internal} wavelength; for a monochromatic calculation
	and in correlated-k mode, the first \emph{internal} wavelength equals the 
	first wavelength output by \code{uvspec}; for spectral calculations, the 
	first wavelength might be a little bit smaller than the first wavelength 
	output by \code{uvspec}; (2) \code{aerosol_tau_file} defines the optical 
	thickness profile at the first \emph{internal} wavelength; or (3) absolute
	optical thickness and wavelength-dependence are defined by \code{aerosol_angstrom}.
	\emph{It is recommended to avoid this option and rather to calculate the aerosol optical 
	properties externally e.g. with mie and to pass them to \code{uvspec} with 
	\code{aerosol_files}}.
	''',
	'aerosol_refrac_index'	: r'''
	Wavelength-independent refractive index of the aerosol; if wavelength-dependence 
	is required, use \code{aerosol_refrac_file} instead. Together with 
	\code{aerosol_sizedist_file} this forms the input to Mie calculations of the 
	aerosol optical properties. Please see the description of \code{aerosol_refrac_file}
	to learn how the optical properties are set up.
	\emph{It is recommended to avoid this option and rather to calculate the aerosol optical 
	properties externally e.g. with mie and to pass them to \code{uvspec} with 
	\code{aerosol_files}}.
	''',
	'aerosol_scale_ssa'	: r'''
	Scale the aerosol single scattering albedo for all wavelengths and altitudes
	with a positive number. If the resulting scaled single scattering
	albedo is larger than 1 it is set to 1.
	\fcode{
	   aerosol_scale_ssa value
	}
	''',
	'aerosol_scale_tau'	: r'''
	Scale the aerosol extinction for all wavelengths and altitudes
	with a positive number.
	\fcode{
	   aerosol_scale_tau value
	}
	''',
	'aerosol_set_gg'	: r'''
	Set the aerosol asymmetry parameter for all wavelengths and altitudes
	to a constant value between -1.0 and 1.0. 
	Please note that this option is only applied if a Henyey-Greenstein 
	phase function is used but not if an explicit phase function is 
	defined e.g. with a aerosol\_files. It doesn't make sense to modify only 
	the first moment of an explicit phase function.  
	\fcode{
	   aerosol_set_gg value
	}
	''',
	'aerosol_set_ssa'	: r'''
	Set the aerosol single scattering albedo for all wavelengths and altitudes
	to a constant value between 0.0 and 1.0.
	\fcode{
	   aerosol_set_ssa value
	}
	''',
	'aerosol_set_tau'	: r'''
	Set the aerosol optical thickness for all wavelengths and altitudes
	to a constant value. The optical thickness defined here is the
	integral from the user-definded \code{altitude} to TOA (top of atmosphere). 
	\fcode{
	   aerosol_set_tau value
	}
	''',
	'aerosol_set_tau550'	: r'''
	Set the aerosol optical thickness at 550nm. Other wavelengths are scaled accordingly.
	Note that this option requires for technical reasons that the
	wavelength interval defined by \code{wavelength} does contain 550nm.
	The optical thickness defined here is the integral from the user-definded 
	\code{altitude} to TOA (top of atmosphere). 
	\fcode{
	   aerosol_set_tau550 value
	}
	''',
	'aerosol_set_tau_at_wvl'	: r'''
	Set the aerosol optical thickness at wavelength lambda (nm). Other wavelengths are scaled accordingly.
	Note that this option requires for technical reasons that the
	wavelength interval defined by \code{wavelength} does contain lambda.
	The optical thickness defined here is the integral from the user-definded 
	\code{altitude} to TOA (top of atmosphere). 
	\fcode{
	   aerosol_set_tau_at wvl lambda tau
	}
	''',
	'aerosol_season'	: r'''
	Specify season to get appropriate aerosol profile.
	\fcode{
	   aerosol_season season
	}
	where \code{season} is either \code{1} or \code{2}:
	\begin{description}
	\item[1] Spring-summer profile.
	\item[2] Fall-winter profile.
	\end{description}
	''',
	'aerosol_sizedist_file'	: r'''
	Aerosol size distribution. Two columns are expected: The radius
	[micrometer] and the particle number. Together with \code{aerosol_refrac_index} or
	\code{aerosol_refrac_file} this forms the input to Mie calculations of the 
	aerosol optical properties. Please note that 
	only the single-scattering albedo, the scattering phase function, and the
	wavelength-dependence of the extinction coefficient are affected by the Mie calculation 
	while the absolute value of the extinction coefficient is taken from other sources;
	generally, the extinction coefficient at the first \emph{internal} wavelength is
	taken from whatever is available (either default \citet{shettle89} or user-defined);
	the extinction at all other wavelengths is scaled according to the Mie calculation. 
	For this reason, the absolute numbers are not relevant - only the shape of the 
	size distribution matters. For details see also the description of \code{aerosol_refrac_file}.
	\emph{It is recommended to avoid this option and rather to calculate the aerosol optical 
	properties externally e.g. with mie and to pass them to \code{uvspec} with 
	\code{aerosol_files}}.
	''',
	'aerosol_species_file'	: r'''
	Specify mass density profiles of a mixture of aerosol types. 
	\fcode{
	aerosol_species_file profile [aero_1 aero_2 ... aero_n]
	}
	where \code{aero_1} to \code{aero_n} are the aerosol species to be included. 
	For each of these species, the optical properties are read from the 
	\code{aerosol_species_library}, e.g. the OPAC data set provided with 
	libRadtran. The profile file needs to include vertical profiles for each
	of these species. This file can be either in \emph{netCDF}-format 
	(automatically recognized filename extension \code{.nc} or \code{.cdf}) 
	or in ASCII format. The format of the ASCII file is: 
	\fcode{
	z1    dens(aero_1, z1) dens(aero_2, z1)  ... dens(aero_n, z1)\\
	z2    dens(aero_1, z2) dens(aero_2, z2)  ...\\
	 .        .           .\\
	 .        .           .\\
	}
	where \code{z} is the height in km, and \code{dens} are the aerosol mass
	densities in g/m3. Please make sure to include one column for each of 
	the species \code{aero_1} to \code{aero_n} listed after 
	\code{aerosol_species_file}. 
	For netCDF input it is also possible to specify the unit 'kg kg$^{-1}$'; the data 
	are then automatically converted to g/m$^3$.
	
	Some default aerosol mixtures are provided, corresponding to the definitions in 
	\citet{hess98:_optic_proper_aeros_cloud}.
	They can simply be invoked by
	\fcode{
	aerosol_species_file mixture_name
	}
	where \code{mixture_name} can be one of the following: 
	\fcode{\\
	continental_clean\\
	continental_average\\
	continental_polluted\\
	urban\\
	maritime_clean\\
	maritime_polluted\\
	maritime_tropical\\
	desert\\
	antarctic
	}
  A variation of the desert mixture containing nonspherical particles is 
  \fcode{
    desert_spheroids
  }
	''',
	'aerosol_species_library'	: r'''
	With this option the \emph{directory} is specified where the optical property
	files for all aerosols species used in the \code{aerosol_species_file} are
	expected: For each species defined in \code{aerosol_species_file},
	\emph{netCDF}-file \emph{species\_name}\code{.nc},
	(e.g. \code{INSO.nc}), which contains the optical properties of the aerosol
	species, has to be provided. The netcdf format is the one produced by 
	the {\sl libRadtran} \code{mie} tool.
	 
	At the libRadtran webpage we provide the OPAC data set 
	\citep{hess98:_optic_proper_aeros_cloud}
	which can be directly used with \code{uvspec}: 
	\fcode{
	aerosol_species_library OPAC
	}
	OPAC contains following aerosol species: 
	\fcode{INSO   insoluble \\
	WASO   water_soluble\\
	SOOT   soot\\
	SSAM   sea_salt_accumulation_mode\\
	SSCM   sea_salt_coarse_mode\\
	MINM   mineral_nucleation_mode\\
	MIAM   mineral_accumulation_mode\\
	MICM   mineral_coarse_mode\\
	MITR   mineral_transported\\
	SUSO   sulfate_droplets\\
	}
  Variations of the mineral aerosol species containing nonspherical particles are:
  \fcode{
    MINM_SPHEROIDS   mineral_nucleation_mode\\
    MIAM_SPHEROIDS   mineral_accumulation_mode\\
    MICM_SPHEROIDS   mineral_coarse_mode\\
    MITR_SPHEROIDS   mineral_transported
  }
  The aspect ratio distribution from \citep{kandler2009} is adapted.
	''',
	'aerosol_ssa_file'	: r'''
	Location of aerosol single scattering albedo file. 
	\fcode{
	   aerosol_ssa_file file
	}
	The file must have two columns.
	Column 1 is the altitude in km. The altitude grid must be exactly equal to 
	the altitude grid specified in the file \code{atmosphere_file}.
	Column 2 is the single scattering albedo of each layer. 
	The single scattering albedo defined with this option is constant with wavelength. If you require spectral dependence please use \code{aerosol_files}.
	Comments start with \code{\#}. Empty lines are ignored.
	''',
	'aerosol_tau_file'	: r'''
	Location of aerosol optical depth file. 
	\fcode{
	   aerosol_tau_file file
	}
	The file must have two columns.
	Column 1 is the altitude in km. The altitude grid must be exactly equal to 
	the altitude grid specified in the file \code{atmosphere_file}.
	Column 2 is the aerosol optical depth of each layer. 
	To allow wavelength-dependent aerosol optical thickness please
	use either \code{aerosol_angstrom} or \code{aerosol_files}.
	Comments start with \code{\#}. Empty lines are ignored.
	''',
	'aerosol_visibility'	: r'''
	Horizontal visibility in km. Affects the profile according to \citet{shettle89}
	and the optical thickness. 
	\fcode{
	   aerosol_visibility value
	}
	''',
	'aerosol_vulcan'	: r'''
	Aerosol situation above 2 km as defined in \citet{shettle89}.
	\fcode{
	   aerosol_vulcan value
	}
	where \code{value} is an integer choosing between the following models
	\begin{description}
	\item[1] Background aerosols.
	\item[2] Moderate vulcanic aerosols.
	\item[3] High vulcanic aerosols.
	\item[4] Extreme vulcanic aerosols.
	\end{description}
	'''
}
