
"""--------------------------------------------------------------------
 * $Id: ic_options.py 2803 2012-09-10 20:17:52Z robert.buras $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""

from option_definition import *

class ic_habit_special_logic(option):
	def canEnable(self, is_set, get_value):
		r = option.canEnable(self, is_set, get_value)
		if is_set("ic_properties") and \
			    get_value("ic_properties")[0] in ("yang", "hey",
							   "key"):
			return r
		else:
			return False

class setup_ic_group():

	group_name = 'Ice cloud'

	def __init__(self):
		ic_documentation = get_ic_documentation()
		#uvspec function to initialize new profile
		isp = addSetting( 'isp', 'get_caoth_index(&Input.caoth,&Input.n_caoth,"ic",1)')
		
		ic_file = option( #TODO: optional argument is first argument, then comes obligatory one
			'ic_file',	#name
			'ic',	#group
			'ic_definitions',	#supgroup
			'Location of file defining ice cloud properties.',	#helpstr
			ic_documentation['ic_file'],	#documentation	
			addToken( 'Input.caoth[isp].filename', file ),	#tokens
			[isp, 
			 addSetting('Input.caoth[isp].source', 'CAOTH_FROM_FILE') ],	#additional settings
			['uvspec'],	#parents
			['ic_files','ic_ipa','ic_ipa_files','mc_icloud_file'],	#enemies  (non_parents)
			['ic_set_tau','ic_set_tau550','ic_set_ssa','ic_set_gg',\
				 'ic_habit',\
				 'ic_scale_tau','ic_scale_ssa','ic_scale_gg',\
				 'ic_level','ic_layer',\
				 'ic_properties','ic_properties_interpolate',\
				 'ic_no_scattering','ic_cloudcover'],	#children
			plot = {'plot_type': '2D',
				'optional_args': {'column_names': (
						"altitude",
						"ice water content",
						"effective radius",)
						  }
				}
		)

		ic_files = option(
			'ic_files',	#name
			'ic',      #group
			'ic_definitions',      #supgroup
			'Location of file specifying cloud extinction coefficient, single scattering albedo, and scattering phase function for each layer.',	#helpstr
			ic_documentation['ic_files'],     #documentation
			addToken( 'Input.caoth[isp].filename', file),	#tokens
			[isp, 
			 addSetting('Input.caoth[isp].source', 'CAOTH_LAYER_FILES'),
			 addSetting('Input.caoth[isp].properties', 'PROP_EXPLICIT')
			 ],     #additional settings
			['uvspec'],     #parents
			['ic_file','ic_ipa','ic_ipa_files','ic_properties','ic_properties_interpolate',\
				 'mc_icloud_file','ic_cloudcover'],     #enemies
			['ic_set_tau','ic_set_tau550','ic_set_ssa','ic_set_gg',\
				 'ic_scale_tau','ic_scale_ssa','ic_scale_gg',\
				 'ic_level','ic_layer',\
				 'ic_no_scattering']	#children
		)

		ic_ipa = option(
			'ic_ipa',	#name
			'ic',      #group
			'ic_definitions',	#supgroup
			'Independent column calculation for a 3D cloud field.',	#helpstr
			ic_documentation['ic_ipa'], #documentation
			addToken( 'Input.caoth[isp].filename', file),	#tokens
			[isp,
			 addSetting('Input.caoth[isp].source', 'CAOTH_FROM_3D_FILE'),
			 addSetting('Input.caoth[isp].ipa', 1),
			 addSetting('Input.ipa', 1, default=0)
			 ],     #additional settings
			['uvspec'],     #parents
			['ic_file','ic_files','ic_ipa_files','mc_icloud_file',\
				 'ic_cloudcover'],     #enemies
			['ic_set_tau','ic_set_tau550','ic_set_ssa','ic_set_gg',\
				 'ic_habit',\
				 'ic_scale_tau','ic_scale_ssa','ic_scale_gg',\
				 'ic_level','ic_layer',\
				 'ic_properties','ic_properties_interpolate',\
				 'ic_no_scattering'],	#children
			mystic = True
		)

		ic_ipa_files = option(
			'ic_ipa_files',	#name
			'ic',      #group
			'ic_definitions',      #supgroup
			'Location of file defining ice cloud property files and the corresponding weights.',	#helpstr
			ic_documentation['ic_ipa_files'], #documentation
			addToken('Input.caoth[isp].filename', file),  #datatype
			[isp,
			 addSetting('Input.caoth[isp].source', 'CAOTH_IPA_FILES'),
			 addSetting('Input.caoth[isp].ipa', 1),
			 addSetting('Input.ipa', 1, default=0),
			 addSetting('Input.caoth[isp].properties', 'PROP_IC_FU')
			 ],     #additional settings
			['uvspec'],     #parents
			['ic_file','ic_files','ic_ipa','mc_icloud_file','ic_cloudcover'],     #enemies
			['ic_set_tau','ic_set_tau550','ic_set_ssa','ic_set_gg',\
				 'ic_habit',\
				 'ic_scale_tau','ic_scale_ssa','ic_scale_gg',\
				 'ic_level','ic_layer',\
				 'ic_properties','ic_properties_interpolate',\
				 'ic_no_scattering']	#children
		)

		ic_cloudcover = option(
			'ic_cloudcover', #name
			'ic',      #group
			'ic_modifications',	#supgroup
			'Fraction of the horizontal sky area which is covered by clouds.',	#helpstr
			ic_documentation['ic_cloudcover'],     #documentation
			addToken( 'Input.caoth[isp].cloudcover', float, valid_range=[0,1] ),	# tokens
			[isp, 
			 addSetting( 'Input.caoth[isp].ipa', 1 ),
			 addSetting( 'Input.ipa', 1, default=0 )
			 ],	#additional settings for uvspec
			['ic_file','mc_icloud_file'],     #parents
			[],     # enemies
			[]      #children
		)

		ic_properties = continious_option(
			'ic_properties',	#name
			'ic',      #group
			'parameterization_def',      #supgroup
			'Define how ice water content and effective droplet radius are translated to optical properties. ',	#helpstr
			ic_documentation['ic_properties'], #documentation
			addToken( 'Input.caoth[isp].prop_name', str,
				  valid_range=['fu', 'echam4', 'mie', 'yang', 'key', 'baum_hufit',
					       'baum', 'hey', file]),     #tokens
			None,   #additional settings
			['ic_file','ic_ipa','ic_ipa_files','mc_icloud_file'],     #parents
			[],     #enemies
			['ic_properties_interpolate'],      #childs
			extra_dependencies=['ic_habit'],
			)

		ic_properties_interpolate = option(
			'ic_properties_interpolate',	#name
			'ic',      #group
			'parameterization_mod',	#supgroup
			'Interpolate ice cloud optical properties.',	#helpstr
			ic_documentation['ic_properties_interpolate'], #documentation
			None, #tokens
			[isp,
			 addSetting( 'Input.caoth[isp].interpolate', 1) ],     #additional settings
			['ic_properties'],     #parents
			[],     #enemies
			[]      #childs
		)

		ic_habit = ic_habit_special_logic(
			'ic_habit',	#name
			'ic',      #group
			'parameterization_def',      #supgroup
			'Ice crystal habit for the \citet{Yang2000}, \citet{Key2002} and \code{hey} parameterizations.',       #helpstr
			ic_documentation['ic_habit'], #documentation
			addToken( 'Input.caoth[isp].ic_habit_name', str,
				  valid_range=['solid-column', 'hollow-column', 'rough-aggregate', 'rosette-4', 'rosette-6', 
					       'plate', 'droxtal']) ,     #tokens
			isp,     #additional settings
			['ic_file','ic_ipa','ic_ipa_files','mc_icloud_file'],     #parents
			[],     #enemies
			['ic_properties_interpolate']      #childs
			)

		ic_set_gg = option(
			'ic_set_gg',	#name
			'ic',      #group
			'parameterization_mod',      #supgroup
			'Set the ice cloud asymmetry factor.',	#helpstr
			ic_documentation['ic_set_gg'], #documentation
			addToken( 'Input.caoth[isp].gg', float, valid_range=[-1,1] ),  #tokens
			isp,     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files'],     #parents
			'',     #non_parents
			[],      #childs
			)

		ic_set_ssa = option(
			'ic_set_ssa',	#name
			'ic',      #group
			'parameterization_mod',      #supgroup
			'Set the ice cloud single scattering albedo.',	#helpstr
			ic_documentation['ic_set_ssa'], #documentation
			addToken( 'Input.caoth[isp].ssa', float, valid_range=[0,1e6] ),  #tokens
			isp,     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files'],     #parents
			'',     #non_parents
			[],      #childs
		)

		ic_set_tau = option(
			'ic_set_tau',	#name
			'ic',      #group
			'parameterization_mod',      #supgroup
			'Set the total ice cloud optical thickness.',	#helpstr
			ic_documentation['ic_set_tau'], #documentation
			addToken( 'Input.caoth[isp].tau', float, valid_range=[0,1e6]),  #tokens
			isp,     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files'],     #parents
			['ic_set_tau550'],     #enemies
			[],      #childs
			)

		ic_set_tau550 = option(
			'ic_set_tau550',	#name
			'ic',      #group
			'parameterization_mod',      #supgroup
			'Set the ice cloud optical thickness at 550nm.',	#helpstr
			ic_documentation['ic_set_tau550'], #documentation
			addToken( 'Input.caoth[isp].tau550', float, valid_range=[0,1e6] ),  #tokens
			isp,     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files'],     #parents
			['ic_set_tau'],     #enemies
			[]      #childs
	   	)
	
		ic_scale_gg = option(
			'ic_scale_gg',	#name
			'ic',      #group
			'parameterization_mod',      #supgroup
			'Scale the ice cloud asymmetry factor.',	#helpstr
			ic_documentation['ic_scale_gg'], #documentation
			addToken( 'Input.caoth[isp].scale_gg', float, valid_range = [0,1e6]),  #tokens
			isp,     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files'],     #parents
			[],     #non_parents
			[],      #childs
		)
		
		ic_scale_ssa = option(
			'ic_scale_ssa',	#name
			'ic',      #group
			'parameterization_mod',      #supgroup
			'Scale the ice cloud single scattering albedo.',	#helpstr
			ic_documentation['ic_scale_ssa'], #documentation
			addToken( 'Input.caoth[isp].scale_ssa', float, valid_range=[0,1e6] ),  #tokens
			isp,     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files'],     #parents
			[],     #non_parents
			[],      #childs
		)
		
		ic_scale_tau = option(
			'ic_scale_tau',	#name
			'ic',      #group
			'parameterization_mod',      #supgroup
			'Scale the ice cloud optical thickness.',	#helpstr
			ic_documentation['ic_scale_tau'], #documentation
			addToken( 'Input.caoth[isp].scale_tau', float, valid_range = [0,1e6]),  #tokens
			isp,     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files','mc_icloud_file'],     #parents
			[],     #non_parents
			[],      #childs
		)
		
		ic_layer = option(
			'ic_layer',	#name
			'ic',      #group
			'microphysics_mod',	#supgroup
			'Interpret cloud properties as layer properties.',	#helpstr
			ic_documentation['ic_layer'], #documentation
			None, #tokens
			[isp, 
			 addSetting('Input.caoth[isp].layer', 'TRUE')],     #additional settings
			['ic_file','ic_files', 'ic_ipa', 'ic_ipa_files','mc_icloud_file'],     #parents
			['ic_level'],     #non_parents
			[],      #childs
		)
		
		ic_level = option(
			'ic_level',	#name
			'ic',      #group
			'microphysics_mod',	#supgroup
			'Interpret cloud properties as level properties.',	#helpstr
			ic_documentation['ic_level'], #documentation
			None, #tokens
			[isp,
			 addSetting( 'Input.caoth[isp].layer', 'FALSE') ],     #additional settings
			['ic_file','ic_files','ic_ipa', 'ic_ipa_files','mc_icloud_file'],     #parents
			['ic_layer'],     #non_parents
			[],      #childs
		)
		
		ic_no_scattering = option(
			'ic_no_scattering',	#name
			'ic',      #group
			'parameterization_mod',	#supgroup
			'Switch off scattering by ice clouds.',	#helpstr
			ic_documentation['ic_no_scattering'], #documentation
			None, #tokens
			[isp,
			 addSetting('Input.caoth[isp].no_scattering', 'TRUE') ],     #additional settings
			['ic_file','ic_files','ic_ipa','ic_ipa_files','mc_icloud_file'],     #parents
			[],     #non_parents
			[],      #childs
		)
	
		mc_icloud_file = conditional_enable_option(
			name='mc_icloud_file', 
			group='ic',
			subgroup='ic_definitions',
			helpstr='Location of 3D file defining ice cloud properties.',
			documentation=ic_documentation['mc_icloud_file'],     
			tokens=addToken( 'Input.caoth[isp].filename', file ),
			settings=[isp,
			 addSetting('Input.caoth[isp].source', 'CAOTH_FROM_3D_FILE') ],
			parents=['uvspec'], 
			childs=['ic_habit','ic_scale_tau',\
				 'ic_level','ic_layer',\
				 'ic_properties','ic_properties_interpolate',\
				 'ic_no_scattering','ic_cloudcover'],
			non_parents=['ic_file','ic_files','ic_ipa','ic_ipa_files'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True,
		)



		self.options = [ic_file,ic_files,ic_ipa,ic_ipa_files,mc_icloud_file,ic_cloudcover,
				ic_properties,ic_properties_interpolate,
				ic_habit,
				ic_set_tau,ic_set_tau550,ic_set_ssa,ic_set_gg,
				ic_scale_tau,ic_scale_ssa,ic_scale_gg,
				ic_level,ic_layer,
				ic_no_scattering]

	def __iter__(self):
		return iter(self.options)

def get_ic_documentation():
	return {
		'ic_cloudcover'	: r'''
				Set the fraction of the horizontal sky area which is covered by clouds.
				\fcode{
				ic_cloudcover value
				}
				When a cloud cover is specified, the result will be calculated by
				the independent pixel approximation (IPA), that is, as weighted average 
				of cloudless sky and overcast sky, where the cloud properties are taken 
				from \code{ic_file}, etc. Please note that, if both \code{wc_cloudcover} and
				\code{ic_cloudcover} are set, both must be equal.

				This option is ignored, if the option \code{cloud_fraction_file} is used.
				''',
		'ic_file'	: r'''
				\option{ic_file}
				Location of file defining ice cloud properties. 
				\fcode{
				ic_file file
				}
				The file must contain three columns. Column 1 is the altitude in km,
				column 2 the ice water content in grams per cubic meter, and column 3
				the effective particle radius in micrometer. Empty lines are
				ignored. Comments start with \code{\#}. Note that the definition of
				cloud altitudes in \code{ic_file} refers to sea level, not to altitude
				above ground. E.g., when \code{altitude} is set to 1.63km, and the
				first cloud level is defined at 3km, the cloud would start at 1.37km
				above ground. An example of an ice cloud is given in
				\file{examples/IC.DAT}.

				Per default the cloud properties are interpreted as layer
				properties. Before version 1.4 the default was level properties: The
				optical depth of a layer was calculated using information from the
				upper and lower levels defining the layer, see \code{ic_layer} and
				\code{ic_level}.  To switch to the old behaviour, use
				\code{ic_level}. See section~\ref{seq:ice_clouds} about ice clouds for
				a realistic example how the contents of the \code{ic_file} are
				converted to optical properties.
				''',
		'ic_files'	: r'''
				A way to specify ice cloud optical depth, single scattering albedo, 
				and phase function moments for each layer. 
				\fcode{
				ic_files file
				}
				The file specified by 
				\code{ic_files} has two columns where column 1 is the altitude in km. The second
				column is the name of a file which defines the optical properties of the level 
				starting at the given altitude. The files specified in the second column must 
				have the following format:
				\begin{description}
				\parameter{Column 1: } 
				The wavelength in nm. These wavelengths may be different from those in \code{solar_file}. 
				Optical properties are interpolated to the requested wavelengths.
				\parameter{Column 2:} 
				The extinction coefficient of the layer in units km-1. 
				\parameter{Column 3:} 
				The single scattering albedo of the layer.
				\parameter{Column 4-(nmom+4):} 
				The moments of the scattering phase function. 
				\end{description}
				Note that if using \code{rte_solver cdisort} or \code{rte_solver
				fdisort2} it makes good sense to make the number of moments larger
				than \code{nstr} because all moments are used in the calculation. For
				\code{rte_solver fdisort1} and \code{rte_solver polradtran} the number
				of moments included in the calculations will be \code{nstr}+1. Higher
				order moments will be ignored for these solvers.  Please note that the
				uppermost line of the \code{ic_files} denotes simply the top altitude
				of the uppermost layer. The optical properties of this line are
				consequently ignored. There are two options for this line: either an
				optical property file with zero optical thickness is specified
				or "NULL" instead.
				''',
		'ic_habit'	: r'''
				Ice crystal habit for the \citet{Yang2000}, \citet{Key2002} and
				\code{hey} parameterizations, see also  \code{ic_properties key/yang/hey}.
				\fcode{
				ic_habit type
				} 
				For Key/Yang \code{type} may be one of \code{solid-column}, 
				\code{hollow-column}, \code{rough-aggregate}, \code{rosette-4}, 
				\code{rosette-6}, \code{plate}, \code{droxtal},
				and \code{spheroid}. Please note that this parameterization is only valid for
				a restricted size range, depending on the habit (see table 1 in 
				\citet{Key2002}. Also, some of the habits are only available for
				wavelengths below 5 micrometer (\code{rosette-4}) while others are only available
				for wavelengths larger than 3 micrometer (\code{droxtal}, \code{spheroid}).
				For \code{hey} the following habits can be chosen: \code{solid-column}, 
				\code{hollow-column}, \code{rough-aggregate}, \code{rosette-6}, 
				\code{plate}, and \code{droxtal}; here all
				habits are available for effective radii from 5 to 90 micrometers in
				the wavelength region from 0.2 to 5 micrometers.
				''',
		'ic_ipa'	: r'''
				Independent column calculation for a 3D cloud field.
				\fcode{
				ic_ipa file
				}
				As argument a name of a 3D cloud file must be given. This file has to 
				be in the format as needed by MYSTIC, see \code{mc_wcloud_file}.
				''',
		'ic_ipa_files'	: r'''
				A two-column file, defining ice cloud property files (see \code{ic_file}) in the first 
				column and the correspoding weights in the second column. 
				\fcode{
				ic_ipa_files file
				}
				The radiative transfer calculation is performed independently for each
				cloud column and the result is the weighted average of all independent
				columns. If \code{ic_ipa_files} and \code{wc_ipa_files} are both
				defined, both must have the same columns in the same order, otherwise
				\code{uvspec} will complain. See
				\file{examples/UVSPEC_WC_IC_IPA_FILES.INP} for an example.
				''',
		'ic_layer'	: r'''
				Interpret ice cloud properties as layer properties (this is the
				default behaviour since version 1.4; see also \code{ic_file}). Cloud
				properties are assumed to be constant within each layer. The layer
				reaches from the level where the properties are defined in the
				\code{ic_file} to the level above that one. For example, the
				following lines
				\begin{Verbatim}
				#      z     IWC    R\_eff
				#     (km)  (g/m^3) (um)    
				       4.000   0.0   0.0
				       3.000   1.0  10.0
				\end{Verbatim}
				define a cloud in the layer between 3 and 4 km with sharp boundaries.
				''',
		'ic_level'	: r'''
				Interpret cloud properties as level properties (this was the default
				behaviour before version 1.4; see also \code{ic_file}). If
				\code{ic_level} is defined, a \code{ic_file} would be interpreted as
				follows:
				\begin{Verbatim}
				#      z     IWC    R\_eff
				#     (km)  (g/m^3) (um)
				      5.000   0      0    
				      4.000   0.2    12.0 
				      3.000   0.1    10.0 
				      2.000   0.1     8.0 
				\end{Verbatim}
				The value 0.2 g/m$^3$ refers to altitude 4.0km, as e.g. in a
				radiosonde profile. The properties of each layer are calculated as
				average over the adjacent levels. E.g. the single scattering
				properties for the model layer between 3 and 4km are obtained by
				averaging over the two levels 3km and 4km. To allow easy definition of
				sharp cloud boundaries, clouds are only formed if both liquid water
				contents above and below the respective layer are larger than
				0. Hence, in the above example, the layers between 2 and 3 as well as
				between 3 and 4km are cloudy while those between 1 and 2km and between
				4 and 5km are not.
				''',
		'ic_no_scattering'	: r'''
				Switch off scattering by ice clouds.
				''',
		'ic_properties'	: r'''
				Defines how ice water content and effective particle radius are translated 
				to optical properties. 
				\fcode{
				ic_properties type
				}
				Possible choices for \code{type} are
				\begin{description}
				\item[fu]
				Parameterization by \citet{Fu1996,Fu1998}, see \code{ic_file}; 
				this is the default setting. Note that this is a parameterization
				which has been created to calculate fluxes but not radiances. 
				Note also that the optical properties in the solar range provided by 
				\citet{Fu1996} are delta-scaled properties (that is, the forward peak of
				the phase function is truncated and optical thickness, asymmetry
				parameter, and single scattering albedo are reduced accordingly),
				whereas \code{uvspec} uses non delta-scaled properties unless the option
				\code{ic_fu_tau scaled} is specified. By default the parameterization
				by \citet{Fu1996} is treated consistently with all other ice cloud
				parameterizations. 
				For wavelengths up to 4 micrometer \citet{Fu1996} is used while for wavelengths 
				larger than 4 micrometer \citet{Fu1998} is chosen. Please note that 
				\citet{Fu1996} is based on ray-tracing calculations while \citet{Fu1998}
				is a mixture of ray-tracing and Mie calculations (which is required for 
				the infrared wavelengths where the geometrical assumption does not hold).
				Hence, both parameterizations are not fully consistent. Rather, differences
				of some \% are to be expected in the wavelength region where both 
				parameterizations overlap. Also, the wavelength dependence in the solar 
				and infrared parts is treated differently: In the solar part \citep{Fu1996}
				the optical properties are defined for wavelength bands - hence they 
				are assumed constant within each band. In the infrared \citep{Fu1998} 
				they are defined at certain wavelengths and linearely interpolated 
				in between. If you use this option, please see also the
				discussion of \code{ic_fu_tau} and \code{ic_fu_reff}.
				The allowed range for the effective radius is from 9.315 -  65.120 micrometer. 
				\item[echam4]
				Use the simple two-band parameterization of the ECHAM4 climate model, described 
				in \cite{Roeckner1996}; this is probably only meaningful if you want to compare
				your results with ECHAM4, the two bands are 0.2 - 0.68 micrometer and 0.68 - 4.0 micrometer.
				Within the two ECHAM4 bands, the optical properties are assumed constant.
				\item[key]
				Parameterization by \citet{Key2002}. This parameterization can also
				be used to calculate radiances because it uses a
				double-Henyey-Greenstein phase function which better represents both
				forward and backward peaks. This parameterization covers the wavelength region  
				from 0.2 to 5.0 micrometer and is available for the following habits:
				solid-column, hollow-column, aggregate, rosette-4, rosette-6, and plate.
				\item[yang]
				Parameterization similar to \citet{Key2002} but based on more recent
				single scattering calculations. Below 3.4 micrometer it actually equals
				the \citet{Key2002} parameterization while from 3.4 - 100 micrometer new 
				coefficients have been calculated with much higher wavelength
				resolution and better accuracy. Hence, \code{yang} should give a reasonably
				consistent approximation from 0.2 - 100 micrometer, suitable for spectrally
				resolved calculations of radiance and irradiance.
				The covered range for the effective radius depends on the \code{ic_habit}.
				(In micrometer: \code{solid-column} [5.96, 84.22], \code{hollow-column} [4.97, 70.24],
				 \code{rough-aggregate} [3.55, 108.10], \code{rosettes-4} [2.77, 45.30], 
				\code{rosettes-6} [2.85, 46.01], \code{plate} [4.87, 48.18], \code{dendrites}
				 [0.45, 1.88], \code{droxtal} [9.48, 293.32], 
				\code{spheroid}  [6.58, 203.39]). 
				\item[baum]
				Use ice cloud parameterization from \citet{baum05a:_bulk,baum05b:_bulk},
				http://www.ssec.wisc.edu/\~baum/Cirrus/IceCloudModels.html.
				In combination with the radiative transfer solvers \code{cdisort}, \ifmystic{\code{montecarlo},}
				 and \code{fdisort2}, accurate phase functions are used.
				\item[baum\_hufit]
				Similar to the option \code{baum} but here the phase function
				is parameterized by 128 Legendre coefficients, calculated with the
				delta-fit method from 
				\citet{Hu2000}. This parameterization covers the region  
				from 0.4 to 2.2 micrometer. If high accuracy is needed e.g. in the vicinity of the halo, 
				the forward peak, or the backscatter peak, \code{ic_properties baum}
				is recommended.
				\item[hey] 
				Use pre-calculated ice cloud optical properties including full phase
				matrices. This option has newly been implemented and is not yet well
				validated. Please check your results carefully!!
				The parameterization
				is currently only available for the spectral region from 0.2 to 5
				micrometers. The single scattering
				properties have been been generated by Hong Gang using the
				models by \citet{Yang2000}. The parameterization is based on
				simple gamma distributions 
				\begin{equation}
				n(r) = n_0 r^{\alpha} \exp\left(-\frac{(\alpha+3)r}{r_e}\right),
				\end{equation}
				where $n_0$ is found by normalization and $\alpha$ is set to 1. In case of
				spherical particles the parameter $r_e$ would be the effective
				radius. For aspherical particles, the parameter $r_e$ is found
				iteratively so that the size distribution yields the required
				effective radius. The
				parameterization is availabe for the following habits: \code{solid-column},
				\code{hollow-column}, \code{rough-aggregate}, \code{rosette-6},
				\code{plate}, and \code{droxtal}. The
				default habit is \code{solid-column}. The habit can be specified using the option
				\code{ic_habit}.
				\item[mie]
				Use pre-calculated Mie tables; useful for \code{correlated_k}; 
				the tables are expected in \code{data_files_path}\file{/correlated_k/.../}.
				%For spectral or pseudo-spectral (\code{correlated_k sbdart}) calculations, 
				%a set of pre-calculated tables is also available; the wavelength grid points of these 
				%data has been carefully selected such that the extinction cross section, 
				%single scattering albedo, and the asymmetry parameter are accurate to 1\% 
				%(compared to the fully-resolved Mie calculation) for all wavelengths
				%between 250nm and 100 micrometer. 
				For spectral or pseudo-spectral
				calculations \code{ic_properties_interpolate} has to be defined explicitely to
				initiate the interpolation of the optical properties to the internal wavelength grid.
				%Please note that this option may be extremely memory-consuming because for each 
				%internal wavelength a full set of Legendre moments of the phase function is 
				%stored (up to several thousands). The Mie tables are not part of the standard distribution 
				%(because of their large size) but they are freely available from http://www.libradtran.org. 
				Note that a Mie calculation assumes spherical ice particles, the scattering function of 
				which differs systematically from non-spherical particles. Hence, \code{ic_properties mie}
				is usually not representative of natural ice clouds.
				\item[filename]
				Read optical properties from specified filename; file format is as produced 
				by the \code{mie} tool of {\sl libRadtran} (see \code{output_user netcdf}). 
				% or by Frank Evans' \code{cloudprp}.
				\end{description}
				The default property is \code{fu}.

				Please note also that, in contrast to spherical particles, there is no unique 
				definition of effective size for non-spherical particles. In particular, the
				above parameterizations use different definitions which, however, differ only by 
				a constant factor. 
				\citet{Yang2000}, cite{Key2002}, and \citet{baum05a:_bulk,baum05b:_bulk}
				 use the general definition 
				\begin{equation}
				   r_{\rm eff} = {{3}\over{4}}{{\int V(h) n(h) dh}\over{\int A(h) n(h) dh}}
				\end{equation}
				  where $h$ is the maximum dimension of an ice crystal, $n(h)$ is the
				  number of particles with maximum dimension $h$ in the size distribution,
				  and $V$ and $A$ are the volume and mean
				  projected area of the particles, respectively. The volume and area are
				  based on the spherical diameter with equivalent volume and the
				  spherical diameter with equivalent projected area as defined by 
				  \citet{Yang2000}. On the other hand, \citet{Fu1996,Fu1998} use
				  hexagonal columns and use the following definition
				\begin{equation}
				   r_{\rm eff} =  {{\int D^2 L n(L) dL}\over{2 \int (D L + {\sqrt{3}\over{4}} D^2) n(L) dL}}
				\end{equation}
				  where $D$ is the width of the ice crystal (that is, the maximum diameter of the 
				  hexagonal area) and $L$ is the length. The integrand in the numerator is proportional 
				  to the volume while that in the denominator is proportional to the projected area. 
				  Evaluating these formulas one finds that, for the same hexagonal particle, the effective
				  radius would be $3 \sqrt{3} / 4 = 1.299$ times larger following the 
				  \citet{Yang2000}, \citet{Key2002} definition rather than the \citet{Fu1996,Fu1998} definition.
				  As an example, an effective radius of 20$\mu m$ with 
				  \code{ic_properties fu} and \code{ic_fu_reff fu} and
				  1.299 $\cdot$ 20$\mu m$ = 26$\mu m$ with \code{ic_properties yang} would give comparable results
				  for hexagonal columns. 
				To use the original definition of the effective radius by \citet{Fu1996,Fu1998} use
				\code{ic_fu_reff fu}!
				''',
		'ic_properties_interpolate'	: r'''
				Interpolate ice cloud optical properties over wavelength; useful for precalculated 
				optical property files defined with \code{ic_properties}. Please note that this option 
				may be extremely memory-consuming because for each internal wavelength a full set of 
				Legendre moments of the phase function is stored (up to several thousands). 
				''',
		'ic_scale_gg'	: r'''
				Scale the ice cloud asymmetry factor for all wavelengths and altitudes
				with a float between 0.0 and 1.0. 
				\fcode{
				ic_scale_gg value
				}
				If you use this option in combination with the ice
				cloud properties by \citet{Fu1996}, please make sure that you understand
				the explanation of \code{ic_fu_tau}.
				''',
		'ic_scale_tau'	: r'''
				Scale the water ice optical thickness for all wavelengths and altitudes.
				with a float between 0.0 and 1000000.0. Also works for 3d clouds.
				\fcode{
				ic_scale_tau value
				}
				If you use this option in combination with the ice
				cloud properties by \citet{Fu1996}, please make sure that you understand
				the explanation of \code{ic_fu_tau}.
				''',
		'ic_scale_ssa'	: r'''
				Scale the ice cloud single scattering albedo for all wavelengths and altitudes
				with a float between 0.0 and 1.0. 
				\fcode{
				ic_scale_ssa value
				}
				If you use this option in combination with the ice
				cloud properties by \citet{Fu1996}, please make sure that you understand
				the explanation of \code{ic_fu_tau}.
				''',
		'ic_set_gg'	: r'''
				Set the ice cloud asymmetry factor for all wavelengths and altitudes
				to a float between -1.0 and 1.0. 
				Please note that this option is only applied if a Henyey-Greenstein 
				phase function is used but not if an explicit phase function is 
				defined e.g. with a ic\_files. It doesn't make sense to modify only 
				the first moment of an explicit phase function.  
				\fcode{
				ic_set_gg value
				}
				If you use this option in combination with the ice
				cloud properties by \citet{Fu1996}, please make sure that you understand
				the explanation of \code{ic_fu_tau}.
				''',
		'ic_set_ssa'	: r'''
				Set the ice cloud single scattering albedo for all wavelengths
				and altitudes to a value between 0.0 and 1.0. 
				\fcode{
				ic_set_ssa value
				}
				If you use this option
				in combination with the ice cloud properties by \citet{Fu1996}, please make
				sure that you understand the explanation of \code{ic_fu_tau}.
				''',
		'ic_set_tau'	: r'''
				Set the total ice cloud optical depth to a constant value for all wavelengths.
				\fcode{
				ic_set_tau value
				}
				The optical thickness defined here is the integral from the surface at the 
				user-defined \code{altitude} to TOA (top of atmosphere). This option is useful 
				only for monochromatic calculations or in wavelength regions 
				where the optical properties of ice clouds can be considered constant, e.g. the 
				ultraviolet region. If you use this option in combination with the ice
				cloud properties by \citet{Fu1996}, please make sure that you understand
				the explanation of \code{ic_fu_tau}.
				''',
		'ic_set_tau550'	: r'''
				Set the ice cloud optical thickness at 550nm. Other wavelengths are scaled accordingly.
				\fcode{
				ic_set_tau550 value
				}
				The optical thickness defined here is the integral from the surface at the 
				user-defined \code{altitude} to TOA (top of atmosphere). 
				Note that this option requires for technical reasons that the
				wavelength interval defined by \code{wavelength} does contain 550nm.
				If you use this option in combination with the ice
				cloud properties by \citet{Fu1996}, please make sure that you understand
				the explanation of \code{ic_fu_tau}.

				''',
		'mc_icloud_file' : r'''
				Define a MYSTIC 3D ice cloud input file.
				\fcode{
				mc_icloud_file file
				}
				The format of the 3D ice cloud file is identical to the 3d water cloud file 
				(see \code{mc_wcloud_file}).
				'''
	}
