"""--------------------------------------------------------------------
 * $Id: mc_options.py 2803 2012-09-10 20:17:52Z robert.buras $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""

from option_definition import *

class setup_mc_group():
	
	group_name = 'Monte Carlo'

	def __init__(self):
		mc_documentation = get_mc_documentation()

		mc_escape = conditional_not_yet_lex2py_option(
			name='mc_escape',
			group='mc',
			helpstr='Calculate MYSTIC radiances via escape probabilities.', 
			documentation=mc_documentation['mc_escape'],
			tokens=addToken("", str),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
			)
		
		mc_vroom = conditional_not_yet_lex2py_option(
			name='mc_vroom',
			group='mc',
			helpstr='Use Variance Reduction Methods in MYSTIC.', 
			documentation=mc_documentation['mc_vroom'],
			tokens=addToken("", str),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
			)
				
		mc_visualize = conditional_not_yet_lex2py_option(
			name='mc_visualize',
			group='mc',
			helpstr='Switch on OpenGL visualization for MYSTIC.', 
			documentation=mc_documentation['mc_visualize'],
			tokens=addToken("", str),
			parents=['uvspec'],
			childs=['mc_hiddenline'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)
				
		mc_hiddenline = conditional_not_yet_lex2py_option(
			name='mc_hiddenline',
			group='mc',
			helpstr='Switch on hidden line removal for the MYSTIC online visualization.', 
			documentation=mc_documentation['mc_hiddenline'],
			tokens=addToken("", str),
			parents=['mc_visualize'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)
				
		mc_truncate = conditional_not_yet_lex2py_option(
			name='mc_truncate',
			group='mc',
			helpstr='Truncate phase function at the specified polar angle mu.', 
			documentation=mc_documentation['mc_truncate'],
			tokens=addToken("", str),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)
				
		mc_backward = conditional_not_yet_lex2py_option(
			name='mc_backward',
			group='mc',
			helpstr='Backward tracing of photons.', 
			documentation=mc_documentation['mc_backward'],
			tokens=addToken("", str),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True,
			childs=['mc_backward_output', 'mc_backward_writeallpixels',
				'mc_backward_writeback','mc_panorama_view', 'mc_sensordirection',
				'mc_sensorposition', 'mc_reference_to_nn','mc_bw_umu_file'
				]
			)

		mc_sensordirection = conditional_not_yet_lex2py_option(
			name='mc_sensordirection',
			group='mc',
			helpstr='Define viewing direction of an irradiance sensor.', 
			documentation=mc_documentation['mc_sensordirection'],
			tokens=addToken("", str),
			parents=['mc_backward'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)

		mc_sensorposition = conditional_not_yet_lex2py_option(
			name='mc_sensorposition',
			group='mc',
			helpstr='Define the position of a sensor.', 
			documentation=mc_documentation['mc_sensorposition'],
			tokens=addToken("", str),
			parents=['mc_backward'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)

		mc_bw_umu_file = conditional_not_yet_lex2py_option(
			name='mc_bw_umu_file',
			group='mc',
			helpstr='Define a different umu and phi for each pixel.', 
			documentation=mc_documentation['mc_bw_umu_file'],
			tokens=addToken("", str),
			parents=['mc_backward'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)

		mc_sample_grid = conditional_not_yet_lex2py_option(
			name='mc_sample_grid',
			group='mc',
			helpstr='Sample grid size (Nx Ny [dx dy]) for MYSTIC.', 
			documentation=mc_documentation['mc_sample_grid'],
			tokens=addToken("", str),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True,
			)

		mc_panorama_view = conditional_not_yet_lex2py_option(
			name='mc_panorama_view',
			group='mc',
			helpstr='Simulate a panorama.', 
			documentation=mc_documentation['mc_panorama_view'],
			tokens=addToken("", str),
			parents=['mc_backward'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True,
			)

		mc_sample_cldprp = conditional_enable_option(
			name='mc_sample_cldprp', 
			group='mc',  
			subgroup='mc_modifications', # Is this used anywhere? 20120817ak
			helpstr='Turns on sampling of cloud properties by photons.',
			documentation=mc_documentation['mc_sample_cldprp'],
			settings=[ addSetting('Input.rte.mc.backward.yes', 1, default=0),
				   addSetting('Input.rte.mc.sample_cldprp', 1) ], 
			parents=['mc_backward'],  
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
			)

		mc_reference_to_nn = conditional_enable_option(
			name='mc_reference_to_nn', 
			group='mc',  
			helpstr='The sampled pixels correspond to he surface pixels.',
			documentation=mc_documentation['mc_reference_to_nn'],
			settings=[ addSetting('Input.rte.mc.reference_to_NN', 1, default=0)], 
			parents=['mc_backward'],  
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)
		
		mc_surface_reflectalways = conditional_enable_option(
			name='mc_surface_reflectalways', 
			group='mc',  
			helpstr='The photon is reflected and the albedo is considered by reducing the photon weight.',
			documentation=mc_documentation['mc_surface_reflectalways'],
			settings=[ addSetting('Input.rte.mc.reflectalways', 1, default=0)], 
			parents=['uvspec'],  
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)
		
		mc_spectral_is = conditional_enable_option(
			name='mc_spectral_is', 
			group='mc', 
			subgroup='mc_modifications',	# Is this used anywhere? 20120817ak
			helpstr='Turns on spectral importance sampling.', 
			documentation=mc_documentation['mc_spectral_is'],     
			settings=[addSetting('Input.rte.mc.spectral_is', True, default=False),
			 addSetting('Input.rte.mc.spectral_is_wvl', -1.)], #
			parents=['uvspec'], 
			childs=['mc_spectral_is_wvl'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)

		mc_spectral_is_wvl = conditional_enable_option(
			name='mc_spectral_is_wvl', 
			group='mc', 
			subgroup='mc_modifications',	# Is this used anywhere? 20120817ak
			helpstr='Specify calculation wavelength for spectral importance sampling.',	#
			documentation=mc_documentation['mc_spectral_is'],     #
			tokens=addToken('Input.rte.mc.spectral_is_wvl', float, valid_range=[-1, 1e6]), 
			settings=addSetting('Input.rte.mc.spectral_is', True, default=False), 
			parents=['mc_spectral_is'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
			)

		mc_absorption = conditional_enable_option(
			name='mc_absorption',
			group='mc',
			helpstr='Calculate MYSTIC absorption.',
			documentation=mc_documentation['mc_absorption'],
			tokens=addToken('Input.rte.mc.abs_unit_string', str, valid_range=['W_per_m2_and_dz', 'W_per_m3', 'K_per_day'] , optional = True ),
			settings=addSetting('Input.rte.mc.absorption', 'MCFORWARD_ABS_ABSORPTION', default='MCFORWARD_ABS_NONE'), 
			parents=['uvspec'], 
			non_parents=['mc_backward'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
		)

		mc_actinic = conditional_enable_option(
			name='mc_actinic',
			group='mc',
			helpstr='Calculate MYSTIC actinic flux.', 
			documentation=mc_documentation['mc_actinic'],
			settings=addSetting('Input.rte.mc.absorption', 'MCFORWARD_ABS_ACTINIC', default='MCFORWARD_ABS_NONE'), 
			non_parents=['mc_backward'],
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
		)

		mc_aerosol_is = conditional_enable_option(
			name='mc_aerosol_is',
			group='mc',
			helpstr='Enable aerosol concentration importance sampling.', 
			documentation=mc_documentation['mc_aerosol_is'], 
			tokens=addToken('Input.rte.mc.filename[FN_MC_AERIS]', file),
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_albedo_file = conditional_enable_option(
			name='mc_albedo_file',
			group='mc', 
			helpstr='Define a MYSTIC 2D albedo input file.',
			documentation=mc_documentation['mc_albedo_file'],
			tokens=addToken( 'Input.rte.mc.filename[FN_MC_ALBEDO]', file ),	
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
		)

		mc_albedo_spectral = conditional_enable_option(
			name='mc_albedo_spectral', 
			group='mc', 
			helpstr='',
			documentation=mc_documentation['mc_albedo_spectral'], 
			settings=addSetting('Input.rte.mc.spectral', 1, 0), 
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_ambrals_file = conditional_enable_option(
			name='mc_ambrals_file',
			group='mc',
			helpstr='Define a MYSTIC 2D AMBRALS BRDF input file.',
			documentation=mc_documentation['mc_ambrals_file'], 
			tokens=addToken('Input.rte.mc.filename[FN_MC_AMBRALS]', file),	
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
		)

		mc_azimuth_old = conditional_enable_option(
			name='mc_azimuth_old',
			group='mc',	#
			helpstr='Use old MYSTIC azimuth convention.',
			documentation=mc_documentation['mc_azimuth_old'], 
			settings=addSetting('Input.rte.mc.azimuth', 'MCAZIMUTH_OLD', default ='MCAZIMUTH_NEW'),	
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_backward_output = conditional_enable_option(
			name='mc_backward_output', 
			group='mc',
			helpstr='Specify quantity to be calculated using backward Monte Carlo.', 
			documentation=mc_documentation['mc_backward_output'], 
			tokens=[ addToken('Input.rte.mc.backward.output_string', str, 
					  valid_range=['edir','edn','eup','fdir','fdn','fup','f','abs','emis','heat'] ),	
				 addToken('Input.rte.mc.abs_unit_string', str, 
					  valid_range=['W_per_m2_and_dz', 'W_per_m3', 'K_per_day'] , optional = True ) ], 
			parents=['mc_backward'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_backward_sunshape_file = conditional_enable_option(
			name='mc_backward_sunshape_file', 
			group='mc', 
			helpstr='', 
			documentation=mc_documentation['mc_backward_sunshape_file'],	#
			tokens=addToken('Input.rte.mc.filename[FN_MC_SUNSHAPE_FILE]', file),	#
			parents=['mc_backward'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_backward_writeallpixels = conditional_enable_option(
			name='mc_backward_writeallpixels', 
			group='mc', 
			helpstr='Write all pixels to the output files',
			documentation=mc_documentation['mc_backward_writeallpixels'],	#
			settings=addSetting('Input.rte.mc.backward.writeallpixels', 1),	#
			parents=['mc_backward'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True
		)

		mc_backward_writeback = conditional_enable_option(
			name='mc_backward_writeback',
			group='mc',
			helpstr='Write extra photon information',
			documentation=mc_documentation['mc_backward_writeback'],
			settings=addSetting('Input.rte.mc.backward.writeback', 1),
			parents=['mc_backward'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_basename = conditional_enable_option(
			name='mc_basename', 
			group='mc', 
			helpstr='Filename for MYSTIC 3D output.',
			documentation=mc_documentation['mc_basename'],
			tokens=addToken('Input.rte.mc.filename[FN_MC_BASENAME]',file),
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_bcond = conditional_enable_option(
			name='mc_bcond',
			group='mc',
			helpstr='', 
			documentation=mc_documentation['mc_bcond'],
			tokens=addToken('Input.rte.mc.bcond_string', str, valid_range=['periodic','mirror','absorb'] ), 
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True
		)

		mc_coherent_backscatter = conditional_enable_option(
			name='mc_coherent_backscatter',
			group='mc', 
			helpstr='Switches on coherent backscattering for lidar and radar.',
			documentation=mc_documentation['mc_coherent_backscatter'], 
			settings=addSetting('Input.rte.mc.coherent_backscatter',1,default=0), 
			parents=['mc_lidar','mc_radar'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_delta_scaling = conditional_enable_option(
			name='mc_delta_scaling',	#
			group='mc', 
			helpstr='Truncate phase function in MYSTIC.',	#
			documentation=mc_documentation['mc_delta_scaling'],	#
			tokens=[ addToken('Input.rte.mc.delta_scaling_mucut', float, optional=True),	
				 addToken('Input.rte.mc.delta_scaling_start', int, optional =True) ],#
			settings=[ addSetting('Input.rte.mc.delta_scaling_mucut', 0.99),	
			addSetting('Input.rte.mc.delta_scaling_start', 0) ], 
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_elevation_file = conditional_enable_option(
			name='mc_elevation_file', 
			group='mc',	#
			helpstr='Define a MYSTIC 2D elevation input file.', 
			documentation=mc_documentation['mc_elevation_file'], 
			tokens=addToken('Input.rte.mc.filename[FN_MC_ELEVATION]',file),
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_emission = conditional_enable_option(
			name='mc_emission',	
			group='mc',
			helpstr='Calculate MYSTIC emission.',	
			documentation=mc_documentation['mc_emission'], 
			tokens=addToken('Input.rte.mc.abs_unit_string', str, 
					valid_range=['W_per_m2_and_dz', 'W_per_m3', 'K_per_day'] , optional = True ),	#
			settings=addSetting('Input.rte.mc.absorption', 'MCFORWARD_ABS_EMISSION', default='MCFORWARD_ABS_NONE'),	
			parents=['uvspec'],
			non_parents=['mc_backward'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_heating = conditional_enable_option(
			name='mc_heating', 
			group='mc', 
			helpstr='Calculate MYSTIC heating rates.', 
			documentation=mc_documentation['mc_heating'], 
			tokens=addToken('Input.rte.mc.abs_unit_string', str, 
					valid_range=['W_per_m2_and_dz', 'W_per_m3', 'K_per_day'] , optional = True ), 
			settings=addSetting('Input.rte.mc.absorption', 'MCFORWARD_ABS_HEATING', default='MCFORWARD_ABS_NONE'),	
			parents=['uvspec'],
			non_parents=['mc_backward'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_ipa = conditional_enable_option(
			name='mc_ipa',
			group='mc', 
			helpstr='Run MYSTIC in independent pixel mode.', 
			documentation=mc_documentation['mc_ipa'],
			settings=addSetting('Input.rte.mc.ipa', 1,default=0),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_maxscatters = conditional_enable_option(
			name='mc_maxscatters',
			group='mc',	#
			helpstr='Photons are destroyed after n scatters. For testing only.',
			documentation=mc_documentation['mc_maxscatters'], 
			tokens=addToken('Input.rte.mc.maxscatters', int, default = 'NOT_DEFINED_INTEGER', valid_range=[0,1e6]), 
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_minscatters = conditional_enable_option(
			name='mc_minscatters',
			group='mc',
			helpstr='Local estimates are not performed before n scatters. For testing only',
			documentation=mc_documentation['mc_minscatters'],
			tokens=addToken('Input.rte.mc.minscatters', int, default=0, valid_range=[0,1e6]),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_minphotons = conditional_enable_option(
			name='mc_minphotons',
			group='mc',
			helpstr='Minimum value of photons to be used per simulation.',	
			documentation=mc_documentation['mc_minphotons'],
			tokens=addToken('Input.rte.mc.minphotons', int, default=0, valid_range=[0,1e6]),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_photons = conditional_enable_option(
			name='mc_photons', 
			group='mc', 
			helpstr='Minimum value of photons to be used per simulation.', 
			documentation=mc_documentation['mc_photons'], 
			tokens=addToken('Input.rte.mc.photons', long, default=0, valid_range=[0,1e10]),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_photons_file = conditional_enable_option(
			name='mc_photons_file',
			group='mc', 
			helpstr='Define a MYSTIC spectral photon file.',
			documentation=mc_documentation['mc_photons_file'],
			tokens=addToken( 'Input.rte.mc.filename[FN_MC_PHOTONS]', file ),	
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_polarisation = conditional_enable_option(
			name='mc_polarisation', 
			group='mc', 
			helpstr='Turn on polarisation for Monte Carlo solver.', 
			documentation=mc_documentation['mc_polarisation'], 
			settings=[addSetting('Input.rte.mc.polarisation', 1,default=0),
				  addSetting('Input.rte.mc.reflectalways', 1,default=0)],
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_radial_pathlength = conditional_enable_option(
			name='mc_radial_pathlength', 
			group='mc', 
			helpstr='I3RC case 7',
			documentation=mc_documentation['mc_radial_pathlength'],
			tokens=[ addToken('Input.rte.mc.Nr', int, default=0,valid_range=[0,1e6]),
				 addToken('Input.rte.mc.Nt', int, default=0,valid_range=[0,1e6]) ],
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_radial_pathlength_dt = conditional_enable_option(
			name='mc_radial_pathlength_dt',
			group='mc', 
			helpstr='Time increment for mc_radial_pathlength',
			documentation=mc_documentation['mc_radial_pathlength_dt'],
			tokens=addToken('Input.rte.mc.dt', float, default=1e-7, valid_range=[0,1e6]),
			parents=['mc_radial_pathlength'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_randomseed = conditional_enable_option(
			name='mc_randomseed', 
			group='mc', 
			helpstr='Set random seed for MYSTIC.',
			documentation=mc_documentation['mc_randomseed'], 
			tokens=addToken('Input.rte.mc.randomseed', int, default=0, valid_range=[0,1e15]),
			parents=['uvspec'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_rpv_file = conditional_enable_option(
			name='mc_rpv_file',
			group='mc', 
			helpstr='Define a MYSTIC 2D rpv albedo input file.',
			documentation=mc_documentation['mc_rpv_file'],
			tokens=addToken( 'Input.rte.mc.filename[FN_MC_RPV]', file ),	
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_rpv_type = conditional_enable_option(
			name='mc_rpv_type',
			group='mc', 
			helpstr='Define a MYSTIC 2D rpv albedo type file.',
			documentation=mc_documentation['mc_rpv_type'],
			tokens=addToken( 'Input.rte.mc.filename[FN_MC_RPVTYPE]', file ),	
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_spherical = conditional_enable_option(
			name='mc_spherical',
			group='mc',
			helpstr='Use MYSTIC spherical 1D geometry.', 
			documentation=mc_documentation['mc_spherical'],
			settings=addSetting('Input.rte.mc.spherical', 1, default=0), 
			parents=['uvspec'], 
			childs=['mc_refraction'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_refraction = conditional_enable_option(
			name='mc_refraction',
			group='mc',
			helpstr='Enable refraction.', 
			documentation=mc_documentation['mc_refraction'],
			settings=addSetting('Input.rte.mc.refraction', 1, default=0), 
			parents=['mc_spherical'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			mystic =True

		)

		mc_std = conditional_enable_option(
			name='mc_std',
			group='mc',
			helpstr='Calculate MYSTIC standard deviation.', 
			documentation=mc_documentation['mc_std'],
			settings=addSetting('Input.rte.mc.std', 1, default=0), 
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

		mc_temperature_file = conditional_enable_option(
			name='mc_temperature_file',
			group='mc', 
			helpstr='Define a MYSTIC 2D temperature input file.',
			documentation=mc_documentation['mc_temperature_file'],
			tokens=addToken( 'Input.rte.mc.filename[FN_MC_TEMPERATURE]', file ),	
			parents=['uvspec'], 
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True

		)

	
		self.options = [mc_photons, mc_photons_file, mc_minphotons, mc_randomseed, mc_std, mc_basename, 
				mc_sample_grid, mc_spherical, mc_refraction,
				mc_polarisation,
				mc_absorption, mc_actinic, mc_emission, mc_heating, 
				mc_escape, mc_vroom,
				mc_backward, mc_backward_output,
				mc_panorama_view, mc_sensordirection, mc_sensorposition,
				mc_backward_writeallpixels,
				mc_spectral_is, mc_spectral_is_wvl,
				mc_albedo_file, mc_ambrals_file, 
				mc_elevation_file,
				mc_rpv_file, mc_rpv_type, mc_surface_reflectalways,
				mc_temperature_file,
				mc_delta_scaling, mc_reference_to_nn, mc_bw_umu_file,
				mc_ipa, mc_minscatters, mc_maxscatters, mc_truncate,
				mc_visualize, mc_hiddenline]


## NOT IN USE?????
#aky	for inpOption in [mc_sample_cldprp, mc_spectral_is, mc_spectral_is_wvl]:
#aky		inpOption['mystic'] = True
	
	def __iter__(self):
		return iter(self.options)


def get_mc_documentation():
	return {
		'mc_escape'  : r'''
		       		Calculate MYSTIC radiances via escape probabilities; slows down the tracing 
		       		but usually speeds up the computation considerably since it reduces
		       		noise. Switched on per default since it should basically be used always 
		       		when calculating radiances. Only meaningful with \code{rte_solver montecarlo}. The syntax is
		       		\fcode{
		       		  mc_escape on/off
		       		} 
		       		''',
		'mc_vroom' : r'''
                		Variance Reduction Optimal Options Method (VROOM). Options are "on"
                		and "off". Needs to be specified if you are calculating radiances and
                		spiky phase functions are present in your atmosphere. If you are using
                		VROOM, please cite: \cite{buras2011}.
                		''',
		'mc_backward'  : r'''
		       		Backward tracing of photons. \code{mc_backward} takes either zero, two or four coordinates:
		       		\fcode{
		       		  mc_backward [ix_start iy_start] [ix_end iy_end]
		       		}
		       		where \code{ix_start}, \code{iy_start} is the index of the sample pixel to be calculated
		       		or the pixel area from \code{ix_start} to \code{ix_end} and \code{iy_start} to \code{iy_end}.
		       		All x-indices must be in the range of 0 ... (\code{Nx}-1) and y-indices the range of 0 ... (\code{Ny}-1).
		       		If no coordinates are specified, all sample pixels will be calculated.
		       		\code{mc_backward} computes radiances and downward diffuse irradiances. If 
		       		a different quantity is required, please use \code{mc_backward_output}.
                		''',
		'mc_panorama_view'  : r'''
		       		Simulate a panorama. Only works with \code{mc_sensorposition},
		       		\code{mc_sample_grid} and \code{mc_backward}. Can best be explained by
		       		following example:

		       		\fcode{\\
		       		mc_panorama_view PHI_1 PHI_2 THETA_1 THETA_2\\
		       		mc_sensorposition X Y Z \\
		       		mc_sample_grid NPHI NTHETA [] []\\
		       		mc_backward NPHI1 NTHETA1 NPHI2 NTHETA2\\
		       		}

		       		or, with numbers:

		       		\fcode{\\
		       		mc_panorama_view 45 135 80 180\\
		       		mc_sensorposition 3000 3000 26 \\
		       		mc_sample_grid 90 100 [] []\\
		       		mc_backward 10 0 10 99\\
		       		}

		       		In this example, \code{mc_panorama_view} defines the camera to cover
		       		the area between 10 degrees below the horizon (80 degrees from nadir,
		       		looking into the ground) and the zenith (180 degrees from nadir), and
		       		looking into the directions between south-west (45) and
		       		north-west (135). The camera is positioned
		       		at x=3km, y=3km and z=26m, see \code{mc_sensorposition}. The number of
		       		pixels that the camera has is defined by \code{mc_sample_grid}, hence
		       		the camera horizontal resolution is 1 degree in the
		       		horizon: (135-45)/90. The camera vertical resolution is also 1
		       		degree: (180-80)/100. This run only calculates one vertical line at
		       		phi=55.5 (see \code{mc_backward}). The square brackets in the line
		       		\code{mc_sample_grid} are usually dx and dy, they are set
		       		automatically in the presence of 3d clouds. If no clouds are present,
		       		you need to set them, if you use \code{mc_elevation_file}, you need to
		       		set them consistent with the elevation file. NOTE: If you have
		       		formerly used \code{mc_panorama}, this is what has changed: The order
		       		of the numbers has changed: Formerly, we wrote \code{mc_panorama MU_1
		       		MU_2 PHI_1 PHI_2}, where MU\_i=cos(THETA\_i). In the options
		       		\code{mc_sample_grid} and \code{mc_backward}, X and Y have been
		       		swapped. Although these changes are far from being nice, they are
		       		necessary to introduce a conform nomenclatur throughout
		       		libRadtran. Sorry for that. (CAUTION! The definition of phi for
		       		panoramas was formerly different than that of \code{phi}!  This has
		       		been changed on 19.11.2010!)
		       		''',
		'mc_sensordirection'  : r'''
		       		Define viewing direction of an irradiance sensor in Monte Carlo backward mode. 
		       		\fcode{
		       		mc_sensordirection x-value y-value z-value
		       		}
		       		Has been introduced for 
		       		irradiance calculations in topography and might not properly work with all options. 
		       		For radiance use the usual \code{umu} and \code{phi}. 
		       		''',
		'mc_sensorposition'  : r'''
		       		Define the position of a sensor. 
		       		\fcode{
		       		mc_sensorposition x-value y-value z-value
		       		}
		       		Has been introduced for irradiance calculations in topography
		       		and might not properly work with all options. 
		       		''',
		'mc_sample_grid'  : r'''
		       		Sample grid size (Nx Ny [dx dy]) for MYSTIC.
		       		\fcode{
		       		mc_sample_grid Nx Ny [dx dy]
		       		}
		       		Only meaningful with \code{rte_solver montecarlo}.
		       		''',
		'mc_bw_umu_file'  : r'''
		       		Define a different umu and phi for each pixel. You specify the file
		       		containing a two-dimensional array with umu and phi for each sample
		       		pixel. Only works in connection with \code{mc_backward}.
		       		\fcode{
		       		mc_bw_umu_file file
		       		}
		       		''',
		'mc_sample_cldprp'  : r'''
			       	Switch on sampling of cloud properties (reff/tau/dlwc) by photons. 
		       		This option needs \code{mc_backward}. The optical properties contributing to 
		       		the result for each pixel are integrated using the photon weight 
		       		$p_{weight,n}$ in the following way:
		       		\begin{equation*}
		       		{\langle r_e \rangle}_\textrm{wc}^\textrm{tau}\,=\,\frac{
		       		\sum\limits_{n=0}^{photons}\,p_{weight,n}\,r_{\textrm{wc},n}}
		       		{\sum\limits_{n=0}^{photons}\,p_{weight,n}}\qquad
		       		r_{\textrm{wc}} \,=\, 
		       		\frac{\int_0^\tau\ \tau_{\textrm{w}}r_{\textrm{ew}}\,\mathrm{d}\tau}
		       		{\int_0^\tau \tau_{\textrm{w}}\,\mathrm{d}\tau}
		       		\end{equation*}
		       		For each internal wavelength, one [basename]\_[wvl].cldprp file with 
		       		the following format is created where values marked with top are taken at 
		       		first cloud contact:
		       		\fcode{
		       		ix iy 
		       		${\langle r_e \rangle}^\textrm{tau}_\textrm{wc}$
		       		${\langle r_e \rangle}^\textrm{tau}_\textrm{ic}$
		       		${\langle r_e \rangle}^\textrm{hit}_\textrm{wc}$
		       		${\langle r_e \rangle}^\textrm{hit}_\textrm{ic}$
		       		${\langle \tau \rangle}_\textrm{wc}$
		       		${\langle \tau \rangle}_\textrm{mc}$
		       		$\nabla LWC$ $\nabla IWC$
		       		}	
		       		Values denoted with \textit{hit} are taken at the last cloud contact 
		       		before hitting the sensor, while the the water content gradient is
		       		taken at the last cloud scattering before hitting the sensor (information
		       		about cloud surface orientation).
		       		''',
		'mc_spectral_is': r'''
		       		\iflidar{
		       		{\sl Experimental option!!!!}\\
		       		Calculate a spectrum with high spectral resolution using an importance sampling method. 
		       		The spectral variation of Rayleigh scattering is included, aerosol and cloud
		       		scattering properties are assumed to be constant over the spectral range that is calculated.
		       		Molecular absoprtion coefficients have to be provided in a file using the option}
		       	 	''',
		'mc_reference_to_nn': r'''
		       	 	Only works with \code{mc_backward}. The sampled pixels correspond to
		       	 	the surface pixels. In other words, the photons are still counted at
		       	 	zout, but the sample pixel which they are counted in corresponds to
		       	 	the pixel at sea level. In other words, the sample grid is projected
		       	 	from nn to zout using umu and phi.
		       	 	''',
		'mc_spectral_is_wvl': r'''
		       		\iflidar{
		       		Specify calculation wavelength for spectral importance sampling.
		       		}
		       		''',
		'mc_surface_reflectalways': r'''
		       		Usually, a photon is either absorbed or reflected at the surface, 
		       		with a probability defined by the surface albedo. If 
		       		\code{mc_surface_reflectalways} is specified, each photon is reflected and 
		       		the albedo is considered by reducing the photon weight. In case of BRDF,
		       		\code{mc_surface_reflectalways} is switched on automatically because the 
		       		other method is no longer implemented for non-Lambertian BRDFs, due to 
		       		implementation and numerical problems. For small albedos, the computational 
		       		time is increased if \code{mc_surface_reflectalways} is used; however, 
		       		the accuracy of the upward radiance (reflected by the surface) is increased 
		       		considerably. In case of clouds, however, computational time might be 
		       		increased considerably without gaining accuracy. 
		       		''',
		'mc_truncate': r'''
		       		Truncate phase function at the specified polar angle mu. 
		       		USE ONLY IF YOU REALLY KNOW WHAT YOU ARE DOING!
		       	 	''',
	'mc_absorption' : r'''
	Calculate MYSTIC absorption of each 3D grid box and write it to file \code{mc_basename}.abs.spc 
	in the following format:
	\fcode{
	lambda ix iy iz absorption
	}
	Only meaningful with \code{rte_solver montecarlo}.
	An optional argument \code{W_per_m2_and_dz} (default), \code{W_per_m3}, or \code{K_per_day} may 
	be specified which converts the result e.g. to heating rates. This option is only available 
	for forward calculations. For backward please use \code{mc_backward_output abs}.
	''',
	'mc_actinic' : r'''
	Calculate MYSTIC actinic flux for each 3D grid box by dividing the absorbed energy by the 
	absorption coeffcient; this method is much better than the traditional photon counting 
	which usually comprises spikes (because in the latter method each photon is weighted with 
	$1/\cos(\theta)$ which may be a very large number); may need a considerable amount of memory, 
	depending on the 3D cloud grid. Only meaningful with \code{rte_solver montecarlo}.
	This option is only available for forward calculations. For backward please use 
	\code{mc_backward_output f}. 
	''',
	'mc_aerosol_is' : r'''
	Enable aerosol concentration importance sampling. The input file
	includes one column containing the desired scaling factors: 
	\fcode{
	mc_aerosol_is file
	}
  	''',
	'mc_albedo_file' : r'''
	Define a MYSTIC 2D albedo input file (default: albedo2D.dat).
	\fcode{
	mc_albedo_file file
	}
	The format of the albedo file is:
	\fcode{
	Nx  Ny  dx  dy
	ix  iy  albedo
	...
	}
	where Nx and Ny are the number of grid boxes in x- and y-direction,
	dx and dy are the size of the pixels in km.
	In the second and the following lines the indices in x- and y-direction
	(ix=1...Nx; iy=1...Ny) and the albedo of the pixel are specified. 
	''',
	'mc_albedo_spectral' : r'''
	''',
	'mc_ambrals_file' : r'''
	Define a MYSTIC 2D AMBRALS BRDF input file. 
	\fcode{
	mc_ambrals_file file
	}
	AMBRALS is the 
	Algorithm for Modeling[MODIS] Bidirectional Reflectance Anisotropies 
	of the Land Surface by \citep{wanner97}. See also 
	http://www-modis.bu.edu/brdf/models.html. The format of the albedo file is:
	\fcode{
	Nx  Ny  dx  dy
	ix  iy  iso vol geo
	...
	}
	where Nx and Ny are the number of grid boxes in x- and y-direction,
	dx and dy are the size of the pixels in km.
	In the second and the following lines the indices in x- and y-direction
	(ix=1...Nx; iy=1...Ny) and the three parameters iso, vol, and geo. 
	
	Optionally, it is now possibly to mix AMBRALS with Cox and Munk
	(CaM). To this end, the format of the albedo file is:
	\fcode{
	Nx  Ny  dx  dy
	ix  iy  iso vol geo isCaM
	...
	}
	where isCaM is 1 if CaM shall be used in the pixel, and 0 if AMBRALS
	shall be used. In case of CaM, the values of iso, vol, and geo are
	ignored. Note that this will only work if the CaM parameters are set
	(that is as minimal configuration u10).
	''',
	'mc_azimuth_old' : r'''
	Use old MYSTIC azimuth convention (0 degree = looking from the
	direction of the sun; 180 degree = looking into the direction of the
	sun; that is, exactly opposite to the disort convention). The MYSTIC
	azimuth was changed March 1, 2004 - hence this option was introduced
	for compatibility reasons.
	''',
	'mc_backward_output' : r'''
	Specify quantity to be calculated using backward Monte Carlo. 
	\fcode{
	mc_backward_output output [unit]
	}
	So far the
	following \code{output} options have been implemented: 
	\begin{description}
	\parameter{edir} 
	  direct horizontal irradiance
	\parameter{edn} 
	  diffuse downward irradiance (default)
	\parameter{eup} 
	  diffuse upward irradiance
	\parameter{f} 
	  actinic flux
	\parameter{abs} 
	  absorption
	\parameter{emis} 
	  emission
	\parameter{heat} 
	  heating rates, that is absorption + emission
	\end{description}
	For \code{abs}, \code{emis}, \code{heat} an optional argument \code{W_per_m2_and_dz} (default), 
	\code{W_per_m3}, or \code{K_per_day} may be specified which converts the result e.g. to heating rates. 
	''',
	'mc_backward_sunshape_file' : r'''
	Path to a 2-coloumned text file containing the extraterrestrial sun shape. 1. column:
	A probability density or if you want to a radiance distribution. Does not need
	to be normalized 2. column: Relative angular distance to center from sun inside sun disc,
	values from 0 to 1 (0: center, 1: limb).  Use \code{mc_sun_radius} to manually 
	set the extent of the sun. If filename is set to "default", or if only sun radius is given
	 a spectrally resolved sun shape as in Koepke2001 is used.
	''',
	'mc_backward_writeallpixels' : r'''
	If set, write all pixels to the output files; otherwise, only those are written 
	which are actually calculated.
	''',
	'mc_backward_writeback' : r'''
	If set, the distribution of photons contributing to the result is written 
	to a file with extension .bac which may be useful for some interpretations
	(it basically tells you where the photons come from whichn contribute to the result). 
	''',
	'mc_basename' : r'''
	Filename for MYSTIC 3D output (default: mc).
	\fcode{
	mc_basename basename
	}
	''',
	'mc_bcond' : r'',
	'mc_coherent_backscatter' : r'''
	Switches on coherent backscattering for lidar and radar. Works only 
	with \code{mc_lidar} or \code{mc_radar}, use only with \code{mc_polarisation}.
	''',
	'mc_delta_scaling' : r'''
	Truncate phase function in MYSTIC: The phasefunctions are set to zero
	for mu>mucut. The extinction coefficient etc. are scaled
	accordingly. Optional settings are \code{mc_delta_scaling mucut} and
	\code{mc_delta_scaling mucut n_start}, where mucut is 0.99 by
	default. The value n\_start defines after which scattering order
	delta-scaling is applied, before that scattering order, the real phase
	function is applied. Default is 0, i.e. the delta-scaled phase
	function is applied from the beginning.
	''',
	'mc_elevation_file' : r'''
	Define a MYSTIC 2D elevation input file (default: elevation2D.dat). 
	\fcode{
	mc_elevation_file file
	}
	The 
	expected format of the elevation file is:
	\fcode{
	Nx  Ny  dx  dy
	ix  iy  elevation
	...
	}
	where Nx and Ny are the number of grid boxes in x- and y-direction,
	dx and dy are the size of the grid boxes in km.
	In the second and the following lines the indices in x- and y-direction
	(ix=1...Nx, iy=1...Ny) and the elevation in km of each point are specified.
	
	Attention: While the other files refer to grid boxes,
	the elevation is defined at grid points. It has to be this 
	way because each "elevation pixel" contains a surface 
	which is defined by the four cornes of the pixel. If the
	grid covers an area of 200x200 km$^2$ and the pixel sizes 
	are dx = 1km and dy = 1km, the elevation has to be defined 
	at 201x201 points (ix = 1 .. Nx, iy = 1 .. Ny).
	
	Also, make sure that the elevation doesn't cross or 
	even touch the 1D surface. Since the default altitude is 0
	that means that the minimum elevation must be larger than 0!
	''',
	'mc_emission' : r'''
	Calculate MYSTIC emission of each grid box; may need a considerable amount of memory,
	depending on the 3D cloud grid. Note that emission is calculated directly without tracing 
	any photons which makes this option very fast. Changing \code{mc_photons} will therefore 
	not affect the result. Only meaningful with \code{rte_solver montecarlo} and \code{source thermal}. 
	An optional argument \code{W_per_m2_and_dz} (default), \code{W_per_m3}, or \code{K_per_day} may 
	be specified which converts the result e.g. to heating rates. This option is only available 
	for forward calculations. For backward please use \code{mc_backward_output emis}.
	''',
	'mc_heating' : r'''
	Calculate MYSTIC heating rates for each grid box; may need a considerable amount of memory,
	depending on the 3D cloud grid. Only meaningful with \code{rte_solver montecarlo}. \code{mc_heating}
	is identical to \code{mc_absorption} for \code{source solar}. For \code{source thermal}, however, 
	the emission of a photon in a grid box is counted as cooling. 
	An optional argument \code{W_per_m2_and_dz} (default), \code{W_per_m3}, or \code{K_per_day} may 
	be specified which converts the result e.g. to heating rates. This option is only available 
	for forward calculations. For backward please use \code{mc_backward_output heat}.
	''',
	'mc_ipa' : r'''
	Run MYSTIC in independent pixel mode. Only meaningful with \code{rte_solver montecarlo}.
	''',
	'mc_maxscatters' : r'''
	If set, photons are destroyed after n scatters. Please note that this
	is only for testing, debugging and process understanding! The result
	will certainly be wrong because photons are lost.
	''',
	'mc_minscatters' :  r'''
	If set, local estimates are not performed before n scatters ( e.g. if
	set to 1, single scattering is not taken into account). Please note
	that this is only for testing, debugging and process understanding!
	The result will certainly be wrong.
			''',
		'mc_minphotons' : r'''
			If set, this is the minimum value of photons to be used per
			simulation. The default value is \code{MIN_MCPHOTONS}, set in
			uvspec.h.
			
			\fcode{
			mc_maxscatters value
			}
			''',
		'mc_photons' : r'''
			Total number of photons to be traced by the Monte Carlo solver, MYSTIC.
			\fcode{
			mc_photons value
			}
			Only meaningful with \code{rte_solver montecarlo}.
			''',
		'mc_photons_file' : r'''
                		Distribution of photons over wavelength bands; to be used with \code{correlated_k}.
                		\fcode{
                			mc_photons_file file
                			}
                		For an example see \file{data/correlated_k/kato2/x_solar.dat}. No error checking!
                		Do only use if you are absolutely sure what you are doing.
                		Only meaningful with \code{rte_solver montecarlo}.
                		''',
		'mc_polarisation' : r'''
			Switch on polarisation for \code{rte_solver montecarlo}.
			''',
		'mc_radial_pathlength' : r'''
				I3RC case 7, laser beam experiment. Radiances are sampled in radial 
				and pathlength elements. Specify the number of radial (Nr) and time (Nt) intervals. 
				The radius increment is calculated from Nr and the domain size. The time interval 
				may  be specified with \code{mc_radial_pathlength_dt}. 
				''',
		'mc_radial_pathlength_dt' : r'''
				Specify time increment for \code{mc_radial_pathlength}. Time is converted to pathlength 
				assuming a speed of light of 3$\cdot$10$^8$~m/s
				''',
                'mc_randomseed' : r'''
                		Provide your own random seed (positive integer) for the random number generator. 
                		\fcode{
                			mc_randomseed value
                			}
                		Usually a random seed is determined from current time plus process id. 
                		This option is useful to re-run a simulation for debugging.
                		''',
		'mc_rpv_file' : r'''
				Define a MYSTIC 2D RPV BRDF input file.
				\fcode{
					mc_rpv_file file
				}
				''',
		'mc_rpv_type' : r'''
				File containing wavelength-dependent RPVs to be used in combination with \code{mc_rpv_file}.
				\fcode{
					mc_rpv_type file
				}
				''',
		'mc_spherical' : '''
				Spherical geometry in MYSTIC. Works only in "1D" - \code{wcloud_file} 
				and \code{icloud_file} are not yet considered. If \code{mc_spherical} is selected
				\code{mc_backward} is switched on automatically. 
				''',
		'mc_refraction' : '''
				Enable refraction for \code{rte_solver montecarlo}. Works only 
				in 1D spherical geometry (with option \code{mc_spherical}).
				''',
		'mc_visualize' : r'''
				Switch on OpenGL visualization for MYSTIC. Try the following keys in the graphics window:

				\begin{description}
				  \item[a]  clouds darker
				  \item[A]  clouds brighter
				  \item[h]  decrease displayed domain height
				  \item[H]  increase displayed domain height
				  \item[r]  turn right
				  \item[l]  turn left
				  \item[u]  turn up
				  \item[d]  turn down
				  \item[k]  "Karussell" (rotate continuously)
				  \item[i]  zoom in
				  \item[o]  zoom out
				  \item[f]  fullscreen
				  \item[q]  quit
				  \item[w]  wireframe (frame cloudy pixels)
				  \item[s]  store and release photon paths; press once to start displaying every 10000th photon pass; press again to save last photon path; press again to release photon
				  \item[b]  step through photon path; store a photon with 's'; press 'b'; use '+' and '-' to step through the photon path forward and backward
				\end{description}
				''',
		'mc_hiddenline' : r'''
				Switch on hidden line removal for the MYSTIC online visualization. Good for topography, not so 
				good for clouds because the latter look much more realistic when the hidden layers are plotted
				in transparent mode.
				''',
		'mc_std' : r'''
				Calculate standard deviation of the average.
				''',
		'mc_temperature_file' : r'''
                		Define a MYSTIC 2D temperature input file.
                		\fcode{
                			mc_temperature_file file
                			}
                		The expected format of the temperature file is:
                			\fcode{
                			Nx  Ny  dx  dy
                			ix  iy  temperature
                			}
                		where Nx and Ny are the number of grid boxes in x- and y-direction,
                		dx and dy are the size of the grid boxes in km.
                		In the second and the following lines the indices in x- and y-direction
                		and the temperature of the pixel are specified. 
                		''',
		}
