"""--------------------------------------------------------------------
 * $Id: output_options.py 2815 2012-09-11 16:50:10Z bernhard.mayer $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""


from option_definition import *

class setup_output_group():

	group_name = 'Output and misc.'

	def __init__(self):
		output_documentation = get_output_documentation()

		reflectivity = option(
			'reflectivity',	#name
			'output',	#group
			'',	#subgroup
			'Calculate transmision/reflectivity instead of absolute quantities.',	#helpstr
			output_documentation['reflectivity'],	#documentation
			None,	#tokens
			addSetting('Input.calibration', 'OUTCAL_REFLECTIVITY', default ='OUTCAL_ABSOLUTE'),	#settings
			['uvspec'],	#parents
			['transmittance'],	#enemies
			''	#childs
			)

		transmittance = option(
			'transmittance',	#name
			'output',	#group
			'',	#subgroup
			'Calculate transmittance/reflectance instead of absolute quantities.',	#helpstr
			output_documentation['transmittance'],	#documentation
			None,	#tokens
			addSetting('Input.calibration', 'OUTCAL_TRANSMITTANCE', default= 'OUTCAL_ABSOLUTE'),	#settings
			['uvspec'],	#parents
			['reflectivity'],	#enemies
			''	#childs
			)
		
		brightness = option(
			'brightness',	#name
			'output',	#group
			'',	#subgroup
			'Convert radiances / irradiances to equivalent brightness temperatures.',	#helpstr
			output_documentation['brightness'],	#documentation
			None,	#tokens
			addSetting('Input.calibration', 'OUTCAL_BRIGHTNESS', default ='OUTCAL_ABSOLUTE'),	#settings
			['uvspec'],	#parents
			'',	#non_parents
			''	#childs
			)

		umu = SignedFloats(
			'umu',	#name
			'output',	#group
			'',	#subgroup
			'Cosine of user output polar angles.',	#helpstr
			documentation=output_documentation['umu'],
			parents=['uvspec'],
			non_parents=[],
			childs=[]
			)

		phi = SignedFloats(
			'phi',	#name
			'output',	#group
			'',	#subgroup
			'User output azimuth angles.',	#helpstr
			documentation=output_documentation['phi'],	#documentation
			parents=['uvspec'],
			)

		output_user = not_yet_lex2py_option(
			name='output_user',
			group='output',
			helpstr='User defined output', 
			documentation=output_documentation['output_user'],
			tokens=addToken("", str),
			parents=['uvspec'],
			)

		prndis = not_yet_lex2py_option(
			name='prndis',
			group='output',
			helpstr='Print various disort output. See disort.doc', 
			documentation=output_documentation['prndis'],
			tokens=addToken("", str),
			)

		heating_rate = option(
			'heating_rate',	#name
			'output',	#group
			'',	#subgroup
			'Calculation of heating rates.',	#helpstr
			output_documentation['heating_rate'],	#documentation
			addToken('Input.heating_string', str, valid_range=['none','local','layer_fd','layer_cd'], optional=True),	#tokens
			addSetting('Input.heating', 'HEAT_LAYER_CD'),	#settings
			['uvspec'],	#parents
			'',	#non_parents
			''	#childs
			)

		zout = not_yet_lex2py_option(
			name='zout',
			group='output',	
			helpstr='Output altitude(s).', 
			documentation=output_documentation['zout'],
			tokens=addToken("", str),
			parents=['uvspec'],
			non_parents=['zout_sea'],
			childs=[]
			)

		zout_sea = not_yet_lex2py_option(
			name='zout_sea',
			group='output',	
			helpstr='Output altitude(s) above sea surface.', 
			documentation=output_documentation['zout_sea'],
			tokens=addToken("", str),
			parents=['uvspec'],
			non_parents=['zout'],
			childs=[]
			)

		output = option(
			'output',	#name
			'output',	#group
			'',	#subgroup
			'Decide how the output from \code{uvspec} is processed.',	#helpstr
			output_documentation['output'],	#documentation
			addToken('Input.output_unit_processing', str, valid_range=['integrate','sum','rgbnorm','rgb_norm','rgb',
										   'per_nm','per_cm-1','per_ck_band'] ),	#tokens
			None,	#settings
			['uvspec'],	#parents
			'',	#non_parents
			''	#childs
			)

		quiet = option(
			'quiet',	#name
			'output',	#group
			'',	#subgroup
			'If specified, informative messages are turned off. See also \code{verbose}.',	#helpstr
			output_documentation['quiet'],	#documentation
			None, #token
			addSetting('Input.quiet', 1, default=0), #setting
			['uvspec'],	#parents
			['verbose'],	#enemies
			''	#childs
			)

		verbose = option(
			'verbose',	#name
			'output',	#group
			'',	#subgroup
			'If specified abundances of informative messages are output to stderr.',	#helpstr
			output_documentation['verbose'],	#documentation
			None, #token
			addSetting('Input.verbose', 1, default=0), #setting
			['uvspec'],	#parents
			['quiet'],	#enemies
			''	#childs
			)

		output_file = option(
			name='output_file',
			group='output', 
			helpstr='Location of the output file', 
			documentation=output_documentation['output_file'], 
			tokens=addToken('Input.filename[FN_OUTFILE]', str),
			parents=['uvspec'],
			)

		output_format = option(
			'output_format',  #name
			'output',      #group
			'',               #supgroup
			'Output format',  #helpstr
			output_documentation['output_format'], #documentation
			addToken('Input.output_format_string', str), #tokens,
			None,
			'',     #parents
			'',     #non_parents 
			''      #childs
			)

		self.options = [output_user, zout, zout_sea, umu, phi, 
				reflectivity, transmittance, brightness, heating_rate, output,
				quiet, verbose,output_file,output_format, prndis ]

	def __iter__(self):
		return iter(self.options)

def get_output_documentation():
	return {
		'transmittance' : r'''
				Calculate transmittance / reflectance instead of absolute quantities. 
				That is, set the extraterrestrial irradiance to 1 and do not correct for Sun-Earth distance:
				\begin{equation}
				   T = \frac{E}{E_0}
				\end{equation}
				  where $E$ is the irradiance / actinic flux / radiance and $E_0$ is the extraterrestrial flux.
				Please note the difference to \code{reflectivity}.
				''',
		'brightness' : r'''
				Convert radiances / irradiances to equivalent brightness temperatures.
				''',
		'output_user' : r'''
				User defined output. 
				%This option is case sensitive.
				Here the user may specify the columns desired for output. 
				\fcode{
				output_user format
				}
				where \code{format} is one or more of the following.
				\begin{description}
				\parameter{lambda} 
				Wavelength in nm.
				\parameter{wavenumber} 
				Wave number in cm$^{-1}$.
				\parameter{sza} 
				solar zenith angle
				\parameter{zout} 
				Output altitude in km.
				\parameter{edir, eglo, edn, eup, enet, esum}
				The direct, global, diffuse downward, and diffuse upward irradiance.
				Net is global - upward, sum is global + upward. 
				\parameter{uu} 
				Radiances:  uu(umu(0),phi(0)) ... uu(umu(0),phi(m)) ... uu(umu(n),phi(0)) ... uu(umu(n),phi(m))
				\parameter{fdir, fglo, fdn, fup, f}
				The direct, global, diffuse downward, diffuse upward, and total actinic flux.
				\parameter{uavgdir, uavgglo, uavgdn, uavgup, uavg} 
				The Direct, global, diffuse downward, diffuse upward, and total diffuse 
				mean intensity (= actinic flux / 4$\pi$).
				\parameter{spher_alb}
				Spherical albedo of the complete atmosphere.
				\parameter{albedo} 
				Albedo.
				\parameter{heat} 
				Heating rate in K/day.
				\end{description}
	
				It is also possible to gain some information about the atmosphere and the clouds:
				\begin{description}
				\parameter{p} 
				pressure [hPa], ,
				\parameter{T, T_d}
				temperature [K], dewpoint temperature [K]
				\parameter{T_sur} 
				surface temperature [K]
				\parameter{theta } 
				potential temperature [K]
				\parameter{theta_e} 
				equivalent potential temperature [K]
				\parameter{n_xxx} 
				number density of the gas xxx [cm$^{-3}$]
				\parameter{rho_xxx} 
				mass density of the gas xxx [kg/m$^3$]
				\parameter{mmr_xxx} 
				mass mixing ratio of the gas xxx [kg/kg]
				\parameter{vmr_xxx} 
				volume mixing ratio of the gas xxx [m$^3$/m$^3$]
				\parameter{rh} 
				relative humidity over water [percent]
				\parameter{rh_ice} 
				relative humidity over ice   [percent]
				\parameter{c_p} 
				specific heat capacity of the air (humidity and temperature dependent)
				\parameter{CLWC} 
				cloud liquid water content   [kg/kg]
				\parameter{CLWD} 
				cloud liquid water density   [g/m$^3$]
				\parameter{CIWC} 
				cloud ice water content      [kg/kg]
				\parameter{CIWD} 
				cloud ice water density      [g/m$^3$]
				\parameter{TCC} 
				total cloud cover            [0-1]
				\end{description}
				where \code{xxx} is one of 
				AIR, O3, O2, H2O, CO2, NO2, BRO, OCLO, HCHO, or O4.
	
				Default output is  
				\fcode{
				output_user lambda lambda, edir, edn, eup, uavgdir, uavgdn, uavgup 
				}
				for \code{fdisort1}, \code{sdisort}, and \code{spsdisort}, whereas the default for 
				\code{twostr} is 
				\fcode{
				output_user lambda, edir, edn, eup, uavg.
				}
				The lines containing radiances and the output of \code{rte_solver polradtran} are not affected. 
				''',
		'prndis' : r'''
				Specify one or more integers between 1 and 7.
				\fcode{
				prndis  value
				}
				Print various disort input and output in disorts own format. See 
				\file{libsrc_f/DISORT2.doc} for more information.
				\strong{Warning:} Produces a lot of output. 
				''',
		'reflectivity' : r'''
				Calculate transmission / reflectivity instead of absolute quantities. 
				For irradiances / actinic fluxes the transmission T is defined as 
				\begin{equation}
				   T = {{E}\over{E_0 \cos{\theta}}}
				\end{equation}
				  where $E$ is the irradiance / actinic flux, $E_0$ is the extraterrestrial flux,
				  and $\theta$ is the solar zenith angle.
				The reflectivity R is defined as 
				\begin{equation}
				   R = {{\pi \cdot L}\over{E_0 \cos{\theta}}}
				\end{equation}
				 where $L$ is the radiance, $E_0$ is the extraterrestrial flux,
				 and $\theta$ is the solar zenith angle.
				Obviously, reflectivities do not depend on Sun-Earth distance. Please 
				note the difference to \code{transmittance}.
				''',
		'heating_rate' : r'''
				Calculation of heating rates. Output is only provided at altitudes specified by \code{zout}.
				To get heating rate profiles a number of altitudes must thus be specified by \code{zout}.
				Heating rates is the change of temperature with time in units of K/day.
				For spectral calculations the default output is a matrix: 
				\begin{Verbatim}
	0.0        zout1          zout2 ...
	lambda1    heating_rates  ...
	lambda2      .
	   .         .
	   .         .
				\end{Verbatim}
				For integrated calculations (\code{output sum} or \code{output integrate}) the default output 
				is in two columns with column 1 being the altitude and column 2 the heating rates.
				The output of \code{heating_rate} can also be specified with the \code{output_user} option.
				Note that heating rates are only well-behaved up to altitudes
				for which the respective correlated-k options are valid. E.g. about 60 km for 
				\code{fu} and about 80 km for \code{kato}, \code{kato2}, \code{kato2.96}, and \code{lowtran}.
				Attention: For spectral calculations, the extraterrestrial spectrum is assumed to be in 
				mW/(m2 nm).
	
				Three different methods are implemented to calculate the heating rate, which can be selected
				with an optional keyword:
				\fcode{
				heating_rate [method]
				}
				where \code{method} may be either \code{layer_cd} (heating rates are derived from centered 
				differences of the flux (the default method), \code{local} (heating rates are derived 
				from the actinic flux), or \code{layer_fd} (heating rates are derived from forward differences of the flux 
                                over one layer. Attention: \code{heating_rate local} introduces new levels into the profile which  
				slightly affects the model output.
				With \code{layer\_fd}, the output is 
				not representative for a \emph{level}, but for the \emph{layer} from the z-level of the line in the output file, 
				where it is written, up to next \emph{output level} above!
				''',
		'output' : r'''
				Decide how the output from \code{uvspec} is processed:
				\fcode{
				output type
				}
				where type is one of
				\begin{description}
				\parameter{sum} 
				Sum output over wavelength. Useful in combination with the 
				\code{correlated_k} option (\code{kato}, \code{kato2}, \code{kato2.96}, 
				\code{Fu}, \code{avhrr_kratz}).
				\parameter{integrate} 
				Integrate output over wavelength for solar and over wavenumber for thermal simulations. 
				Useful for spectral calculations and \code{correlated_k lowtran}.
				\parameter{per_nm } 
				Output is given in W/(m$^2$ nm) or mW/(m$^2$ nm) (W or mW is determined by the extraterrestrial spectrum.)
				\parameter{per_cm$^{-1}$ } 
				Output is given in W/(m$^2$ cm$^{-1})$ or mW/(m$^2$ cm$^{-1}$).
				\parameter{per_ck_band } 
				Output is given in W/m$^2$ or mW/m$^2$ per correlated-k band. (This option can not be used for 
				spectral calculations and \code{correlated_k} LOWTRAN in the solar range.)
				\parameter{none} 
				No processing - output spectral information (default).
				\end{description}
				''',
		'umu' : r'''
			Cosine of output polar angles in increasing order, starting with 
			negative (downwelling radiance, looking upward) values (if any) and on through 
			positive (upwelling radiance, looking downward) values. 
			Must not be zero.
			\fcode{
			umu values
			}
			''',
		'phi' : r'''
			Azimuth output angles (in degrees) in increasing order. 
			\fcode{
			phi values
			}
			The radiance is output at \code{phi} and \code{umu}.
			\begin{itemize}
			\item Sensor in the North (looking South):  0 deg 
			\item Sensor in the East  (looking West):   90 deg 
			\item Sensor in the South (looking North):  180 deg 
			\item Sensor in the West  (looking East):   270 deg 
			\end{itemize}
			For all one-dimensional solvers the absolute azimuth does not matter, but only the relative azimuth \code{phi}-\code{phi0}.
			''',
		'quiet' : r'''
			If specified, informative messages are turned off. See also \code{verbose}.
			''',
		'verbose' : r'''
			If specified abundances of informative messages are output to stderr. To make
			use of this information, you may want to write the standard \code{uvspec} output to 
			one file and the diagnostic messages to another. To do so, try
			\code{(./uvspec < uvspec.inp > uvspec.out) >\& verbose.txt} (depending on your shell you 
			might need a slightly different syntax). The irradiances 
			and radiances will be written to \file{uvspec.out} while all 
			diagnostic messages go into \file{verbose.txt}. See also \code{quiet}.
			''',

		'output_file'	: r'''
			\fcode {             
			    output_file filename
			}
			uvspec output will be written to \code{filename}. The format can be set by the option 
			\code{output_format} (default is ascii).  
			''',
		'zout'	: r'''
			This option is used to specify the output altitudes in km \emph{above surface altitude}. 
			One or more altitudes may be specified in increasing magnitude. 
			\fcode{
			zout 0 1 2 3 4 5 ...
			}
			Output altitudes must be within the range defined in the
			\code{atmosphere_file}. Note that \code{zout} does not restructure the
			atmosphere model. Hence, if you specify \code{zout 0.730} and have your
			atmosphere model in \code{atmosphere_file} go all the way down to 
			sea level, i.e. 0.0km., output is presented at 0.730km and calculations
			performed with an atmosphere between 0.0 and 0.730 km (and above of course). 
			If you want calculations done for e.g. an elevated site you have to restructure
			the atmosphere model and make sure it stops at the appropriate altitude.
			This you may either due by editing the atmosphere file or by using 
			\code{altitude}. Note that for \code{rte_solver polradtran} the atmosphere file must 
			contain the altitudes specified by \code{zout}. You can also use \code{toa} for 
			\code{top of atmosphere} and \code{sur} for \code{surface altitude} and \code{cpt} 
			for \code{cold point tropopause}.

			Instead of specifying the altitudes in km, it is also possible to use keywords as 
			argument for this option. Possible keywords are \code{atm_levels}, 
			\code{all_levels}, \code{model_levels}, \code{model_layers}, and \code{model_levels_and_layers}. 
			For \code{atm_levels}, all levels from the \code{atmosphere_file} are used as output levels.  
			For \code{all_levels}, all levels (including levels from \code{atmosphere_file}, 
			\code{dens_file}, cloud files, \code{altitude} options) are used as output levels. 
			For \code{model_levels}, \code{model_layers}, \code{model_levels_and_layers} 
			the levels, layers, or both from the \code{ECMWF_atmosphere_file} are used as output level.
			Usage e.g.:
			\fcode{
			zout model_levels [nlev_max]
			}
			With the optional argument \code{nlev_max} the user may specify the number of zout layers 
			from the ground.
			''',
		'zout_sea'	: r'''
			like zout, but \emph{above sea surface}
			''',
		'output_format'    : r'''
			\fcode {
			    output_format format
			}
			where \code{format} is either \code{ascii} (default) or \code{flexstor}. 
			Note that \code{flexstor} does not currently work when \code{umu} and/or \code{phi} is specified.
			There is also the
			possibility to write uvspec simulation results to an existing netCDF file. For
			that \code{format} must be \code{netCDF} and the option \code{output_file} must be given
			and point to a file that contains a lat/lon/time grid.
			If \code{format} is set to \code{sat picture} then \code{output_file} must be given and point
			to a NetCDF-File that contains a pixel x/pixel y/time grid.
			'''
		}
