"""--------------------------------------------------------------------
 * $Id: solver_options.py 2815 2012-09-11 16:50:10Z bernhard.mayer $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""


from option_definition import *


class setup_solver_group():
	
	group_name = 'Solver'

	def __init__(self):
		solver_documentation = get_solver_documentation()

		rte_solver = continious_option(
			'rte_solver',	#name
			'solver',	#group
			'',	#subgroup
			'Radiative transfer equation solver.',	#helpstr,
			solver_documentation['rte_solver'],	#documentaion
			addToken('Input.rte.solver_name', str, 
				 valid_range=['cdisort', 'disort2', 'disort',
					      'sos', 'polradtran', 
					      'twostr', 'ctwostr',
					      'montecarlo', 'mystic', 'tzs',
					      'sssi', 'sss', 'rodents',
					      'twostrebe', 'sslidar',
					      'null' ]),	#tokens
#				 valid_range=['sdisort', 'spsdisort',
#					      'fdisort1', 'fdisort2',
#					      'cdisort', 'disort2', 'disort',
#					      'sos', 'polradtran', 'twostrpp',
#					      'twostreampp', 'twostream',
#					      'twostr', 'ctwostr',
#					      'montecarlo', 'mystic', 'tzs',
#					      'sssi', 'sss', 'rodents',
#					      'twostrebe', 'sslidar',
#					      'null' ]),	#tokens
			None,	#settings
			['uvspec'],	#parents
			'',	#non_parents
			['polradtran_quad_type','polradtran_aziorder',
			 'polradtran_nstokes','polradtran_src_code',
			 'cdisort_pseudospherical', 'cdisort_spherical_albedo', 'ctwostr_pseudospherical',
			 'polradtran_max_delta_tau','fisot','raman','raman_original',
			 'nstr','deltam','nrefrac','cox_and_munk_pcl',
			 # Mystic options are not enabled unless solver is mystic of monte carlo, see mc_options.py
			 'mc_escape','mc_vroom','mc_backward','mc_spectral_is',
			 'mc_spectral_is_wvl','mc_absorption','mc_actinic',
			 'mc_albedo_file','mc_ambrals_file',
			 'mc_backward_output', 'mc_backward_writeallpixels', 
			 'mc_basename',
			 'mc_delta_scaling', 'mc_elevation_file', 'mc_emission', 'mc_heating',
			 'mc_ipa','mc_maxscatters','mc_minscatters','mc_minphotons','mc_photons', 
			 'mc_photons_file','mc_polarisation',
			 #'mc_radial_pathlength', 'mc_radial_pathlength_dt','mc_aerosol_is','mc_sample_cldprp',
			 #'mc_albedo_spectral',  'mc_bcond','mc_coherent_backscatter','mc_backward_sunshape_file', 
			 #'mc_backward_writeback','mc_azimuth_old',
			 'mc_wcloud_file','mc_icloud_file',
			 'mc_randomseed', 'mc_rpv_file','mc_rpv_type','mc_spherical','mc_std',
			 'mc_temperature_file','mc_sample_grid','mc_sensordirection','mc_sensorposition',
			 'mc_reference_to_nn','mc_bw_umu_file','mc_truncate',
			 'mc_surface_reflectalways','mc_refraction','mc_visualize','mc_hiddenline']  
			)

		source = option(
			name='source',
			group='solver',
			helpstr='Radiation source type.',
			documentation=solver_documentation['source'],
			tokens=addToken('Input.source_name', str, valid_range=['solar','thermal']),
			settings=addSetting('Input.source', 'SRC_NONE', default='SRC_SOLAR'), 
			parents=['uvspec'],
			)

		cdisort_pseudospherical = conditional_enable_option(
			name='cdisort_pseudospherical',
			group='solver',
			helpstr='Invokes pseudospherical geometry for cdisort solver.',
			documentation=solver_documentation['cdisort_pseudospherical'],
			settings=addSetting('Input.rte.cdisort_pseudospherical', 1),
			parents=['rte_solver'],
			speaker='rte_solver',
			enable_values=("cdisort","cdisort2")
			)
		ctwostr_pseudospherical = conditional_enable_option(
			name='ctwostr_pseudospherical',
			group='solver',
			helpstr='Invokes pseudospherical geometry for twostr solver.',
			documentation=solver_documentation['ctwostr_pseudospherical'],
			settings=addSetting('Input.rte.ctwostr_pseudospherical', 1),
			parents=['rte_solver'],
			speaker='rte_solver',
			enable_values=("ctwostr",)
			)

		cdisort_spherical_albedo = conditional_enable_option(
			name='cdisort_spherical_albedo',
			group='solver',
			helpstr='Calculate spherical albedo.',
			documentation=solver_documentation['cdisort_spherical_albedo'],
			settings=[addSetting('Input.rte.ibcnd', 1),
				  addSetting('Input.rte.nphi', 0)],
			parents=['rte_solver'],
			speaker='rte_solver',
			enable_values=("cdisort","cdisort2")
			)

		fisot = conditional_enable_option(
			name='fisot',
			group='solver',
			helpstr='Specifies that isotropic '
			'illumination is included at top-boundary.',
			documentation=solver_documentation['fisot'],
			settings=addSetting('Input.rte.fisot', 1),
			parents=['rte_solver'],
			speaker='rte_solver',
			enable_values=("disort", "sdisort", "spsdisort",
				      "fdisort1", "fdisort2","cdisort",
				      "cdisort2", "twostr","twostrpp",
				      "twostream","twostreampp")
			)

		raman = conditional_enable_option(
			name='raman',
			group='solver',
			helpstr='Include first order rotational Raman scattering.',
			documentation=solver_documentation['raman'],
			settings=[addSetting('Input.raman', 'RAMAN_CALC'),
				  addSetting('Input.ck_scheme', 'CK_RAMAN'),
				  addSetting('Input.rte.solver', 'CDISORT'),
				  addSetting('Input.processing', 'PROCESS_RAMAN')],
			parents=['rte_solver'],
			childs=['raman_original'],
			speaker='rte_solver',
			enable_values=("cdisort", "cdisort2"),
			)

		raman_original = conditional_enable_option(
			name='raman_original',
			group='solver',
			helpstr='Calculate each wavelength at the time. CPU expensive.',
			documentation=solver_documentation['raman_original'],
			settings=addSetting('Input.raman_original', 1),
			parents=['raman'],
			speaker='rte_solver',
			enable_values=("cdisort", "cdisort2"),
			)

		nstr = conditional_enable_option(
			name='nstr',
			group='solver',
			subgroup='',
			helpstr='Number of streams.',
			documentation=solver_documentation['nstr'],
			tokens=addToken('Input.rte.nstr', int, default=6),
			settings=None,
			parents=['rte_solver'],
			non_parents='',
			childs='',
			speaker='rte_solver',
			enable_values=("disort", "sdisort", "spsdisort",
				      "fdisort1", "fdisort2","cdisort",
				      "cdisort2","sos","polradtran"),
			)

		polradtran_quad_type = conditional_enable_option(
			name='polradtran_quad_type',
			group='solver',
			helpstr='Type of quadrature used',
			documentation=solver_documentation['polradtran_quad_type'],
			tokens=addToken('Input.rte.pol_quad_type', str),
			parents=['rte_solver'],
			speaker="rte_solver",
			enable_values=("polradtran",)
			)

		polradtran_max_delta_tau = conditional_enable_option(
			name='polradtran_max_delta_tau',	#name
			group='solver',	#group
			helpstr='Initial layer thickness for doubling.',	#helpstr,
			documentation=solver_documentation['polradtran_max_delta_tau'],	#documentaion
			tokens=addToken('Input.rte.pol_max_delta_tau', float, default=1e-5 ),	#tokens (double??)
			parents=['rte_solver'],	#parents
			speaker="rte_solver",
			enable_values=("polradtran",)
			)


		polradtran_src_code = conditional_enable_option(
			name='polradtran_src_code',	#name
			group='solver',	#group
			helpstr='Radiation sources.',
			documentation=solver_documentation['polradtran_src_code'],	#documentaion
			tokens=addToken( 'Input.rte.pol_src_code', int, default=1, valid_range=[0,1,2,3]),	#tokens
			parents=['rte_solver'],	#parents
			speaker="rte_solver",
			enable_values=("polradtran",)

			)

		polradtran_nstokes = conditional_enable_option(
			name='polradtran_nstokes',	#name
			group='solver',	#group
			helpstr='Number of Stokes parameters.',	#helpstr,
			documentation=solver_documentation['polradtran_nstokes'],	#documentaion
			tokens=addToken('Input.rte.pol_nstokes', int, default=1, valid_range=[1,2,3]),	#tokens
			parents=['rte_solver'],	#parents
			speaker="rte_solver",
			enable_values=("polradtran",)

			)

		polradtran_aziorder = conditional_enable_option(
			name='polradtran_aziorder',	#name
			group='solver',	#group
			helpstr='Order of Fourier azimuth series.',	#helpstr,
			documentation=solver_documentation['polradtran_aziorder'],	#documentaion
			tokens=addToken('Input.rte.pol_aziorder', int, default=0),	#tokens
			parents=['rte_solver'],	#parents
			speaker="rte_solver",
			enable_values=("polradtran",)

			)

		nrefrac = conditional_enable_option(
			name='nrefrac',	#name
			group='solver',	#group
			helpstr='Refraction for rte_solver sdisort.',	#helpstr,
			documentation=solver_documentation['nrefrac'],	#documentaion
			tokens=addToken('Input.rte.nrefrac', int, default=0, valid_range=[0,1,2]),	#tokens
			parents=['rte_solver'],	#parents
			speaker="rte_solver",
			enable_values=("sdisort",)
			)

		nscat = option(
			'nscat',	#name
			'solver',	#group
			'',	#subgroup
			'Order of scattering.',	#helpstr,
			solver_documentation['nscat'],	#documentaion
			addToken('Input.rte.nscat', int, default=20),	#tokens
			None,	#settings
			['rte_solver'],	#parents
			'',	#non_parents
			''	#childs
			)

		deltam = conditional_enable_option(
			name='deltam',	#name
			group='solver',	#group
			helpstr='Turn delta-M scaling on/off.',	#helpstr,
			documentation=solver_documentation['deltam'],	#documentaion
			tokens=addToken('Input.rte.deltam_switch', str, valid_range=['on','off']),	#tokens
			parents=['rte_solver'],	#parents
			speaker="rte_solver",
			enable_values=("disort", "sdisort", "spsdisort",
				       "fdisort1", "twostr", "twostrpp",
				       "twostream","twostreampp")
			)
		

		self.options = [rte_solver, source, raman, raman_original, 
				cdisort_pseudospherical, cdisort_spherical_albedo, ctwostr_pseudospherical,
				fisot, nstr, deltam, nscat, nrefrac,
				polradtran_src_code, polradtran_nstokes, 
				polradtran_quad_type, polradtran_aziorder, polradtran_max_delta_tau ]

	def __iter__(self):
		return iter(self.options)


def get_solver_documentation():
	return {
		'polradtran_max_delta_tau' : r'''
			       	Initial layer thickness for doubling; governs accuracy, 10E-5 should be
		       		adequate. Do not go beyond half the real precision, i.e. 10e-8 for REAL*8.
	       			Default 1.e-05.
	       			\fcode{
		       		polradtran_max_delta_tau value
		       		}
	                        This option is only relevant for \code{rte_solver polradtran}.
		       		''',
		'rte_solver' : r'''
				Set the radiative transfer equation solver to be used. 
				\fcode{
				rte_solver type
				}
				If not specified the default \code{rte_solver} is \code{cdisort}.
				Choices for \code{type} are
				\begin{description}
				\parameter{cdisort}
				C-version of the disort algorithm, translated from Fortran by Tim Dowling. 
				This is the recommended discrete ordinate code in {\sl libRadtran}.
				For documentation see \file{src\_f/DISORT2.doc} as well as the papers and
				the DISORT report at
				ftp://climate1.gsfc.nasa.gov/wiscombe/Multiple\_Scatt/.  The intensity
				correction can be performed according to \citet{Nakajima1988} using
				\code{disort_icm moments} (like in the original code), or with the 
				improvements described in (Buras, Dowling, Emde, in preparation; default). Can be run in  
				plane-parallel geometry (default) or in pseudo-spherical geometry 
				(using \code{cdisort_pseudospherical}).
				\parameter{ctwostr}
				C-version of the two--stream radiative transfer solver described by
				\citet{Kylling1995}. Can be run in plane-parallel geometry (default)
				or in pseudo-spherical geometry (using \code{ctwostr_pseudospherical}
				).
				\parameter{disort}
				Same as \code{cdisort} which is the recommended discrete ordinate code 
				in {\sl libRadtran}. Before {\sl libRadtran} 1.6 \code{cdisort} invoked 
				the Fortran code disort 1.3.
				\parameter{disort2}
				Same as \code{cdisort} which is the recommended \emph{discrete ordinate code}
				in {\sl libRadtran}. Before {\sl libRadtran} 1.6 \code{cdisort} invoked 
				the Fortran code disort 2.0.
				
				\parameter{fdisort1}
				The standard plane--parallel disort algorithm by \citet{Stamnes1988c},
				version 1.3 -- provided for compatibility reasons. Use only if you have troubles with the default			         
			        \code{disort} or for historical reasons. For documentation see \file{src\_f/DISORT.doc} as well 
				as the papers and the DISORT report at
				ftp://climate1.gsfc.nasa.gov/wiscombe/Multiple\_Scatt/. To optimize
				for computational time and memory, please adjust the parameters in
				\file{src\_f/DISORT.MXD} for your application and re-compile. For your
				application please use \code{rte_solver fdisort2} which is the advanced
				version, unless you e.g. want to explore how a specific feature of
				\code{fdisort2} (e.g. the \citet{Nakajima1988} intensity correction)
				improves the \code{fdisort1} result.
				\parameter{fdisort2}
				Version 2 of the Fortran algorithm disort -- provided for compatibility reasons.
				Use only if you have troubles with the default \code{disort} or for historical reasons. For documentation see
				\file{src\_f/DISORT2.doc} as well as the papers and the DISORT report
				at ftp://climate1.gsfc.nasa.gov/wiscombe/Multiple\_Scatt/
				\code{fdisort2} has several improvements compared to its 'ancestor'
				\code{fdisort1} (version 1.3). To optimize for computational time and
				memory, please adjust the parameters in \code{src_f/DISORT.MXD} for
				your application and re-compile.
	
				Note! \code{fdisort2} is a new version of the original disort code
				which was implemented in summer 2009. It uses phase functions to
				calculate the intensity corrections by \citet{Nakajima1988} instead of
				Legendre moments. Hence it needs cloud properties files which contain
				the phase functions. It is still possible to use the old version of
				disort2, you need to specify \code{disort_icm moments}.
				\parameter{sdisort}
				Pseudospherical disort as described by \citet{Dahlback1991}. Double
				precision version. To optimize for computational time and memory,
				please adjust the parameters in \file{src\_f/DISORT.MXD} for your
				application and re-compile.  \parameter{spsdisort} Pseudospherical
				disort as described by \citet{Dahlback1991}, single precision
				version. \strong{Warning:} it is not recommended to use
				\code{spsdisort} for really large solar zenith angles nor for cloudy
				conditions. For large optical thickness it is numerically unstable and
				may produce wrong results. To optimize for computational time and
				memory, please adjust the parameters in \file{src_f/DISORT.MXD} for
				your application and re-compile.  \parameter{polradtran} The
				plane-parallel radiative transfer solver of \citet{Evans1991}.
				Includes polarization. The full implementation of the polRadtran
				solver in \code{uvspec} is quite new (version 1.4). If you find
				unusual behaviour, please contact the {\sl libRadtran} authors.
				\parameter{twostr}
				The two--stream radiative transfer solver described by
				\citet{Kylling1995}, in pseudo-spherical geometry.
				\parameter{twostrpp}
				The two--stream radiative transfer solver described by
				\citet{Kylling1995}, in plane-parallel geometry.
				\parameter{rodents}
				Delta-Eddington two--stream code (RObert's Delta-EddingtoN Two-Stream), plane-parallel.
				\undocumented{
				 \parameter{twostrebe}
				 Delta-Eddington two--stream solver by Bernhard Mayer. \code{zout_interpolate} is activated automatically.
				}
				\parameter{sslidar}
				A simple single scattering lidar simulator by Robert Buras.
				\parameter{sos}
				A scalar pseudospherical succesive orders of scattering code. Works
				for solar zenith angles smaller than 90 degrees. Can calculate
				azimuthally averaged radiances. Set \code{nscat} to specify the order
				of scattering.
				\ifmystic{
				\parameter{montecarlo}
				The MYSTIC Monte Carlo code. Monte Carlo is
				the method of choice (1) for horizontally
				inhomogeneous problems; (2) whenever
				polarization is involved; (3) for applications
				where spherical geometry plays a role; and (4)
				whenever sharp features of the scattering
				phase function play a role, like for the 
				calculation of the backscatter glory or the
				aureole.  
				\parameter{mystic}
				Same as \code{montecarlo}.
				}
				\parameter{tzs} TZS stands for "thermal, zero scattering" and is a
				very fast analytical solution for the special case of thermal emission
				in a non-scattering atmosphere. Please note that TZS does only
				radiance calculations at top of the atmosphere.
				\parameter{sss}
				SSS stands for "solar, single scattering" and is an analytical single
				scattering approximation which might be reasonable for an optically
				thin atmosphere. Please note that SSS does only radiance calculations
				at top of the atmosphere. This is an experimental solver - be careful!
				\parameter{null}
				The NULL solver does not solve the radiative transfer
				equation. However, it sets up the optical properties, and does the
				post-processing; useful if you are either interested in the overhead
				time required by a particular model input or if you are simply
				interested in the optical properties, as output by \code{verbose}.
				\end{description}
				Default: \code{cdisort}
				''',
		'polradtran_src_code' : r'''
				Radiation sources included by
				\fcode{
				polradtran_src_code value
				}
				which may be
				\begin{description}
				\item[0] none
				\item[1] solar
				\item[2] thermal
				\item[3] both
				\end{description}
				Default 1.
				This option is only relevant for \code{rte_solver polradtran}.
				''',
		'raman' : r'''
				The \code{raman} option includes single order rotational Raman scattering in 
				the calculation. The solution treats Raman as a perturbation similar to the 
				approaches of \citet{Vountas1998} and \citet{Spurr2008}.
				
				The \code{raman} option may only be used for spectral calculation.

				The cdisort radiative transfer solver with a general source is needed to solve the radiative transfer
				equation including Raman scattering. This solver is automagically invoked when specifying 
				the \code{raman} option. It is thus not neccessary to set the \code{rte_solver}.

				The \code{raman} option is optimized with respect to speed. The optimized implementation
				should be just as accurate as the original version. To use the original version
				invode \code{raman_original}.

				Please note that while the \code{raman} option has been extensively tested and verified, it 
				is nevertheless a new option, hence, use it with care.
				''',
		'raman_original' : r'''
				Treats each wavelength individually and is thus accurate, but 
				computationally very expensive. 
				''',
		'cdisort_pseudospherical' : r'''
				Invokes pseudo-spherical geometry in cdisort. Default is plane-parallel.
				''',
		'cdisort_spherical_albedo' : r'''
				Calculate spherical albedo using \code{cdisort}. When this option is enabled,
				only the spherical albedo is calculated. The output is enabled by 
				\fcode{output_user spher_alb}
				''',
		'ctwostr_pseudospherical' : r'''
				Invokes pseudo-spherical geometry in ctwostr. Default is plane-parallel.
				''',
		'fisot' : r'''
				Specifies that isotropic illumination is used at top-boundary instead
				of beam source. Useful for those who want to calculate the reflectance 
				for a homogeneous or inhomogeneous atmosphere. The intensity is still set by
				\code{solar_file}.
				''',
		'nstr' : r'''
				Number of streams used to solve the radiative transfer equation.
				\fcode{
				nstr value
				}
				Default is 6 for fluxes and 16 for radiances.
				(For \code{rte_solver fdisort1}, \code{fdisort2} and \code{cdisort} only
				even nstr are possible.)
				''',
		'polradtran_quad_type' : r'''
				Type of quadrature used:
				\fcode{
				polradtran_quad_type type
				}
				where \code{type} is one of
				\begin{description}
				\item[G] gaussian
				\item[D] double gaussian, 
				\item[L] Lobatto
				\item[E (default)] extra-angle(s), this must be used of \code{polradtran} is used in 
				combination with \code{umu}. Will internally use Gaussian scheme (\code{G}). 
				See also radtran documentation (\file{libsrc_f/README.polRadtran}).
				\end{description}
				Default E.                                  
				This option is only relevant for \code{rte_solver polradtran}.
				''',
		'source' : r'''
				Set the radiation source type
				\fcode{
				source  type
				}
				where \code{type} is either \code{solar} or \code{thermal}.
				Solar radiation is per default output in W/(m2 nm) for spectral and 
				\code{correlated_k} LOWTRAN calculations. For all other \code{correlated_k} options
				the output is integrated over the wavelength band.
				Thermal radiation is per default output in W/(m2 cm-1), if the bandwidth 
				is equal to 1 cm-1 (default for \code{correlated_k LOWTRAN} calculations). 
				Otherwise the output is the integrated flux over the
				wavenumber interval specified by \code{thermal_bandwith}, \code{thermal_bands_file},
				or by the \code{correlated_k} option (\code{kato}, \code{kato2}, \code{kato2.96}, 
				\code{fu}, or \code{avhrr_kratz}).
				''',
		'nrefrac' : r'''
				For the \code{rte_solver sdisort} refraction may be included by 
				\fcode{
				nrefrac value
				}
				where \code{value} has the meaning
				\begin{description}
				\item[0] No refraction, default.
				\item[1] Refraction included using fast, but harsh method.
				\item[2] Refraction included using slow, but accurate method.
				\end{description}
				If refraction is included also set parameter \code{refraction_file}.
				''',
		'polradtran_nstokes' : r'''
				Number of Stokes parameters
				\fcode{
				polradtran_nstokes value
				}
				where \code{value} is one of
				\begin{description}
				\item[1] for I (no polarization, default)
				\item[2] for I,Q,U (Since V is very small in the atmosphere, it makes sense
				           to compute only I,Q,U. This saves computation time and
					   memory).
				\item[3] for I,Q,U,V
				\end{description}
				Default is 1.
				''',
		'nscat' : r'''
				The order of scattering for the \code{sos} radiative transfer equation solver. 
				\fcode{
				nscat value
				}
				Default is 20. May also be used with the \code{sdisort} solver. If set to 1 \code{sdisort} 
				will run in single scattering mode while if set in to 2, \code{sdisort} runs in full multiple 
				scattering mode.
				''',
		'deltam' : r'''
				Turn delta-M scaling on/off. Set to either \code{on} or
				\code{off}. Note that for the \code{rte_solver cdisort} and
				\code{rte_solver fdisort2} delta-M scaling is hardcoded to be always
				on.
				''',
		'polradtran_aziorder' : r'''
				Order of Fourier azimuth series
				\fcode{
				polradtran_aziorder value
				}
				The value \code{0} (default for irradiance) is the azimuthally symmetric case.
				For radiance computation a higher order is required, thus the default for radiances is 4. 
				This option is only relevant for \code{rte_solver polradtran}.
				'''
		}
