
"""--------------------------------------------------------------------
 * $Id: spectral_options.py 2756 2012-08-02 13:38:21Z svn-kylling $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""

from option_definition import *

class setup_spectral_group():

	group_name = 'Spectral'

	def __init__(self):
		spectral_documentation = get_spectral_documentation()

		filter_function_file = option(
			'filter_function_file', #name
			'spectral', #group
			'', #subgroup
			'If specified, the calculated spectrum is multiplied with a filter function defined in file.', #helpstr
			spectral_documentation['filter_function_file'], #documentation
			addToken('Input.filename[FN_FILTERFUNCTION]',file), #token
			None, #setting
			['uvspec'], #parents
			['slit_function_file','thermal_bands_file'], #enemies
			['filter_function_normalize'] #children
			)
		filter_function_normalize = option(
			'filter_function_normalize', #name
			'spectral', #group
			'', #subgroup
			'If specified, the integral of the filter function over wavelength is normalized.', #helpstr
			spectral_documentation['filter_function_normalize'], #documentation
			None, #token
			addSetting('Input.filter_function_normalize', 1, default=0), #setting
			['filter_function_file'], #parents
			'', #non_parents
			'' #childs
			)
		slit_function_file = option(
			'slit_function_file', #name
			'spectral', #group
			'', #subgroup
			'If specified, the calculated spectrum is convolved with the function found in the slit_function_file.', #helpstr
			spectral_documentation['slit_function_file'], #documentation
			addToken('Input.filename[FN_SLITFUNCTION]', file), #token
			addSetting('Input.convolve', 1, default=0), #setting
			['uvspec'], #parents
			['filter_function_file','thermal_bands_file'], #enemies
			'' #childs
			)
		solar_file = option(
			'solar_file', #name
			'spectral', #group
			'', #subgroup
			'Location of file holding the extraterrestrial spectrum.', #helpstr
			spectral_documentation['solar_file'], #documentation
			[ addToken('Input.extraterrestrial_filename', file), 
			  addToken('Input.solar_file_spectrum_unit', str, valid_range=['per_nm', 'per_cm-1', 'per_ck_band'], optional=True ) ], #token
			None, #setting
			['uvspec'], #parents
			'', #non_parents
			'' #childs
			)
		spline = option(
			'spline', #name
			'spectral', #group
			'', #subgroup
			'Spline interpolate the calculated spectrum.', #helpstr
			spectral_documentation['spline'], #documentation
			[ addToken('Input.spline_lambda_0', float, valid_range=[0,1e6] ), #TODO:Valid_range from option wavelength.
			  addToken('Input.spline_lambda_1', float, valid_range=[0,1e6] ),
			  addToken('Input.spline_lambda_step', float, valid_range=[0,1e6]) ], #token
			addSetting('Input.spline', 1, default=0), #setting
			['uvspec'], #parents
			['spline_file'], #enemies
			'' #childs
			)
		spline_file = option(
			'spline_file', #name
			'spectral', #group
			'', #subgroup
			'Spline interpolate to arbitrary wavelengths, in nm, given as a single column in file.', #helpstr
			spectral_documentation['spline_file'], #documentation
			addToken('Input.filename[FN_SPLINE]', file), #token
			addSetting('Input.spline', 1, default=0), #setting
			['uvspec'], #parents
			['spline'], #enemies
			'' #childs
			)
		thermal_bands_file = option(
			'thermal_bands_file', #name
			'spectral', #group
			'', #subgroup
			'File with the center wavelengths and the wavelength band intervals for calculations in the thermal range.', #helpstr
			spectral_documentation['thermal_bands_file'], #documentation
			addToken('Input.filename[FN_WLBANDS]', file), #token
			None, #setting
			['uvspec'], #parents
			['filter_function_file','slit_function_file'], #non_parents
			'' #childs
			)
		transmittance_wl_file = option(
			'transmittance_wl_file', #name
			'spectral', #group
			'', #subgroup
			'Location of file with wavelength grid used for the internal transmittance calculations.', #helpstr
			spectral_documentation['transmittance_wl_file'], #documentation
			addToken('Input.filename[FN_WLTRANS]', file), #token
			None, #setting
			['uvspec'], #parents
			['thermal_bands_file'], #enemies
			'' #childs
			)
		wavelength = option(
			'wavelength', #name
			'spectral', #group
			'', #subgroup
			'Set the wavelength range by specifying first and last wavelength in nm.', #helpstr
			spectral_documentation['wavelength'], #documentation
			[ addToken('Input.wl.start', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6]),
			  addToken('Input.wl.end', float, default='NOT_DEFINED_FLOAT', valid_range=[0,1e6], optional=True) ], #token  TODO:argument 2 should be bigger that argument 1
			None, #setting
			['uvspec'], #parents
			['wavelength_index'], #enemies
			'' #childs
			)
		wavelength_index = option(
			'wavelength_index', #name
			'spectral', #group
			'', #subgroup
			'Set the wavelengths to be selected.', #helpstr
			spectral_documentation['wavelength_index'], #documentation
			[ addToken('Input.wl.start_index', int, default='NOT_DEFINED_INTEGER'),
			  addToken('Input.wl.end_index', int, default='NOT_DEFINED_INTEGER', optional=True) ], #token
			None, #setting
			['uvspec'], #parents
			['wavelength'], #enemies
			'' #childs
			)

		correlated_k = option(
			'correlated_k',	#name
			'spectral',      #group
			'',      #supgroup
			'Set correlated-k scheme. ',	#helpstr
			spectral_documentation['correlated_k'], #documentation
			addToken( 'Input.ck_scheme_name', str,
				  valid_range=['kato', 'kato2', 'kato2.96','fu','avhrr_kratz','sbdart', file]),     #tokens
			None,   #additional settings
			['uvspec'],     #parents
			[],     #enemies
			[]      #childs
			)

		self.options = [solar_file, wavelength, wavelength_index,
				correlated_k,
				slit_function_file, spline, spline_file,
				filter_function_file, filter_function_normalize, 		
				thermal_bands_file, transmittance_wl_file]

	def __iter__(self):
		return iter(self.options)

def get_spectral_documentation():
	return {
		'filter_function_file' : r'''
				If specified, the calculated spectrum is multiplied with a filter function 
				defined in file.
				\fcode{
				filter_function_file file
				}
				The file must contain two columns. 
				Column 1 is the wavelength, in nm. Column 2 is the corresponding filter 
				function value. Comments start with \code{\#}. Empty lines are ignored.
				In combination with \code{output sum} or \code{output integrate} this option 
				is useful e.g. to calculate weighted irradiances or actinic fluxes or 
				to simulate broadband or satellite observations.
				''',
		'filter_function_normalize' : r'''
				If specified, the integral of the filter function over wavelength is normalized such
			  that \code{output integrate} gives radiative properties per wavelength, averaged over
			  the filter function.
				''',
		'slit_function_file' : r'''
				If specified, the calculated spectrum is convolved with the function found
				in the \file{slit_function_file}. 
				\fcode{
				slit_function_file file 
				}
				The file must contain two columns. Column 1 is the wavelength, in nm, and relative to the
				center wavelength. Column 2 is the corresponding slit function value. It must
				be unity at the maximum. The wavelength steps in the slit function file must
				be equidistant. Comments start with \code{\#}. Empty lines are ignored. Please 
				note that prior to convolution the spectrum is interpolated to the wavelength
				steps of the slit function. For this reason, make sure that the resolution
				of the slit function is high enough even if the slit function is e.g. a 
				simple triangle which could in principle be described with 3 grid points. For an 
				example see \file{examples/TRI_SLIT.DAT} and the \code{make_slitfunction} tool.
				''',
		'solar_file' : r'''
				Location of file holding the extraterrestrial spectrum.
				\fcode{
				solar_file filename [unit]
				}
				The file must contain two columns. Column 1 is the wavelength in nm, and column 2 
				the corresponding extraterrestrial flux. The user may freely use any units
				he/she wants for the extraterrestrial flux. The wavelength  specified grid
				defines the wavelength resolution at which results are returned. However, 
				the wavelength range is determined by \code{wavelength}. \code{solar_file} may be 
				omitted for thermal radiation calculations (\code{source thermal}) as well as 
				\code{transmittance} and \code{reflectivity} calculations. If omitted, the 
				output resolution equals the internal wavelength grid which the model chooses 
				for the radiative transfer calculation.
				Comments start with \code{\#}. Empty lines are ignored.
	
				For some purpuses it is useful to tell libRadtran the units of the spectrum.
				This can be done with the optional second argument. If \code{unit} is set to \code{per_nm}
				libRadtran assumes that the unit of the spectrum is W/(m2 nm), if set to \code{per_cm-1}
				it assumes W/(m2 cm-1).
				Note that \code{solar_file} is ignored if \code{correlated_k} is specified.
				''',
		'spline' : r'''
				\fcode{
				spline lambda_0 lambda_1 lambda_step
				}
				Spline interpolate the calculated spectrum between wavelengths \code{lambda_0} 
				and \code{lambda_1} in steps of \code{lambda_step},
				in nm. Specified as e.g.
				\fcode{
				spline 290. 365. 0.5
				}
				Here, the calculated spectrum is interpolated to wavelengths 290.0, 290.5, 291.0, 
				..., 364.5, 365.0. For interpolation to arbitrary wavelengths use \code{spline_file}.
				The specified wavelength interval must be within the one specified by \code{wavelength}.
				''',
		'spline_file' : r'''
				Spline interpolate to arbitrary wavelengths, in nm, given as a single column in file 
				\file{spline_file}. 
				\fcode{
				spline_file file
				}
				The specified wavelengths must be within the range specified 
				by \code{wavelength}. Comments start with \code{\#}. Empty lines are ignored.
				''',
		'thermal_bands_file' : r'''
				File with the center wavelengths and the wavelength band intervals to be used for 
				calculations in the thermal range. 
				\fcode{
				thermal_bands_file file
				}
				The following three columns are expected:
				center (or reference) wavelength, lower wavelength limit, upper wavelength limit [nm].
				\code{thermal_bands_file} defines the wavelength grid for the radiative transfer 
				calculation. The RTE solver is called for each of the wavelengths in the first column. 
				The atmospheric (scattering, absorption, etc) properties are also evaluated at these 
				wavelengths. For thermal radiation calculations, the Planck function is integrated 
				over the wavelength bands defined in the second and third columns. The result will
				therefore be a band-integrated irradiance which does only make sense when the 
				\code{solar_file} grid equals the \code{thermal_bands_file} grid.
				''',
		'transmittance_wl_file' : r'''
				Location of single column file that sets the wavelength grid used for the 
				internal transmittance calculations. 
				\fcode{
				transmittance_wl__file file
				}
				The wavelengths must be in nm.
				Do not use this option unless you know what you are doing.
				Comments start with \code{\#}. Empty lines are ignored.
				''',
		'wavelength' : r'''
				Set the wavelength range by specifying first and last wavelength in nm. 
				\fcode{
				wavelength lambda_0 lambda_1
				}
				The default output wavelength grid is that defined in \code{solar_file}, 
				unless \code{spline} is specified. Note that the radiative transfer calculations 
				are done on an internal grid which can be influenced with \code{transmittance_wl_file}
				or \code{molecular_tau_file}
				''',
		'wavelength_index' : r'''
				Set the wavelengths to be selected. To be used together with predefined wavelength grids,
				such as \code{transmittance_wl_file}, \code{molecular_tau_file} and particularly 
				useful in combination with the \code{correlated_k} option where often only a 
				specified number of wavelength bands is required. E.g., in combination with
				\code{correlated_k AVHRR_KRATZ}, \code{wavelength_index 15 15} will select wavelength
				index 15 which corresponds to channel 4, or \code{wavelength_index 10 14} will select 
				those bands required for channel 3. Indices start from 1.
				''',
		'correlated_k' : r'''
				To calculate integrated shortwave or longwave irradiance, or to simulate 
				satellite instrument channels, use
				\fcode{
				   correlated_k type
				}
				to choose between the following types of correlated-k schemes:
				\begin{description}
				\item[kato] 
				\citet{Kato1999b} correlated-k distribution, shortwave; based on HITRAN 96. Please note that the 
				bands above 2.5 micrometer are not very reliable which, however, this has only little impact
				on integrated shortwave radiation.
				\item[kato2] 
				\citet{Kato1999b}, shortwave; optimized version (Seiji Kato, personal communication, 2003);
				please note that \code{kato2} only has 148 subbands (that is, calls to the \code{rte_solver}) 
				compared to 575 for \code{kato} which translates to a decrease in computational speed by 
				up to a factor of 4 with only little increase in uncertainty. The absorption data are
				based on HITRAN 2000. Please note that the bands above 2.5 micrometer are not very reliable which, 
				however, this has only little impact on integrated shortwave radiation.
				\item[kato2.96]
				\citet{Kato1999b}, shortwave; optimized version (Seiji Kato, personal communication, 2003);
				similar to \code{kato2} but based on HITRAN96. Please note that the bands above 2.5 micrometer 
				are not very reliable which, however, has only little impact on integrated shortwave radiation.
				\item[fu]
				\citet{fu92,fu93}, shortwave and longwave; fast parameterization, developed for climate models.
				\item[avhrr\_kratz] 
				\citet{Kratz1995}, AVHRR instrument channels
				\item[lowtran] 
				Gas absorption parameterization from LOWTRAN; code adopted from SBDART \citep{Ricchiazzi1998b}; 
				please see the section on "Spectral resolution".
				\item[sbdart] 
				Identical to LOWTRAN.
				\end{description}
				If \code{correlated_k} kato/kato2/kato2.96/fu/avhrr\_kratz is specified, the extraterrestrial 
				flux is taken from
				internally defined files specific for each parameterization, not 
				from \code{solar_file}. The output is the integrated irradiance for 
				each band. To get e.g. integrated shortwave irradiance, simply add all 
				bands of the \citet{Kato1999b} or the \citet{fu92,fu93}
				parameterization. The five AVHRR channels are weighted sums of the
				libRadtran output. Examples how to integrate the output in the
				\code{avhrr_kratz} case are included in the \code{uvspec} self check
				which is initiated with  
				\code{make check}.
				'''
		}
