
"""--------------------------------------------------------------------
 * $Id: surface_options.py 2769 2012-08-17 13:37:01Z svn-kylling $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""


from option_definition import *

class setup_surface_group():

	group_name = 'Surface'

	def __init__(self):
		surface_documentation = get_surface_documentation()

		albedo = option(
			name='albedo',
			group='surface',
			helpstr='Lambertian surface albedo',
			documentation=surface_documentation['albedo'], 
			tokens=addToken('Input.alb.albedo', float, default=0.0, valid_range=[0,1]),
			settings=addSetting('Input.alb.source', 'ALBEDO_CONSTANT', default='NOT_DEFINED_INTEGER'), 
			parents=['uvspec'], 
			non_parents=['albedo_file','albedo_library','brdf_ambrals',
 				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'],
			childs=''
			)

		albedo_file = option(
			name='albedo_file',
			group='surface',
			helpstr='Location of surface albedo file for wavelength dependent surface albedo.',
			documentation=surface_documentation['albedo_file'], 
			tokens=addToken('Input.filename[FN_ALBEDO]', file), 
			settings=[ addSetting('Input.alb.source', 'ALBEDO_FROM_ALBEDO_FILE', default='NOT_DEFINED_INTEGER'), 
				   addSetting('Input.alb.surf_type_map', False, default=False), 
				   addSetting('Input.alb.albedo', -1.0, default=0.0) ],
			parents=['uvspec'],  
			non_parents=['albedo','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'], 
			)

		albedo_library = option(
			name='albedo_library',
			group='surface',
			helpstr='Spectral albedos of different surface types.',
			documentation=surface_documentation['albedo_library'], 
			tokens=addToken('Input.alb.library', str, valid_range=['IGBP',file]),
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library'],
			childs=['surface_type']
			)

		brdf_ambrals = option(
			name='brdf_ambrals',
			group='surface',
			helpstr='Three-parameter BRDF fit for vegetated and non-vegetated surfaces.',
			documentation=surface_documentation['brdf_ambrals'],
			tokens=[ addToken('Input.ambrals.iso', float, default='NOT_DEFINED_FLOAT'), 
				 addToken('Input.ambrals.vol', float, default='NOT_DEFINED_FLOAT'), 
				 addToken('Input.ambrals.geo', float, default='NOT_DEFINED_FLOAT') ], 
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','albedo_library',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'], 
			)

		cox_and_munk_pcl = conditional_enable_option(
			name='cox_and_munk_pcl',
			group='surface',
			helpstr='Pigment concentration for ocean BRDF.',
			documentation=surface_documentation['cox_and_munk_pcl'], 
			tokens=addToken('Input.cm.pcl', float, default='NOT_DEFINED_FLOAT', valid_range=[0,100]),
			settings=addSetting('Input.disort2_brdf', 'BRDF_CAM', default='BRDF_NONE'), 
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'],
			speaker='rte_solver',
			enable_values=("disort","disort2")
			)
		
		cox_and_munk_sal = option(
			name='cox_and_munk_sal',
			group='surface',
			helpstr='Salinity for ocean BRDF.',
			documentation=surface_documentation['cox_and_munk_sal'], 
			tokens=addToken('Input.cm.sal', float, default='NOT_DEFINED_FLOAT', valid_range=[0,55]), 
			settings=addSetting('Input.disort2_brdf', 'BRDF_CAM', default='BRDF_NONE'), 
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'],
			)
		
		cox_and_munk_solar_wind = option(
			name='cox_and_munk_solar_wind',
			group='surface',
			helpstr='Wind azimuth identical to the incoming photon azimuth.',
			documentation=surface_documentation['cox_and_munk_solar_wind'],
			settings=[ addSetting('Input.disort2_brdf', 'BRDF_CAM', default='BRDF_NONE'), 
				   addSetting('Input.cm.solar_wind', 1, default=0) ],
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'],
			)
		
		cox_and_munk_u10 = option(
			name='cox_and_munk_u10',
			group='surface',
			helpstr='Wind speed for ocean BRDF.',
			documentation=surface_documentation['cox_and_munk_u10'], 
			tokens=addToken('Input.cm.u10', float, default='NOT_DEFINED_FLOAT', valid_range=[1,1e6]), 
			settings=addSetting('Input.disort2_brdf', 'BRDF_CAM', default='BRDF_NONE'), 
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'],
			)
		
		cox_and_munk_u10_map = option(
			name='cox_and_munk_u10_map',
			group='surface',
			helpstr='Location of netCDF file with wind speed for ocean BRDF.', 
			documentation=surface_documentation['cox_and_munk_u10_map'], 
			tokens=addToken('Input.filename[FN_U10_MAP]', file), 
			settings=addSetting('Input.disort2_brdf', 'BRDF_CAM', default='BRDF_NONE'), 
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'],
			)
		
		cox_and_munk_uphi = option(
			name='cox_and_munk_uphi',
			group='surface',
			helpstr='Wind direction for ocean BRDF.',
			documentation=surface_documentation['cox_and_munk_uphi'], 
			tokens=addToken('Input.cm.uphi', float, default='0.0', valid_range=[0,1e6]), 
			settings=addSetting('Input.disort2_brdf', 'BRDF_CAM', default='BRDF_NONE'), 
			parents=['uvspec'],
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'rpv_file', 'rpv_k', 'rpv_rho0', 'rpv_theta', 'rpv_sigma', 'rpv_t1', 'rpv_t2', 
				     'rpv_scale', 'rpv_library','surface_type'],
			)
		
		earth_radius = option(
			name='earth_radius',
			group='surface',
			helpstr='Radius of the Earth in km',
			documentation=surface_documentation['earth_radius'], 
			tokens=addToken('Input.r_earth', float, default=6370.0),
			parents=['uvspec'], 
			)

		fluorescence = option(
			name='fluorescence',
			group='surface',
			helpstr='Bottom surface isotropic fluorescence source.',
			documentation=surface_documentation['fluorescence'], 
			tokens=addToken('Input.flu.fluorescence', float, default=0.0),
			settings=addSetting('Input.flu.source', 'FLUORESCENCE_CONSTANT', default='NOT_DEFINED_INTEGER'), 
			parents=['uvspec'], 
			non_parents=['fluorescence_file'],
			childs=''
			)

		fluorescence_file = option(
			name='fluorescence_file',
			group='surface',
			helpstr='Bottom surface isotropic fluorescence source file, wavelength dependent.',
			documentation=surface_documentation['fluorescence_file'], 
			tokens=addToken('Input.filename[FN_FLUORESCENCE]', file), 
			settings=[ addSetting('Input.flu.source', 'FLUORESCENCE_FROM_FLUORESCENCE_FILE', default='NOT_DEFINED_INTEGER')],
			parents=['uvspec'],  
			non_parents=['fluorescence_file'], 
			)

		rpv_file = option(
			name='rpv_file',
			group='surface',
			helpstr='File containing the RPV BDRF parameterization.', 
			documentation=surface_documentation['rpv_file'], 
			tokens=addToken('Input.filename[FN_RPV]', file), 
			settings=addSetting('Input.rpv.source', 'RPV_FROM_RPV_FILE', default='RPV_CONSTANT'), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)
		
		rpv_k = option(
			name='rpv_k',
			group='surface',
			helpstr='Constant RPV k.',
			documentation=surface_documentation['rpv_k'], 
			tokens=addToken('Input.rpv.k', float, default=0.0), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)

		rpv_rho0 = option(
			name='rpv_rho0', 
			group='surface',
			helpstr='Constant RPV rho0.',
			documentation=surface_documentation['rpv_rho0'], 
			tokens=addToken('Input.rpv.rho0', float, default='0.0'),
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)
		
		rpv_theta = option(
			name='rpv_theta',
			group='surface',
			helpstr='Constant RPV theta.', 
			documentation=surface_documentation['rpv_theta'], 
			tokens=addToken('Input.rpv.theta', float, default='0.0'), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)
		
		rpv_sigma = option(
			name='rpv_sigma', 
			group='surface',
			helpstr='Constant RPV sigma.',
			documentation=surface_documentation['rpv_sigma'], 
			tokens=addToken('Input.rpv.sigma', float, default='0.0'), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)
		
		rpv_t1 = option(
			name='rpv_t1', 
			group='surface',
			helpstr='Constant RPV t1.', 
			documentation=surface_documentation['rpv_t1'], 
			tokens=addToken('Input.rpv.t1', float, default='0.0'), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)
		
		rpv_t2 = option(
			name='rpv_t2', 
			group='surface',
			helpstr='Constant RPV t2.', 
			documentation=surface_documentation['rpv_t2'], 
			tokens=addToken('Input.rpv.t2', float, default='0.0'), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)

		rpv_scale = option(
			name='rpv_scale',	
			group='surface',
			helpstr='Apply a constant scaling factor for the RPV BRDF.',	
			documentation=surface_documentation['rpv_scale'], 
			tokens=addToken('Input.rpv.scale', float, default='1.0'), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			)

		rpv_library = option(
			name='rpv_library',
			group='surface',
			helpstr='Collections of spectral BRDFs of different surface types.', 
			documentation=surface_documentation['rpv_library'], 
			tokens=addToken('Input.rpv.library', str, valid_range=['IGBP', file]), 
			parents=['uvspec'], 
			non_parents=['albedo','albedo_file','albedo_library','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi',
				     'surface_type'],
			childs=['surface_type']
			)
		
		surface_type = option(
			name='surface_type',	
			group='surface',
			helpstr='Select surface type.',
			documentation=surface_documentation['surface_type'], 
			tokens=addToken('Input.alb.surface', int, default='NOT_DEFINED_INTEGER', valid_range=[0,99]), 
			settings=addSetting('Input.alb.surf_type_map', False), 
			non_parents=['albedo','albedo_file','brdf_ambrals',
				     'cox_and_munk_pcl', 'cox_and_munk_sal', 'cox_and_munk_solar_wind', 
				     'cox_and_munk_u10', 'cox_and_munk_u10_map', 'cox_and_munk_uphi'],
			parents=['albedo_library','rpv_library'], 
			)
		
		surface_temperature = option(
			name='surface_temperature', 
			group='surface',
			helpstr='Surface temperature.',
			documentation=surface_documentation['surface_temperature'], 
			tokens=addToken('Input.surface_temperature', float, default='NOT_DEFINED_FLOAT'), 
			parents=['uvspec'], 
			)

		self.options = [albedo, albedo_file, albedo_library, brdf_ambrals,
				cox_and_munk_pcl, cox_and_munk_sal, cox_and_munk_solar_wind, 
				cox_and_munk_u10, cox_and_munk_u10_map, cox_and_munk_uphi, 
				rpv_file, rpv_k, rpv_rho0, rpv_theta, rpv_sigma, rpv_t1, rpv_t2, 
				rpv_scale, rpv_library, 
				fluorescence, fluorescence_file,
				surface_type, surface_temperature, earth_radius  ]

	def __iter__(self):
		return iter(self.options)

def get_surface_documentation():
	return {
		'albedo' : r'''
				The Lambertian surface albedo
				\fcode{
				   albedo value
				}
				where \code{value} is a number between 0.0 and 1.0,
				constant for all wavelengths. For wavelength dependent surface albedo 
				use \code{albedo_file}. The default albedo is 0.0.
				''',
		'albedo_file' : r'''
				Location of surface albedo file for wavelength dependent surface albedo. 
				\fcode{
				   albedo_file file
				}
				The file must have two columns.
				Column 1 is the wavelength in nm, and column 2 the corresponding 
				Lambertian surface albedo. An arbitrary wavelength grid may be chosen as the 
				albedo will be interpolated linearely to the wavelength grid used for the 
				radiation calculation. Comments start with \code{\#}. Empty lines are ignored.
				A large collection of spectral albedos are available e.g. at http://speclib.jpl.nasa.gov/ 
				\citep{Baldridge2009}.
				''',
		'albedo_library' : r'''
				Albedo libraries are a collection of spectral albedos of different surface types. 
				This option must be used either with \code{surface_type} 
				or \code{surface_type_map}, in order to select the specific surface type.
				There are two possibilities for libraries: 
				the built-in IGBP library or a user defined albedo library.
	
				The built-in library of the International Geosphere Biosphere Programme is 
				selected with 
				\fcode{
				     albedo_library IGBP
				}
				The IGBP library contains 20 surface types which are set by \code{surface_type}:
				\fcode{
				 1 evergreen_needle_forest\\
				 2 evergreen_broad_forest\\
				 3 deciduous_needle_forest\\
				 4 deciduous_broad_forest\\
				 5 mixed_forest\\
				 6 closed_shrub\\
				 7 open_shrubs\\
				 8 woody_savanna\\
				 9 savanna\\
				10 grassland\\
				11 wetland\\
				12 cropland\\
				13 urban\\
				14 crop_mosaic\\
				15 antarctic_snow\\
				16 desert\\
				17 ocean_water\\
				18 tundra\\
				19 fresh_snow\\
				20 sea_ice\\
				}
				Surface types 1 - 17 are defined by the International Geosphere Biosphere 
				Programme (IGBP); additionally there are tundra, fresh\_snow, and sea\_ice surface types.
				The spectral albedo of the ground is determined as a function of solar 
				zenith angle, precitable water, and clouds. The spectral resolution 
				equals the grid of the correlated-k Fu/Liou parameterisation.
				This library originates from the NASA CERES/SARB Surface Properties Project, 
				see \citet{Belward1996}.
	
				For creating your own albedo library use \code{albedo_library path}, where \code{path} is 
				the path of the directory where the albedo data is stored. The files are expected to have the 
				names \code{albedo_01.dat, albedo_02.dat, ...} If \code{surface_type 1} is specified
				the albedo from \code{albedo_01.dat} will be used, and so on.  
				Each file is required to have two columns:
				Column 1 is the wavelength in nm, and column 2 the corresponding 
				Lambertian surface albedo. The wavelength grid may be freely set. The 
				albedo will be interpolated linearely to the wavelength grid used for the 
				radiation calculation. Comments start with \code{\#}. Empty lines are ignored.
				This option is similar to \code{albedo_file}, except that it offers an
				easy way to use the option \code{surface_type_map} in combinition with albedo files.
				''',
		'brdf_ambrals' : r'''
				AMBRALS BRDF, a three-parameter BRDF fit for vegetated and 
				non-vegetated surfaces \citep{wanner97}.
				\fcode{
				 brdf_ambrals iso_value vol_value geo_value
				}
				Specify iso, vol, and geo. May be combined with \ifmystic{\code{montecarlo}},
				\code{cdisort}, and \code{fdisort2}.
				''',
		'earth_radius' : r'''
				Specify the earth radius in km. 
				\fcode{
				    earth_radius value
				}
				This is needed by all solvers in spherical geometry\ifmystic{, e.g. \code{mystic}
				in combination with option \code{mc_spherical}}. The default value is 6370 km.
				''',
		'fluorescence' : r'''
				Specifies the magnitude of a bottom surface isotropic fluorescence source.
				\fcode{
				   fluorescence value
				}
				where \code{value} is a number greater or equal to 0.0, 
				constant for all wavelengths.
				Must be used together with  \code{solar_file}. The units of the fluorescence should 
				obviously be the same as for the solar source in \code{solar_file}.
				For wavelength dependent fluorescence use \code{fluorescence_file}. The default fluorescence is 0.0. 
				Currently only works with the \code{cdisort} solver.
				''',
		'fluorescence_file' : r'''
				Location of fluorescence file for wavelength dependent fluorescence emission 
				from the bottom surface. 
				\fcode{
				   fluorescence_file file
				}
				The file must have two columns.
				Column 1 is the wavelength in nm, and column 2 the corresponding 
				fluorescence. An arbitrary wavelength grid may be chosen as the 
				fluorescence will be interpolated linearely to the wavelength grid used for the 
				radiation calculation. Comments start with \code{\#}. Empty lines are ignored.
				Currently only works with the \code{cdisort} solver. Furthermore. if \code{raman} is 
				not set, \code{transmittance_wl_file} 
				must be specified with the same resolution as the \code{solar_file}, and the first value
				must be the value specified by \code{wavelength}. The units of the fluorescence should 
				obviously be the same as for the \code{solar_file}.
				''',
		'surface_type' : r'''
				With this option the \code{surface_type} is selected.
				This option can be used with \code{albedo_library} in order to select a spectral albedo 
				or with \code{rpv_library} in order to select a BRDF function.
				\fcode{
				surface_type surface_type_number
				}
				where \code{surface_type_number} is an integer starting from 0,
				where 0 refers to a black surface and the following numbers to 
				the entries in the specified \code{library}.
				''',
		'rpv_file' : r'''
				4 column file, containing the Rahman, Pinty, and Verstraete (RPV) BDRF
				parameterization \citep{Rahman1993}.
				\fcode{
				rpv_file file
				}
				Bidirectional reflectance distribution functions for a variety of
				surfaces are given in the paper. This option is only supported with
				solvers: \code{cdisort}, \code{fdisort2}\ifmystic{ and \code{rte_solver
				montecarlo} (MYSTIC)}. The columns of the input file are wavelength
				[nm], rho0, k, and theta.  The parameters are interpolated linearly to
				the internal wavelength grid.  To make sure that the results are
				reasonable, specify the RPV data on a wavelength grid similar or equal
				to that used internally for the radiative transfer calculation!
				Optionally, a fifth column with a constant scaling factor may be
				defined.  If it has seven columns, the fifth to seventh are sigma, t1,
				t2, and if it has eight, the eighth is scale again.
				''',
		'rpv_k' : r'''
				Constant RPV k, see \code{rpv_file}.
				\fcode{
				rpv_k value
				}
				\code{rpv_k} overwrites the wavelength-dependent
				value defined in \code{rpv_file}.
				''',
		'rpv_rho0' : r'''
				Constant RPV rho0, see \code{rpv_file}. 
				\fcode{
				rpv0 value
				}
				\code{rpv_rho0} overwrites the wavelength-dependent
				value defined in \code{rpv_file}.
				''',
		'rpv_theta' : r'''
				Constant RPV theta, see \code{rpv_file}. 
				\fcode{
				rpv_theta value
				}
				\code{rpv_theta} overwrites the wavelength-dependent
				value defined in \code{rpv_file}.
				''',
		'rpv_sigma' : r'''
				Constant RPV sigma, to be used for snow \citep{Degunther2000b}.
				\fcode{
				rpv_sigma value
				}
				A wavelength dependent sigma is not yet available.
				''',
		'rpv_t1' : r'''
				Constant RPV t1, to be used for snow \citep{Degunther2000b}.
				\fcode{
				rpv_t1 value
				}
				A wavelength dependent sigma is not yet available.
				''',
		'rpv_t2' : r'''
				Constant RPV t2, to be used for snow \citep{Degunther2000b}.
				\fcode{
				rpv_t2 value
				}
				A wavelength dependent sigma is not yet available.
				''',
		'rpv_scale' : r'''
				Apply a constant scaling factor for the RPV BRDF. 
				\fcode{
				rpv_scale value
				}
				Required e.g. if the the albedo should be set to a certain value. This
				factor is only used by \code{rte_solver cdisort}, \code{rte_solver
				fdisort2}\ifmystic{ and \code{rte_solver montecarlo}}.
				''',
		'rpv_library' : r'''
				The rpv libraries are collections of spectral BRDFs of different surface types, 
				This option must be used either with \code{surface_type} 
				or \code{surface_type_map}, in order to select the specific surface type.
							
				For using a \code{rpv_library} write 
				\fcode{
				rpv_library library_path 
				}
				where \code{library_path} is the path of the directory, where the BRDF data is stored. The files are 
				expected to have the names \code{IGBP.01.rpv, IGBP.02.rpv, ...} If \code{surface_type 1} is specified
				the BRDF from \code{IGBP.01.rpv} will be used, and so on.
				Each file must have the structure like an \code{rpv_file}.
				(This option is quite the same as \code{rpv_file}, except that it offers you an
				easy way to use the option \code{surface_type_map} in combinition with your \code{rpv_files}.)
				
				%ak \emph{For our group there is also a built-in library for IGBP surface types.}
				\fcode{
				rpv_library IGBP
				}
				The built-in library contains the first 17 surface types see
				\code{albedo_library}.  The data is given for the wavelengths 443nm,
				565nm, 670nm, and 865nm. Stay near this wavelength in order to get
				reasonable results. In future this the rpv-library will be NDVI
				dependent, but until now the most common NDVI class is selected
				automatically.
				''',
		'cox_and_munk_pcl' : r'''
				Pigment concentration for \citet{cox54a,cox54b} ocean BRDF (in mg/m$^{-3}$).
				\fcode{
				   cox_and_munk_pcl value
				}
	
				At present only available with \code{rte_solver cdisort}, 
				\ifmystic{\code{rte_solver montecarlo}}
				and \code{rte_solver fdisort2}.  The
				number of streams (\code{nstr}) is automatically increased to 16 if
				\code{cox_and_munk} BRDF is switched on, to avoid numerical
				problems. The default value is 0.01 mg/m$^{-3}$. To switch on Cox and
				Munk BRDF, specify any of the \code{cox_and_munk} options and define
				at least \code{cox_and_munk_u10}.
				''',
		'cox_and_munk_sal' : r'''
				Salinity for \citet{cox54a,cox54b} ocean BRDF (in "per mille", 0.1\%; this 
				unit is equivalent to the other common units for salinity, 
				ppt - parts per thousand, psu - practical salinity unit).
				\fcode{
				   cox_and_munk_sal value
				}
				At present only available with \code{rte_solver cdisort}, 
				\ifmystic{\code{rte_solver montecarlo}}
				and \code{rte_solver fdisort2}.  The
				number of streams (\code{nstr}) is automatically increased to 16 if
				\code{cox_and_munk} BRDF is switched on, to avoid numerical
				problems. The default value is 34.3. To switch on Cox and Munk BRDF,
				specify any of the \code{cox_and_munk} options and define at least
				\code{cox_and_munk_u10}.
				''',
		'cox_and_munk_solar_wind' : r'''
				Use old definition of wind direction for Monte Carlo simulations. If this switch is set,
				the wind azimuth is identical to the incoming photon azimuth. Else, the wind azimuth is set by
				\code{cox_and_munk_uphi} or is 0 by default.
				''',
		'cox_and_munk_u10' : r'''
				Wind speed for \citet{cox54a,cox54b} ocean BRDF (in m/s).
				\fcode{
				   cox_and_munk_u10 value
				}
				At present only available with \code{rte_solver cdisort}, 
				\ifmystic{\code{rte_solver montecarlo},}
				 and \code{rte_solver fdisort2}.  The wind
				speed is the most important parameter affecting ocean BRDF.  The
				minimum allowed wind speed is 1 m/s because otherwise the strong
				specular reflection causes numerical problems. If a lower value is
				specified, the wind speed is automatically set to 1m/s. Also, the
				number of streams (\code{nstr}) is automatically increased to 16 if
				\code{cox_and_munk} BRDF is switched on, to avoid numerical problems.
				To switch on Cox and Munk BRDF, specify any of the \code{cox_and_munk}
				options and define at least \code{cox_and_munk_u10}.
				''',
		'cox_and_munk_u10_map' : r'''
				Specify wind speed (in m/s) for the \citet{cox54a,cox54b}
				ocean BRDF with the help of an \emph{netCDF} file, which is used in combinition 
				with the options \code{latitude}, \code{longitude}, and \code{time}.
				\fcode{
				cox_and_munk_u10_map file
				}
				where \code{file} is the location of the \emph{netCDF} file. 
				{\sl libRadtran} reads the value at the nearest pixel to the given \code{latitude} and 
				\code{longitude}. No spatial interpolation or averaging of the values is done.
	
				The file must contain the elements of the wind vector \code{U10} and
				\code{V10}.  These must be specified as functions of \code{latitude}
				and \code{longitude} \code{U10(lat, lon)}, \code{V10(lat, lon)}, or
				additionally may also depent on time \code{U10(time, lat, lon)},
				\code{V10(time, lat, lon)}.  If the variable time is present in the
				file, the wind speed will be interpolated according to the option
				\code{time_interpolate}. All grids must be provided as \code{double
				lat(lat)}, \code{double lon(lon)}, and \code{double time(time)}.
				''',
		'cox_and_munk_uphi' : r'''
				Wind direction for \citet{cox54a,cox54b} ocean BRDF. 
				\fcode{
				   cox_and_munk_uphi value
				}
				Default value is 0 degrees,
				which is wind from the South. 90 degrees corresponds to wind from the West, etc. (Honestly, this was
				never truly validated. It could possibly be that 0 is wind from the North, 90 is wind from the East, etc.)
				''',
				'surface_temperature' : r'''
				Surface temperature, used for thermal infrared calculations. 
				\fcode{
				surface_temperature value
				}
				If not specified, the
				temperature of the lowest atmospheric level is used as surface temperature.
				'''
}
