
"""--------------------------------------------------------------------
 * $Id: wc_options.py 2803 2012-09-10 20:17:52Z robert.buras $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------"""

from option_definition import *

class setup_wc_group():

	group_name = 'Water cloud'

	def __init__(self):
		wc_documentation = get_wc_documentation()
		#uvspec function to initialize new profile
		isp = addSetting( 'isp', 'get_caoth_index(&Input.caoth,&Input.n_caoth,"wc",1)')
		
		wc_file = option( #TODO: optional argument is first argument, then comes obligatory one
			'wc_file',	#name
			'wc',	#group
			'wc_definitions',	#supgroup
			'Location of file defining water cloud properties.',	#helpstr
			wc_documentation['wc_file'],	#documentation	
			addToken( 'Input.caoth[isp].filename', file ),	#tokens
			[isp, 
			 addSetting('Input.caoth[isp].source', 'CAOTH_FROM_FILE') ],	#additional settings
			['uvspec'],	#parents
			['wc_files','wc_ipa','wc_ipa_files','mc_wcloud_file'],	#enemies  (non_parents)
			['wc_set_tau','wc_set_tau550','wc_set_ssa','wc_set_gg',\
				 'wc_scale_tau','wc_scale_ssa','wc_scale_gg',\
				 'wc_level','wc_layer',\
				 'wc_properties','wc_properties_interpolate',\
				 'wc_no_scattering','wc_cloudcover'],	#children
			plot = {'plot_type': '2D',
				'optional_args': {'column_names': (
						"altitude",
						"liquid water content",
						"effective radius",)
						  }
				}
			)

		wc_files = option(
			'wc_files',	#name
			'wc',      #group
			'wc_definitions',      #supgroup
			'Location of file specifying cloud extinction coefficient, single scattering albedo, and scattering phase function for each layer.',	#helpstr
			wc_documentation['wc_files'],     #documentation
			addToken( 'Input.caoth[isp].filename', file),	#tokens
			[isp, 
			 addSetting('Input.caoth[isp].source', 'CAOTH_LAYER_FILES'),
			 addSetting('Input.caoth[isp].properties', 'PROP_EXPLICIT')
			 ],     #additional settings
			['uvspec'],     #parents
			['wc_file','wc_ipa','wc_ipa_files','mc_wcloud_file','wc_properties','wc_properties_interpolate',\
				 'wc_cloudcover'],     #enemies
			['wc_set_tau','wc_set_tau550','wc_set_ssa','wc_set_gg',\
				 'wc_scale_tau','wc_scale_ssa','wc_scale_gg',\
				 'wc_level','wc_layer',\
				 'wc_no_scattering']	#children
			#aky		['microphysics_mod', 'parameterization_def', 'parameterization_mod']	#childs
		)

		wc_ipa = option(
			'wc_ipa',	#name
			'wc',      #group
			'wc_definitions',	#supgroup
			'Independent column calculation for a 3D cloud field.',	#helpstr
			wc_documentation['wc_ipa'], #documentation
			addToken( 'Input.caoth[isp].filename', file),	#tokens
			[isp,
			 addSetting('Input.caoth[isp].source', 'CAOTH_FROM_3D_FILE'),
			 addSetting('Input.caoth[isp].ipa', 1),
			 addSetting('Input.ipa', 1, default=0)
			 ],     #additional settings
			['uvspec'],     #parents
			['wc_file','wc_files','wc_ipa_files','mc_wcloud_file',\
				 'wc_cloudcover'],     #enemies
			#aky		['wc_file','wc_files','wc_ipa_files'],     #non_parents
			['wc_set_tau','wc_set_tau550','wc_set_ssa','wc_set_gg',\
				 'wc_scale_tau','wc_scale_ssa','wc_scale_gg',\
				 'wc_level','wc_layer',\
				 'wc_properties','wc_properties_interpolate',\
				 'wc_no_scattering'],	#children
			#aky		['microphysics_mod', 'parameterization_def', 'parameterization_mod'],	#childs
			mystic = True
		)

		wc_ipa_files = option(
			'wc_ipa_files',	#name
			'wc',      #group
			'wc_definitions',      #supgroup
			'Location of file defining water cloud property files and the corresponding weights.',	#helpstr
			wc_documentation['wc_ipa_files'], #documentation
			addToken('Input.caoth[isp].filename', file),  #datatype
			[isp,
			 addSetting('Input.caoth[isp].source', 'CAOTH_IPA_FILES'),
			 addSetting('Input.caoth[isp].ipa', 1),
			 addSetting('Input.ipa', 1, default=0),
			 addSetting('Input.caoth[isp].properties', 'PROP_WC_HU')
			 ],     #additional settings
			['uvspec'],     #parents
			['wc_file','wc_files','wc_ipa','mc_wcloud_file','wc_cloudcover'],     #enemies
			['wc_set_tau','wc_set_tau550','wc_set_ssa','wc_set_gg',\
				 'wc_scale_tau','wc_scale_ssa','wc_scale_gg',\
				 'wc_level','wc_layer',\
				 'wc_properties','wc_properties_interpolate',\
				 'wc_no_scattering']	#children
			#aky		['microphysics_mod', 'parameterization_def', 'parameterization_mod']	#childs	#TODO: wc_properties hu wird gesetzt
		)

		wc_cloudcover = option(
			'wc_cloudcover', #name
			'wc',      #group
			'wc_modifications',	#supgroup
			'Fraction of the horizontal sky area which is covered by clouds.',	#helpstr
			wc_documentation['wc_cloudcover'],     #documentation
			addToken( 'Input.caoth[isp].cloudcover', float, valid_range=[0,1] ),	# tokens
			[isp, 
			 addSetting( 'Input.caoth[isp].ipa', 1 ),
			 addSetting( 'Input.ipa', 1, default=0 )
			 ],	#additional settings for uvspec
			['wc_file','mc_wcloud_file'],     #parents
			[],     # enemies
			[]      #children
		)

		wc_properties = option(
			'wc_properties',	#name
			'wc',      #group
			'parameterization_def',      #supgroup
			'Define how liquid water content and effective droplet radius are translated to optical properties. ',	#helpstr
			wc_documentation['wc_properties'], #documentation
			addToken( 'Input.caoth[isp].prop_name', str,
				  valid_range=['hu', 'echam4', 'mie', file]),     #tokens
			None,   #additional settings
			['wc_file','wc_ipa','wc_ipa_files','mc_wcloud_file'],     #parents
			[],     #enemies
			['wc_properties_interpolate']      #childs
			)

		wc_properties_interpolate = option(
			'wc_properties_interpolate',	#name
			'wc',      #group
			'parameterization_mod',	#supgroup
			'Interpolate water cloud optical properties.',	#helpstr
			wc_documentation['wc_properties_interpolate'], #documentation
			None, #tokens
			[isp,
			 addSetting( 'Input.caoth[isp].interpolate', 1) ],     #additional settings
			['wc_properties'],     #parents
			[],     #enemies
			[]      #childs
		)

		wc_set_gg = option(
			'wc_set_gg',	#name
			'wc',      #group
			'parameterization_mod',      #supgroup
			'Set the water cloud asymmetry factor.',	#helpstr
			wc_documentation['wc_set_gg'], #documentation
			addToken( 'Input.caoth[isp].gg', float, valid_range=[-1,1] ),  #tokens
			isp,     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files'],     #parents
			'',     #non_parents
			[],      #childs
			)

		wc_set_ssa = option(
			'wc_set_ssa',	#name
			'wc',      #group
			'parameterization_mod',      #supgroup
			'Set the water cloud single scattering albedo.',	#helpstr
			wc_documentation['wc_set_ssa'], #documentation
			addToken( 'Input.caoth[isp].ssa', float, valid_range=[0,1e6] ),  #tokens
			isp,     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files'],     #parents
			'',     #non_parents
			[],      #childs
		)

		wc_set_tau = option(
			'wc_set_tau',	#name
			'wc',      #group
			'parameterization_mod',      #supgroup
			'Set the total water cloud optical thickness.',	#helpstr
			wc_documentation['wc_set_tau'], #documentation
			addToken( 'Input.caoth[isp].tau', float, valid_range=[0,1e6]),  #tokens
			isp,     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files'],     #parents
			['wc_set_tau550'],     #enemies
			[],      #childs
			)

		wc_set_tau550 = option(
			'wc_set_tau550',	#name
			'wc',      #group
			'parameterization_mod',      #supgroup
			'Set the water cloud optical thickness at 550nm.',	#helpstr
			wc_documentation['wc_set_tau550'], #documentation
			addToken( 'Input.caoth[isp].tau550', float, valid_range=[0,1e6] ),  #tokens
			isp,     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files'],     #parents
			['wc_set_tau'],     #enemies
			[]      #childs
	   	)
	
		wc_scale_gg = option(
			'wc_scale_gg',	#name
			'wc',      #group
			'parameterization_mod',      #supgroup
			'Scale the water cloud asymmetry factor.',	#helpstr
			wc_documentation['wc_scale_gg'], #documentation
			addToken( 'Input.caoth[isp].scale_gg', float, valid_range = [0,1e6]),  #tokens
			isp,     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files'],     #parents
			[],     #non_parents
			[],      #childs
		)
		
		wc_scale_ssa = option(
			'wc_scale_ssa',	#name
			'wc',      #group
			'parameterization_mod',      #supgroup
			'Scale the water cloud single scattering albedo.',	#helpstr
			wc_documentation['wc_scale_ssa'], #documentation
			addToken( 'Input.caoth[isp].scale_ssa', float, valid_range=[0,1e6] ),  #tokens
			isp,     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files'],     #parents
			[],     #non_parents
			[],      #childs
		)
		
		wc_scale_tau = option(
			'wc_scale_tau',	#name
			'wc',      #group
			'parameterization_mod',      #supgroup
			'Scale the water cloud optical thickness.',	#helpstr
			wc_documentation['wc_scale_tau'], #documentation
			addToken( 'Input.caoth[isp].scale_tau', float, valid_range = [0,1e6]),  #tokens
			isp,     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files','mc_wcloud_file'],     #parents
			[],     #non_parents
			[],      #childs
		)
		
		wc_layer = option(
			'wc_layer',	#name
			'wc',      #group
			'microphysics_mod',	#supgroup
			'Interpret cloud properties as layer properties.',	#helpstr
			wc_documentation['wc_layer'], #documentation
			None, #tokens
			[isp, 
			 addSetting('Input.caoth[isp].layer', 'TRUE')],     #additional settings
			['wc_file','wc_files', 'wc_ipa', 'wc_ipa_files','mc_wcloud_file'],     #parents
			['wc_level'],     #non_parents
			[],      #childs
		)
		
		wc_level = option(
			'wc_level',	#name
			'wc',      #group
			'microphysics_mod',	#supgroup
			'Interpret cloud properties as level properties.',	#helpstr
			wc_documentation['wc_level'], #documentation
			None, #tokens
			[isp,
			 addSetting( 'Input.caoth[isp].layer', 'FALSE') ],     #additional settings
			['wc_file','wc_files','wc_ipa', 'wc_ipa_files','mc_wcloud_file'],     #parents
			['wc_layer'],     #non_parents
			[],      #childs
		)
		
		wc_no_scattering = option(
			'wc_no_scattering',	#name
			'wc',      #group
			'parameterization_mod',	#supgroup
			'Switch off scattering by water clouds.',	#helpstr
			wc_documentation['wc_no_scattering'], #documentation
			None, #tokens
			[isp,
			 addSetting('Input.caoth[isp].no_scattering', 'TRUE') ],     #additional settings
			['wc_file','wc_files','wc_ipa','wc_ipa_files','mc_wcloud_file'],     #parents
			[],     #non_parents
			[],      #childs
		)
	
	
		mc_wcloud_file = conditional_enable_option(
			name='mc_wcloud_file', 
			group='wc',
			subgroup='wc_definitions',
			helpstr='Location of 3D file defining water cloud properties.',
			documentation=wc_documentation['mc_wcloud_file'],     
			tokens=addToken( 'Input.caoth[isp].filename', file ),
			settings=[isp,
			 addSetting('Input.caoth[isp].source', 'CAOTH_FROM_3D_FILE') ],
			parents=['uvspec'], 
			childs=['wc_scale_tau',\
				 'wc_level','wc_layer',\
				 'wc_properties','wc_properties_interpolate',\
				 'wc_no_scattering','wc_cloudcover'],
			non_parents=['wc_file','wc_files','wc_ipa','wc_ipa_files'],
			speaker='rte_solver',
			enable_values=("mystic","montecarlo"),
			threedmystic =True,
			)

#		mc_wcloud_file = option( #TODO: optional argument is first argument, then comes obligatory one
#			'mc_wcloud_file',	#name
#			'wc',	#group
#			'wc_definitions',	#supgroup
#			,	#helpstr
#			wc_documentation['mc_wcloud_file'],	#documentation	
#			addToken( 'Input.caoth[isp].filename', file ),	#tokens
#			[isp,
#			 addSetting('Input.caoth[isp].source', 'CAOTH_FROM_3D_FILE') ],	#additional settings
#			['uvspec'],	#parents
#			['wc_file','wc_files','wc_ipa','wc_ipa_files'],	#enemies  (non_parents)
#			['wc_scale_tau',\
#				 'wc_level','wc_layer',\
#				 'wc_properties','wc_properties_interpolate',\
#				 'wc_no_scattering','wc_cloudcover'],	#children
#		)

		self.options = [wc_file,wc_files,wc_ipa,wc_ipa_files,mc_wcloud_file,wc_cloudcover,
				wc_set_tau,wc_set_tau550,wc_set_ssa,wc_set_gg,
				wc_scale_tau,wc_scale_ssa,wc_scale_gg,
				wc_level,wc_layer,
				wc_properties,wc_properties_interpolate,
				wc_no_scattering]
		
	def __iter__(self):
		return iter(self.options)

def get_wc_documentation():
	return {
		'wc_cloudcover'	: r'''
				Set the fraction of the horizontal sky area which is covered by clouds.
				\fcode{
				wc_cloudcover value
				}
				When a cloud cover is specified, the result will be calculated by
				the independent pixel approximation (IPA), that is, as weighted average 
				of cloudless sky and overcast sky, where the cloud properties are taken 
				from \code{wc_file}, etc. Please note that, if both \code{wc_cloudcover} and
				\code{ic_cloudcover} are set, both must be equal.
				
				This option is ignored, if the option \code{cloud_fraction_file} is used.
				''',
		'wc_file'	: r'''
				Location of file defining water cloud properties. 
				\fcode{
				wc_file file
				}
				The file must contain three columns: Column 1 is the altitude in km,
				column 2 the liquid water content (LWC) in grams per cubic meter, and
				column 3 the effective droplet radius in micrometer. Empty lines are
				ignored. Comments start with \code{\#}.  Note that the definition of
				cloud altitudes in \code{wc_file} refers to sea level, not to altitude
				above ground. E.g., when \code{altitude} is set to 1.63km, and the
				first cloud level is defined at 3km, the cloud would start at 1.37km
				above ground.  An example of a cloud is given in
				\file{examples/WC.DAT}.
				
				Per default the cloud properties are interpreted as layer
				properties. Before version 1.4 the default was level properties: The
				optical depth of a layer was calculated using information from the
				upper and lower levels defining the layer, see \code{wc_layer} and
				\code{wc_level}.  To switch to the old behaviour, use
				\code{wc_level}. See section~\ref{seq:wc} about water clouds for a
				realistic example how the contents of the \code{wc_file} are converted
				to optical properties.
				''',
		'wc_files'	: r'''
				A way to specify cloud extinction coefficient, single scattering albedo, 
				and scattering phase function for each layer. 
				\fcode{
				wc_files file
				}
				The file specified by 
				\code{wc_files} has two columns where column 1 is the altitude in km. The second
				column is the name of a file which defines the optical properties of the layer 
				starting at the given altitude. The files specified in the second column must 
				have the following format:
				\begin{description}
				\item{Column 1: } 
				The wavelength in nm. These wavelengths may be different from those in \code{solar_file}. 
				Optical properties are interpolated to the requested wavelengths.
				\item{Column 2:} 
				The extinction coefficient of the layer in units km-1. 
				\item{Column 3:} 
				The single scattering albedo of the layer.
				\item{Column 4-(nmom+4):} 
				The moments of the scattering phase function. 
				\end{description}
				Note that if using the \code{rte_solver cdisort} or \code{rte_solver
				fdisort2} it makes good sense to make the number of moments larger
				than \code{nstr} because all moments are used in the calculation. For
				\code{rte_solver fdisort1} and \code{rte_solver polradtran} the number
				of moments included in the calculations will be \code{nstr}+1. Higher
				order moments will be ignored for these solvers.  Please note that the
				uppermost line of the \code{wc_files} denotes simply the top altitude
				of the uppermost layer. The optical properties of this line are
				consequently ignored. There are two options for this line: either an
				optical property file with zero optical thickness is specified
				or "NULL" instead.
				''',
		'wc_ipa'	: r'''
				Independent column calculation for a 3D cloud field.
				\fcode{
				wc_ipa file
				}
				As argument a name of a 3D cloud file must be given. This file has to
				be in the format as needed by MYSTIC, see \code{mc_wcloud_file}. See
				\file{examples/UVSPEC_WC_IPA.INP} for an example.
				''',
		'wc_ipa_files'	: r'''
				A two-column file, defining water cloud property files (see \code{wc_file}) in the first 
				column and the corresponding weights in the second column. 
				\fcode{
				wc_ipa_files file
				}
				The radiative transfer calculation is performed independently for each
				cloud column and the result is the weighted average of all independent
				columns. If \code{ic_ipa_files} and \code{wc_ipa_files} are both
				defined, both must have the same columns in the same order, otherwise
				\code{uvspec} will complain.  See
				\file{examples/UVSPEC_WC_IPA_FILES.INP} for an example.
				''',
		'wc_layer'	: r'''
				Interpret cloud properties as layer properties (this is the default
				behaviour since version 1.4; see also \code{wc_file}). Cloud
				properties are assumed to be constant over the layer. The layer
				reaches from the level, where the properties are defined in the
				\code{wc_file} to the level above that one.  For example, the
				following lines
				\begin{Verbatim}
				#      z     LWC    R_eff
				#     (km)  (g/m^3) (um) 
				       4.000   0.0   0.0
				       3.000   1.0  10.0
				\end{Verbatim}
				define a cloud in the layer between 3 and 4 km with sharp boundaries.
				''',
		'wc_level'	: r'''
				Interpret cloud properties as level properties (this was the default
				behaviour before version 1.4; see also \code{wc_file}). If
				\code{wc_level} is defined, a \code{wc_file} would be interpreted as
				follows:
				\begin{Verbatim}
				#      z     LWC    R\_eff
				#     (km)  (g/m^3) (um)  
				       5.000    0      0   
				       4.000   0.2   12.0 
				       3.000   0.1   10.0 
				       2.000   0.1    8.0 
				\end{Verbatim}
				The value 0.2 g/m$^3$ refers to altitude 4.0km, as e.g. in a
				radiosonde profile. The properties of each layer are calculated as
				average over the adjacent levels. E.g. the single scattering
				properties for the model layer between 3 and 4km are obtained by
				averaging over the two levels 3km and 4km. To allow easy definition of
				sharp cloud boundaries, clouds are only formed if both liquid water
				contents above and below the respective layer are larger than
				0. Hence, in the above example, the layers between 2 and 3 as well as
				between 3 and 4km are cloudy while those between 1 and 2km and between
				4 and 5km are not.
				''',
		'wc_no_scattering'	: r'''
				Switch off scattering by water clouds.
				''',
		'wc_properties'	: r'''
				Possible choices for \code{type} are
				\begin{description}
				\parameter{hu} 
				Parameterization by \citet{Hu1993}; 
				this is the default setting. Note that 
				the parameterization is somewhat different for \code{correlated_k FU} than for all other 
				cases because in the latter case the parameterization from the newer (March 2000) 
				Fu and Liou code is used while otherwise the data are taken from the original 
				paper by \citet{Hu1993}. Note that this parameterization has been
				developed to calculate irradiances, hence it is less suitable for radiances.
				This is due to the use of the Henyey-Greenstein phase function as an approximation
				of the real Mie phase function.
				\parameter{echam4} 
				Use the very simple two-band parameterization of the ECHAM4 climate model, described 
				in \cite{Roeckner1996}; this is probably only meaningful if you want to compare
				your results with ECHAM4, the two bands are 0.2 - 0.68 micrometer and 0.68 - 4.0 micrometer;
				within these bands, the optical properties are assumed constant.
				\parameter{mie} 
				Use pre-calculated Mie tables; useful for \code{correlated_k}; 
				the tables are expected in \code{data_files_path}\file{/correlated_k/.../}. \\
				For spectral or pseudo-spectral (\code{correlated_k sbdart}) calculations, 
				a set of pre-calculated tables is also available.
				%CE: commented some parts. Should be rewritten when new Mie tables are well
				% optimized.
				% the wavelength grid points of these 
				%data has been carefully selected such that the extinction cross section, 
				%single scattering albedo, and the asymmetry parameter are accurate to 1\% 
				%(compared to the fully-resolved Mie calculation) for all wavelengths
				%between 250nm and 100 micrometer. 
				For spectral or pseudo-spectral
				calculations \code{wc_properties_interpolate} has to be defined explicitely to
				initiate the interpolation of the optical properties to the internal wavelength grid.
				%Please note that this option may be extremely memory-consuming because for each 
				%internal wavelength a full set of Legendre moments of the phase function is 
				%stored (up to several thousands). 
				The Mie tables are not part of the standard distribution 
				(because of their large size) but they are freely available from http://www.libradtran.org. 
				This is the correct option to calculate radiances, to be preferred over the 
				Henyey-Greenstein approach of \citet{Hu1993}.
				\parameter{filename} 
				Read optical properties from specified filename; file format is as produced 
				by the \code{mie}-tool of the {\sl libRadtran} package (see \code{output_user netcdf}).
				\end{description}
				''',
		'wc_properties_interpolate'	: r'''
				Interpolate water cloud optical properties over wavelength; useful for precalculated 
				optical property files defined with \code{wc_properties}. Please note that this option 
				may be extremely memory-consuming because for each internal wavelength a full set of 
				Legendre moments of the phase function is stored (up to several thousands). 
				''',
		'wc_scale_gg'	: r'''
				Scale the water cloud asymmetry factor for all wavelengths and altitudes
				with a float between 0.0 and 1.0.
				\fcode{
				wc_scale_gg value
				}
				''',
		'wc_scale_tau'	: r'''
				Scale the water cloud optical thickness for all wavelengths and altitudes.
				with a float between 0.0 and 1000000.0. Also works for 3d clouds.
				\fcode{
				wc_scale_tau value
				}
				''',
		'wc_scale_ssa'	: r'''
				Scale the water cloud single scattering albedo for all wavelengths and altitudes
				with a float between 0.0 and 1.0.
				\fcode{
				wc_scale_ssa value
				}
				''',
		'wc_set_gg'	: r'''
				Set the water cloud asymmetry factor for all wavelengths and altitudes
				to a float between -1.0 and 1.0. 
				Please note that this option is only applied if a Henyey-Greenstein 
				phase function is used but not if an explicit phase function is 
				defined e.g. with a wc\_files. It doesn't make sense to modify only 
				the first moment of an explicit phase function.  
				\fcode{
				wc_set_gg value
				}
				This option is useful only for monochromatic 
				calculations or in wavelength regions where the optical 
				properties of water clouds can be considered constant, 
				e.g. the ultraviolet range. 
				''',
		'wc_set_ssa'	: r'''
				Set the water cloud single scattering albedo for all wavelengths
				and altitudes to a float between 0.0 and 1.0. 
				\fcode{
				wc_set_ssa value
				}
				This option is useful only for monochromatic 
				calculations or in wavelength regions where the optical properties of water clouds 
				can be considered constant, e.g. the ultraviolet range.
				''',
		'wc_set_tau'	: r'''
				Set the total water cloud optical thickness to a constant value for all wavelengths.
				\fcode{
				wc_set_tau value
				}
				The optical thickness defined here is the integral from the surface at the 
				user-defined \code{altitude} to TOA (top of atmosphere). This option is useful only 
				for monochromatic calculations or in wavelength regions where the optical properties 
				of water clouds can be considered constant, e.g. the ultraviolet range.
				''',
		'wc_set_tau550'	: r'''
				Set the water cloud optical thickness at 550nm. 
				\fcode{
				wc_set_tau550 value
				}
				The optical thickness defined here 
				is the integral from the surface at the user-defined \code{altitude} 
				to TOA (top of atmosphere). Other wavelengths are scaled accordingly.
				Note that this option requires for technical reasons that the
				wavelength interval defined by \code{wavelength} does contain 550nm.
				''',
		'mc_wcloud_file' : r'''
				Define a MYSTIC 3D water cloud input file.
				\fcode{
				mc_wcloud_file file
				}
				The expected format of the cloud file is: 
				\fcode{
				Nx  Ny  Nz flag \\
				dx  dy  z(1) z(2) ... z(n) \\
				ix  iy  iz  ext  g     ssa  (if flag == 1)\\
				ix  iy  iz  ext  reff       (if flag == 2)\\
				ix  iy  iz  LWC  reff       (if flag == 3)\\
				}
				where \code{Nx}, \code{Ny} and \code{Nz} are the number of grid boxes in 
				\code{x}, \code{y}, and \code{z}-direction.
				The parameter \code{flag} determines the format of the 3rd and 
				following lines. In the second line \code{dx} and \code{dy} are the sizes of the boxes in x- 
				and y-direction in km. 
				In the third and following lines the indices \code{ix}, \code{iy}, and \code{iz} specify cloudy pixels. 
				The optical properties of the cloud, are given by the other parameters in the line, 
				where \code{ext} is the extinction coefficient [1/km], \code{g} the asymmetry parameter,
				\code{reff} the effective radius [micrometer], and \code{ssa} the single scattering albedo.
				The conversion from microphysical to optical properties is defined by \code{wc_properties} 
				(identical to the 1D case). Note that the first dimension (x)
				propagates east, and the second dimension (y) north. For more
				information see section~\ref{sec:mystic}.
				'''
                	}
