/*--------------------------------------------------------------------
 * $Id: lex_starter.l 2809 2012-09-11 08:51:28Z robert.buras $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2012 by Arve Kylling, Bernhard Mayer,
 *                            Claudia Emde, Robert Buras
 *
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/
/* the "incl" state is used for picking up the name
 * of an include file
 */
%x include

%{

#include <getopt.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sun.h>
#include <string.h>
#include <float.h>
       
#include "uvspec.h"
#include "ascii.h"
#include "atmosphere.h"
#include "ancillary.h"
#include "netCDF_functions.h"
#include "solver.h"
#include "cdisort.h"
#include "uvspecrandom.h"

#define Q(x) #x
#define QUOTE(x) Q(x)

#define THEVERSION QUOTE(LRTVERSION)

/* #define COPYPROTECTION 1 */

/* following line needed to suppress compiler warning about unused function "input" */
#define YY_NO_INPUT

#define QSORT_CAST (int (*)(const void *, const void *))

#if HAVE_OPENGL
  extern int GLmystic_hiddenline; /* hidden line removal */
#endif

static float *add_cmu_to_umu(int nstr, int numu, int maxumu, float* oldumu, int* cmuind, int* umuind);

static float  yytext2float(char* yytext, int which_token, int max_token);
static float* yytext2floats(char* yytext, int nary);
static double yytext2double(char* yytext, int which_token, int max_token);
static int    yytext2int(char* yytext, int which_token, int max_token);
static int*   yytext2integers(char* yytext, int nary);
static char*  yytext2string(char* yytext, int which_token, int max_token);
static int    yytext2ntokens(char* yytext);
static int    yytextCstring(char *dest, char* yytext, int which_token, int max_token);
static void free_output(input_struct input, output_struct *output);
#ifndef UVSPEC_FUNCTION
static int print_output(input_struct input, output_struct *output);
#else
static int print_output(input_struct input, output_struct *output, char *outfilename);
#endif
void write_flexstor_output     (input_struct Input, output_struct *output, FILE *f);
void write_polradtran_output   (input_struct Input, output_struct *output, FILE *f);
int write_header              (input_struct Input, output_struct *output, FILE *f);
void write_spectral_heat       (input_struct Input, output_struct *output, FILE *f);
int write_user_defined_output (input_struct Input, output_struct *output, FILE *f);
int check_if_result_exists     (int *output_user, int n_output_user, int solver, int **valid_result, int quiet);
int write_netCDF_output        (input_struct Input, output_struct *output);

char *s, *t, *u;
int ntokens;
int mol_id;
float tmpfloat;
int   tmpint;
float* tmp;
int*   tmpi;
int ierror=0;
int line_number;
input_struct Input;
output_struct Output;
int i=0, j=0, isp=0, iscr=0;
int iscloudcover=0, isipafiles=0;
int position=0;
float epsilon=1E-7;
  
#define EPSILON 1.E-6        /*relative rounding error for floats */
#define MAX_INCLUDE_DEPTH 10
YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
int include_stack_ptr = 0;

#ifndef PI
#define PI 3.14159265358979323846264338327
#endif

/* This is a fix needed at MIM, need to test whether this also works on DLR */
#if COMPILING_CONDOR_AT_MIM
#define fscanf __fscanf
#endif

#define UVSPEC_TOKEN " |\t "

%}

%option noyywrap
%option nounput

DIGIT    [0-9]
SPACE    [ \t]
CCHAR    [#]
COMMENT  ({CCHAR}.*)
INTEGER  {DIGIT}+
SPACE_INTEGER {SPACE}+{INTEGER}
INTEGERS {SPACE_INTEGER}+ 
EXPON   ([DdEe][+-]?{DIGIT}+)
FLOAT    ((({DIGIT}*+"."{DIGIT}*)|({DIGIT}*+"."{DIGIT}*{EXPON}))|({INTEGER}))
SPACE_FLOAT {SPACE}+{FLOAT}
FLOATS   {SPACE_FLOAT}+ 
SIGN     [-+]
SIGNED_FLOAT {SIGN}?{FLOAT}
SPACE_SIGNED_FLOAT {SPACE}+{SIGNED_FLOAT}
SIGNED_FLOATS {SPACE_SIGNED_FLOAT}+
SIGNED_INTEGER {SIGN}?{INTEGER}
WORD     [_\.\/A-Za-z][_\.\/A-Za-z0-9\-]*
DIM      [1,3][d,D]
SPACE_PROFILETYPE {SPACE}+({DIM}|{WORD})
SPACE_WORD {SPACE}+{WORD}
WORDS {SPACE_WORD}*
FLOAT_OR_WORD ({SPACE_FLOAT}|{SPACE_WORD})
FLOATS_OR_WORDS {FLOAT_OR_WORD}*
SIGNED_FLOAT_OR_WORD ({SPACE_SIGNED_FLOAT}|{SPACE_WORD})
SIGNED_FLOATS_OR_WORDS {SIGNED_FLOAT_OR_WORD}*

%%

  /* These are the allowed input lines in the uvspec input file 
     The input line specifications must come before the eating of
     blanks, otherwise things get messed up 
     Obviously this whole lot could have been handled more properly
     using bison etc., but then rather few would know what was going 
     on??? 
   */


 /* PYTHON_OPTIONS */
absorption{SPACE}+{WORD}{SPACE}+{WORD} { s = (char *) calloc (strlen(yytext)+1,sizeof(char));
            strcpy (s, yytext);
            ntokens = yytext2ntokens(s);
            free(s);
            fflush (stderr);
            s = yytext2string(yytext,2,ntokens+1);
            t = yytext2string(yytext,3,ntokens+1);
	    if (strncasecmp(t,"on",2)==0) {
	      if (     strncasecmp("O4",  s,2)==0) {Input.ck_o4abs   = 1;}
	      else if (strncasecmp("N2",  s,2)==0) {Input.ck_n2abs   = 1;}
	      else if (strncasecmp("CO" , s,2)==0) {Input.ck_coabs   = 1;}
	      else if (strncasecmp("SO2", s,3)==0) {Input.ck_so2abs  = 1;}
	      else if (strncasecmp("NH3", s,3)==0) {Input.ck_nh3abs  = 1;}
	      else if (strncasecmp("NO",  s,2)==0) {Input.ck_noabs   = 1;}
	      else if (strncasecmp("HNO3",s,4)==0) {Input.ck_hno3abs = 1;}
	      else {fprintf(stderr,"Unknown 'absorption %s' on line %d\n", s, line_number+1); ierror++;}
            }
	    else if (strncasecmp(t,"off",3)==0) {
	      if (     strncasecmp("O4",  s,2)==0) {Input.ck_o4abs   = 0;}
	      else if (strncasecmp("N2",  s,2)==0) {Input.ck_n2abs   = 0;}
	      else if (strncasecmp("CO" , s,2)==0) {Input.ck_coabs   = 0;}
	      else if (strncasecmp("SO2", s,3)==0) {Input.ck_so2abs  = 0;}
	      else if (strncasecmp("NH3", s,3)==0) {Input.ck_nh3abs  = 0;}
	      else if (strncasecmp("NO",  s,2)==0) {Input.ck_noabs   = 0;}
	      else if (strncasecmp("HNO3",s,4)==0) {Input.ck_hno3abs = 0;}
	      else {fprintf(stderr,"Unknown 'absorption %s' on line %d\n", s, line_number+1); ierror++;}
           }
	   free(s); free(t);}

aerosol_species_file{SPACE_WORD}{SPACE_WORD}+ {
				     ntokens = yytext2ntokens(yytext);
                                     yytextCstring (Input.aer.filename[FN_AER_SPECIES], yytext, 1+1, ntokens+1);
				     Input.aer.n_species = ntokens - 1; /* one token for type */
                                     Input.aer.species_names = (char **) calloc (Input.aer.n_species, sizeof(char *));
                                     for (i=3;i<=ntokens+1; i++) {
				         Input.aer.species_names[i-3] = yytext2string(yytext, i, ntokens+1);
				     }
                                     Input.aer.spec =1; } 

albedo_map{WORDS}{SPACE_FLOAT}*    { if (Input.alb.source != NOT_DEFINED_INTEGER) {
                                       fprintf (stderr, "*** Warning, 'albedo_map' on line %d\n",line_number+1);
                                       fprintf (stderr, "*** another albedo was already defined in the input file!\n"); 
                                     }
                                     ntokens = yytext2ntokens(yytext);
                                     if ( ntokens < 1 ||  3 < ntokens ) {
                                       fprintf (stderr, "Error, 'albedo map' on line %d requires ONE or TWO optional arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     Input.alb.source = ALBEDO_FROM_ALBEDO_MAP;
                                     yytextCstring (Input.filename[FN_ALBEDO_MAP], yytext, 1+1, ntokens+1);

                                     if (ntokens > 1)
                                       yytextCstring (Input.alb.netCDF_alb_name,     yytext, 2+1, ntokens+1);
                                     if (ntokens > 2)
                                       Input.alb.scale_factor  =   yytext2float(     yytext, 3+1, ntokens+1); 
                                   }
albedo_surface{SPACE_INTEGER} {
                                     Input.alb.surface = yytext2int(yytext,2,2);
                                     Input.alb.surf_type_map = FALSE;}
altitude_map{SPACE_WORD}+{SPACE_FLOAT}*  {ntokens = yytext2ntokens(yytext);
                                    if ( ntokens < 1 ||  3 < ntokens ) {
                                      fprintf (stderr, "Error, 'altitude map' on line %d requires ONE or TWO optional arguments,\n",line_number+1); 
                                      ierror++;
                                    }
                                    Input.alt.source = ALT_FROM_MAP; 
                                    yytextCstring (Input.filename[FN_ALTITUDE_MAP], yytext, 1+1, ntokens+1);

                                    if (ntokens > 1)
                                      yytextCstring (Input.alt.netCDF_alt_name,     yytext, 2+1, ntokens+1);
                                    if (ntokens > 2)
                                      Input.alt.scale_factor  =   yytext2float(     yytext, 3+1, ntokens+1); 
                                   }
angstrom{SIGNED_FLOATS}             {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.aer.alpha    = yytext2float(s,2,3); 
                                     Input.aer.beta     = yytext2float(yytext,3,3);
				     free(s);
                                     Input.aer.spec++;}
atm_z_grid{SIGNED_FLOATS}           {Input.atm.nz_atm_forced_sea = yytext2ntokens(yytext);
                                     Input.atm.z_atm_forced_sea = yytext2floats(yytext, Input.atm.nz_atm_forced_sea);
                                     /* reverse order of the levels */
                                     for (i=0;i<Input.atm.nz_atm_forced_sea/2;i++){
                                       tmpfloat = Input.atm.z_atm_forced_sea[i];
                                       Input.atm.z_atm_forced_sea[i] = Input.atm.z_atm_forced_sea[Input.atm.nz_atm_forced_sea-1-i];
                                       Input.atm.z_atm_forced_sea[Input.atm.nz_atm_forced_sea-1-i] = tmpfloat;
                                     }
                                    }

bpdf_tsang_u10{SPACE}+{FLOAT}       {ntokens = yytext2ntokens(yytext); 
                                       if ( ntokens !=1 ) {
                                         fprintf (stderr, "Error, 'bpdf_tsang_u10' on line %d requires wind speed as argument.\n",line_number+1); 
                                         ierror++;
                                       }
                                       else {  
                                        Input.bpdf.type = BPDF_TSANG; 
                                        Input.bpdf.u10 = yytext2float(yytext,2,2); 
                                       }
                                    }  
cloud_fraction_file{SPACE}+{WORD}   {yytextCstring(Input.filename[FN_CLOUD_FRACTION],yytext,2,2); }
cloud_fraction_map{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_CLOUD_FRACTION_MAP],yytext,2,2); }
cloud_overlap{SPACE}+{WORD}         {s = yytext2string(yytext,2,2);
                                      if      (strlen(s)==4  && strncasecmp("rand", s, 4)==0) 
				                      {Input.cloud_overlap = CLOUD_OVERLAP_RAND;}
				      else if (strlen(s)==3  && strncasecmp("max", s, 3)==0) 
				                      {Input.cloud_overlap = CLOUD_OVERLAP_MAX;
						      Input.ipa=1; Output.nipa=2;}
				      else if (strlen(s)==7  && strncasecmp("maxrand", s, 7)==0)
				                      {Input.cloud_overlap = CLOUD_OVERLAP_MAXRAND;
						      Input.ipa=1; Output.nipa=2;}
				      else if (strlen(s)==3  && strncasecmp("off", s, 3)==0)
				                      {Input.cloud_overlap = CLOUD_OVERLAP_OFF;}
				      else  {fprintf(stderr,"Unknown 'cloud overap scheme %s' on line %d\n", s, line_number+1); ierror++;}		      
				     free(s);
				     }
cox_and_munk_pcl_map{WORDS}{SPACE_FLOAT}* {Input.disort2_brdf = BRDF_CAM;
                                     ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 3) {
                                       fprintf (stderr, "Error, 'cox_and_munk_pcl_map' on line %d requires ONE or TWO arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     yytextCstring (Input.filename[FN_PIGMENTS_MAP], yytext, 1+1, ntokens+1); 
                                     if (ntokens > 1 )
                                       yytextCstring (Input.cm.pcl_netCDF_name,      yytext, 2+1, ntokens+1);
                                     if (ntokens > 2 )
                                       Input.cm.pcl_scale_factor = yytext2float    ( yytext, 3+1, ntokens+1); 
                                    }
cox_and_munk_sal_map{WORDS}{SPACE_FLOAT}* {Input.disort2_brdf = BRDF_CAM; 
                                     ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 3) {
                                       fprintf (stderr, "Error, 'cox_and_munk_sal_map' on line %d requires ONE or TWO arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     yytextCstring (Input.filename[FN_SALINITY_MAP], yytext, 1+1, ntokens+1);
                                     if (ntokens > 1 )
                                       yytextCstring (Input.cm.sal_netCDF_name,      yytext, 2+1, ntokens+1);
                                     if (ntokens > 2 )
                                       Input.cm.sal_scale_factor = yytext2float    ( yytext, 3+1, ntokens+1); 
                                    }
crs_file{SPACE}+{WORD}{SPACE}+{WORD} {
                                     s = yytext2string(yytext,2,4);
				     if      (strncasecmp("O3",s,2)==0)   {yytextCstring(Output.crs.filename[MOL_O3],yytext,3,4);}
				     else if (strncasecmp("O2",s,2)==0)   {yytextCstring(Output.crs.filename[MOL_O2],yytext,3,4);}
				     else if (strncasecmp("H2O",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_H2O],yytext,3,4);}
				     else if (strncasecmp("CO2",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_CO2],yytext,3,4);}
				     else if (strncasecmp("NO2",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_NO2],yytext,3,4);}
				     else if (strncasecmp("BRO",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_BRO],yytext,3,4);}
				     else if (strncasecmp("OCLO",s,4)==0) {yytextCstring(Output.crs.filename[MOL_OCLO],yytext,3,4);}
				     else if (strncasecmp("HCHO",s,4)==0) {yytextCstring(Output.crs.filename[MOL_HCHO],yytext,3,4);}
				     else if (strncasecmp("O4",s,2)==0)   {yytextCstring(Output.crs.filename[MOL_O4],yytext,3,4);}
				     else if (strncasecmp("SO2",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_SO2],yytext,3,4);}
                                     else {fprintf(stderr,"Unknown 'crs_file %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
                                     }
data_files_path{SPACE}+{WORD}       {yytextCstring (Input.filename[FN_PATH],
						    yytext, 2, 2); 
                                     /* append slash */
                                     if (Input.filename[FN_PATH][strlen(Input.filename[FN_PATH])-1]!= '/')
				       strcat (Input.filename[FN_PATH], "/");}
dens_column{SPACE}+{WORD}{SPACE}+{FLOAT}{SPACE_WORD}? {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);
				     s =  yytext2string(yytext,2,ntokens+1);
				     if      (strncasecmp("O3",s,2)==0)   {mol_id = MOL_O3;  }
                                     else if (strncasecmp("O2",s,2)==0)   {mol_id = MOL_O2;  }
                                     else if (strncasecmp("H2O",s,3)==0)  {mol_id = MOL_H2O; }
                                     else if (strncasecmp("CO2",s,3)==0)  {mol_id = MOL_CO2; }
                                     else if (strncasecmp("NO2",s,3)==0)  {mol_id = MOL_NO2; }
                                     else if (strncasecmp("BRO",s,3)==0)  {mol_id = MOL_BRO; }
                                     else if (strncasecmp("OCLO",s,4)==0) {mol_id = MOL_OCLO;}
                                     else if (strncasecmp("HCHO",s,4)==0) {mol_id = MOL_HCHO;}
                                     else if (strncasecmp("O4",s,2)==0)   {mol_id = MOL_O4;}
				     else if (strncasecmp("SO2", s,3)==0) {mol_id = MOL_SO2;}
                                     else {fprintf(stderr,"Unknown 'dens_column %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
                                     Input.atm.column[mol_id] = yytext2float(yytext, 3, ntokens+1);
				     if (ntokens == 3) {
   				        s =  yytext2string(yytext,4,ntokens+1);
				        if      (strncasecmp("DU"  ,s,2)==0)                               {Input.atm.unit_column[mol_id]= DU; }
				        else if (strncasecmp("CM_2",s,4)==0 || strncasecmp("CM-2",s,4)==0) {Input.atm.unit_column[mol_id]= CM_2; }
                                        else {fprintf(stderr,"Unknown 'dens_column %s' on line %d\n", s, line_number+1); ierror++;}
                                        free(s);
                                     }
                                     }
dens_file{SPACE}+{WORD}{SPACE}+{WORD}{SPACE_WORD}? { s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);
                                     fflush (stderr);
                                     s = yytext2string(yytext,2,ntokens+1);
				     if      (strncasecmp("O3"  ,s,2)==0) {mol_id = MOL_O3;}
				     else if (strncasecmp("O2"  ,s,2)==0) {mol_id = MOL_O2;}
				     else if (strncasecmp("H2O" ,s,3)==0) {mol_id = MOL_H2O;}
				     else if (strncasecmp("CO2" ,s,3)==0) {mol_id = MOL_CO2;}
				     else if (strncasecmp("NO2" ,s,3)==0) {mol_id = MOL_NO2;}
				     else if (strncasecmp("BRO" ,s,3)==0) {mol_id = MOL_BRO;}
				     else if (strncasecmp("OCLO",s,4)==0) {mol_id = MOL_OCLO;}
				     else if (strncasecmp("HCHO",s,4)==0) {mol_id = MOL_HCHO;}
				     else if (strncasecmp("O4",  s,2)==0) {mol_id = MOL_O4;}
				     else if (strncasecmp("SO2", s,3)==0) {mol_id = MOL_SO2;}
                                     else {fprintf(stderr,"Unknown 'dens_file %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
				     if (ntokens == 3) {
   				        s =  yytext2string(yytext,3,ntokens+1);
				        if      (strncasecmp("CM_3",s,4)==0 || strncasecmp("CM-3",s,4)==0 || 
                                                 strncasecmp("cm_3",s,4)==0 || strncasecmp("cm-3",s,4)==0) {Input.atm.unit_profile[mol_id]= CM_3; }   
				        else if (strncasecmp("M_3" ,s,3)==0 || strncasecmp("M-3" ,s,3)==0 || 
                                                 strncasecmp("m_3" ,s,3)==0 || strncasecmp("m-3" ,s,3)==0) {Input.atm.unit_profile[mol_id]= M_3; }
				        else if (strncasecmp("MMR" ,s,3)==0)                               {Input.atm.unit_profile[mol_id]= MMR; }
				        else if (strncasecmp("VMR" ,s,3)==0)                               {Input.atm.unit_profile[mol_id]= VMR; }
                                        else if (strncasecmp("RH"  ,s,3)==0) {
                                          if (mol_id != MOL_H2O) {
                                            fprintf(stderr, "Error, unit \"relative humidity\" only possible for H2O dens_files.\n");
                                            fprintf(stderr, "See 'dens_column ... %s' in line %d\n", s, line_number+1);
                                            ierror++;
                                          }
                                          else 
                                            Input.atm.unit_profile[mol_id]= RH; 
                                        }
                                        else {fprintf(stderr,"Unknown 'dens_column %s' on line %d\n", s, line_number+1); ierror++;}
                                        free(s);
                                     }
                                     if (ntokens == 2)
                                       position = 3;
                                     else if (ntokens == 3)
                                       position = 4;
                                     yytextCstring(Input.atm.filename[mol_id],  yytext, position, ntokens+1);
                                     }
disort_icm{SPACE}+{WORD}             {s =  yytext2string(yytext,2,2);
				     if (strncasecmp("phase",s,5)==0) {Input.rte.disort_icm = DISORT_ICM_PHASE;}
				     else if (strncasecmp(s,"moments",7)==0) {Input.rte.disort_icm = DISORT_ICM_MOMENTS;}
				     else if (strncasecmp(s,"off",3)==0) {Input.rte.disort_icm = DISORT_ICM_OFF;}
                                     else {fprintf(stderr,"Unknown 'disort_icm %s' on line %d\n", s, line_number+1); ierror++;}
				     free(s);}

ECHAM_atmosphere_file{SPACE}+{WORD} {Input.atm.rs_source=RS_FROM_ECHAM;
                                      yytextCstring(Input.filename[FN_ECHAM],yytext,2,2);
                                      Input.atm.n_rs_gas=0;
                                    Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_H2O; Input.atm.n_rs_gas++; mol_id = MOL_H2O;
                                      Input.atm.rs_unit[mol_id] = MMR;
                                     }

ECHAM_clouds_file{SPACE}+{WORD}     {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"wc",1);
                                     Input.caoth[isp].source = CAOTH_FROM_ECHAM;
                                     Input.caoth[isp].layer=TRUE;
                                     yytextCstring (Input.caoth[isp].filename,yytext,2,2);
                                     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"ic",1);
                                     Input.caoth[isp].source = CAOTH_FROM_ECHAM;
                                     Input.caoth[isp].layer=TRUE;
                                     yytextCstring (Input.caoth[isp].filename,yytext,2,2);
				    }
ECHAM_sza{SPACE}+{WORD}             {yytextCstring(Input.filename[FN_SZA],yytext,2,2);
                                     Input.atm.sza = NOT_DEFINED_FLOAT;
                                     Input.atm.sza_source = SZA_ECHAM;
                                    }
ECMWF_atmosphere_file{SPACE}+{WORD} {Input.atm.rs_source=RS_FROM_ECMWF;
                                     yytextCstring(Input.filename[FN_ECMWF],yytext,2,2);
                                     Input.atm.n_rs_gas=0;
                                     Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_O3;
				     Input.atm.n_rs_gas++;
				     mol_id = MOL_O3;
                                     Input.atm.rs_unit[mol_id] = MMR;
                                     Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_H2O;
				     Input.atm.n_rs_gas++;
				     mol_id = MOL_H2O;
                                     Input.atm.rs_unit[mol_id] = MMR;
                                    }
ECMWF_ozone_climatology             {Input.atm.ECMWF_ozone_climatology=TRUE;
                                    }
ECMWF_wc_file{SPACE}+{WORD}         {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"wc",1);
                                     Input.caoth[isp].source = CAOTH_FROM_ECMWF;
                                     yytextCstring (Input.caoth[isp].filename,yytext,2,2);
                                    }
ECMWF_ic_file{SPACE}+{WORD}         {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"ic",1);
                                     Input.caoth[isp].source = CAOTH_FROM_ECMWF;
                                     yytextCstring (Input.caoth[isp].filename,yytext,2,2);
                                    }
ECMWF_ic_reff{SPACE_WORD}{FLOAT_OR_WORD}* {ntokens = yytext2ntokens(yytext);
                                     if (ntokens < 1 || ntokens > 2) {
                                       fprintf (stderr, "Error, wrong number of arguments for option 'ECMWF_ic_reff' on line %d \n",line_number+1); 
                                       ierror++;
                                     }
                                     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"ic",1);
                                     t = yytext2string(yytext, 2,ntokens+1);
				     if      (strncasecmp("fixed",t,5)==0)  {Input.caoth[isp].reff_prop = REFF_FIXED;
                                       if (ntokens != 2) {
                                         fprintf (stderr, "Error, wrong number of arguments for option 'ECMWF_ic_reff fixed' on line %d \n",line_number+1); 
                                         ierror++;
                                       }
                                       else Input.caoth[isp].reff =  yytext2float(yytext,2+1,ntokens+1);    
                                     }
				     else if (strncasecmp("file",t,5)==0)  {Input.caoth[isp].reff_prop = REFF_FILE;
                                       if (ntokens != 2) {
                                         fprintf (stderr, "Error, wrong number of arguments for option 'ECMWF_ic_reff file' on line %d \n",line_number+1); 
                                         ierror++;
                                       }
                                       yytextCstring(Input.filename[FN_IC_REFF],yytext,2+1,ntokens+1);    
                                     }
				     else if (strncasecmp("Ou"   ,t,2)==0)  {Input.caoth[isp].reff_prop = REFF_OU;}
				     else {fprintf(stderr,"Unknown 'ECMWF_ic_reff %s' on line %d\n", s, line_number+1);
				           ierror++;}
                                     free(t);
                                    }
ECMWF_wind_file{WORDS}              {ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 2) {
                                       fprintf (stderr, "Error, to many arguments for option 'ECMWF_wind map' on line %d \n",line_number+1); 
                                       ierror++;
                                     }
                                     /* Input.wind.source = WIND_FROM_WIND_MAP; */
                                     yytextCstring (Input.filename[FN_ECMWF_WIND_MAP], yytext, 1+1, ntokens+1);
                                    }
emissivity_map{WORDS}{SPACE_FLOAT}* {ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 3) {
                                       fprintf (stderr, "Error, 'emissivity_map' on line %d requires ONE or TWO optional arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     Input.alb.source = ALBEDO_FROM_EMISSIVITY_MAP;
                                     yytextCstring (Input.filename[FN_ALBEDO_MAP], yytext, 1+1, ntokens+1);

                                     if (ntokens > 1)
                                       yytextCstring (Input.alb.netCDF_alb_name,     yytext, 2+1, ntokens+1);
                                     if (ntokens > 2)
                                       Input.alb.scale_factor  =   yytext2float(     yytext, 3+1, ntokens+1); 
                                   }
flexstor                            {Input.output_format  = OUTPUT_FORMAT_FLEXSTOR; }
fu_h2o_continuum{FLOAT_OR_WORD}     {s = yytext2string(yytext,2,2);
                                     if      (strncasecmp(s,"off",3)==0)  Input.ck_h2ocont = 0;
				     else if (strncasecmp(s,"on", 2)==0)  Input.ck_h2ocont = 1;
				     else { 
                                       tmpfloat = yytext2float(yytext,2,2);
                                       fprintf(stderr,"%f %f\n", fabs(tmpfloat-2.1), fabs(tmpfloat-2.4));
                                       if      (fabs(tmpfloat-2.1) < epsilon)  Input.ck_h2ocont = 1;
				       else if (fabs(tmpfloat-2.4) < epsilon)  Input.ck_h2ocont = 2;
                                       else {
                                         fprintf(stderr,"Unknown 'h2o_continuum %s' on line %d\n", s, line_number+1); 
                                         ierror++;
                                       }
                                     }
                                     fprintf(stderr,"Input.ck_h2ocont = %d\n", Input.ck_h2ocont);
				     free(s);}
header                              {Input.header    = 1; }
ichap{SPACE}+{INTEGER}              {Input.rte.ichap = yytext2int(yytext,2,2);}
ic_file{SPACE_PROFILETYPE}?+{SPACE_WORD} {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"ic",1);
                                     ntokens = yytext2ntokens(yytext);
                                     if (ntokens == 1) {
                                       Input.caoth[isp].source = CAOTH_FROM_FILE;
                                       yytextCstring (Input.caoth[isp].filename,yytext,2,2);
				     }
                                     else {
                                       t = yytext2string(yytext, 2, ntokens+1);
                                       if      ( strncasecmp("1d",t, 2)==0 ) {
                                         Input.caoth[isp].source = CAOTH_FROM_FILE; }
                                       else if ( strncasecmp("3d",t, 2)==0 ) {
                                         Input.caoth[isp].source = CAOTH_FROM_3D_FILE; }
                                       else if ( strncasecmp("ipa_files",t, 9)==0 ) {
                                         Input.caoth[isp].source = CAOTH_IPA_FILES;
                                         Input.caoth[isp].ipa=1;
					 Input.ipa=1;
                                         Input.caoth[isp].properties = PROP_IC_FU; }
                                       else if ( strncasecmp("moments",t, 7)==0 ) {
                                         Input.caoth[isp].source = CAOTH_LAYER_FILES;
                                         Input.caoth[isp].properties = PROP_EXPLICIT; }
                                       yytextCstring (Input.caoth[isp].filename,yytext,3,3);
                                     }
                                    }
ic_fu_reff{SPACE}+{WORD}            {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"ic",1);
                                     s =  yytext2string(yytext,2,2);
                                     if (strncasecmp("fu",s,2)==0) Input.caoth[isp].fu2yang = 0;
                                     else {fprintf(stderr,"Unknown 'ic_fu_reff %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
				    }
ic_fu_tau{SPACE}+{WORD}             {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"ic",1);
                                     s =  yytext2string(yytext,2,2);
				     if      (strncasecmp("unscaled",s,8)==0)  Input.caoth[isp].unscaled = 1;
				     else if (strncasecmp("scaled",s,6)==0)    Input.caoth[isp].unscaled = 0;
                                     else {fprintf(stderr,"Unknown 'ic_fu_tau %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
				    }
ic_saturate{SPACE}+{WORD}+{SPACE_FLOAT}* {fprintf(stderr,"ic_saturate obsolete!"); ierror++;}
include                             BEGIN(include);
ipa_3d                              {Input.ipa3d=1;}
latitude{SIGNED_FLOATS_OR_WORDS}    {ntokens = yytext2ntokens(yytext);                                     
                                     s = yytext2string(yytext,1+1,ntokens+1);
                                     if      (strcmp("N",s) == 0 || strcmp("n",s) == 0) tmpint = 1;
                                     else if (strcmp("S",s) == 0 || strcmp("s",s) == 0) tmpint =-1;
                                     else                                               tmpint = 0;
                                     tmpfloat = yytext2float (yytext,1+abs(tmpint)+1,ntokens+1);
                                     Input.latitude = tmpfloat;               /*       already  in degrees */
                                     if (ntokens > 1+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,2+abs(tmpint)+1,ntokens+1);
                                       Input.latitude += tmpfloat / 60.0;     /*   60.0 minutes to degrees */
                                     }
                                     if (ntokens > 2+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,3+abs(tmpint)+1,ntokens+1);
                                       Input.latitude += tmpfloat / 3600.0;   /* 3600.0 seconds to degrees */
                                     }
                                     if (tmpint != 0) Input.latitude *= tmpint;
                                     free(s);
                                    }
longitude{SIGNED_FLOATS_OR_WORDS}   {ntokens = yytext2ntokens(yytext);                                     
                                     s = yytext2string(yytext,1+1,ntokens+1);
                                     if      (strcmp("E",s) == 0 || strcmp("e",s) == 0) tmpint = 1;
                                     else if (strcmp("W",s) == 0 || strcmp("w",s) == 0) tmpint =-1;
                                     else                                               tmpint = 0;
                                     tmpfloat = yytext2float (yytext,1+abs(tmpint)+1,ntokens+1);
                                     Input.longitude = tmpfloat;              /*        already in degrees */
                                     if (ntokens > 1+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,2+abs(tmpint)+1,ntokens+1);
                                       Input.longitude += tmpfloat / 60.0;    /*   60.0 minutes to degrees */
                                     }
                                     if (ntokens > 2+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,3+abs(tmpint)+1,ntokens+1);
                                       Input.longitude += tmpfloat / 3600.0;  /* 3600.0 seconds to degrees */
                                     }
                                     if (tmpint != 0) Input.longitude *= tmpint;
                                     free(s);
                                    }

mc_backward{SPACE_INTEGER}?{SPACE_INTEGER}?{SPACE_INTEGER}?{SPACE_INTEGER}? {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     Input.rte.mc.backward.yes = 1;
                                     if (ntokens != 0 && ntokens != 2 && ntokens != 4) {
				       fprintf (stderr, "Error, need either 0, 2, or 4 integers as arguments for mc_backward!\n");
				       exit (1);
                                     }
				     if (ntokens == 2) {
                                       tmpi   = yytext2integers(yytext, 2); 
                                       Input.rte.mc.backward.islower = tmpi[0];
                                       Input.rte.mc.backward.jslower = tmpi[1];
                                       Input.rte.mc.backward.isupper = Input.rte.mc.backward.islower;
                                       Input.rte.mc.backward.jsupper = Input.rte.mc.backward.jslower;
				       free(tmpi);
                                     }
                                     if (ntokens == 4) {
                                       tmpi   = yytext2integers(yytext, 4); 
                                       Input.rte.mc.backward.islower = tmpi[0];
                                       Input.rte.mc.backward.jslower = tmpi[1];
                                       Input.rte.mc.backward.isupper = tmpi[2];
                                       Input.rte.mc.backward.jsupper = tmpi[3];
				       free(tmpi);
                                     }
				     free(s);}
mc_blitz_position{SPACE_FLOAT}{SPACE_FLOAT}{SPACE_FLOAT}{SPACE_FLOAT}{SPACE_FLOAT}{SPACE_FLOAT} {
                                     s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     if (ntokens !=6) {
                                       fprintf(stderr,"Wrong number of arguments for 'mc_blitz_position' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
                                     Input.source=SRC_BLITZ;
				     Input.rte.mc.blitz_position = calloc (6, sizeof(double));
				     for (i=0;i<6; i++)
				       Input.rte.mc.blitz_position[i] = yytext2double(yytext,2+i,ntokens+1);
				     Input.rte.numu   = 1;
				     Input.rte.maxumu = 1;
				     Input.rte.umu    = calloc(1, sizeof(float));
				     Input.rte.umu[0] = 0.0;
				     Input.rte.nphi   = 1;
				     Input.rte.maxphi = 1;
				     Input.rte.phi    = calloc(1, sizeof(float));
				     Input.rte.phi[0] = 0.0;
				     free(s); }
mc_bw_umu_file{SPACE}+{WORD}        {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_UMU], s); free(s);
				     Input.rte.numu   = 1;
				     Input.rte.maxumu = 1;
				     Input.rte.umu    = calloc(1, sizeof(float));
				     Input.rte.umu[0] = 0.0;
				     Input.rte.nphi   = 1;
				     Input.rte.maxphi = 1;
				     Input.rte.phi    = calloc(1, sizeof(float));
				     Input.rte.phi[0] = 0.0;
                                    }
mc_cloud_grid{SPACE_INTEGER}{SPACE_INTEGER}{SPACE_INTEGER} {
                                     fprintf (stderr, " ... option mc_cloud_grid is obsolete!\n"); ierror++;}
mc_new_delta_scaling {fprintf(stderr,"mc_new_delta_scaling obsolete, use mc_delta_scaling! read documentation for correct usage"); ierror++;}
mc_escape{SPACE_WORD}*             {
                                    s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				    strcpy (s, yytext);
                                    ntokens = yytext2ntokens(s);
                                    free(s);            
                                    if (ntokens == 0) { Input.rte.mc.escape = 1;
                                                        Input.rte.mc.locest = 0;}
                                    if (ntokens == 1) {
				      s = yytext2string(yytext,2,ntokens+1);
				      if (strncasecmp(s,"on",2)==0) {
				        Input.rte.mc.escape = 1;
				        Input.rte.mc.locest = 0;
				      } else {
				        Input.rte.mc.escape = 0;
                                      }				      
                         	      free(s);
				    }
                                   }
mc_hiddenline                       {
                                     #if HAVE_OPENGL
                                       GLmystic_hiddenline = 1;
                                     #endif
                                    }
mc_jacobian                         {Input.rte.mc.jacobian = 1; }
mc_jacobian_std                     {Input.rte.mc.jacobian_std=1;}
mc_lidar{SPACE_WORD}?               {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     ntokens = yytext2ntokens(yytext);
				     if (ntokens == 0) {
				       #if HAVE_LIDAR
				       Input.rte.mc.locest = MCLIDAR_SPACE;
				       #else
				       Input.rte.mc.locest = -1;
				       #endif
				     }
				     else {
                                       s =  yytext2string(yytext,2,2);
                                       Input.rte.mc.locest=-1;
				       #if HAVE_LIDAR
                                       if      (strncasecmp("ascope",s,6) == 0) Input.rte.mc.locest = MCLIDAR_ASCOPE;
                                       if      (strncasecmp("polarize",s,8) == 0) Input.rte.mc.locest = MCLIDAR_POLARIZE;
                                       if      (strncasecmp("space",s,5) == 0) Input.rte.mc.locest = MCLIDAR_SPACE;
                                       if      (strncasecmp("falcon",s,6) == 0) Input.rte.mc.locest = MCLIDAR_FALCON;
                                       if      (strncasecmp("simple",s,6) == 0) Input.rte.mc.locest = MCLIDAR_SIMPLE;
                                       if      (strncasecmp("pilot",s,5) == 0) Input.rte.mc.locest = MCLIDAR_PILOT;
                                       if      (strncasecmp("moon",s,4) == 0) Input.rte.mc.locest = MCLIDAR_MOON;
                                       if      (strncasecmp("test",s,4) == 0) Input.rte.mc.locest = MCLIDAR_TEST;
				       #endif
				     }
				     if (Input.rte.mc.locest == -1) {
				       fprintf(stderr,"Error, unknown 'mc_lidar %s' on line %d\n", s, line_number+1);
				       ierror++;
				     }
                                     Input.source=SRC_LIDAR;
				     Input.rte.mc.escape = 0;
                                     free(s);
                                    }
mc_lidar_abserr{SPACE}+{FLOAT}      {fprintf(stderr,"'mc_lidar_abserr on line %d is obsolete option!\n", line_number+1); ierror++;}
mc_lidar_file{SPACE}+{WORD}         {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_LIDAR], s); free(s);}
mc_momcld_file{SPACE}+{WORD}        {fprintf (stderr, "Error, option mc_momcld_file does not exist anymore.\n");
                                     fprintf (stderr, "Use wc_properties instead!\n"); exit(-1);}
mc_panorama{SIGNED_FLOATS}          {fprintf (stderr, "Error, option mc_panorama does not exist anymore.\n");
                                     fprintf (stderr, "Use mc_panorama_view instead!\n"); exit(-1);}
mc_panorama_view{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     Input.rte.mc.panorama = PAN_MODE_CAMERA;
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     if (ntokens != 4) {
                                       fprintf(stderr,"Wrong number of arguments for 'mc_panorama_view' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
				     Input.rte.mc.pan_angles = calloc (4, sizeof(double));
                                     Input.rte.mc.pan_angles[0] = yytext2double(yytext, 2,5);
                                     Input.rte.mc.pan_angles[1] = yytext2double(yytext, 3,5);
                                     Input.rte.mc.pan_angles[2] = yytext2double(yytext, 4,5);
                                     Input.rte.mc.pan_angles[3] = yytext2double(yytext, 5,5);
				     Input.rte.numu   = 1;
				     Input.rte.maxumu = 1;
				     Input.rte.umu    = calloc(1, sizeof(float));
				     Input.rte.umu[0] = 0.0;
				     Input.rte.nphi   = 1;
				     Input.rte.maxphi = 1;
				     Input.rte.phi    = calloc(1, sizeof(float));
				     Input.rte.phi[0] = 0.0;
				     free(s); }
mc_panorama_alignment{SPACE_WORD}   {
                                     s =  yytext2string(yytext,2,2);
				     if      (strncasecmp("mu",s,2)==0) Input.rte.mc.pan_alignment = PAN_ALIGNMENT_MU;
				     else if (strncasecmp("sun",s,3)==0) Input.rte.mc.pan_alignment = PAN_ALIGNMENT_SUN;
				     else if (strncasecmp("zenith",s,6)==0) Input.rte.mc.pan_alignment = PAN_ALIGNMENT_ZENITH;
                                     else {fprintf(stderr,"Unknown 'mc_panorama_alignment option %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
				    }
mc_panorama_distr_photons_over_pixel {Input.rte.mc.pan_distr_photons_over_pixel = 1; }
mc_panorama_no_pixel                {Input.rte.mc.pan_no_pixel = 1; }
mc_panorama_quicklook               {Input.rte.mc.pan_quicklook = 1; }
mc_panorama_vr_sc0 		    {fprintf(stderr,"Error, option 'mc_panorama_vr_sc0' on line %d is obsolete!\n", line_number+1); ierror++; }
mc_panorama_with_direct_rad         {Input.rte.mc.pan_with_direct_rad = 1; }
mc_pcld_file{SPACE}+{WORD}          {fprintf(stderr,"Error, option 'mc_pcld_file' on line %d is obsolete!\n", line_number+1); ierror++; }
mc_radar                            {
                                     #if HAVE_LIDAR
                                     Input.rte.mc.locest = MCRADAR;
                                     #else
                                     Input.rte.mc.locest = -1;
                                     #endif
                                     if (Input.rte.mc.locest == -1) {
                                       fprintf(stderr,"Error, cannot use radar without lidar\n");
                                       ierror++;
                                     }
                                     Input.source=SRC_LIDAR;
                                     Input.rte.mc.escape = 0;
                                    }
mc_readrandomseed                   {Input.rte.mc.readrandomseed = 1; }
mc_readrandomstatus                 {Input.rte.mc.readrandomstatus = 1; Input.rte.mc.readrandomseed = 1; }
mc_relerr{SPACE}+{FLOAT}            {fprintf(stderr,"'mc_relerr on line %d is obsolete option!\n", line_number+1); ierror++;}
mc_ris_factor{SPACE_FLOAT}?         {ntokens = yytext2ntokens(yytext);
                                     if (ntokens == 1) {
                                       Input.rte.mc.ris_factor=yytext2double(yytext,2,2);
                                     }
                                     else {
                                       Input.rte.mc.ris_factor=0;
                                     }
                                    }
mc_ris_optical_depth{SPACE_FLOAT}   {Input.rte.mc.ris_optical_depth=yytext2double(yytext,2,2);}
mc_sample_grid{SPACE_INTEGER}{SPACE_INTEGER}{SPACE_FLOAT}*{SPACE_FLOAT}* {
                                     s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     if (ntokens !=2 && ntokens != 4) {
                                       fprintf(stderr,"Wrong number of arguments for 'mc_sample_grid' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
				     Input.rte.mc.Nx_sample = yytext2int(yytext,2,ntokens+1);
				     Input.rte.mc.Ny_sample = yytext2int(yytext,3,ntokens+1);
                                     if (ntokens == 4) {
					Input.rte.mc.dx_sample = yytext2double(yytext,4,ntokens+1);
					Input.rte.mc.dy_sample = yytext2double(yytext,5,ntokens+1);
				     }
				     free(s); }
mc_satellite_position{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     if (ntokens != 3) {
                                       fprintf(stderr,"Wrong number of arguments for 'mc_satellite_position' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
                                     Input.rte.mc.sensorposition=MC_SENSORPOSITION_SPHERICAL;
                                     Input.rte.mc.sensorposition_lon=yytext2float(yytext,2,4);
                                     Input.rte.mc.sensorposition_lat=yytext2float(yytext,3,4);
                                     Input.rte.mc.sensorposition_alt=yytext2float(yytext,4,4);
				     free(s); }
mc_satellite_view{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT}{SPACE_SIGNED_FLOAT} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     Input.rte.mc.panorama = PAN_MODE_SATELLITE;
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     if (ntokens != 4) {
                                       fprintf(stderr,"Wrong number of arguments for 'mc_satellite_view' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
				     Input.rte.mc.pan_angles = calloc (4, sizeof(double));
                                     Input.rte.mc.pan_angles[0] = yytext2double(yytext, 2,5);
                                     Input.rte.mc.pan_angles[1] = yytext2double(yytext, 3,5);
                                     Input.rte.mc.pan_angles[2] = yytext2double(yytext, 4,5);
                                     Input.rte.mc.pan_angles[3] = yytext2double(yytext, 5,5);
				     free(s); }
mc_sensordirection{SIGNED_FLOATS}   {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     if (yytext2ntokens(s)!=3) {
				       fprintf(stderr,"Need 3 coordinates for mc_sensordirection\n");
                                       ierror++;
                                     }
                                     else {
                                       Input.rte.mc.sensordirection=1;
                                       Input.rte.mc.sensordirection_dx=yytext2float(yytext,2,4);
                                       Input.rte.mc.sensordirection_dy=yytext2float(yytext,3,4);
                                       Input.rte.mc.sensordirection_dz=yytext2float(yytext,4,4);
                                     } 
				     free(s); }
mc_sensorposition{SIGNED_FLOATS}    {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     if (yytext2ntokens(s)!=3) {
				       fprintf(stderr,"Need 3 coordinates for mc_sensorposition\n");
                                       ierror++;
                                     }
                                     else {
                                       Input.rte.mc.sensorposition=MC_SENSORPOSITION_CARTESIAN;
                                       Input.rte.mc.sensorposition_x=yytext2float(yytext,2,4);
                                       Input.rte.mc.sensorposition_y=yytext2float(yytext,3,4);
                                       Input.rte.mc.sensorposition_z=yytext2float(yytext,4,4);
                                     } 
				     free(s); }
mc_spherical3D                      {Input.rte.mc.spherical3D=1;}
mc_spherical3D_scene{SIGNED_FLOATS} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     if (yytext2ntokens(s)!=4) {
				       fprintf(stderr,"Need 4 coordinates for mc_spherical3d_scene\n");
                                       ierror++;
                                     }
                                     else {
                                       Input.rte.mc.spherical3D_scene=1;
                                       Input.rte.mc.spherical3D_scene_lon_min=yytext2float(yytext,2,5);
                                       Input.rte.mc.spherical3D_scene_lon_max=yytext2float(yytext,3,5);
                                       Input.rte.mc.spherical3D_scene_lat_min=yytext2float(yytext,4,5);
                                       Input.rte.mc.spherical3D_scene_lat_max=yytext2float(yytext,5,5);
                                     } 
				     free(s); }
mc_sun_position{SIGNED_FLOATS} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     if (yytext2ntokens(s)!=2) {
				       fprintf(stderr,"Need 2 coordinates for mc_sun_position\n");
                                       ierror++;
                                     }
                                     else {
                                       Input.atm.sza_source = SZA_DIRECT_INPUT;
                                       Input.atm.phi0_spher = yytext2float(yytext,2,3);
				       Input.atm.sza_spher  = yytext2float(yytext,3,3);
				       
                                     } 
				     free(s); }
mc_sun_radius{SPACE}+{FLOAT}        {Input.rte.mc.sun_radius = yytext2float(yytext,2,2); } /*Bernhard Reinhardt */
mc_surfaceparallel                  {Input.rte.mc.surfaceparallel = 1;}
mc_tipa{SPACE}+{WORD}               {s =  yytext2string(yytext,2,2); Input.rte.mc.ipa=1;
                                     if      (strncasecmp("dirdiff",s,7)==0) Input.rte.mc.tipa = TIPA_DIRDIFF; /* 1 */
                                     else if (strncasecmp("dir3d"  ,s,5)==0) Input.rte.mc.tipa = TIPA_DIR3D;   /* 3 */
                                     else if (strncasecmp("dir"    ,s,3)==0) {fprintf(stderr,"'mc_tipa %s' on line %d, %s is obsolete option!\n", s, line_number+1, s); ierror++;}
				     else {fprintf(stderr,"Unknown 'mc_tipa %s' on line %d\n", s, line_number+1);
				     ierror++;}
                                     free(s);
				    } /* ulrike */
mc_truncate{SPACE_SIGNED_FLOAT}?   {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                    ntokens = yytext2ntokens(s);
				    if (ntokens == 0) {
				        Input.rte.mc.truncate=0.99;
				    }
				    else {
				        Input.rte.mc.truncate = yytext2float(yytext,2,2);
				    }
				    free(s);
				   }
mc_visualize                        {
                                     #if HAVE_OPENGL
                                      Input.rte.mc.visualize = 1; 
				     #else
 				      Input.rte.mc.visualize = 0; 
	                              fprintf (stderr, "*** Warning, you selected mc_visualize but OpenGL is not available.\n");
				      fprintf (stderr, "*** Ignoring mc_visualize - please install OpenGL if you need it.\n");
                                     #endif
                                    }
mc_vroom{SPACE_WORD}*               {
                                     s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);            
                                     if (ntokens == 0) { Input.rte.mc.vroom = 1;}
                                     if (ntokens == 1) {
				       s = yytext2string(yytext,2,ntokens+1);
				       if (strncasecmp(s,"on",2)==0) {
				         Input.rte.mc.vroom = 1;
				       } else {
				         Input.rte.mc.vroom = 0;
                                       }				      
                         	       free(s);
				     }
                                    }
no2_column_du{SPACE}+{SIGNED_FLOAT} {Input.atm.column[MOL_NO2] = yytext2float(yytext,2,2); 
                                     Input.atm.unit_column[MOL_NO2] = DU;}
no2_column_moleccm-2{SPACE}+{SIGNED_FLOAT} {Input.atm.column[MOL_NO2] = yytext2float(yytext,2,2);
                                     Input.atm.unit_column[MOL_NO2] = CM_2;}
no2_crs{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncasecmp("Burrows",s,7)==0) {Input.no2_spectrum = NO2_GOME;}
				     else if (strncasecmp(s,"Bogumil",7)==0)    {Input.no2_spectrum = NO2_BREMEN;}
				     else {fprintf(stderr,"Unknown 'no2_crs %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
optimize_fortran{SPACE_INTEGER}*    {ntokens = yytext2ntokens(yytext); 
                                     if ( ntokens != 0 &&  ntokens != 1 ) {
                                       fprintf (stderr, "Error, wrong number of arguments %d for 'optimize_fortran' on line %d\n", 
                                                 ntokens, line_number+1); 
                                       ierror++;
                                     }
                                     Input.optimize_fortran = TRUE;
                                     if (ntokens == 1)
                                       Input.optimize_delta = yytext2int(yytext,2,ntokens+1); 
                                     }
output_user{WORDS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     Input.output_user_flag = 1; 
                                     Input.n_output_user = yytext2ntokens(s); 
				     free(s);
				     Input.output_user     = calloc (Input.n_output_user, sizeof (int));
				     Input.output_user_gas = calloc (Input.n_output_user, sizeof (int));
				     for (i=2;i<=Input.n_output_user+1; i++) {
				       t = yytext2string(yytext, i, Input.n_output_user+1);  
				       if      (strcmp("lambda",t)==0 ||  
                                                strcmp("wavelength",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVE;}
				       else if (strcmp("lambda_min",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVE_MIN;}
				       else if (strcmp("lambda_max",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVE_MAX;}
				       else if (strcmp("wavenumber",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVENUMBER;}
				       else if (strcmp("wavenumber_min",t)==0){Input.output_user[i-2] = OUTPUT_USER_WAVENUMBER_MIN;}
				       else if (strcmp("wavenumber_max",t)==0){Input.output_user[i-2] = OUTPUT_USER_WAVENUMBER_MAX;}
				       else if (strcmp("zout",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_ZOUT_SUR;}
				       else if (strcmp("zout_sur",t)==0)      {Input.output_user[i-2] = OUTPUT_USER_ZOUT_SUR;}
				       else if (strcmp("zout_sea",t)==0)      {Input.output_user[i-2] = OUTPUT_USER_ZOUT_SEA;}
				       else if (strcmp("z_sur",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_Z_SUR;}
				       else if (strcmp("p",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_P;}
				       else if (strcmp("T",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_T;}
				       else if (strcmp("T_d",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_T_D;}
				       else if (strcmp("T_sur",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_T_SUR;}
				       else if (strcmp("theta",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_THETA;}
				       else if (strcmp("theta_e",t) ==0)      {Input.output_user[i-2] = OUTPUT_USER_THETA_E;}
				       else if (strcmp("edir",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_EDIR;}
				       else if (strcmp("eglo",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_EGLO;}
				       else if (strcmp("edn",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_EDN;}
				       else if (strcmp("eup",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_EUP;}
				       else if (strcmp("enet",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_ENET;}
				       else if (strcmp("esum",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_ESUM;}
				       else if (strcmp("fdir",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_FDIR;}
				       else if (strcmp("fglo",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_FGLO;}
				       else if (strcmp("fdn",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_FDN;}
				       else if (strcmp("fup",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_FUP;}
				       else if (strcmp("f",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_F;}
				       else if (strcmp("uavgdir",t) ==0)      {Input.output_user[i-2] = OUTPUT_USER_UDIR;}
				       else if (strcmp("uavgglo",t) ==0)      {Input.output_user[i-2] = OUTPUT_USER_UGLO;}
				       else if (strcmp("uavgdn",t)  ==0)      {Input.output_user[i-2] = OUTPUT_USER_UDN;}
				       else if (strcmp("uavgup",t)  ==0)      {Input.output_user[i-2] = OUTPUT_USER_UUP;}
				       else if (strcmp("uavg",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_U;}
                                       else if (strcmp("uu",t)      ==0)      {Input.output_user[i-2] = OUTPUT_USER_UU;}
				       else if (strcmp("albedo",t)  ==0)      {Input.output_user[i-2] = OUTPUT_USER_ALB;}
				       else if (strcmp("sza",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_SZA;}
				       else if (strcmp("c_p",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_C_P;}
				       else if (strcmp("heat",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_HEAT;}
				       else if (strcmp("emis",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_EMIS;}
				       else if (strcmp("abs",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_ABS;}
				       else if (strcmp("w_rad",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_W_RAD;}
				       else if (strcmp("m_rad",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_M_RAD;}
                                       else if (strncasecmp("n_",  t,2)==0)   {Input.output_user[i-2] = OUTPUT_USER_N;}
                                       else if (strncasecmp("rho_",t,4)==0)   {Input.output_user[i-2] = OUTPUT_USER_RHO;}
                                       else if (strncasecmp("vmr_",t,4)==0)   {Input.output_user[i-2] = OUTPUT_USER_VMR;}
                                       else if (strncasecmp("mmr_",t,4)==0)   {Input.output_user[i-2] = OUTPUT_USER_MMR;}
                                       else if (strcmp("rh",t)        ==0)    {Input.output_user[i-2] = OUTPUT_USER_RH;}
                                       else if (strcmp("rh_ice",t)    ==0)    {Input.output_user[i-2] = OUTPUT_USER_RH_ICE;}
                                       else if (strcasecmp("clwc",t)  ==0)    {Input.output_user[i-2] = OUTPUT_USER_CLWC;}
                                       else if (strcasecmp("clwd",t)  ==0)    {Input.output_user[i-2] = OUTPUT_USER_CLWD;}
                                       else if (strcasecmp("tclw",t)  ==0)    {Input.output_user[i-2] = OUTPUT_USER_TCLW;}
                                       else if (strcasecmp("reff_water",t)==0){Input.output_user[i-2] = OUTPUT_USER_REFF_WAT;}
                                       else if (strcasecmp("ciwc",t)  ==0)    {Input.output_user[i-2] = OUTPUT_USER_CIWC;}
                                       else if (strcasecmp("ciwd",t)  ==0)    {Input.output_user[i-2] = OUTPUT_USER_CIWD;}
                                       else if (strcasecmp("tciw",t)  ==0)    {Input.output_user[i-2] = OUTPUT_USER_TCIW;}
                                       else if (strcasecmp("reff_ice",t) ==0) {Input.output_user[i-2] = OUTPUT_USER_REFF_ICE;}
                                       else if (strcasecmp("tcw",t)  ==0)     {Input.output_user[i-2] = OUTPUT_USER_TCW;}
                                       else if (strcasecmp("cc",t)   ==0)     {Input.output_user[i-2] = OUTPUT_USER_CC;}
                                       else if (strcasecmp("tcc",t)   ==0)    {Input.output_user[i-2] = OUTPUT_USER_TCC;}
                                       else if (strcasecmp("clouds",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_CLOUDS;}
                                       else if (strcmp("u",t)         ==0)    {Input.output_user[i-2] = OUTPUT_USER_WIND_U;}
                                       else if (strcmp("v",t)         ==0)    {Input.output_user[i-2] = OUTPUT_USER_WIND_V;}
                                       else if (strcmp("w",t)         ==0)    {Input.output_user[i-2] = OUTPUT_USER_WIND_W;}
                                       else if (strcmp("dtheta_dx",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_DTDX;}
                                       else if (strcmp("dtheta_dy",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_DTDY;}
                                       else if (strcmp("dtheta_dz",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_DTDZ;}
                                       else if (strcmp("heat_ad_x",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD_X;}
                                       else if (strcmp("heat_ad_y",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD_Y;}
                                       else if (strcmp("heat_ad_z",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD_Z;}
                                       else if (strcmp("heat_ad",t)   ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD;}
                                       else if (strcmp("spher_alb",t)   ==0)  {Input.output_user[i-2] = OUTPUT_USER_SPHER_ALB;;}
				       else {fprintf(stderr,"Unknown 'output_user %s' on line %d\n", t, line_number+1);
				       ierror++;}
                                       if (Input.output_user[i-2] == OUTPUT_USER_N || 
                                           Input.output_user[i-2] == OUTPUT_USER_RHO || 
                                           Input.output_user[i-2] == OUTPUT_USER_VMR || 
                                           Input.output_user[i-2] == OUTPUT_USER_MMR) {
                                         if      ((s = strstr(t,"air")) != NULL || (s = strstr(t,"AIR")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_AIR;
                                         else if ((s = strstr(t,"o3"))  != NULL || (s = strstr(t,"O3"))  != NULL)
                                            Input.output_user_gas[i-2] = MOL_O3;
                                         else if ((s = strstr(t,"h2o")) != NULL || (s = strstr(t,"H2O")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_H2O;
                                         else if ((s = strstr(t,"co2")) != NULL || (s = strstr(t,"CO2")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_CO2;
                                         else if ((s = strstr(t,"no2")) != NULL || (s = strstr(t,"NO2")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_NO2;
                                         else if ((s = strstr(t,"bro")) != NULL || (s = strstr(t,"BRO")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_BRO;
                                         else if ((s = strstr(t,"oclo"))!= NULL || (s = strstr(t,"OCLO"))!= NULL)
                                            Input.output_user_gas[i-2] = MOL_OCLO;
                                         else if ((s = strstr(t,"hcho"))!= NULL || (s = strstr(t,"HCHO"))!= NULL)
                                            Input.output_user_gas[i-2] = MOL_HCHO;
                                         else if ((s = strstr(t,"o4"))  != NULL || (s = strstr(t,"O4"))  != NULL)
                                            Input.output_user_gas[i-2] = MOL_O4;
                                         else if ((s = strstr(t,"so2"))  != NULL || (s = strstr(t,"SO2"))  != NULL)
                                            Input.output_user_gas[i-2] = MOL_SO2;
                                         else if ((s = strstr(t,"o2"))  != NULL || (s = strstr(t,"O2"))  != NULL)
                                            Input.output_user_gas[i-2] = MOL_O2;
                                         else {
                                           fprintf(stderr,"Unknown 'output_user %s' on line %d\n", t, line_number+1);
				           ierror++;
                                         }
                                       }                                         
                                       free(t);
				     }
                                    }

ozone_column{SPACE}+{SIGNED_FLOAT}  {Input.atm.column[MOL_O3] = yytext2float(yytext,2,2); 
                                     Input.atm.unit_column[MOL_O3] = DU;}
pressure_out{SIGNED_FLOATS_OR_WORDS}|p_out{SIGNED_FLOATS_OR_WORDS} {
                                      Input.atm.nzout = yytext2ntokens(yytext);
                                      Input.atm.press_zout = calloc (Input.atm.nzout,sizeof(float));
                                      for (i=2;i<=Input.atm.nzout+1; i++) {
                                       Input.atm.zout_source = OUTLEVEL_PRESS;
				       t = yytext2string(yytext, i, Input.atm.nzout+1);
				       if      (strncasecmp("toa",t,3)==0) {Input.atm.press_zout[i-2] = ZOUT_TOA;}
				       else if (strncasecmp("sur",t,3)==0) {Input.atm.press_zout[i-2] = ZOUT_SURFACE;}
				       else if (strncasecmp("cpt",t,3)==0) {Input.atm.press_zout[i-2] = ZOUT_CPT;}
                                       else                                {Input.atm.press_zout[i-2] = yytext2float(yytext,i,Input.atm.nzout+1);}
                                       free(t);
				      }
				     }
prndis{INTEGERS}                     {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				      strcpy (s, yytext);
				      Input.rte.nprndis  = yytext2ntokens(s); 
				      Input.rte.prndis   = 
				        yytext2integers(yytext, Input.rte.nprndis); 
				      free(s); }
profile_file{SPACE_WORD}{SPACE_PROFILETYPE}{SPACE_WORD} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 3) {
                                       fprintf (stderr, "Error, 'profile_file' on line %d needs to have 3 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     s = yytext2string(yytext,2,4);
                                     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,s,0);
                                     t = yytext2string(yytext, 3, ntokens+1);
                                     if      ( strncasecmp("1d",t, 2)==0 ) {
                                       Input.caoth[isp].source = CAOTH_FROM_FILE; }
                                     else if ( strncasecmp("3d",t, 2)==0 ) {
                                       Input.caoth[isp].source = CAOTH_FROM_3D_FILE; }
                                     else if ( strncasecmp("ipa_files",t, 9)==0 ) {
                                       Input.caoth[isp].source = CAOTH_IPA_FILES;
                                       Input.caoth[isp].ipa=1;
				       Input.ipa=1;
				       if      ( strncasecmp("wc",s,2)==0 )
				         Input.caoth[isp].properties = PROP_WC_HU;   
				       else if ( strncasecmp("ic",s,2)==0 )
				         Input.caoth[isp].properties = PROP_IC_FU; 
                                     }
                                     else if ( strncasecmp("moments",t, 7)==0 ) {
                                       Input.caoth[isp].source = CAOTH_LAYER_FILES;
                                       Input.caoth[isp].properties = PROP_EXPLICIT;
                                     }
				     else {
                                       fprintf (stderr, "Error, 'profile_file' on line %d has invalid argument 2: %s\n",line_number+1,t);
                                       ierror++;
                                     }
                                     yytextCstring (Input.caoth[isp].filename,yytext,4,4);
				     free(s);
                                    }
profile_layer{SPACE}+{WORD}         {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,2),0);
                                     Input.caoth[isp].layer=TRUE;}
profile_level{SPACE}+{WORD}         {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,2),0);
                                     Input.caoth[isp].layer=FALSE;}
profile_no_scattering               {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,2),0);
                                     Input.caoth[isp].no_scattering=TRUE;}
profile_properties{WORDS}           {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf (stderr, "Error, 'profile_properties' on line %d needs to have 2 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     s = yytext2string(yytext,2,3);
                                     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,s,0);
                                     t = yytext2string(yytext, 3, ntokens+1);
                                     if      (strncasecmp("hu",t,2)==0)
                                       Input.caoth[isp].properties = PROP_WC_HU;
				     else if (strncasecmp("echam4",t,6)==0)
                                       if      (strncasecmp("wc",s,2)==0)
				         Input.caoth[isp].properties = PROP_WC_ECHAM4;
                                       else if (strncasecmp("ic",s,2)==0)
				         Input.caoth[isp].properties = PROP_IC_ECHAM4;
				       else {
                                         fprintf (stderr, "Error, profile properties for ECHAM4 does not work with profile %s! Need to use 'wc' or 'ic'! (line %d)\n",Input.caoth[isp].name,line_number+1); 
                                         ierror++;
				       }
				     else if (strncasecmp("mie",t,3)==0)
				       Input.caoth[isp].properties = PROP_WC_MIE;
				     else if (strncasecmp("fu",t,2)==0)
				       Input.caoth[isp].properties = PROP_IC_FU;
				     else if (strncasecmp("yang",t,4)==0)
				       Input.caoth[isp].properties = PROP_IC_YANG;
				     else if (strncasecmp("key",t,3)==0)
				       Input.caoth[isp].properties = PROP_IC_KEY;
				     else if (strncasecmp("baum_hufit",t,10)==0)
				       Input.caoth[isp].properties = PROP_IC_BAUMHUFIT;
				     else if (strncasecmp("baum",t,4)==0)
				       Input.caoth[isp].properties = PROP_IC_BAUM;
				     else if (strncasecmp("hey",t,3)==0)
				       Input.caoth[isp].properties = PROP_IC_HEY;
				     else  {
                                       Input.caoth[isp].properties = PROP_FILE;
				       strcpy (Input.caoth[isp].filename_prop, t);
				     }
				     switch (Input.caoth[isp].properties) {
				     case PROP_WC_MIE:
				     case PROP_WC_HU:
				       if (strncasecmp("ic",s,2)==0) {
                                         fprintf (stderr, "Error, profile properties %s is a water cloud property and is not allowed for profile 'ic'! Define other profile name (e.g. 'wc')! (line %d)\n",t,line_number+1);
                                         ierror++;
				       }
				       break;
				     case PROP_IC_FU:
				     case PROP_IC_YANG:
				     case PROP_IC_KEY:
				     case PROP_IC_BAUMHUFIT:
				     case PROP_IC_BAUM:
				     case PROP_IC_HEY:
				       if (strncasecmp("wc",s,2)==0) {
                                         fprintf (stderr, "Error, profile properties %s is an ice cloud property and is not allowed for profile 'wc'! Define other profile name (e.g. 'ic')! (line %d)\n",t,line_number+1);
                                         ierror++;
				       }
				     }
                                     free(s);free(t);
				    }
profile_properties_interpolate{SPACE_WORD} {s = yytext2string(yytext,2,2);
                                     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,s,0);
                                     Input.caoth[isp].interpolate=1;
				     free(s);}
profile_scale_gg{SPACE_WORD}{SPACE_FLOAT} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf (stderr, "Error, 'profile_scale_gg' on line %d needs to have 2 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,3),0);
                                     Input.caoth[isp].scale_gg   = yytext2float(yytext,3,3); }
profile_scale_ssa{SPACE_WORD}{SPACE_FLOAT} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf (stderr, "Error, 'profile_scale_ssa' on line %d needs to have 2 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,3),0);
                                     Input.caoth[isp].scale_ssa  = yytext2float(yytext,3,3); }
profile_set_gg{SPACE_WORD}{SPACE_FLOAT} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf (stderr, "Error, 'profile_set_gg' on line %d needs to have 2 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,3),0);
                                     Input.caoth[isp].gg   = yytext2float(yytext,3,3); }
profile_set_ssa{SPACE_WORD}{SPACE_FLOAT} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf (stderr, "Error, 'profile_set_ssa' on line %d needs to have 2 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,3),0);
                                     Input.caoth[isp].ssa  = yytext2float(yytext,3,3); }
profile_set_tau{SPACE_WORD}{SPACE_FLOAT} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf (stderr, "Error, 'profile_set_tau' on line %d needs to have 2 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,3),0);
                                     Input.caoth[isp].tau  = yytext2float(yytext,3,3); }
profile_set_tau550{SPACE_WORD}{SPACE_FLOAT} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf (stderr, "Error, 'profile_set_tau550' on line %d needs to have 2 arguments\n",line_number+1); 
                                       ierror++;
                                     }
				     isp=get_caoth_index(&Input.caoth,&Input.n_caoth,
							   yytext2string(yytext,2,3),0);
                                     Input.caoth[isp].tau550  = yytext2float(yytext,3,3); }
radiosonde{SPACE}+{WORD}{WORDS}     {Input.atm.rs_source=RS_FROM_FILE;
                                     s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);
				     s =  yytext2string(yytext,2,ntokens+1);
				     strcpy (Input.filename[FN_RADIOSONDE], s);
                                     Input.atm.n_rs_gas=0;
                                     free(s);
                                     for (i=3;i<=ntokens+1; i++) {
				       t = yytext2string(yytext, i, ntokens+1);  
				       if      (strcmp("O3",t)  ==0||strcmp("o3",t)  ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_O3;  Input.atm.n_rs_gas++; mol_id = MOL_O3;  }
				       else if (strcmp("O2",t)  ==0||strcmp("o2",t)  ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_O2;  Input.atm.n_rs_gas++; mol_id = MOL_O2;  }
				       else if (strcmp("H2O",t) ==0||strcmp("h2o",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_H2O; Input.atm.n_rs_gas++; mol_id = MOL_H2O; }
				       else if (strcmp("CO2",t) ==0||strcmp("co2",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_CO2; Input.atm.n_rs_gas++; mol_id = MOL_CO2; }
				       else if (strcmp("NO2",t) ==0||strcmp("no2",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_NO2; Input.atm.n_rs_gas++; mol_id = MOL_NO2; }
				       else if (strcmp("BRO",t) ==0||strcmp("bro",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_BRO; Input.atm.n_rs_gas++; mol_id = MOL_BRO; }
				       else if (strcmp("OCLO",t)==0||strcmp("oclo",t)==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_OCLO;Input.atm.n_rs_gas++; mol_id = MOL_OCLO;}
				       else if (strcmp("HCHO",t)==0||strcmp("hcho",t)==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_HCHO;Input.atm.n_rs_gas++; mol_id = MOL_HCHO;}
				       else if (strcmp("O4",t)==0  ||strcmp("o4",t)==0)   {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_O4;  Input.atm.n_rs_gas++; mol_id = MOL_O4;}
				       else if (strcmp("SO2",t)==0 ||strcmp("so2",t)==0)  {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_SO2; Input.atm.n_rs_gas++; mol_id = MOL_SO2;}
				       else if (strcmp("CM_3",t)==0||strcmp("cm-3",t)==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++; 
                                                                                            else Input.atm.rs_unit[mol_id] = CM_3; }
				       else if (strcmp("M_3",t) ==0||strcmp("m-3",t) ==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++;
                                                                                            else Input.atm.rs_unit[mol_id] = M_3;  }
				       else if (strcmp("MMR",t) ==0||strcmp("mmr",t) ==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++;
                                                                                            else Input.atm.rs_unit[mol_id] = MMR;  }
				       else if (strcmp("VMR",t) ==0||strcmp("vmr",t) ==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++;
                                                                                            else Input.atm.rs_unit[mol_id] = VMR;  }
                                       else if (strcmp("RH" ,t) ==0||strcmp("rh",t) == 0) {
                                         if (mol_id==0 || mol_id>=MOL_NN) 
                                           ierror++;
                                         else if (mol_id != MOL_H2O) {
                                           fprintf(stderr,"Error, unit \"relative humidity\" only possible for H2O dens_files.\n");
                                           fprintf(stderr,"See 'dens_column ... %s' in line %d\n", s, line_number+1);
                                           ierror++;
                                         }
                                         else 
                                           Input.atm.rs_unit[mol_id]= RH; 
                                        }
				       else {fprintf(stderr,"Unknown 'radiosonde ... %s' on line %d\n", t, line_number+1);
				             ierror++;}
                                       free(t);
				     }
                                     if (Input.atm.n_rs_gas>2)
                                     for (i=0;i<Input.atm.n_rs_gas-1;i++) {
                                       for (j=i+1;j<Input.atm.n_rs_gas;j++)
                                         if (Input.atm.rs_gas[i] == Input.atm.rs_gas[j]) {
                                           fprintf(stderr, "Error in input file, 'radiosonde', on line %d\n", line_number+1);
                                           fprintf(stderr, "At least one species is specified more than once!\n");
                                           ierror++;
                                         }
                                     }  
                                     if (ierror!=0) {
                                       fprintf(stderr,"Error in input file, 'radiosonde', on line %d\n",line_number+1);  
                                       fprintf(stderr,"Please check the arguments and their order (first species, than unit)\n"); 
                                     }
                                    }
radiosonde_levels_only|ECMWF_levels_only {Input.atm.rs_add_upper_levels = FALSE;
                                    }
refractive_index_file{SPACE}+{WORD} {	Input.filename[FN_REFIND] = yytext2string(yytext, 2, 2); }
satellite_geometry{SPACE_WORD}      {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 1) {
                                       fprintf(stderr,"Error, wrong number of arguments for 'satellite_geometry' on line %d\n", line_number+1);
                                       fprintf(stderr,"usage: \n"); 
                                       fprintf(stderr,"   satellite_geometry  netCDF_file  \n"); 
                                       ierror++;
                                     }
                                     yytextCstring(Input.filename[FN_SATGEOMETRY],yytext,2,ntokens+1);
                                    }
satellite_pixel{SPACE_INTEGER}{SPACE_INTEGER} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf(stderr,"Error, wrong number of arguments for 'satellite_pixel' on line %d\n", line_number+1);
                                       fprintf(stderr,"usage: \n"); 
                                       fprintf(stderr,"   satellite_pixel  pixel_nr_x  pixel_nr_y  \n"); 
                                       ierror++;
                                     }
                                     Input.sat_pixel_x = yytext2int(yytext,2,ntokens+1); 
                                     Input.sat_pixel_y = yytext2int(yytext,3,ntokens+1);
                                    }
sslidar_area{SPACE}+{FLOAT}         {Input.sslidar_area = yytext2float(yytext,2,2); }
sslidar_E0{SPACE}+{FLOAT}           {Input.sslidar_E0 = yytext2float(yytext,2,2); }
sslidar_eff{SPACE}+{FLOAT}          {Input.sslidar_eff = yytext2float(yytext,2,2); }
sslidar_nranges{SPACE_INTEGER}      {Input.sslidar_nranges = yytext2int(yytext,2,2);}
sslidar_polarisation                {Input.sslidar_polarisation = 1;}
sslidar_position{SPACE}+{FLOAT}     {Input.sslidar_position = yytext2float(yytext,2,2); }
sslidar_range{SPACE}+{FLOAT}        {Input.sslidar_range = yytext2float(yytext,2,2); }
surface_temperature_map{WORDS}      {ntokens = yytext2ntokens(yytext);
                                     if (ntokens < 1 || ntokens > 2) {
                                       fprintf (stderr, "Error, 'surface_temperature_map' on line %d has one mandatory and one optional argument\n",line_number+1); 
                                       fprintf (stderr, "optional argument is variable name of the surface temperature in the netCDF file \n");
                                       ierror++;
                                     }
                                     yytextCstring (Input.filename[FN_SURFACE_TEMP_MAP],yytext, 1+1, ntokens+1);
                                     if (ntokens == 2)
                                       yytextCstring (Input.netCDF_name_surf_T,yytext, 1+1, ntokens+1); 
                                    }
surface_type_map{WORDS}             {ntokens = yytext2ntokens(yytext);
                                     Input.alb.surf_type_map = TRUE;
                                     yytextCstring (Input.filename[FN_SURFACE_TYPE_MAP],yytext, 1+1, ntokens+1);
                                     if (ntokens > 1) {
                                       if (ntokens != 2) {
                                         fprintf (stderr, "Error, 'surface_type_map' on line %d requires NO or ONE optional arguments,\n",line_number+1); 
                                         fprintf (stderr, "optional argument is variable name of the albedo in the netCDF file \n");
                                         ierror++;
                                       }
                                       else
                                         yytextCstring (Input.alb.netCDF_surf_name,yytext, 1+1, ntokens+1);  
                                     }
                                    }
thermal_bandwidth{SPACE_FLOAT}{WORDS} {ntokens = yytext2ntokens(yytext);
                                     Input.bandwidth = yytext2double(yytext,1+1,ntokens+1);
                                     if (ntokens==2) {
                                       s =  yytext2string(yytext,2+1,ntokens+1);
				       if      (strcmp("nm",s)==0)   {Input.bandwidth_unit = UNIT_PER_NM ;}
				       else if (strcmp("cm-1",s)==0) {Input.bandwidth_unit = UNIT_PER_CMm1;}
				       else {
                                         fprintf(stderr,"Error, unknown unit %s for 'thermal_bandwidth' on line %d\n", s, line_number+1);
				         ierror++;
                                       }
                                       free(s);
                                     } 
                                    }
time{WORDS}{FLOATS}                 {ntokens = yytext2ntokens(yytext);
                                     tmpint=0;
                                     s =  yytext2string(yytext,2,ntokens+1);
                                     if      (strncasecmp(s,"UTC",3) == 0) { Input.time_zone = TIME_UTC; tmpint=1; }
				     else if (strncasecmp(s,"LAT",3) == 0) { Input.time_zone = TIME_LAT; tmpint=1; }
                                     else {Input.time_zone = TIME_UTC; tmpint=0;}
                                     if (Input.time_zone == TIME_UTC) {
                                      if (ntokens == 6+tmpint) {
                                        Input.UTC.tm_year  = yytext2int  (yytext, 2+tmpint, ntokens+1) - 1900; /* !!! years after 1900             !!! */
                                        Input.UTC.tm_mon   = yytext2int  (yytext, 3+tmpint, ntokens+1) -    1; /* !!! month after January 0 ... 11 !!! */
                                        Input.UTC.tm_mday  = yytext2int  (yytext, 4+tmpint, ntokens+1);   
                                        Input.UTC.tm_hour  = yytext2int  (yytext, 5+tmpint, ntokens+1); 
                                        Input.UTC.tm_min   = yytext2int  (yytext, 6+tmpint, ntokens+1);
                                        Input.UTC.tm_sec   = yytext2float(yytext, 7+tmpint, ntokens+1);                                     
                                      }
                                      else {fprintf(stderr,"Error, wrong number of arguments %d for %s on line %d\n",ntokens, s, line_number+1);
                                            fprintf(stderr,"  Please specify the time with the format: YYYY MM DD HH MM SS.\n");
				            ierror++;
                                      }
                                     }
                                     else if (Input.time_zone == TIME_LAT) {
                                      if (ntokens == 6+tmpint) {
                                        Input.LAT.tm_year  = yytext2int  (yytext, 2+tmpint, ntokens+1) - 1900; /* !!! years after 1900             !!! */
                                        Input.LAT.tm_mon   = yytext2int  (yytext, 3+tmpint, ntokens+1) -    1; /* !!! month after January 0 ... 11 !!! */
                                        Input.LAT.tm_mday  = yytext2int  (yytext, 4+tmpint, ntokens+1);   
                                        Input.LAT.tm_hour  = yytext2int  (yytext, 5+tmpint, ntokens+1); 
                                        Input.LAT.tm_min   = yytext2int  (yytext, 6+tmpint, ntokens+1);
                                        Input.LAT.tm_sec   = yytext2float(yytext, 7+tmpint, ntokens+1);                                     
                                      }
                                      else {fprintf(stderr,"Error, wrong number of arguments %d for %s on line %d\n",ntokens, s, line_number+1);
                                            fprintf(stderr,"  Please specify the time with the format: YYYY MM DD HH MM SS.\n");
				            ierror++;
                                      }
                                     }
                                     else {
                                      fprintf(stderr,"Error, unknown time zone %d, please report this program bug!\n", Input.time_zone);
				      ierror++;
                                     }
                                     free(s);
                                    } 
time_interpolate{WORDS}             {Input.atm.time_interpolate = TIME_INTERPOLATION;}
time_interval{SIGNED_FLOATS_OR_WORDS} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2 && ntokens != 3) {
                                       fprintf(stderr,"Error, wrong number of arguments for 'time_interval' on line %d\n", line_number+1);
                                       fprintf(stderr,"usage: \n"); 
                                       fprintf(stderr,"   time_interval  dtime_start  dtime_end [unit]\n"); 
                                       ierror++;
                                     }
                                     Input.delta_time_start = yytext2float(yytext,1+1,ntokens+1);
                                     Input.delta_time_end   = yytext2float(yytext,1+2,ntokens+1);
                                     if ( ntokens == 3 ) {
                                       s =  yytext2string(yytext,1+3,ntokens+1);
				       if      (strncasecmp("s"  ,s,1) == 0 ) {/* nothing to do */}
				       else if (strncasecmp("min",s,3) == 0 ) {Input.delta_time_start*=  60.0; Input.delta_time_end*=  60.0;}
				       else if (strncasecmp("h"  ,s,1) == 0 ) {Input.delta_time_start*=3600.0; Input.delta_time_end*=3600.0;}
				       else {
                                         fprintf(stderr,"Error, unknown unit %s for 'time_interval' on line %d\n", s, line_number+1);
				         ierror++;
                                       }
                                       free(s);
                                     }
                                    }
tipa{SPACE}+{WORD}                  {s =  yytext2string(yytext,2,2);
                                     if      (strncasecmp("dirdiff",s,7)==0) Input.tipa = TIPA_DIRDIFF; /* 1 */
				     else {                                  Input.tipa = TIPA_DIR;}    /* 2 */
                                     free(s);
				    }


wvn{SPACE}+{FLOAT}{SPACE}+{FLOAT}?      {ntokens = yytext2ntokens(yytext);
				     Input.wl.start = yytext2float(yytext,2,ntokens+1); 
                                     if (ntokens == 1)
                                       Input.wl.end   = Input.wl.start;
                                     else if (ntokens == 2)
                                       Input.wl.end   = yytext2float(yytext,3,ntokens+1);
                                     else {
                                       fprintf(stderr,"Wrong number of arguments for 'wavelength' on line %d\n", line_number+1);
				       ierror++;
				     }
                                    }
wvn_index{SPACE}+{INTEGER}{SPACE}+{INTEGER}?  {ntokens = yytext2ntokens(yytext); 
				     Input.wl.start_index = yytext2int(yytext,2,ntokens+1); 
                                     if (ntokens == 1)
                                       Input.wl.end_index = Input.wl.start_index;
				     else if (ntokens == 2) 
                                       Input.wl.end_index = yytext2int(yytext,3,ntokens+1);
                                     else {
                                       fprintf(stderr,"Wrong number of arguments for 'wavelength_index' on line %d\n", line_number+1);
				       ierror++;
				     }
                                    }
wc_file{SPACE_PROFILETYPE}+{SPACE_WORD} {isp=get_caoth_index(&Input.caoth,&Input.n_caoth,"wc",1);
                                       t = yytext2string(yytext, 2, 3);
                                       if      ( strncasecmp("1d",t, 2)==0 ) {
                                         Input.caoth[isp].source = CAOTH_FROM_FILE; }
                                       else if ( strncasecmp("3d",t, 2)==0 ) {
                                         Input.caoth[isp].source = CAOTH_FROM_3D_FILE; }
                                       else if ( strncasecmp("ipa_files",t, 9)==0 ) {
                                         Input.caoth[isp].source = CAOTH_IPA_FILES;
                                         Input.caoth[isp].ipa=1;
                                         Input.ipa=1;
                                         Input.caoth[isp].properties = PROP_WC_HU; }
                                       else if ( strncasecmp("moments",t, 7)==0 ) {
                                         Input.caoth[isp].source = CAOTH_LAYER_FILES;
                                         Input.caoth[isp].properties = PROP_EXPLICIT; }
                                       yytextCstring (Input.caoth[isp].filename,yytext,3,3);
                                    }
wc_saturate{SPACE}+{WORD}+{SPACE_FLOAT}* {fprintf(stderr,"wc_saturate obsolete!"); ierror++;}
write_ext_to_file                   {Input.write_ext_to_file=1;}
write_output_as_netcdf              {Input.write_output_as_netcdf=1;}
z_interpolate{WORDS}                {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s); 
                                     free(s);
                                     switch (ntokens) {
                                     case 0:
                                       Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX;
                                       break;
                                     case 1:
                                       t = yytext2string(yytext, 2, ntokens+1);
                                       if      (strncasecmp("default",t, 7)==0)  { Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX; }
                                       else if (strncasecmp("lin_mix",t, 7)==0 || strncasecmp("linmix",t, 6)==0)
                                                                                 { Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX; }
                                       else if (strncasecmp("log_lin",t, 7)==0 || strncasecmp("loglin" ,t, 6)==0)  
                                                                                 { Input.atm.z_interpolate = Z_INTERPOLATE_LOGLIN; }
                                       else if (strncasecmp("spline" ,t, 6)==0)  { Input.atm.z_interpolate = Z_INTERPOLATE_SPLINE; }
                                       else {fprintf (stderr, "Error, unknown 'z_interpolate %s' on line %d\n", t, line_number+1);
				             ierror++;}
                                       free(t);
                                       break;
                                     case 2:
				       t =  yytext2string(yytext,2,ntokens+1);
				       if      (strcmp("O3",t)  ==0 || strcmp("o3",t)  ==0)  mol_id = MOL_O3; 
                                       else if (strcmp("O2",t)  ==0 || strcmp("o2",t)  ==0)  mol_id = MOL_O2; 
                                       else if (strcmp("H2O",t) ==0 || strcmp("h2o",t) ==0)  mol_id = MOL_H2O; 
                                       else if (strcmp("CO2",t) ==0 || strcmp("co2",t) ==0)  mol_id = MOL_CO2; 
                                       else if (strcmp("NO2",t) ==0 || strcmp("no2",t) ==0)  mol_id = MOL_NO2; 
                                       else if (strcmp("BRO",t) ==0 || strcmp("bro",t) ==0)  mol_id = MOL_BRO; 
                                       else if (strcmp("OCLO",t)==0 || strcmp("oclo",t)==0)  mol_id = MOL_OCLO; 
                                       else if (strcmp("HCHO",t)==0 || strcmp("hcho",t)==0)  mol_id = MOL_HCHO; 
                                       else if (strcmp("O4",t)==0   || strcmp("o4",t)==0)    mol_id = MOL_O4; 
                                       else if (strcmp("SO2",t)==0  || strcmp("so2",t)==0)   mol_id = MOL_SO2; 
                                       else if (strcmp("T" ,t)==0 || strcmp("t" ,t)==0)  mol_id = -1;
                                       else {fprintf (stderr,"Error, unknown z_interpolate option: %s on line %d\n", t, line_number+1);
				             ierror++;}
                                       free(t);  
                                       t =  yytext2string(yytext,3,ntokens+1);
                                       if (mol_id > 0) {
                                         if      (strcmp("SPLINE",t)==0 || strcmp("spline",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_SPLINE;  
                                         else if (strcmp("LINEAR",t)==0 || strcmp("linear",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LINEAR; 
                                         else if (strcmp("LOG",t)   ==0 || strcmp("log"   ,t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LOG;
                                         else if (strcmp("LINMIX",t)==0 || strcmp("linmix",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LINMIX; 
                                         else if (strcmp("LOG_SPLINE",t)==0 || strcmp("log_spline",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LOG_SPLINE;
                                         else {fprintf (stderr,"Error, unknown z_interpolate option: %s on line %d\n", t, line_number+1);
				               ierror++;}
                                       }
                                       else if (mol_id == -1) {
                                         if      (strcmp("SPLINE",t)==0 || strcmp("spline",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_SPLINE;  
                                         else if (strcmp("LINEAR",t)==0 || strcmp("linear",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LINEAR; 
                                         else if (strcmp("LOG",t)   ==0 || strcmp("log"   ,t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LOG;
                                         else if (strcmp("LINMIX",t)==0 || strcmp("linmix",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LINMIX; 
                                         else if (strcmp("LOG_SPLINE",t)==0 || strcmp("log_spline",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LOG_SPLINE;
                                         else {fprintf (stderr,"Error, unknown z_interpolate option: %s on line %d\n", t, line_number+1);
				               ierror++;}
                                       }
                                       free(t);
                                       break;
                                     default:
                                       fprintf (stderr,"Error, z_interpolate has wrong number of arguments on line %d.\n", line_number+1);
                                       ierror++;
                                     }
				    }
zout{SIGNED_FLOATS_OR_WORDS}|zout_sur{SIGNED_FLOATS_OR_WORDS}|zout_sea{SIGNED_FLOATS_OR_WORDS}  {
                                     /* determine input option */
                                     ntokens = yytext2ntokens(yytext);
                                     s = yytext2string(yytext, 1, ntokens+1);
                                     if      ( strncasecmp("zout"    ,s,4) == 0 && strlen(s) == 4 ) {Input.atm.zout_source = OUTLEVEL_ZOUT_ABOVE_SUR;}
				     else if ( strncasecmp("zout_sur",s,8) == 0 && strlen(s) == 8 ) {Input.atm.zout_source = OUTLEVEL_ZOUT_ABOVE_SUR;}
                                     else if ( strncasecmp("zout_sea",s,8) == 0 && strlen(s) == 8 ) {Input.atm.zout_source = OUTLEVEL_ZOUT_ABOVE_SEA;}
                                     else {fprintf (stderr,"Error, program bug during reading input file: %s on line %d\n", yytext, line_number+1);
				           ierror++;}
                                     free(s);

                                     /* check for keywords */
                                     t = yytext2string(yytext, 2, ntokens+1);
				     if      (strncasecmp("atm_levels",  t,10)==0)            {Input.atm.zout_source = OUTLEVEL_ATM_LEVELS;}
				     else if (strncasecmp("all_levels",  t,10)==0)            {Input.atm.zout_source = OUTLEVEL_ALL_LEVELS;}
				     else if (strncasecmp("model_levels",t,13)==0)            {Input.atm.zout_source = OUTLEVEL_MODEL_LEVELS;}
				     else if (strncasecmp("model_layers",t,13)==0)            {Input.atm.zout_source = OUTLEVEL_MODEL_LAYERS;}
				     else if (strncasecmp("model_levels_and_layers",t,24)==0) {Input.atm.zout_source = OUTLEVEL_MODEL_LEVELS_AND_LAYERS;}
                                     else {
                                       Input.atm.nzout = yytext2ntokens(yytext);
                                       if ( Input.atm.zout_source == OUTLEVEL_ZOUT_ABOVE_SUR) {
                                         Input.atm.zout_sur = calloc (Input.atm.nzout,sizeof(float));
                                         for (i=2;i<=Input.atm.nzout+1; i++) {
				           u = yytext2string(yytext, i, Input.atm.nzout+1);
				           if      (strncasecmp("toa"         ,u, 3)==0) {Input.atm.zout_sur[i-2] = ZOUT_TOA;}
				           else if (strncasecmp("sur"         ,u, 3)==0) {Input.atm.zout_sur[i-2] = ZOUT_SURFACE;}
                                           else if (strncasecmp("cpt"         ,u, 3)==0) {Input.atm.zout_sur[i-2] = ZOUT_CPT;}
                                           else {Input.atm.zout_sur[i-2] = yytext2float(yytext,i,ntokens+1);}
                                           free(u);
                                         }
                                       }
                                       else if ( Input.atm.zout_source == OUTLEVEL_ZOUT_ABOVE_SEA ) {
                                         Input.atm.zout_sea = calloc (Input.atm.nzout,sizeof(float));
                                         for (i=2;i<=Input.atm.nzout+1; i++) {
				           u = yytext2string(yytext, i, Input.atm.nzout+1);
				           if      (strncasecmp("toa"         ,u, 3)==0) {Input.atm.zout_sea[i-2] = ZOUT_TOA;}
				           else if (strncasecmp("sur"         ,u, 3)==0) {Input.atm.zout_sea[i-2] = ZOUT_SURFACE;}
                                           else if (strncasecmp("cpt"         ,u, 3)==0) {Input.atm.zout_sea[i-2] = ZOUT_CPT;}
                                           else {Input.atm.zout_sea[i-2] = yytext2float(yytext,i,ntokens+1);}
                                           free(u);
                                         }
                                       }
                                     }
                                     free(t);

                                     /* check: these keyword do not allow further levels */
                                     if (Input.atm.zout_source == OUTLEVEL_ATM_LEVELS ||
                                         Input.atm.zout_source == OUTLEVEL_ALL_LEVELS ||
                                         Input.atm.zout_source == OUTLEVEL_MODEL_LEVELS ||
                                         Input.atm.zout_source == OUTLEVEL_MODEL_LAYERS ||
                                         Input.atm.zout_source == OUTLEVEL_MODEL_LEVELS_AND_LAYERS) {
                                       if (ntokens != 1 && ntokens != 2) {
                                         fprintf (stderr,"Error, using '%s %s' in line %d accept no further levels!!!\n", s, t, line_number+1);
				         ierror++;
                                       }
                                       if (ntokens == 2)
                                         Input.atm.nzout = yytext2int(yytext,2+1,ntokens+1);
                                     }
				    }


{SPACE}+       ;         /* eat up blanks */ 

{COMMENT}*     ;         /* eat up comments */

\n                        { ++line_number;};

{WORD}                    {fprintf(stderr,"Unknown command on line %d\n", line_number+1);
                           fprintf(stderr,"%s\n", yytext);
                           ierror++;}


<include>[ \t]*      /* eat the whitespace */
<include>[^ \t\n]+   { /* got the include file name */
        if (include_stack_ptr >= MAX_INCLUDE_DEPTH) {
            fprintf (stderr, "Includes nested too deeply");
            exit (1);
        }

        include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;

        yyin = fopen(yytext, "r");

        if (!yyin) {
            fprintf (stderr, "Error including %s\n", yytext);
            exit(1);
        }

        yy_switch_to_buffer (yy_create_buffer (yyin, YY_BUF_SIZE));

        BEGIN(INITIAL);
        }

<<EOF>> {
        if (--include_stack_ptr < 0) {
            yyterminate();
        }
        else {
            yy_delete_buffer (YY_CURRENT_BUFFER);
            yy_switch_to_buffer (include_stack[include_stack_ptr]);
        }
        }

%%

/*************************************************************/
/* print usage information                                   */
/*************************************************************/

#ifndef UVSPEC_FUNCTION
static void version()
{
  fprintf (stderr, "uvspec, version %s\n", THEVERSION);
}

static void usage()
{

  fprintf (stderr, "\n");  
  version();
  fprintf (stderr, "\nCalculates the radiation field of the Earth\'s atmosphere.\n");
  fprintf (stderr, "See libRadtran documentation for more information.\n\n");
  fprintf (stderr, "\n");
  fprintf (stderr, "Radiative transfer is a complex problem. Make sure\n");
  fprintf (stderr, "you understand both the problem you try to solve and the\n");
  fprintf (stderr, "tools you use to solve it.\n\n");
  fprintf (stderr, "\n");

  fprintf (stderr, "Usage: uvspec [-h] < input_file > Output_file\n");
  fprintf (stderr, " -h             Print this message.\n");  
  fprintf (stderr, " -v             Print version number.\n");  
  fprintf (stderr, " -c             Checks if the syntax of the input file is correct,\n");  
  fprintf (stderr, "                then exit. NO radiation calculated.\n");  
  fprintf (stderr, " -i input_file  Alternative way to specifiy the input file.\n");  
  fprintf (stderr, " -f input_file  Same as -i.\n");  

  fprintf (stderr, "\n");  
}
#endif

#ifndef UVSPEC_FUNCTION
int main(int argc, char **argv) 
#else
int calluvspec (char *outfilename)
#endif
{
  

/* @11c@

% @include mystic.sdoc

<lpdoc>
\section{Radiative transfer tool - \code{uvspec}}
\label{sec:uvspec_options}

The \code{uvspec} input file consists of single line entries, each making
up a complete input to the \code{uvspec} program. First on the line comes
the parameter name, followed by one or more parameter values. The 
parameter name and the parameter values are separated by white space.
Filenames are entered without any surrounding single or double quotes.
Comments are introduced by a \code{\#}. Blank lines are ignored. 
The order of the lines is not important, with one exception: if the
same input option is used more than once, the second one will usually
over-write the first one. Be aware that also options in another \code{included} 
input file will overwrite options specified before.

The various input parameters are described in detail below.
\begin{description}

\option{absorption}
Switch off absorption by individual minor trace gases which are currently only
included when \code{correlated_k lowtran} is chosen. The syntax is
\fcode{
  absorption species on/off
}
where species may be one of O4, N2, CO, SO2, NH3, NO, HNO3. By default all
are switched on. 

This option may also be used to turn on/off absorption by O4 in spectral 
resolution. It is on by default.

\option{albedo_map}
\emph{This option is preliminary and still subject to change (no wavelength dependency yet)!}
It gives the possibility to specify a wavelength independent albedo with the help of 
a \emph{netCDF} file, which is used in combinition 
with the options \code{latitude}, \code{longitude}, and \code{time}.
\fcode{
albedo_map file [variable_name]
}
Here \code{file} is the location of the \emph{netCDF} file,
The optional argument allows the name of the albedo variable in 
the \emph{netCDF} file to be specified (the default name is \code{AL}).
The albedo must be provided as function of \code{latitude} and \code{longitude} \code{AL(lat, lon)}, 
and may also depend on time \code{AL(time, lat, lon)}. 
The latitude, longitude, and time grids must be provided as doubles 
\code{double lat(lat)}, \code{double lon(lon)}, and \code{double time(time)}.
\code{uvspec} reads the value at the nearest pixel to the given \code{latitude} and 
\code{longitude}. No spatial interpolation or averaging of the values are performed.
If a time-dependent albedo is provided, the albedo data nearest to the specified time 
will be selected (or linear interpolated if \code{time_interpolate} is switched on). 

\option{altitude_map}
Specifies an altitude map which is used in combinition with 
\code{latitude}, \code{longitude} in order to 
select the altitude for the simulation.
No interpolation is done between the pixels of the map.
The format of the call is:
\fcode{
altitude_map file [variable_name]
}
where \code{file} is the location of the altitude map file. The map is expected to 
be in \emph{netCDF} format. The file must contain \code{double lat(lat)}, 
\code{double lon(lon)}, and the altitude variable, where \code{variable_name} is the 
name of the surface elevation variable in the \emph{netCDF} file. The default name is Z.
The altitude variable must be \code{altitude(lat, lon)}. 
For format discribtion see also the example map included in {\sl libRadtran}, 
\file{data/altitude/ELEVATION_GTOPO_10min.cdf}.
To use this map in \code{uvspec}, you may also use \code{altitude_map GTOPO}. This map has a resolution 
of 10 arc minutes and the unit of the altitude is meter. Please note that this resolution
might not ne adequate for your application. 
If an altitude in the map is below the lowest level of the 
\code{atmosphere_file}, the atmospheric profiles are extrapolated assuming a constant 
gradient for temperature and mixing ratios.

\option{angstrom}
Deprecated option. Same as \code{aerosol_angstrom}.

\option{atm_z_grid}
With this option the vertical resolution of the \code{atmosphere_file} data is changed to 
the levels (in km above sea surface) given as argument. This might be useful in oder to reduce the number 
of levels (save computational time) \ifmystic{or in order to easily adjust the atmosphere profile to the resolution 
of a Monte Carlo cloud file \code{mc_wcloud_file} or \code{mc_icloud_file}}.
\fcode{
atm_z_grid 0 2 4 6 8 10 20 30 ...
}

\ifmystic{
\option{bpdf_tsang_u10}
Wind speed for ocean BPDF (in m/s) at present only available with 
\code{rte_solver montecarlo}.
\fcode{
bpdf\_tsang\_u10 value
}
The BPDF model has been developed by \citet{Tsang1985}. 
The wind speed is the most important parameter affecting the ocean reflectance
matrix. The BPDF model also takes into account shadowing by surface waves. 
The model has been implemented in a FORTRAN routine by Mishchenko 
(http://www.giss.nasa.gov/staff/mmishchenko/brf/)
which has been included into {\sl libRadtran}.
}
\option{cloud_fraction_file}
File containing a cloud fraction profile. 
\fcode{
 cloud_fraction_file file
}
Two columns are expected:
altitude [km] and cloud fraction, including ice and water clouds. If \code{cloud_fraction_file}
is defined, effective cloud properties are calculated assuming either random overlap or maximum random overlap
of the cloud layers (see also \code{cloud_overlap}). An example is provided in \file{examples/CF.DAT}.

\undocumented{
\option{cloud_fraction_map}
Undocumented option.
}

\option{cloud_overlap}
Cloud overlap assumption. 
\fcode{
 cloud_overlap type
}
Following types are implemented:
\begin{description}
\item[rand] Random overlap of cloud layers 
\item[maxrand]  Maximum random overlap scheme
\item[max]      Maximum overlap scheme
\item[off]      Turn off cloud overlap for ECMWF clouds                                        
\end{description}
Per default the \code{cloud_overlap} scheme is switched off.
\option{cox_and_munk_pcl_map}
A possibility to specify pigment concentration (in mg/m3) for the Cox and Munk 
ocean BRDF with the help of an \emph{netCDF} file, which is used in combinition 
with options \code{latitude}, \code{longitude}, and \code{time}.
\fcode{
cox_and_munk_pcl_map file [variable_name]
}
where \code{file} is the location of the \emph{netCDF} file. 
{\sl libRadtran} reads the value at the nearest pixel to the given \code{latitude} and 
\code{longitude}. No spatial interpolation or averaging of the values is done.

The default name of the pigment concentration variable is
\code{chlorophyll}, but can be changed with the optional argument
\code{variable_name}.  The pigment concentration must be provided as
function of \code{latitude} and \code{longitude},
\code{chlorophyll(lat, lon)}, or additionally may also depend on time
\code{chlorophyll(time, lat, lon)}.  If a time-dependent pigment
concentration is specified, the pigment concentration will be
interpolated according to the option \code{time_interpolate}. All
grids must be provided in the file as \code{double lat(lat)},
\code{double lon(lon)}, and \code{double time(time)}.

\option{cox_and_munk_sal_map}
Specify ocean salinity (in ppt) for the \citet{cox54a,cox54b}
ocean BRDF with the help of an \emph{netCDF} file, which is used in combinition 
with the options \code{latitude}, \code{longitude}, and \code{time}.
\fcode{
cox_and_munk_pcl_map file [variable_name]
}
where \code{file} is the location of the \emph{netCDF} file. 
{\sl libRadtran} reads the value at the nearest pixel to the given \code{latitude} and 
\code{longitude}. No spatial interpolation or averaging of the values is done.

The expected name of the pigment concentration variable is per default
\code{salinity}, but can be changed with the optional argument
\code{variable_name}.  The pigment concentration must be provided as
function of \code{latitude} and \code{longitude}, \code{salinity(lat,
lon)}, or additionally may also depent on time \code{salinity(time,
lat, lon)}.  If a time-dependent salinity is specified, the salinity
will be interpolated according to the option
\code{time_interpolate}. All grids must be provided as \code{double
lat(lat)}, \code{double lon(lon)}, and \code{double time(time)}.

\option{crs_file}
May be used to specify cross sections of O3, NO2, 
BRO, OCLO, HCHO, O4 and SO2 to be used instead of those supplied with 
{\sl libRadtran}. No temperature dependence may be specified. Use as follows:
\fcode{
crs_file NO2 ../examples/no2_crs.dat
}
The \code{NO2}, \code{O3,BRO}, \code{OCLO}, \code{HCHO}, \code{O4} or \code{SO2}  must be 
specified to identify the
species for which the cross section applies.
The cross section file has two columns:
\begin{description}
\item[1] wavelength (nm)
\item[2] cross section (cm$^2$)
\end{description}
\option{data_files_path}
The path to the directory where all \code{uvspec} internal data files live, e.g.
the files that are in the subdirectories of the \file{data} directory
that comes with the \code{uvspec} distribution. 
\fcode{
   data_files_path path
}
The default for \code{path} is \code{../data/}.

\option{dens_file}
Specify density profiles (or matrix, see below) of various traces gases to be included in the radiative 
transfer calculation. The entry of the input file looks like:
\fcode{
dens_file gas_species [unit] filename
}
At the moment following gas\_species are included: ozone (O3), nitrogen dioxide (NO2), water vapor (H2O),
bromine oxide (BRO), chlorine dioxide (OCLO), formaldehyde (HCHO), carbon dioxide (CO2), 
sulphur dioxide (SO2), and the oxygen dimer (O4). 
The gas species is identified 
by their abbrevations given in the parenthesis above.
\code{unit} is an optional argument to define the unit of the density. The profiles can
be given in particles per cm3 (cm$^{-3}$), in particles per m3 (m$^{-3}$), as volume mixing ratio (vmr), as mass mixing 
ratio (mmr), or as relative humidity (only for water). The default for \code{unit} is cm$^{-3}$.
The model expects a density file with two columns:
\begin{description}
\item[1] Altitude above sea level in km.
\item[2] The density of trace gas [in the specified unit]
\end{description}
The altitude grid may be different from that in \code{atmosphere_file}. All densities inside the range 
of the \code{dens_file} are replaced. For all other altitudes the values from the 
\code{atmosphere_file} are used. If the density is specified as -1 at a level, 
the value from \code{atmosphere_file} is used.

To scale the profile to a total column value use \code{dens_column}.

For airmass factor calculations it is for some species necessary to account for the 
variation of the profile with sza. This may be accomplished by specifying a \code{dens_file} 
in the following format:
\fcode{
0.0       SZA1      SZA2 ...\\
z(1)    dens(1,1)    ...\\
z(2)      .           .\\
 .        .           .
}
where z(i) are the altitude levels above sea level in km, SZA is the solar zenith 
angle in degrees, and dens is the density [in the specified unit] of the trace gases as 
function of solar zenith angle and altitude. 
The matrix may only be specified for one specie. It may however be combined with profiles
of other species. 
A density matrix can only be used in connection with \code{rte_solver sdisort}!

\option{disort_icm}
Intensity correction method for \code{rte_solver cdisort} or
\code{rte_solver fdisort2}. Valid options are \code{phase}, i.e.~the
phase function is used for the Nakajima intensity correction, and
\code{moments}, i.e.~the Legendre moments are used for the
correction. Optionally, the option \code{off} turns off the intensity
correction. Default is \code{phase}.

 
\undocumented{
\option{ECHAM_atmosphere_file}
Undocumented option.
}
\undocumented{
\option{ECHAM_clouds_file}
Undocumented option.
}
\undocumented{
\option{ECHAM_sza}
Undocumented option.
}

\option{ECMWF_atmosphere_file}
Reads in combination with the options \code{latitude},
\code{longitude}, and \code{time} (all mandatory) the pressure,
temperature, ozone, and water vapour from an ECMWF \emph{netCDF} data
file and will combine it with the data given by the
\code{atmosphere_file}.
\fcode{
ECMWF_atmosphere_file file
}
No spatial interpolation of the values is done.  The atmospheric data
nearest to the specified \code{time} will be selected (or linearly
interpolated if \code{time_interpolate} is switched on).  Atmospheric
profiles, which are not provided by the ECMWF file (O2, CO2, NO2) are
taken from the \code{atmosphere_file}. Per default, also the
atmosphere above the ECMWF data is taken from the
\code{atmosphere_file}. In order to avoid this, please have a look at
the option: \code{ECMWF_levels_only}.

\option{ECMWF_levels_only}
The atmosphere considered in the simulation has the same height range
as the data in the
\code{ECMWF_atmosphere_file}/\code{radiosonde}-file. No further levels
are added above those.  This option has only an effect in combination
with \code{ECMWF_atmosphere_file} or \code{radiosonde}
(this option is identical to \code{radiosonde_levels_only}).

\option{ECMWF_ic_file}
Reads in combination with the options \code{latitude}, \code{longitude}, and \code{time} 
(all mandatory) the pressure, temperature, and cloud ice water content (CIWC) and cloud 
cover (CC) from an ECMWF \emph{netCDF} data file.
\fcode{
ECMWF_ic_file file
}
No spatial interpolation of the values is done.
The data nearest to the specified \code{time} will be selected (or linearly interpolated 
if \code{time_interpolate} is switched on).
In order to use the ECMWF data without cloud overlap assumption, use \code{cloud_overlap off}.

\option{ECMWF_ic_reff}
\emph{This option is preliminary and still subject to change!}
The ECMWF data only contains cloud water content, but no effective radius.
With this option, the effective radius can be specified. 
There are two possibilities:
For a fixed effective radius use the keyword \code{fixed} 
and specify the \code{reff} in micrometer.
\fcode{
ECMWF_ic_reff fixed reff
}
In order to use the parametrisation by \citet{Ou1995} use the keyword \code{Ou}.
\fcode{
ECMWF_ic_reff Ou
}
The default option is Ou.

\option{ECMWF_ozone_climatology}
The Integrated Forecast System (IFS) of the ECMWF uses a ozone
climatology for radiative transfer instead of the ozone simulated by
the IFS. If this option is activated the ozone profile of the
\code{atmosphere_file} or \code{ECMWF_atmosphere_file} is replaced by
the ozone climatology by Fortuin and Langematz (1995).
(If there is also a \code{dens_file} for ozone, it modifies the ozone
climatology profile.)

\option{ECMWF_wc_file}
Reads in combination with the options \code{latitude},
\code{longitude}, and \code{time} (all mandatory) the pressure,
temperature, and cloud liqid water content (CLWC) and cloud cover (CC)
from an ECMWF \emph{netCDF} data file.
\fcode{
ECMWF_wc_file file
}
No spatial interpolation of the values is done.  The data nearest to
the specified \code{time} will be selected (or linearly interpolated
if \code{time_interpolate} is switched on).  In order to use the ECMWF
data without cloud overlap assumption, use \code{cloud_overlap off}.

\option{ECMWF_wind_file}
Reads in combination with the options \code{latitude},
\code{longitude}, and \code{time} (all mandatory) the wind components
U, V, and W from an ECMWF \emph{netCDF} data file.
\fcode{
ECMWF_wind_file file
}
The data nearest to the specified \code{time} will be selected (or
linearly interpolated, if \code{time_interpolate} is switched on).

\option{emissivity_map}
\emph{This option is preliminary and still subject to change (no wavelength dependency yet)!}
Specify a wavelength independent emissivity with the help of 
an \emph{netCDF} file, which is used in combinition 
with the options \code{latitude}, \code{longitude}, and \code{time}.
\fcode{
emissivity_map file [variable_name]
}
where \code{file} is the location of the \emph{netCDF} file. 
With the optional argument \code{variable_name} the name of the emissivity variable in 
the \emph{netCDF} file can be specified. (By default the expected name is \code{EMIS}.)
The emissivity must be specified as function of \code{latitude} and \code{longitude} \code{EMIS(lat, lon)}, 
or additionally may also depent on time \code{EMIS(time, lat, lon)}. 
All grids must be provided as 
\code{double lat(lat)}, \code{double lon(lon)}, and \code{double time(time)}.
{\sl libRadtran} reads the value at the nearest pixel to the given \code{latitude} and 
\code{longitude}. No spatial interpolation or averaging of the values is done.
If the variable time is present in the file, the emissivity data nearest to the specified time 
will be selected (or interpolated if \code{time_interpolate} is switched on). 

\option{flexstor}
Deprecated option use \code{output flexstor} instead.
Provide output in flexstor format. Must not be combined with \code{header}.
Also, does not currently work when \code{umu} and/or \code{phi} is specified.

%CE commented for ESAS Light
\undocumented{
\option{fu_h2o_continuum}
Undocumented option to switch the H2O continuum on or off or select a specific 
version of the continuum.
}

\option{header}
Include information on some of the input parameters in the output. May not be combined
with \code{flexstor}. Please note that the information provided is rather 
incomplete because this option was introduced quite early and was never updated. 
For a more complete information please use the \code{verbose} option.

%CE: Commented for ESA version
\undocumented{
\option{ichap}
Undocumented option for sdisort.
}

\option{ic_fu_tau}
Specify if the \citet{Fu1996} optical properties are delta-scaled or not. 
With 
\fcode{
ic_fu_tau scaled
}
 delta-scaling is switched on, with
\fcode{
ic_fu_tau unscaled
}
it is switched off. The default is without
delta-scaling. Please note that this was changed on July 22, 2008: 
Before, delta-scaling was switched on by default which might have
caused some confusion, because irradiance calculations were not
consistent with the other ice cloud parameterizations implemented in
\code{uvspec}. 
Using the \citet{Fu1996} parameterization in combination with one of 
\code{ic_set_tau/tau550/gg/ssa} or \code{ic_scale_gg/ssa} you now get
consistent results with all other ice cloud parameterizations.

%THIS IS THE OLD DOCUMENTATION (BEFORE CHANGE OF DELTA-SCALING
%				      DEFAULT).
%
%It has been confirmed that the difference with and 
%without delta-scaling is typically on the order of less than 1 - 2\% 
%and therefore it was decided to switch delta-scaling off by default.
%Please note that this has nothing to do with the internal delta-scaling
%by the solvers: e.g. disort2 always applies it's own internal delta-scaling 
%anyway. (THE FOLLOWING NEEDS TO BE MODIFIED).
%If you define a cloud only by its microphysical properties (ice water
%content, effective radius), delta-scaling should certainly be switched
%on and you do not need to read further. 
%If, however, you want to
%use the Fu (1996) parameterization in combination with one of 
%\code{ic_set_tau/tau550/gg/ssa} or \code{ic_scale_gg/ssa} it might be
%reasonable to switch delta-scaling off and you should make sure that
%you understand the following. Citing from Fu (1996): "For nonspherical
%particles in cirrus clouds, it is found that a simple representation
%of the scattering phase function through the asymmetry factor is
%inadequate (Fu and Takano 1994). As demonstrated in appendix A, the
%fraction of scattered energy residing in the forward peak, f, needs to
%be removed from the scattering parameters to incorporate the strong
%forward peak contribution in multiple scattering." Or in other words,
%the sharp forward peak is truncated and added to the unscattered
%direct radiation. The remaining phase function (excluding the sharp
%forward peak) can be safely approximated by a Henyey-Greenstein
%function. The scaling implies a reduction of the optical thickness,
%the asymmetry parameter, and the single scattering albedo. This
%reduction can be rather severe, e.g. a factor of about 3 for the
%optical thickness in the visible spectral range. This implies
%seemingly inconsistent optical properties: For idential IWC content
%and effective radius, \code{ic_properties key/yang} would give an
%(unscaled) optical thickness about three times higher than
%\code{ic_properties fu}. The effect on the radiation field, however,
%will be comparable, due the consistent scaling of optical thickness,
%asymmetry parameter, and single scattering albedo. If you, however,
%adjust the optical thickness using e.g. \code{ic_set_tau}, the effect
%on the radiation field will be completely different because the
%(unscaled) optical thickness by Key (2002) has a completely different
%meaning as the (scaled) optical thickness by Fu (1996). In such cases
%it might be reasonable to switch scaling off. This is a complicated
%and confusing topic and it is suggested that you play around a bit
%with the options, read the Fu (1996) paper, and make heavy use of the
%\code{verbose} feature.

\option{ic_fu_reff}
If
\fcode{
ic_fu_reff fu
}
 is specified, the  
parameterization uses the original definition of the effective radius
as specified in \citet{Fu1996,Fu1998}. By default it
uses the same definition of the effective radius as the \citet{Key2002},
\citet{Yang2000} and \citet{baum05a:_bulk,baum05b:_bulk}
parameterizations; see discussion of \code{ic_properties}.

\option{ic_saturate}
Obsolete option.

\option{include}
Include a file into the \code{uvspec} input. 
\fcode{
include file
}
Works exactly like the C \code{\#include} or 
the Fortran \code{INCLUDE} statements.

\ifthreedmystic{
\option{ipa_3d}
This option can be used to carry out a 3D calculation with a 1D-solver, where a 
3D-cloud file as used for MYSTIC (flag 3) serves as an input file. Further, a sample grid needs to be specified
via \code{mc_sample_grid}, where the grid and the resolution has to be the same as in the 3D-cloud file
(Nx, Ny, dx, dy). To calculate heating rates, \code{mc_backward_output heat K_per_day} must stand in the input
file. 
}

\option{latitude}
This option can be used to specify the latitude of the location to simulate.
(This option only has an effects, if \code{longitude} is specified, too.)
\fcode{
latitude deg [min] [sec]
}
where \code{deg min sec} is the position in degrees, arc minutes, and arc seconds north.
\code{deg} might also be a float number. \code{min} and \code{sec} may be obmitted.
The \code{latitude} information will be used for the following: 

\code{latitude} in combination with \code{longitude}, \code{time}, and any \code{map}-option
is used to select the location where to read the input data.

\code{latitude} in combination with \code{longitude} and \code{time} is used to calculate 
the solar zenith angle, if no \code{sza} is specified (see also \code{time_interval}).

\code{latitude} in combination with \code{longitude} and \code{time} is used to choose a
suitable default atmosphere file, if no \code{atmosphere_file} is specified. 

\option{longitude}
This option can be used to specify the longitude of the location to simulate.
(This option only has an effects, if \code{latitude} is specified, too.)
\fcode{
longitude deg [min] [sec]
}
where \code{deg min sec} is the position in degrees, arc minutes, and arc seconds east.
\code{deg} might also be a float number. \code{min} and \code{sec} may be obmitted.
The \code{longitude} information will be used for the following: 

\code{longitude} in combination with \code{latitude}, \code{time}, and any \code{map}-option
is used to select the location where to read the input data.

\code{longitude} in combination with \code{latitude} and \code{time} is used to calculate 
the solar zenith angle, if no \code{sza} is specified (see also \code{time_interval}).

\code{longitude} in combination with \code{latitude} and \code{time} is used to choose a
suitable default atmosphere file, if no \code{atmosphere_file} is specified. 



%CE Commented for ESA version
\undocumented{
\option{mc_bcond}
Undocumented option to define MYSTIC boundary conditions: periodic (default),
absorbing, mirror (not really supported - don't expect meaningful results). 
}

\ifthreedmystic{
\option{mc_blitz_position}

Define lightning as source. The position of the the lightning light
source is defined as a simple line, going from [ X\_1 Y\_1 Z\_1 ] to [
X\_2 Y\_2 Z\_2 ] (in units of m). Usage:
\fcode{
mc_blitz_position X_1 Y_1 Z_1 X_2 Y_2 Z_2
}
Only works in forward mode, for radiances.
}

\iflidar{
\option{mc_jacobian}

Calculate jacobi matrix. Only meaningful with \code{rte_solver
montecarlo}.

This option works with \code{mc_lidar}, or with \code{mc_backward}. In
the first case, it delivers the jacobi matrix for an elastic or HSRL
lidar. In the latter case, it delivers the jacobi matrix for height
resolved changes in molecular absorption and scattering coefficients.
}
\iflidar{
\option{mc_jacobian_std}

Calculate jacobi matrix standard deviation. Only meaningful with
\code{mc_jacobian}.
}

\undocumented{
\option{mc_tipa}

Run MYSTIC in tilted independent pixel mode (\code{mc_ipa} is set
automatically).  Only meaningful with \code{rte_solver
montecarlo}. Choose either \code{mc_tipa dir} or \code{mc_tipa
dirdiff}. The latter is the analogon to \code{tipa dirdiff}, where the
3d-cloud-matrix is tilted according to the solar zenith angle and is
then used as input for all the following calculations. In
\code{mc_tipa dir}, the tilted cloud-matrix is used only to calculate
the optical depth along the solar beam and which is used for further
calculations together with the original cloud-matrix.
}

\iflidar{
\option{mc_lidar}
Use local estimator to simulate a lidar. If \code{mc_lidar} is set,
you need to provide a lidar file for \code{mc_lidar_file}. A detailed
documentation is available on request from Robert Buras.
}

\iflidar{
\option{mc_lidar_file}
File containing positions, looking directions, and opening angles of
lasers and detectors for lidar simulations in MYSTIC.  Only meaningful
with \code{mc_lidar}.
\fcode{
mc_lidar_file file
}
}

\ifthreedmystic{
\option{mc_panorama_alignment}

Use this option only together with \code{mc_panorama_view}. This
option has been introduced to facilitate e.g.~the calculation of
circum solar radiation. Three sub-options are available, \code{sun},
\code{mu}, and \code{zenith}. When set \code{sun}, the camera is
aligned to sun (i.e.~using the options \code{sza} and
\code{phi0}). $\theta$ and $\mu$ as given by the option
\code{mc_panorama_view} now relates to the sun's position. $\mu$ sets
cosine of the angular distance from the center of the sun. $\mu=$-1
implies looking directly into the sun. With $\phi$ we select the
direction around the sun on a circle with angular distance ${\rm
acos}(\mu)$ from the center of the sun. $\phi=0$ implies that the
camera pixel is looking in a direction exactly below the sun, at
$\phi=180$ the camera pixel will be looking exactly above the
sun. Increasing $\phi$ means the direction of looking moves clock-wise
around the sun-camera-line (seen from the sun). If the sun is in the
zenith, the definition of $\phi$ is identical to that in \code{phi}.

The sub-option \code{mu} aligns the camera geometry according to the
options \code{umu} and \code{phi}. Be aware that \code{phi0} and
\code{phi} are defined differently!

The sub-option \code{zenith} makes only sense when using
\code{mc_spherical3D}. Then the camera is aligned to the zenith with
respect to the lon-lat position of the camera.
}

\ifthreedmystic{
\option{mc_panorama_distr_photons_over_pixel}

Use this option only together with
\code{mc_panorama_view}. With only \code{mc_panorama_view} all photons for the same pixel
are emitted into exactly the same direction. The finite solid angle that
is covered by one pixel is not accounted for. With 
\code{mc_panorama_distr_photons_over_pixel} the photons are distributed
over the FOV covered by the pixel. 
}

\ifthreedmystic{
\option{mc_panorama_no_pixel}
Use this option only together with \code{mc_panorama_view}. Useful for
calculation of radiance distributions which shall be exactly
charted. Originally \code{mc_panorama_view} was meant to calculate images
like produced by a CCD camera. You set the edges of your desired image
and the resolution of the sensor. MYSTIC will then calculate the
radiances at the center of each pixel. This implies that values for
the edges of the image as given by the input file are never
computed. The border angles for which radiances are computed are
shifted half a pixel to the inside of the domain. With
\code{mc_panorama_no_pixel} the edging values as given in the input
file are directly hit. A short example: Assume you only have three
pixels. You choose your edges to be at $\phi=0$ and $\phi=180$. With
\code{mc_panorama_no_pixel} MYSTIC will calculate radiances at
$\phi=0$, $\phi=90$ and $\phi=180$. Without this option you will get
values for $\phi=30$, $\phi=90$ and $\phi=150$.
}

\ifthreedmystic{
\option{mc_panorama_quicklook}

Use this option only together with \code{mc_panorama_view}. This
option allows to do a fast calculation in order to see whether you
have set your parameters right. All panorama pixels are run in a fast
mode: only one photon is started per pixel, it is moved till to a
fixed optical depth (0.1), and then a local estimate is performed. In
case the local estimate is done scattering (i.e.~the photon has not
hit a surface before reaching tau=0.1), isotropic scattering is
assumed. Then the photon is killed. We suggest you turn off Rayleigh
scattering and aerosols while using this option. The result lets you
see roughly where your panorama is looking at surface, sunlit clouds,
shaded clouds, or slanted clouds.
}

\ifthreedmystic{
\option{mc_panorama_with_direct_rad} Output is direct plus diffuse radiance.
Use together with \code{mc_panorama_distr_photons_over_pixel}.
}

% CE: Commented out for ESA version, probably not working
\undocumented{
\option{mc_pcld_file}
Cloud phase function file for MYSTIC.
Only meaningful with \code{rte_solver montecarlo}.
\emph{This is an experimental option}.
\fcode{
mc_pcld_file file
}
}

\iflidar{
\option{mc_radar}
Switch on radar, internally it's lidar with radar configuration, use with
\code{mc_lidar_file}, \code{mc_ris_factor} and switch \code{mc_vroom} off.
}

\ifthreedmystic{
\option{mc_readrandomseed}
Read from file the random seed (positive integer) for the random number generator. 
\fcode{
mc_readrandomseed file
}
Usually a random seed is determined from current time plus process id. 
This option is useful to re-run a simulation for debugging.
}

\ifthreedmystic{
\option{mc_readrandomstatus}
Read from file the random status for the random number generator. 
\fcode{
mc_readrandomstatus file
}
This option is useful to re-run a simulation for debugging, especially if
the buggy photon appears only late in a long simulation. This option automatically
toggles on \code{mc_readrandomseed}.
}

\ifthreedmystic{
\option{mc_ris_factor}
Use with \code{mc_radar} to get good statistics, artificially enhances
the optical depth of the clouds for microwave wavelengths. Either use
it with a number to set the ris-factor manually or don't set a number
to enable automatic determination of the ris-factor. In the latter
case, the ris-factor is scaled so that the optical depth of the
complete range equals 1. To get a different optical depth, use
\code{mc_ris_optical_depth}.
\fcode{
mc_ris_factor (value)
}
}

\ifthreedmystic{
\option{mc_ris_optical_depth}
Use with \code{mc_ris_factor} to automatically determine a ris-factor
for an optical depth other than 1.
\fcode{
mc_ris_optical_depth value
}
}

\iflidar{
\option{mc_satellite_position}

Define the position of the satellite. See \code{mc_satellite_view} for
description.
}

\iflidar{
\option{mc_satellite_view}

Simulate a satellite image. Only works with
\code{mc_satellite_position}/\code{mc_sensorposition},
\code{mc_sample_grid}, \code{mc_backward} and
\code{mc_spherical3D}. Can best be explained by following example:

\fcode{\\
mc_spherical3D \\
mc_satellite_view PHI_W PHI_E THETA_S THETA_N \\
mc_sample_grid N_PHI N_THETA \\
mc_satellite_position SAT_LON SAT_LAT SAT_ALT \\
mc_sun_position SUN_LON SUN_LAT \\
mc_backward \\
}

In this example, the earth is a sphere (not plane parallel). The
satellite is in zenith above longitude SAT\_LON degrees, latitude
SAT\_LAT degrees, and at an altitude of SAT\_ALT
meters. (Alternatively, you can use \code{mc_sensorposition S_X S_Y
S_Z}, with [S\_X,0,0] being toward MSG, [0,S\_Y,0] being above the
Maledives, and [0,0,S\_Z] being above the north pole.) The image taken
has a resolution of N\_THETA times N\_PHI, and the picture (taken with
a camera with spherical picture geometry) spans from PHI\_W to PHI\_E
degrees (positive being east of nadir) and from THETA\_S to THETA\_N
degrees (positive being north of nadir). In this example, the sun is
in zenith at the latitude SUN\_LAT degrees (positive meaning north) and
longitude SUN\_LON degrees (positive meaning east). See also
\code{mc_panorama_view}, which is a similar option.  Example for MSG:

\fcode{\\
mc_spherical3D \\
mc_satellite_view -10 10 -10 10 \\
mc_sample_grid 4000 4000 \\
mc_satellite_position 0 0 3.6e7 \\
mc_sun_position -10 23\# about 1PM UTC on mid summer \\
mc_backward \\
}

Only meaningful with \code{rte_solver montecarlo}.
}

\iflidar{
\option{mc_spectral_is}
{\sl Experimental option!!!!}\\
Calculate a spectrum with high spectral resolution using an importance sampling method. 
The spectral variation of Rayleigh scattering is included, aerosol and cloud
scattering properties are assumed to be constant over the spectral range that is calculated.
Molecular absorption coefficients have to be provided in a file using the option
 
\code{molecular_tau_file}. 
Rayleigh scattering coefficients may be specified using the option
\code{rayleigh_crs} \iflidar{(Note that \code{rayleigh_tau_file} is not implemented with \code{mc_spectral_is}).} 
One wavelength contained in the \code{molecular_tau_file} needs to be provided with 
the option \code{wavelength}, for this wavelength the calculation is performed as usual. All 
other wavelengths are calculated using an importance sampling method. 
This method is very efficient for calculations with very high spectral resolution, e.g. 
simulations for trace gas retrievals. The method works also in combination with 
\code{mc_polarisation}. The output is given as normalized radiances and is printed 
to the file \code{mc.is.spc}. In order to obtain calibrated radiances these radiances need to be
multiplied with the extraterrestrial irradiance. 
}
 
\iflidar{
\option{mc_spherical3D}
3D spherical geometry in MYSTIC. Only works with
\code{mc_satellite_position}/\code{mc_sensorposition}, and
\code{mc_satellite_view}, and only for solar backward. You have to set
the sun position using \code{mc_sun_position}. Still very
experimental. Does not work with \code{mc_elevation_file}.
}

\iflidar{
\option{mc_spherical3D_scene}
Define scene on globe. Only works with \code{mc_spherical3D}. Without
this option, 3d clouds and 2d albedo maps are folded around the full
globe when using \code{mc_spherical3D}. This option defines the limits
of the clouds and albedo maps in terms of longitude and latitude. Example:

\fcode{\\
mc_spherical3D_scene LON_W LON_E LAT_S LAT_N \\
}

The scene spans from LON\_W degrees in the west to LON\_E degrees in
the east (positive values are east, negative are west of Greenwich)
and from LAT\_S degrees in the south to LAT\_N degrees in the
north (positive are northern hemisphere, negative are southern
hemisphere).
}

\iflidar{
\option{mc_sun_position}

Sun position when using \code{mc_spherical3D}. See \code{mc_satellite_view}
for an example and explanation.
}

\ifthreedmystic{
\option{mc_sun_radius}
At the moment only useful together with \code{mc_panorama_view}.
Set the angular radius of the sun in degrees. If omitted the radius is calculated 
(more or less correctly) via the earth-sun distance (not well tested).
If no \code{mc_backward_sun_shape_file} is given a spectral sun shape according to
Koepke2001 is used.
}

\ifthreedmystic{
\option{mc_surfaceparallel}
Calculate irradiance parallel to the surface instead of horizonal irradiance. This
option is obviously only interesting for topograpy and only for calculations at
the surface. For other levels the option is ignored. 
}

\option{no2_column_du}
Obsolete, use \code{dens_column} instead.
%Set the NO2 column to a given value. The column is integrated between the 
%user-defined \code{altitude} and TOA (top of atmosphere). The value must be in 
%Dobson units. If value is negative or \code{no2_column_du}
%is not specified, the NO2 column is not scaled.

\option{no2_column_moleccm-2}
Obsolete, use \code{dens_column} instead.
%Set the NO2 column to a given value. The column is integrated between the 
%user-defined \code{altitude} and TOA (top of atmosphere). The value must be in 
%molecules / cm2. If value is negative or \code{no2_column_moleccm-2}
%is not specified, the NO2 column is not scaled.

\option{no2_crs}
Choose between the various no2 cross sections by
\fcode{
no2_crs type
}
where \code{type} is one of
\begin{description}
\parameter{Burrows} 
\citet{Burrows1998} no2 cross section.
\parameter{Bogumil} 
no2 cross section from \citet{Bogumil2003}.
\end{description}
\citet{Bogumil2003} is default.

\option{optimize_fortran}
When this option is activated, the minimum dimensions of the FORTRAN arrays 
for the specific input conditions are determined and written to \code{stderr}.
Using \code{worldloop} two tolerance levels are added to the minimum number 
of atmospheric levels, as this number may change during \code{worldloop} calculations.

\option{ozone_column}
Obsolete, use \code{dens_column} instead.
%Set the ozone column to a given value. The column is integrated between the 
%user-defined \code{altitude} and TOA (top of atmosphere). The value must be in 
%Dobson units. If value is negative or \code{ozone_column}
%is not specified, the ozone column is not scaled.

%\option{p_out}
%Undocumented option. Same as \code{pressure_out}.

\option{pressure_out}
Specify the output levels in pressure coordinates. The syntax is  
\fcode{
pressure_out p1 p2 ...
}
where '\code{p1 p2 ...}' are the output levels in hPa. 
The pressure output levels must be sorted in decreasing order. 
Output pressure levels must be within the range defined in the
\code{atmosphere_file}. You can also use \code{toa} for 
\code{top of atmosphere} and \code{sur} for \code{surface altitude} and \code{cpt} 
for \code{cold point tropopause}.

\option{profile_file}
Define file containing properties of clouds, aerosols, hydrometeors,
etc. This option is a generalization of the options \code{wc_file},
\code{ic_file}, \code{wc_files}, etc.

Usage:
\fcode{
profile_file typename type file
}

\code{typename} describes the name of the profile; typically this is
describes what kind of particles are dealt with here. Examples
are "wc", "ice\_clouds", "aer", "drizzle". The \code{typename} is
needed to refer to this profile when using other options, such as
\code{profile_properties}. Note that \code{typename} "wc" and "ic"
have special effects (i.e.~default properties, and "ic" properties are
not allowed with "wc" files, and vice versa).

\code{type} defines the file type, which can be one of the
following (described for the example of water clouds):

\begin{description}
\item[1d] Location of file defining one-dimensional profile.
See \code{wc_file} for a description on the file structure.

\ifthreedmystic{
\item[3d] Define a 3D input file. See
\code{mc_wcloud_file} for a description of the expected file format.
In case the \code{rte_solver} is not \code{mystic}, independent column
calculations are performed for the 3D field. (See
\file{examples/UVSPEC_WC_IPA.INP} for an example.)

This option is identical to e.g.~\code{mc_wcloud_file} in case of
\code{rte_solver mystic}, and identical to e.g.~\code{wc_ipa} else.
}

\item[ipa\_files] A two-column file, defining one-dimensional profile
files (see \code{type 1d}) in the first column and the corresponding
weights in the second column.

See \code{wc_ipa_files} for a more thorough description.

\item[moments] A way to specify extinction coefficient, single
scattering albedo, and scattering phase function for each layer.
See \code{wc_files} for a description of the file format.
\end{description}

\option{profile_layer}
Interpret properties of profile \code{typename} as layer
properties. (default). See \code{wc_layer} for more explanations.

Usage:
\fcode{
profile_layer typename
}

\option{profile_level}
Interpret properties of profile \code{typename} as level properties. See
\code{wc_level} for more explanations.

Usage:
\fcode{
profile_level typename
}

\option{profile_no_scattering}
Switch off scattering on profile \code{typename}. Usage:
\fcode{
profile_no_scattering typename
}

\option{profile_properties}
Define how liquid/ice water content/mass concentration and effective
particle radius are translated to optical properties for profile
\code{typename}. This option is a generalization of the options
\code{wc_properties} and \code{ic_properties}.

Usage:
\fcode{
profile_properties typename property
}

\code{typename} describes the name of the profile; it must be identical
to the one defined in \code{profile_file}.

Possible choices for \code{property} are
\begin{description}
\parameter{hu} 
Parameterization by \citet{Hu1993} for water clouds; 
this is the default setting. Note that 
the parameterization is somewhat different for \code{correlated_k FU} than for all other 
cases because in the latter case the parameterization from the newer (March 2000) 
Fu and Liou code is used while otherwise the data are taken from the original 
paper by \citet{Hu1993}. Note that this parameterization has been
developed to calculate irradiances, hence it is less suitable for radiances.
This is due to the use of the Henyey-Greenstein phase function as an approximation
of the real Mie phase function.
\parameter{echam4} 
Use the very simple two-band parameterization of the ECHAM4 climate
model, described in \cite{Roeckner1996}; this is probably only
meaningful if you want to compare your results with ECHAM4, the two
bands are 0.2 - 0.68 micrometer and 0.68 - 4.0 micrometer; within
these bands, the optical properties are assumed constant.

This \code{type} is only available for \code{name} "wc"
and "ic". Dependent on which \code{name} is defined, the
appropriate parameterization is chosen.
\parameter{mie} 
Use pre-calculated Mie tables for water and ice clouds; useful for \code{correlated_k}; 
the tables are expected in \code{data_files_path}\file{/correlated_k/.../}. \\
For spectral or pseudo-spectral (\code{correlated_k sbdart}) calculations, 
a set of pre-calculated tables is also available.
%CE: commented some parts. Should be rewritten when new Mie tables are well
% optimized.
% the wavelength grid points of these 
%data has been carefully selected such that the extinction cross section, 
%single scattering albedo, and the asymmetry parameter are accurate to 1\% 
%(compared to the fully-resolved Mie calculation) for all wavelengths
%between 250nm and 100 micrometer. 
For spectral or pseudo-spectral
calculations \code{wc_properties_interpolate} has to be defined explicitely to
initiate the interpolation of the optical properties to the internal wavelength grid.
%Please note that this option may be extremely memory-consuming because for each 
%internal wavelength a full set of Legendre moments of the phase function is 
%stored (up to several thousands). 
The Mie tables are not part of the standard distribution
(because of their large size) but they are freely available from http://www.libradtran.org. 
This is the correct option to calculate radiances, to be preferred over the 
Henyey-Greenstein approach of \citet{Hu1993}.

This \code{type} is only available for \code{name} "wc"
and "ic". Dependent on which \code{name} is defined, the
appropriate parameterization is chosen.

Note that a Mie calculation assumes spherical ice particles, the
scattering function of which differs systematically from non-spherical
particles. Hence, \code{ic_properties mie} is usually not
representative of natural ice clouds.
\parameter{filename} 
Read optical properties from specified filename; file format is as produced 
by the \code{mie}-tool of the {\sl libRadtran} package (see \code{output_user netcdf}).
\item[fu]
Parameterization by \citet{Fu1996,Fu1998} for ice particles, see \code{ic_file}; 
this is the default setting. Note that this is a parameterization
which has been created to calculate fluxes but not radiances. 
Note also that the optical properties in the solar range provided by 
\citet{Fu1996} are delta-scaled properties (that is, the forward peak of
the phase function is truncated and optical thickness, asymmetry
parameter, and single scattering albedo are reduced accordingly),
whereas \code{uvspec} uses non delta-scaled properties unless the option
\code{ic_fu_tau scaled} is specified. By default the parameterization
by \citet{Fu1996} is treated consistently with all other ice cloud
parameterizations. 
For wavelengths up to 4 micrometer \citet{Fu1996} is used while for wavelengths 
larger than 4 micrometer \citet{Fu1998} is chosen. Please note that 
\citet{Fu1996} is based on ray-tracing calculations while \citet{Fu1998}
is a mixture of ray-tracing and Mie calculations (which is required for 
the infrared wavelengths where the geometrical assumption does not hold).
Hence, both parameterizations are not fully consistent. Rather, differences
of some \% are to be expected in the wavelength region where both 
parameterizations overlap. Also, the wavelength dependence in the solar 
and infrared parts is treated differently: In the solar part \citep{Fu1996}
the optical properties are defined for wavelength bands - hence they 
are assumed constant within each band. In the infrared \citep{Fu1998} 
they are defined at certain wavelengths and linearely interpolated 
in between. If you use this option, please see also the
discussion of \code{ic_fu_tau} and \code{ic_fu_reff}.
The allowed range for the effective radius is from 9.315 -  65.120 micrometer. 
\item[key]
Parameterization by \citet{Key2002} for ice particles. This parameterization can also
be used to calculate radiances because it uses a
double-Henyey-Greenstein phase function which better represents both
forward and backward peaks. This parameterization covers the wavelength region  
from 0.2 to 5.0 micrometer and is available for the following habits:
solid-column, hollow-column, aggregate, rosette-4, rosette-6, and plate.
\item[yang]
Parameterization similar to \citet{Key2002} for ice particles but based on more recent
single scattering calculations. Below 3.4 micrometer it actually equals
the \citet{Key2002} parameterization while from 3.4 - 100 micrometer new 
coefficients have been calculated with much higher wavelength
resolution and better accuracy. Hence, \code{yang} should give a reasonably
consistent approximation from 0.2 - 100 micrometer, suitable for spectrally
resolved calculations of radiance and irradiance.
The covered range for the effective radius depends on the \code{ic_habit}.
(In micrometer: \code{solid-column} [5.96, 84.22], \code{hollow-column} [4.97, 70.24],
 \code{rough-aggregate} [3.55, 108.10], \code{rosettes-4} [2.77, 45.30], 
\code{rosettes-6} [2.85, 46.01], \code{plate} [4.87, 48.18], \code{dendrites}
 [0.45, 1.88], \code{droxtal} [9.48, 293.32], 
\code{spheroid}  [6.58, 203.39]). 
\item[baum]
Use ice cloud parameterization from \citet{baum05a:_bulk,baum05b:_bulk},
http://www.ssec.wisc.edu/\~baum/Cirrus/IceCloudModels.html.
In combination with the radiative transfer solvers \code{cdisort}, \ifmystic{\code{montecarlo},}
 and \code{fdisort2}, accurate phase functions are used.
\item[baum\_hufit]
Similar to the option \code{baum} but here the phase function
is parameterized by 128 Legendre coefficients, calculated with the
delta-fit method from 
\citet{Hu2000}. This parameterization covers the region  
from 0.4 to 2.2 micrometer. If high accuracy is needed e.g. in the vicinity of the halo, 
the forward peak, or the backscatter peak, \code{ic_properties baum}
is recommended.
\item[hey] 
Use pre-calculated ice cloud optical properties including full phase
matrices. This option has newly been implemented and is not yet well
validated. Please check your results carefully!!
The parameterization
is currently only available for the spectral region from 0.2 to 5
micrometers. The single scattering
properties have been been generated by Hong Gang using the
models by \citet{Yang2000}. The parameterization is based on
simple gamma distributions 
\begin{equation}
n(r) = n_0 r^{\alpha} \exp\left(-\frac{(\alpha+3)r}{r_e}\right),
\end{equation}
where $n_0$ is found by normalization and $\alpha$ is set to 1. In case of
spherical particles the parameter $r_e$ would be the effective
radius. For aspherical particles, the parameter $r_e$ is found
iteratively so that the size distribution yields the required
effective radius. The
parameterization is availabe for the following habits: \code{solid-column},
\code{hollow-column}, \code{rough-aggregate}, \code{rosette-6},
\code{plate}, and \code{droxtal}. The
default habit is \code{solid-column}. The habit can be specified using the option
\code{ic_habit}.
\end{description}

The default property is \code{hu} in case of \code{name} "wc" and
\code{fu} in case of \code{name} "wc" and \code{fu}. Else the
property needs to be specified!

A note on ice particles:

Please note also that, in contrast to spherical particles, there is no unique 
definition of effective size for non-spherical particles. In particular, the
above parameterizations use different definitions which, however, differ only by 
a constant factor. 
\citet{Yang2000}, cite{Key2002}, and \citet{baum05a:_bulk,baum05b:_bulk}
 use the general definition 
\begin{equation}
   r_{\rm eff} = {{3}\over{4}}{{\int V(h) n(h) dh}\over{\int A(h) n(h) dh}}
\end{equation}
  where $h$ is the maximum dimension of an ice crystal, $n(h)$ is the
  number of particles with maximum dimension $h$ in the size distribution,
  and $V$ and $A$ are the volume and mean
  projected area of the particles, respectively. The volume and area are
  based on the spherical diameter with equivalent volume and the
  spherical diameter with equivalent projected area as defined by 
  \citet{Yang2000}. On the other hand, \citet{Fu1996,Fu1998} use
  hexagonal columns and use the following definition
\begin{equation}
   r_{\rm eff} =  {{\int D^2 L n(L) dL}\over{2 \int (D L + {\sqrt{3}\over{4}} D^2) n(L) dL}}
\end{equation}
  where $D$ is the width of the ice crystal (that is, the maximum diameter of the 
  hexagonal area) and $L$ is the length. The integrand in the numerator is proportional 
  to the volume while that in the denominator is proportional to the projected area. 
  Evaluating these formulas one finds that, for the same hexagonal particle, the effective
  radius would be $3 \sqrt{3} / 4 = 1.299$ times larger following the 
  \citet{Yang2000}, \citet{Key2002} definition rather than the \citet{Fu1996,Fu1998} definition.
  As an example, an effective radius of 20$\mu m$ with 
  \code{ic_properties fu} and \code{ic_fu_reff fu} and
  1.299 $\cdot$ 20$\mu m$ = 26$\mu m$ with \code{ic_properties yang} would give comparable results
  for hexagonal columns. 
To use the original definition of the effective radius by \citet{Fu1996,Fu1998} use
\code{ic_fu_reff fu}!

\option{profile_properties_interpolate}
Interpolate optical properties for profile \code{typename} over
wavelength; useful for precalculated optical property files defined
with \code{profile_properties}. Please note that this option may be
extremely memory-consuming because for each internal wavelength a full
set of Legendre moments of the phase function is stored (up to several
thousands).
\fcode{
profile_properties_interpolate typename
}

\option{profile_scale_gg}
Scale the asymmetry factor of profile \code{typename} for all wavelengths and altitudes
with a float between 0.0 and 1.0.
\fcode{
profile_scale_gg typename value
}
See \code{wc_scale_gg} for more explanations.

\option{profile_scale_ssa}
Scale the single scattering albedo of profile \code{typename} for all wavelengths and altitudes
with a float between 0.0 and 1.0.
\fcode{
profile_scale_ssa typename value
}
See \code{wc_scale_ssa} for more explanations.

\option{profile_set_gg}
Set the asymmetry factor of profile \code{typename} for all wavelengths and altitudes
to a float between -1.0 and 1.0. 
\fcode{
profile_set_gg typename value
}
See \code{wc_set_gg} for more explanations.

\option{profile_set_ssa}
Set the single scattering albedo of profile \code{typename} for all wavelengths
and altitudes to a float between 0.0 and 1.0. 
\fcode{
profile_set_ssa typename value
}
See \code{wc_set_ssa} for more explanations.

\option{profile_set_tau}
Set the total optical thickness of profile \code{typename} to a constant value for all wavelengths.
\fcode{
profile_set_tau typename value
}
See \code{wc_set_tau} for more explanations.

\option{profile_set_tau550}
Set the water optical thickness of profile \code{typename} at 550nm.
\fcode{
profile_set_tau550 typename value
}
See \code{wc_set_tau550} for more explanations.

\option{radiosonde}
This option allows to change the temperature and pressure profile, and optionally to 
specify one or more density profiles. The entry in the input file looks like this:
\fcode{
radiosonde filename [gas_species] [unit] ...
}
Currently the following gas\_species are included: ozone (O3), nitrogen dioxide (NO2), 
water vapor (H2O), bromine oxide (BRO), chlorine dioxide (OCLO), formaldehyde (HCHO), 
carbon dioxide (CO2), sulphur dioxide (SO2), and the oxygen dimer (O4). 
Each gas species is identified by its abbrevations given in parentheses above.
Unit is an optional argument to defines the unit of the density. The profiles can
be given in particles per cm$^3$ (CM-3), in particles per m$^3$ (M-3), as volume 
mixing ratio (VMR), as mass mixing ratio in kg/kg (MMR), or as relative humidity 
(RH) (only for water). The default unit is RH for water vapour, 
MMR for ozone, and CM3 for all other gases.
The radiosonde file must have (2 + number of gases) columns:
\begin{description}
\item[1] pressure in hPa
\item[2] temperature in Kelvin
\item[3, 4, ...] density of trace gas in the specified unit
\end{description}
A new z-grid will be calculated, starting at \code{altitude} and assuming a linear temperature variation
between levels. The air density will be recalculated according to the ideal gas law, and the density of 
the well mixed gases O2 and CO2 will be scaled accordingly.
The atmospheric data above the radiosonde data is taken from the \code{atmosphere_file} level by level, starting 
at the first pressure level above the radiosonde data. The z-grid of the \code{atmosphere file} in 
this height region is shifted accordingly.
Also if the density in the radiosonde file is specified as -1 at a level, 
the value from the \code{atmosphere_file} is used.
Possible calls are
\fcode{
radiosonde ../examples/radiosonde.dat
}
just in order to change the temperature and pressure profile, or
\fcode{
radiosonde ../examples/radiosonde2.dat H2O RH O3 MMR NO2
}
where water vapour density will be given as relative humidity, ozone as mass mixing ratio, 
and NO2 in cm$^{-3}$ (default).

\option{radiosonde_levels_only}
The atmosphere considered in the simulation has the same height range as the data in 
the \code{radiosonde}-file. No further levels are added above those.
This option has only an effect in combination with \code{radiosonde}.

%CE Commented for ESA version
\undocumented{
\option{refractive_index_file}
Undocumented option to read a refractive index profile. 
\fcode{
refractive_index_file file
}
Actually not the refractive index profile 
but refractive index - 1 must be provided for better accuracy. The file must have two columns: 
\code{altitude [km]   refractive_index-1}
}


\option{satellite_geometry}
With this option the satellite geometry is determinded. The argument for this option
\fcode{
satellite_geometry  netCDF_file 
}
is the location of a \code{netCDF_file}, which must contain latitude and longitude position
as well as zenith and azimuth viewing angle for each pixel.

%ak20100306 What is below does not belong in the general Users Guide yet.
%For format specification see the example file data/satellite/MSG\_seviri/MSG\_seviri\_geometry.nc.
%This option has to be used in combination with \code{satellite_pixel}.
%You can also use the abbreviation
%\code{satellite_geometry MSG}
%for MSG simulation, \emph{but in order to use this, you have to copy the netCDF file}
%\fcode{
%cp -r /data/A3/satellite_geometry/satellites libRadtran/data/
%}
%\emph{which is not in cvs version, as it is too large.}

\option{satellite_pixel}
This option specifies which pixel of the satellite image that should be simulated. 
\fcode{
satellite_pixel  pixel_x pixel_y
}
The arguments \code{pixel_nr_x} and \code{pixel_nr_y} specifies the pixel position in the native 
system of the satellite, which is determinded by the option \code{satellite_geometry}.

\option{sslidar_area}

Set area of single scattering lidar in units of square meters (solver
\code{sslidar}).

\option{sslidar_E0}

Set Laser pulse energy for single scattering lidar in units of
Joule (solver \code{sslidar}). (You can also use a \code{solar_file}
instead... not yet implemented.)

\option{sslidar_eff}

Set lidar efficiency for single scattering lidar (solver
\code{sslidar}).

\option{sslidar_nranges}

Set number of range bins for single scattering lidar (solver
\code{sslidar}).

\option{sslidar_polarisation}

Turn on polarisation measurement for lidar (solver \code{sslidar}).

\option{sslidar_position}

Set lidar position for single scattering lidar in units of
km (solver \code{sslidar}).

\option{sslidar_range}

Set lidar range bin width for single scattering lidar in units of
km (solver \code{sslidar}).

\option{surface_temperature_map}
Specify a surface\_temperature map with a \emph{netCDF} file which is used in combinition 
with the options \code{latitude}, \code{longitude}, and \code{time}.
\fcode{
surface_temperature_map file [variable_name]
}
where \code{file} is the location of the \emph{netCDF} file. 
\code{libRadtran} reads the value at the nearest pixel to the given \code{latitude} and 
\code{longitude}. No spatial interpolation or averaging of the values is done.

\option{surface_type_map}
Specify a surface type map, which is used in combinition with 
\code{albedo_library}, \code{latitude}, and \code{longitude} in order to 
select the surface type relevant for the simulation.
No pixel interpolation is done.
The format of the call is:
\fcode{
surface_type_map file [variable_name]
}
where \code{file} is the location of the surface type map file. The map is expected to 
be in \emph{netCDF} format. The file must contain the variables \code{double lat(nlat)}, 
\code{double lon(nlon)}, and \code{byte surface_type (nlat, nlon)}. If the name of the 
surface type variable is different, the optional argument can be used in order to specify 
the variable name. For format specification see also 
\file{data/albedo/IGBP_map/SURFACE_TYPE_IGBP_10min.cdf}.

For using the IGBP map, the call is \code{surface_type_map IGBP}. This map has a resolution of 10 
minutes and contains the surface types 1 to 18 defined in the \code{albedo_library IGBP}. 
Fresh snow and sea ice are not included, as their extent is too variable. Attention:  
That implies e.g. that the Arctic is considered ocean\_water and not sea\_ice!

Locations on the pixel boundaries are interpreted as the pixel northward and
eastward respectively. E.g. location 0 N, 0 E is interpreted like the pixel ranging from
0 to 10min North and from 0 to 10min East.

\option{thermal_bandwidth}
Specify a constant bandwidth in cm-1 for thermal calculations. 
\fcode{
thermal_bandwidth value
}
The default is 1 cm-1.
This option is ignored if used together with \code{correlated_k kato/kato2/kato2.96/fu/avhrr_kratz}.

\option{time}
Specifies the time to simulate.
\fcode{
time YYYY MM DD hh mm ss
}
where \code{YYYY} is the year, \code{MM} the month, \code{DD} the day, 
\code{hh} the hour, \code{mm} the minute, \code{ss} the second in UTC.
The time information will be used for a couple of things: 

\code{time} in combination with \code{latitude}, \code{longitude}, and any \code{map}-option
is used to select the location where to read the input data.

\code{time} is used to correct extraterrestrial 
irradiance for the Sun-Earth distance with the day of year. If not given, 
the Earth-Sun distance is 1 AU (i.e. equinox distance).

\code{time} in combination with \code{latitude} and \code{longitude} is used to calculate 
the solar zenith angle if no \code{sza} is specified.

\code{time} in combination with \code{latitude} and \code{longitude} is used to choose a
suitable default atmosphere file, if no \code{atmosphere_file} is specified.

\code{time} in combination with an \code{ECMWF_atmosphere_file} is used to choose
a date in the ECMWF input file. 

\option{time_interpolate}
If a map option is used in combination with \code{time}, the data, which is nearest to the 
specified \code{time} is used for the simulation. This means \code{time_interpolate} is switched 
off per default.

If this option is switched on, the data fields stored in the \emph{netCDF} files are interpolated 
to the specified \code{time}. (Be aware, that this might cause strange effects for data field 
of moving properties. E.g. an interpolated cloud field might have double horizontal extent, 
but only half the optical depth.)

\option{time_interval}
This option can be used in order to calculate an effective solar zenith angle 
for a time interval, instead of a distinct point in time. 
The cosine of the solar zenith angle is here replaced by its time average. 
The azimuth of the sun is replaced by an average of the azimuth position weighted 
with the cosine of the solar zenith angle.
\fcode{time_interval dtime_start dtime_end [unit]\\
time_interval    -180        180     min
}
the time interval reaches from \code{time} + \code{dtime_start} to 
\code{time} + \code{dtime_end}, in the example from 180 minutes before \code{time}
to 180 minutes after \code{time}. The \code{unit} argument is optional, and can be one of the 
following: \code{s} (seconds), \code{min} (minutes), or \code{h} (hour). The default is \code{s}.
This option makes only an effect in combination with \code{time}, \code{latitude}, \code{longitude}, 
and only has an effect for solar simulations (and of course if no \code{sza} defined).

\undocumented{
\option{tipa}

With the option \code{tipa dirdiff}, the tilted independent
pixel (column) approximation (TIPA, the same as TICA) is carried out,
where a 3D-cloud file serves as input and the columns are tilted
towards the sun (see, e.g., Wapler and Mayer, 2008).  \code{tipa
dirdiff} works with all 1D-rte-solvers.  When choosing
tipa (\code{tipa dir}), from the "tilted clouds" only the optical
depth for the calculation of the direct radiation is derived and used
later to calculate the diffuse radiation. To calculate the diffuse
radiation, the original cloud-matrix is still necessary /
maintained. \code{tipa dir} works only with Roberts (Buras)
delta--Eddington two--stream model \code{rodents}.  For both tipa
options, a \code{mc_sample_grid} has to be specified as Nx Ny dx dy. A
3d-cloud-file (with flag 3) serves as input (see
\code{mc_wcloud_file}). The output is written into mc.flx(.spc),
however, a basename via \code{mc_basename} can be given. \code{ipa_3d}
is turned on automatically when one of the tipa options is choosen.
}

\option{wc_saturate}
Obsolete option.

\iflidar{
\option{write_ext_to_file}
Option needed by ARLEM when calling uvspec. Writes extinction
information to file instead of to output.
}

\iflidar{
\option{write_output_as_netcdf} Option only works with
\code{mc_lidar}. Output is written into netcdf file rather than dozens
of ASCII files. Saves disk space.
}

\option{wvn}
Deprecated option. Same as \code{wavelength}.

%DOCUMENTATION END
\end{description} % @c11\_03@ */


/*
</lpdoc>
 */


  /***************** End of documentation *********************/

  int i=0;
  int brdftype=0;
  int status=0;
  int io=0;
  int check_input_file_only = 0;

#ifndef UVSPEC_FUNCTION
#ifdef COPYPROTECTION
  /* copy protection :-) */
  struct tm *tmstruct = calloc (1, sizeof(struct tm));
  time_t t=0;
#endif
#endif

#if HAVE_LIBNETCDF
  int ncid=0;
  int id_var_test=0;
#endif

  char infilename[FILENAME_MAX]="";


  char randomseedfilename [FILENAME_MAX] = "randomseed";
  FILE *randomseedfile=NULL;
#ifdef WRITERANDOMSTATUS
  FILE *randomstatusfile=NULL;
#endif
  int rseed=0;

  char tmp_filename [FILENAME_MAX] = "";
  FILE *f=NULL;

  /* char base[255]=""; */

  int time_UTC_in_s = NOT_DEFINED_INTEGER;
  int time_LAT_in_s = NOT_DEFINED_INTEGER;

  
#ifndef UVSPEC_FUNCTION
  int c=0;
#endif

  char function_name[]="main";
  char file_name[]="uvspec_lex.l";

  /* Initialization */
  
  /* allocate memory for filenames */

  Input.filename         = calloc (FN_NN,     sizeof(char *));
  Input.aer.filename     = calloc (FN_AER_NN, sizeof(char *));
  Input.atm.filename     = calloc (MOL_NN,    sizeof(char *));
  Input.rte.mc.filename  = calloc (FN_MC_NN,  sizeof(char *));
  Output.crs.filename    = calloc (MOL_NN,    sizeof(char *));

  for (i=0; i<FN_NN; i++)
    Input.filename[i]        = calloc (FILENAME_MAX+1, sizeof(char));
  
  for (i=0; i<FN_AER_NN; i++) 
    Input.aer.filename[i]    = calloc (FILENAME_MAX+1, sizeof(char));

  for (i=0; i<MOL_NN; i++) 
    Input.atm.filename[i]    = calloc (FILENAME_MAX+1, sizeof(char));
 
  for (i=0; i<FN_MC_NN; i++)
    Input.rte.mc.filename[i] = calloc (FILENAME_MAX+1, sizeof(char));

  for (i=0; i<MOL_NN; i++) 
    Output.crs.filename[i]   = calloc (FILENAME_MAX+1, sizeof(char));

  s = NULL; 
  s = getenv("LIBRADTRAN_DATA_FILES");
  if ( s == NULL ) { s = "../data/"; }
  strcpy (Input.filename[FN_PATH], s);
  
  if (Input.filename[FN_PATH][strlen(Input.filename[FN_PATH])-1]!='/') {
    strcat(Input.filename[FN_PATH],"/");
  }

  Input.quiet             = 0;
  Input.verbose           = 0;
  Input.output_format     = OUTPUT_FORMAT_ASCII;
  Input.output_format_string=NULL;
  Input.header            = 0;
  Input.spectrum_unit     = UNIT_NOT_DEFINED;        /* default: unit of solar spectrum is unknown */
  Input.output_unit       = UNIT_NOT_DEFINED;        /* default: output units are unknown, same as unit of solar spectrum */
  Input.bandwidth         = 1.0;                     /* default: bandwidth in cm-1 (thermal) or 1nm (solar) */
  Input.bandwidth_unit    = UNIT_NOT_DEFINED;        /* cm-1 for thermal, nm for solar */
  Output.print_phi        = 1;                       /* default: print radiances   */
  Input.ck_scheme         = CK_NONE;
  Input.ck_h2ocont        = 1;                       /* default is h2o-continuum is on, version 2.1 */
  Input.ck_o4abs          = 1;
  Input.ck_n2abs          = 1;
  Input.ck_coabs          = 1;
  Input.ck_so2abs         = 1;
  Input.ck_nh3abs         = 1;
  Input.ck_noabs          = 1;
  Input.ck_hno3abs        = 1;

  Input.n_caoth           = 0;
  Input.i_ic              = -1;
  Input.i_wc              = -1;
  
  Input.cloud_overlap     = CLOUD_OVERLAP_OFF;
  Output.cf.cf_total      = NOT_DEFINED_FLOAT;
  Output.cf.nlev          = 0; 

  Input.ipa3d             = 0;
  Input.tipa              = 0;
  Input.rte.mc.tipa       = 0;
  Input.ozone_spectrum    = O3_MOLINA;
  Input.no2_spectrum      = NO2_BREMEN;
  Input.so2_spectrum      = SO2_BREMEN;
  Input.co2_spectrum      = CO2_YOSHINO;
  Input.o2_spectrum       = O2_DEFAULT;
  Input.no2_spectrum      = NO2_GOME;
  Input.bro_spectrum      = BRO_IASB;
  Input.oclo_spectrum     = OCLO_WAHNER;
  Input.hcho_spectrum     = HCHO_IASB;
  Input.o4_spectrum       = O4_GREENBLATT;

  Input.UTC.tm_year       = NOT_DEFINED_INTEGER;
  Input.UTC.tm_mon        = NOT_DEFINED_INTEGER;
  Input.UTC.tm_mday       = NOT_DEFINED_INTEGER;
  Input.UTC.tm_hour       = NOT_DEFINED_INTEGER;
  Input.UTC.tm_min        = NOT_DEFINED_INTEGER;
  Input.UTC.tm_sec        = NOT_DEFINED_FLOAT;

  Input.LAT.tm_yday       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_year       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_mon        = NOT_DEFINED_INTEGER;
  Input.LAT.tm_mday       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_hour       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_min        = NOT_DEFINED_INTEGER;
  Input.LAT.tm_sec        = NOT_DEFINED_FLOAT;

  Input.raman               = RAMAN_NONE;
  Input.raman_fast          = 0;
  Input.raman_original      = 0;
  Input.delta_wvl_raman  = -1;
  Input.rayleigh_crs        = RAYLEIGH_BODHAINE;

  Input.aer.standard      = 0;
  strcpy(Input.aer.filename[FN_AER_SPECIES_LIB],"OPAC");  /* default aerosol library   */


  Input.latitude          = NOT_DEFINED_FLOAT; 
  Input.longitude         = NOT_DEFINED_FLOAT; 
  Input.sat_pixel_x       = NOT_DEFINED_FLOAT;
  Input.sat_pixel_y       = NOT_DEFINED_FLOAT;

  Input.alb.surface       = NOT_DEFINED_INTEGER;    /* there is no default */
  Input.alb.scale_factor  = 1.0;
  Input.alb.netCDF_alb_name  = calloc (FILENAME_MAX+1, sizeof(char));
  Input.alb.netCDF_surf_name = calloc (FILENAME_MAX+1, sizeof(char));

  Input.flu.source        = NOT_DEFINED_INTEGER;    /* first of all, no defined albedo, default setting see below */
  Input.flu.fluorescence  = 0.0;

  Input.alt.source        = ALT_NOT_DEFINED;
  Input.alt.scale_factor  = 1.0;
  Input.alt.netCDF_alt_name  = calloc (FILENAME_MAX+1, sizeof(char));

  Input.netCDF_name_surf_T = calloc (FILENAME_MAX+1, sizeof(char));

  Input.atm.rs_source     = RS_NO_DATA;                /* default no radiosonde-file */
  Input.atm.nz_atm_forced_sea = NOT_DEFINED_INTEGER;   /* no forced atmosphere level */

  Input.delta_time_start  = 0.0;  /* no time period for averaging */
  Input.delta_time_end    = 0.0;  /* no time period for averaging */
  Input.atm.sza_spher     = 0.0;
  Input.atm.phi0_spher    = 0.0;

  Input.atm.nzout         = NOT_DEFINED_INTEGER;
  Input.atm.zout_source   = OUTLEVEL_ZOUT_ABOVE_SUR;
  Output.atm.nz_model_level = NOT_DEFINED_INTEGER;
  Output.atm.nz_model_layer = NOT_DEFINED_INTEGER;
  Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX; /* alternative default: Z_INTERPOLATE_LOGLIN */
  Input.atm.interpol_method_press  = NOT_DEFINED_INTEGER;
  Input.atm.interpol_method_temper = NOT_DEFINED_INTEGER;
  Input.atm.interpol_method_refind = NOT_DEFINED_INTEGER;   
  Input.atm.interpol_method_gas = calloc(MOL_NN, sizeof(int));
  Input.atm.well_mixed_gas      = calloc(MOL_NN, sizeof(int));
  for (i=0; i<MOL_NN; i++) {
    Input.atm.interpol_method_gas[i] = NOT_DEFINED_INTEGER;
    Input.atm.well_mixed_gas     [i] = NOT_DEFINED_INTEGER;
  }
  Input.atm.ECMWF_new_format = NOT_DEFINED_INTEGER;
  Input.atm.ECMWF_ozone_climatology = FALSE;
  Input.atm.time_interpolate = TIME_NEAREST_DATE;  
  Output.atm.microphys.z_cpt_sea  = NOT_DEFINED_FLOAT; /* cold point tropopause height      */ 
  Output.atm.microphys.temper_cpt = NOT_DEFINED_FLOAT; /* cold point tropopause temperature */ 
  Output.atm.microphys.press_cpt  = NOT_DEFINED_FLOAT; /* cold point tropopause pressure    */ 

  Input.cm.pcl_netCDF_name  = calloc (FILENAME_MAX+1, sizeof(char));
  Input.cm.pcl_scale_factor = 1.0;
  Input.cm.sal_netCDF_name  = calloc (FILENAME_MAX+1, sizeof(char));
  Input.cm.sal_scale_factor = 1.0;

  Input.bpdf.type           = BPDF_NONE; 
  Input.bpdf.u10            = NOT_DEFINED_FLOAT;

  Input.rte.disort_icm    = DISORT_ICM_PHASE;
  Input.rte.cdisort_pseudospherical = 0;
  Input.rte.ctwostr_pseudospherical = 0;
  Input.rte.ibcnd         = 0; 
  Input.rte.ichap         = 1;
  Input.rte.maxphi        = 0;
  Input.rte.solver        = CDISORT;
  Input.rte.pol_stokes[0] = 'I';
  Input.rte.pol_stokes[1] = 'Q';
  Input.rte.pol_stokes[2] = 'U';
  Input.rte.pol_stokes[3] = 'V';

  Input.rte.mc.abs_unit      = MCABS_UNIT_W_PER_SQUARE_M_AND_DZ;
  Input.rte.mc.photons       = 100000;
  Input.rte.mc.Nx_sample     = 0;
  Input.rte.mc.Ny_sample     = 0;
  Input.rte.mc.jacobian      = 0;
  Input.rte.mc.jacobian_std  = 0;
  Input.rte.mc.delta_scaling_mucut = NOT_DEFINED_INTEGER; /* TZ ds */
  Input.rte.mc.delta_scaling_start = -1;
  Input.rte.mc.truncate            = NOT_DEFINED_INTEGER;
  Input.rte.mc.bcond         = MCBCOND_PERIODIC;
  Input.rte.mc.reflectalways = 0;
  Input.rte.mc.refraction    = 0;
  Input.rte.mc.ris_factor    = 1.0;
  Input.rte.mc.ris_optical_depth = 1.0;
  Input.rte.mc.spectral_is   = 0;
  Input.rte.mc.spherical     = 0;
  Input.rte.mc.spherical3D   = 0;
  Input.rte.mc.spherical3D_scene         = 0;
  Input.rte.mc.spherical3D_scene_lon_min = 0;
  Input.rte.mc.spherical3D_scene_lon_max = 0;
  Input.rte.mc.spherical3D_scene_lat_min = 0;
  Input.rte.mc.spherical3D_scene_lat_max = 0;
  Input.rte.mc.std           = 0;
  Input.rte.mc.randomseed    = NOT_DEFINED_INTEGER;
  Input.rte.mc.readrandomseed   = 0;
  Input.rte.mc.readrandomstatus = 0;
  Input.rte.mc.escape = 1;
  Input.rte.mc.vroom  = -1;
  Input.rte.mc.sensorposition     = MC_SENSORPOSITION_NONE;
  Input.rte.mc.sensorposition_x   = 0;
  Input.rte.mc.sensorposition_y   = 0;
  Input.rte.mc.sensorposition_z   = 0;
  Input.rte.mc.sensorposition_lon = 0;
  Input.rte.mc.sensorposition_lat = 0;
  Input.rte.mc.sensorposition_alt = 0;
  
  Input.rte.mc.panorama = PAN_MODE_NONE;
  Input.rte.mc.pan_alignment = PAN_ALIGNMENT_NONE;
  Input.rte.mc.pan_no_pixel = 0;
  Input.rte.mc.pan_quicklook = 0;
  Input.rte.mc.pan_distr_photons_over_pixel = 0;
  Input.rte.mc.pan_with_direct_rad = 0;
  Input.rte.mc.polarisation = 0;
  Input.rte.mc.nstokes = 1;
  Input.rte.mc.reference_to_NN = 0;

  Input.rte.mc.locest        = 0;

  Input.rte.mc.blitz_position = NULL;

  Input.rte.mc.backward.output = MCBACKWARD_EDN;
  Input.rte.mc.backward.islower = -999;
  Input.rte.mc.backward.jslower = -999;
  Input.rte.mc.backward.isupper = -999;
  Input.rte.mc.backward.jsupper = -999;
  
  Input.rte.mc.backward.absorption = 0;

  Input.optimize_fortran  = FALSE;  /* no verbose output for FORTRAN optimization */
  Input.optimize_delta    = 0;      /* no security levels */

  Input.write_ext_to_file = 0;
  Input.write_output_as_netcdf = 0;

  Input.rte.nphi          = 0;
  Input.rte.numu          = 0;
  Input.rte.nprndis       = 0;
  Input.rte.deltam        = 1;
  /* Define polradtran defaults for polarized radiance simulations, this is 
  the standard application for the polradtran solver */ 
  Input.rte.pol_quad_type = "G";
  Input.mixing_ratio = calloc(MX_NN, sizeof(float));
  for (i=0; i<MX_NN; i++)
    Input.mixing_ratio[i] = NOT_DEFINED_INTEGER;
  Input.atm.column       = calloc(MOL_NN, sizeof(float));
  for (i=0; i<MOL_NN; i++) 
    Input.atm.column[i] = NOT_DEFINED_INTEGER;
  Input.atm.unit_column           = calloc(MOL_NN, sizeof(int));
  Input.atm.unit_column[MOL_AIR]  = CM_2;
  Input.atm.unit_column[MOL_O3]   = DU;
  Input.atm.unit_column[MOL_O2]   = CM_2;
  Input.atm.unit_column[MOL_H2O]  = CM_2;
  Input.atm.unit_column[MOL_CO2]  = CM_2;
  Input.atm.unit_column[MOL_NO2]  = CM_2;
  Input.atm.unit_column[MOL_BRO]  = CM_2;
  Input.atm.unit_column[MOL_OCLO] = CM_2;
  Input.atm.unit_column[MOL_HCHO] = CM_2;
  Input.atm.unit_column[MOL_O4]   = CM_2;
  Input.atm.unit_column[MOL_SO2]  = CM_2;
  Input.atm.unit_profile           = calloc(MOL_NN, sizeof(int));
  Input.atm.unit_profile[MOL_AIR]  = CM_3;
  Input.atm.unit_profile[MOL_O3]   = CM_3;
  Input.atm.unit_profile[MOL_O2]   = CM_3;
  Input.atm.unit_profile[MOL_CO2]  = CM_3;
  Input.atm.unit_profile[MOL_NO2]  = CM_3;
  Input.atm.unit_profile[MOL_BRO]  = CM_3;
  Input.atm.unit_profile[MOL_OCLO] = CM_3;
  Input.atm.unit_profile[MOL_HCHO] = CM_3;
  Input.atm.unit_profile[MOL_O4]   = CM_3;
  Input.atm.unit_profile[MOL_SO2]  = CM_3;
  Input.atm.well_mixed_gas [MOL_AIR]  = YES;
  Input.atm.well_mixed_gas [MOL_O3]   = NO;
  Input.atm.well_mixed_gas [MOL_O2]   = YES;
  Input.atm.well_mixed_gas [MOL_H2O]  = NO;
  Input.atm.well_mixed_gas [MOL_CO2]  = YES;
  Input.atm.well_mixed_gas [MOL_NO2]  = NO;
  Input.atm.well_mixed_gas [MOL_BRO]  = NO;
  Input.atm.well_mixed_gas [MOL_OCLO] = NO;
  Input.atm.well_mixed_gas [MOL_HCHO] = NO;
  Input.atm.well_mixed_gas [MOL_O4]   = NO;
  Input.atm.well_mixed_gas [MOL_SO2]  = NO;
  Input.atm.n_rs_gas = 0;
  Input.atm.rs_gas  = calloc(MOL_NN, sizeof(int));
  Input.atm.rs_unit = calloc(MOL_NN, sizeof(int));
  Input.atm.rs_unit[MOL_AIR]  = CM_3;
  Input.atm.rs_unit[MOL_O3]   = MMR;
  Input.atm.rs_unit[MOL_O2]   = CM_3;
  Input.atm.rs_unit[MOL_H2O]  = RH;
  Input.atm.rs_unit[MOL_CO2]  = CM_3;
  Input.atm.rs_unit[MOL_NO2]  = CM_3;
  Input.atm.rs_unit[MOL_BRO]  = CM_3;
  Input.atm.rs_unit[MOL_OCLO] = CM_3;
  Input.atm.rs_unit[MOL_HCHO] = CM_3;
  Input.atm.rs_unit[MOL_O4]   = CM_3;
  Input.atm.rs_unit[MOL_SO2]  = CM_3;
  Input.atm.rs_add_upper_levels = TRUE;
  Input.atm.mol_mass = calloc(MOL_NN, sizeof(float));
  Input.atm.mol_mass[MOL_AIR]  = mol_mass_air; /* 28.977  from function ck.c                    in g/MOL */
  Input.atm.mol_mass[MOL_O3]   = 48.000;       /*         from function ck.c                    in g/MOL */
  Input.atm.mol_mass[MOL_O2]   = 31.998609;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_H2O]  = mol_mass_wv;  /* 18.015  from function ck.c                    in g/MOL */
  Input.atm.mol_mass[MOL_CO2]  = 44.009646;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_NO2]  = 46.005333;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_BRO]  = 95.902831;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_OCLO] = 67.451347;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_HCHO] = 30.026294;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_O4]   = 63.979110;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_SO2]  = 64.062998;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */

  Input.r_earth = 6370.0;

  Input.processing         = PROCESS_NONE;
  Input.n_output_user      = NOT_DEFINED_INTEGER;
  Input.output_user_flag   = 0;

  Input.sslidar_E0=0.1; /* Joule */
  Input.sslidar_position=0.0; /* km */
  Input.sslidar_nranges=100;
  Input.sslidar_range=0.1; /* km */
  Input.sslidar_eff=0.5;
  Input.sslidar_area=1.0; /* m^2 */
  Input.sslidar_polarisation=0;


  /*PYTHON INITIALISATIONS*/




  
#ifndef UVSPEC_FUNCTION
  while ((c=getopt (argc, argv, "hvcf:i:")) != EOF)  {
    switch(c)  {
    case 'h': 
      usage();
      return (0);
      break;
    case 'c': 
      check_input_file_only = TRUE;
      break;
    case 'f':
    case 'i':
      strcpy (infilename, optarg);
      break;
    case 'v': 
      version();
      return (0);
      break;
    default:
      usage();
      return (0);
    }
  }
#endif

  /* copy protection :-) */
#ifndef UVSPEC_FUNCTION
#ifdef COPYPROTECTION
  tmstruct->tm_mday=1;   /* day of the month 1-31      */
  tmstruct->tm_mon=3;    /* months since January 0-11  */
  tmstruct->tm_year=109; /* years since 1900           */
  
  t=mktime (tmstruct);
 
  fprintf (stderr, "%s\n", asctime (tmstruct));

  if ((int) time(NULL) > t) {
    remove(argv[0]);
    return -1;
  }
#endif  
#endif  

  /***********************/
  /* Read the input file */
  /***********************/

  if (strlen(infilename)>0)
    yyin = fopen (infilename, "r");

  yylex();

  if (ierror!=0)  {
    fprintf (stderr, "Error in input file! Please check your input file and retry!\n");
    return -1;
  }
  /* Everything is fine ininput file. Thus exit if only checking file */
  if (check_input_file_only)  { return 0; }

  if (strlen(infilename)>0)
    fclose (yyin);


  /*****************************************/
  /* check combinition of parameters       */
  /* and do input dependent initialisation */
  /*****************************************/

  /*initializing Input.rte.solver*/
  if (Input.rte.solver_name != NULL) {
     if      (strncasecmp(Input.rte.solver_name,"sdisort",      7 ) == 0 && strlen(Input.rte.solver_name) ==  7 )  Input.rte.solver = SDISORT;
     else if (strncasecmp(Input.rte.solver_name,"spsdisort",    9 ) == 0 && strlen(Input.rte.solver_name) ==  9 )  Input.rte.solver = SPSDISORT;
     else if (strncasecmp(Input.rte.solver_name,"fdisort1",     8 ) == 0 && strlen(Input.rte.solver_name) ==  8 )  Input.rte.solver = DISORT;
     else if (strncasecmp(Input.rte.solver_name,"fdisort2",     8 ) == 0 && strlen(Input.rte.solver_name) ==  8 )  Input.rte.solver = DISORT2;
     else if (strncasecmp(Input.rte.solver_name,"cdisort",      7 ) == 0 && strlen(Input.rte.solver_name) ==  7 )  Input.rte.solver = CDISORT;
     else if (strncasecmp(Input.rte.solver_name,"disort2",      7 ) == 0 && strlen(Input.rte.solver_name) ==  7 )  Input.rte.solver = CDISORT;
     else if (strncasecmp(Input.rte.solver_name,"disort",       6 ) == 0 && strlen(Input.rte.solver_name) ==  6 )  Input.rte.solver = CDISORT;
     else if (strncasecmp(Input.rte.solver_name,"sos",          3 ) == 0 && strlen(Input.rte.solver_name) ==  3 )  Input.rte.solver = SUCORD;
     else if (strncasecmp(Input.rte.solver_name,"polradtran",  10 ) == 0 && strlen(Input.rte.solver_name) == 10 )  Input.rte.solver = POLRADTRAN;
     else if (strncasecmp(Input.rte.solver_name,"twostrpp",     8 ) == 0 && strlen(Input.rte.solver_name) ==  8 )  Input.rte.solver = TWOSTRPP;
     else if (strncasecmp(Input.rte.solver_name,"twostreampp", 11 ) == 0 && strlen(Input.rte.solver_name) == 11 )  Input.rte.solver = TWOSTRPP;
     else if (strncasecmp(Input.rte.solver_name,"twostream",    9 ) == 0 && strlen(Input.rte.solver_name) ==  9 )  Input.rte.solver = TWOSTR;
     else if (strncasecmp(Input.rte.solver_name,"twostr",       6 ) == 0 && strlen(Input.rte.solver_name) ==  6 )  Input.rte.solver = TWOSTR;
     else if (strncasecmp(Input.rte.solver_name,"ctwostr",      7 ) == 0 && strlen(Input.rte.solver_name) ==  7 )  Input.rte.solver = CTWOSTR;
     else if (strncasecmp(Input.rte.solver_name,"montecarlo",  10 ) == 0 && strlen(Input.rte.solver_name) == 10 )  Input.rte.solver = MONTECARLO;
     else if (strncasecmp(Input.rte.solver_name,"mystic",       6 ) == 0 && strlen(Input.rte.solver_name) ==  6 )  Input.rte.solver = MONTECARLO;
     else if (strncasecmp(Input.rte.solver_name,"tzs",          3 ) == 0 && strlen(Input.rte.solver_name) ==  3 )  Input.rte.solver = TZS;
     else if (strncasecmp(Input.rte.solver_name,"sssi",         4 ) == 0 && strlen(Input.rte.solver_name) ==  4 )  Input.rte.solver = SSSI;
     else if (strncasecmp(Input.rte.solver_name,"sss",          3 ) == 0 && strlen(Input.rte.solver_name) ==  3 )  Input.rte.solver = SSS;
     else if (strncasecmp(Input.rte.solver_name,"rodents",      7 ) == 0 && strlen(Input.rte.solver_name) ==  7 )  Input.rte.solver = RODENTS;
     else if (strncasecmp(Input.rte.solver_name,"twostrebe",    9 ) == 0 && strlen(Input.rte.solver_name) ==  9 )  Input.rte.solver = TWOSTREBE;
     else if (strncasecmp(Input.rte.solver_name,"sslidar",      7 ) == 0 && strlen(Input.rte.solver_name) ==  7 )  Input.rte.solver = SSLIDAR;
 
     else if (strncasecmp(Input.rte.solver_name,"null",         4 ) == 0 && strlen(Input.rte.solver_name) ==  4 )  Input.rte.solver = NULLSOLVER;
     else {fprintf(stderr,"Unknown 'rte_solver %s'\n", Input.rte.solver_name);
           ierror++;}
     free(Input.rte.solver_name);
  }

  /*initialize Input.heating*/
  if (Input.heating_string != NULL ) {
    if      (strncasecmp(Input.heating_string,"none",4)==0)           Input.heating = NO_HEAT;
    else if (strncasecmp(Input.heating_string,"local",5)==0)          Input.heating = HEAT_LOCAL;
    else if (strncasecmp(Input.heating_string,"layer_fd",8)==0)       Input.heating = HEAT_LAYER_FD;
    else if (strncasecmp(Input.heating_string,"layer_cd",8)==0)       Input.heating = HEAT_LAYER_CD;
    else {fprintf(stderr,"Unknown 'heating %s'. Use 'heating layer_cd' instead.\n", Input.heating_string);
      ierror++;}
    free(Input.heating_string);
  }

  /* initialize wavelength end if not specified */
  if ( Input.wl.end < 0 && Input.wl.start > 0 )
	Input.wl.end = Input.wl.start;
  if ( Input.wl.end_index < 0 && Input.wl.start_index > 0 )
	Input.wl.end_index = Input.wl.start_index;

  /* initialization for solar_file optional argument*/ 
  if (Input.solar_file_spectrum_unit != NULL) {
	if (strncasecmp("per_nm", Input.solar_file_spectrum_unit, 6) == 0) Input.spectrum_unit= UNIT_PER_NM;
	else if (strncasecmp("per_cm-1" , Input.solar_file_spectrum_unit, 8) == 0) Input.spectrum_unit= UNIT_PER_CMm1;
	else if (strncasecmp("per_ck_band", Input.solar_file_spectrum_unit,11) == 0) Input.spectrum_unit= UNIT_PER_CK_BAND;
	else {
		fprintf(stderr,"Unknown optional argument for 'solar_file' %s\n", Input.solar_file_spectrum_unit);
		ierror++; }
	free(Input.solar_file_spectrum_unit);
  }

  /*initialization of Input_source*/
  if ( Input.source_name != NULL ) {
     if      (strncasecmp(Input.source_name,"solar",5)  ==0)  Input.source = SRC_SOLAR;
     else if (strncasecmp(Input.source_name,"thermal",7)==0)  Input.source = SRC_THERMAL;
     else {
	fprintf(stderr,"Unknown 'source %s'\n", Input.source_name);
        ierror++;}
     free(Input.source_name);
  }

  /* initialization of Input.tre.deltam*/
  if ( Input.rte.deltam_switch != NULL ) {
     if (strncasecmp("on", Input.rte.deltam_switch,2)==0) {Input.rte.deltam = 1;}
     else if (strncasecmp("off", Input.rte.deltam_switch,3)==0) {Input.rte.deltam = 0;}
     else {fprintf(stderr,"Unknown 'deltam %s'\n", Input.rte.deltam_switch); ierror++;}
     free(Input.rte.deltam_switch);
  }

  /* initialization of correlated-k scheme */
  if ( Input.ck_scheme_name != NULL ) {
     if      (strlen(Input.ck_scheme_name)==8 && strncasecmp("kato2.96",Input.ck_scheme_name,8)==0) {Input.ck_scheme = CK_KATO2_96;}
     else if (strlen(Input.ck_scheme_name)==5 && strncasecmp("kato2",Input.ck_scheme_name,5)==0)    {Input.ck_scheme = CK_KATO2;}
     else if (strlen(Input.ck_scheme_name)==4 && strncasecmp("kato",Input.ck_scheme_name,4)==0)     {Input.ck_scheme = CK_KATO;}
     else if (strncasecmp("fu",Input.ck_scheme_name,2)==0)                       {Input.ck_scheme = CK_FU;}
     else if (strncasecmp("avhrr_kratz",Input.ck_scheme_name,11)==0)             {Input.ck_scheme = CK_AVHRR_KRATZ;}
     else if (strncasecmp("sbdart",Input.ck_scheme_name,6)==0 || strncasecmp("lowtran",Input.ck_scheme_name,7)==0) {
        Input.ck_scheme = CK_LOWTRAN;}
     else {Input.ck_scheme = CK_GENERIC; 
        strcpy (Input.filename[FN_CK_GENERIC], Input.ck_scheme_name);}
  }

  /* initialization of ozone_spectrum */
  if (Input.ozone_spectrum_type != NULL) {
     if (strncasecmp("Bass_and_Paur", Input.ozone_spectrum_type,13)==0) {Input.ozone_spectrum = O3_BASSPAUR;}
     else if (strncasecmp("Molina", Input.ozone_spectrum_type,6)==0)    {Input.ozone_spectrum = O3_MOLINA;}
     else if (strncasecmp("Daumont", Input.ozone_spectrum_type,7)==0)   {Input.ozone_spectrum = O3_DAUMONT;}
     else if (strncasecmp("Bogumil", Input.ozone_spectrum_type,7)==0)   {Input.ozone_spectrum = O3_BREMEN;}
     else {
	fprintf(stderr,"Unknown 'o3_crs %s'\n", Input.ozone_spectrum_type);
     	ierror++;}
     free(Input.ozone_spectrum_type);
  }

  /* initialization of Rayleigh_spectrum */
  if (Input.rayleigh_crs_type != NULL) {
     if      (strncasecmp("Bodhaine", Input.rayleigh_crs_type,8)==0)  Input.rayleigh_crs = RAYLEIGH_BODHAINE;
     else if (strncasecmp("Nicolet", Input.rayleigh_crs_type,7)==0)   Input.rayleigh_crs = RAYLEIGH_NICOLET;
     else if (strncasecmp("Penndorf", Input.rayleigh_crs_type,8)==0)  Input.rayleigh_crs = RAYLEIGH_PENNDORF;
     else {
	fprintf(stderr,"Unknown 'rayleigh_crs %s'\n", Input.rayleigh_crs_type);
	ierror++;}
     free(Input.rayleigh_crs_type);
  }

  /* initialization for albedo_library option */
  if (Input.alb.library != NULL) {
    if (strncasecmp("IGBP", Input.alb.library, 4)==0) {Input.alb.source = ALBEDO_IGBP_LIBRARY;}
    else {
         Input.alb.source = ALBEDO_USER_LIBRARY;
         yytextCstring (Input.filename[FN_ALBEDO_LIB_PATH],Input.alb.library, 2, 2);
         /* append slash */
         if (Input.filename[FN_ALBEDO_LIB_PATH][strlen(Input.filename[FN_ALBEDO_LIB_PATH])-1]!= '/')
	     strcat (Input.filename[FN_ALBEDO_LIB_PATH], "/");
    }
    free(Input.alb.library);
  }
  /* Initialize Input.processing or Input.Output_unit from option output{WORD} */
  if ( Input.output_unit_processing != NULL) {
     if      (strncasecmp(Input.output_unit_processing,"integrate",9) == 0) Input.processing = PROCESS_INT;
     else if (strncasecmp(Input.output_unit_processing,"sum",3)       == 0) Input.processing = PROCESS_SUM;
     else if (strncasecmp(Input.output_unit_processing,"rgbnorm",7)   == 0) Input.processing = PROCESS_RGBNORM;
     else if (strncasecmp(Input.output_unit_processing,"rgb_norm",8)  == 0) Input.processing = PROCESS_RGBNORM;
     else if (strncasecmp(Input.output_unit_processing,"rgb",3)       == 0) Input.processing = PROCESS_RGB;
     else if (strncasecmp(Input.output_unit_processing,"per_nm",6)    == 0)    Input.output_unit= UNIT_PER_NM;
     else if (strncasecmp(Input.output_unit_processing,"per_cm-1",8)  == 0)    Input.output_unit= UNIT_PER_CMm1;
     else if (strncasecmp(Input.output_unit_processing,"per_ck_band",11) == 0) Input.output_unit= UNIT_PER_CK_BAND;
     else {fprintf(stderr,"Unknown 'output %s'\n", Input.output_unit_processing);
         ierror++;}
     free(Input.output_unit_processing);
  }

  /* initialization for albedo_library option */
  if (Input.rpv.library != NULL) {
    if (strncasecmp("IGBP", Input.rpv.library, 4)==0) {Input.rpv.source = RPV_IGBP_LIBRARY;}
    else {
         Input.rpv.source = RPV_USER_LIBRARY;
         yytextCstring (Input.filename[FN_RPV_LIB_PATH],yytext, 1+1, ntokens+1);
         /* append slash */
         if (Input.filename[FN_RPV_LIB_PATH][strlen(Input.filename[FN_RPV_LIB_PATH])-1]!= '/')
	       strcat (Input.filename[FN_RPV_LIB_PATH], "/");
    }
    free(Input.rpv.library);
  }

  /*Initializing Monte Carlo options */

  /* initializing Input.rte.mc.abs_unit from option mc_absorption */
  if ( Input.rte.mc.abs_unit_string != NULL ) {
    if      (strcmp(Input.rte.mc.abs_unit_string, "W_per_m2_and_dz") == 0) Input.rte.mc.abs_unit = MCABS_UNIT_W_PER_SQUARE_M_AND_DZ;
    else if (strcmp(Input.rte.mc.abs_unit_string, "W_per_m3")        == 0) Input.rte.mc.abs_unit = MCABS_UNIT_W_PER_CUBIC_M;
    else if (strcmp(Input.rte.mc.abs_unit_string, "K_per_day")       == 0) Input.rte.mc.abs_unit = MCABS_UNIT_K_PER_DAY;
    else {
       fprintf(stderr,"Error, unknown 'mc_absorption %s'\n", Input.rte.mc.abs_unit_string); 
       ierror++;
    }
    free(Input.rte.mc.abs_unit_string);
  }

  /*initialize Input.rte.mc.backward.output from option mc_backward_output */
  if ( Input.rte.mc.backward.output_string != NULL ) {
     if	    (strncasecmp(Input.rte.mc.backward.output_string, "edir",4)==0) Input.rte.mc.backward.output = MCBACKWARD_EDIR;
     else if(strncasecmp(Input.rte.mc.backward.output_string, "edn",3) ==0) Input.rte.mc.backward.output = MCBACKWARD_EDN;
     else if(strncasecmp(Input.rte.mc.backward.output_string, "eup",3) ==0) Input.rte.mc.backward.output = MCBACKWARD_EUP;
     else if(strncasecmp(Input.rte.mc.backward.output_string, "fdir",4)==0) Input.rte.mc.backward.output = MCBACKWARD_FDIR;
     else if(strncasecmp(Input.rte.mc.backward.output_string, "fdn",3) ==0) Input.rte.mc.backward.output = MCBACKWARD_FDN;
     else if(strncasecmp(Input.rte.mc.backward.output_string, "fup",3) ==0) Input.rte.mc.backward.output = MCBACKWARD_FUP;
     else if(strncasecmp(Input.rte.mc.backward.output_string, "f",1)==0) {
      Input.rte.mc.backward.output = MCBACKWARD_ACT;
      Input.rte.mc.backward.absorption = 1;
     }
     else if(strncasecmp(Input.rte.mc.backward.output_string, "abs",3)==0) {
      Input.rte.mc.backward.output = MCBACKWARD_ABS;
      Input.rte.mc.backward.absorption = 1;
     }
     else if(strncasecmp("emis",Input.rte.mc.backward.output_string,4)==0) {
      Input.rte.mc.backward.output = MCBACKWARD_EMIS;
      Input.rte.mc.backward.absorption = 1;
     }
     else if(strncasecmp("heat",Input.rte.mc.backward.output_string,4)==0) {
      Input.rte.mc.backward.output = MCBACKWARD_HEAT;
      Input.rte.mc.backward.absorption = 1;
     }
     else {
      fprintf (stderr, "Error, unknown backward quantity %s\n", Input.rte.mc.backward.output_string);
      ierror++;
     }
     free(Input.rte.mc.backward.output_string);
  }

  /*initialize Input.rte.mc.bcond from option mc_bcond */
  if ( Input.rte.mc.bcond_string != NULL ) {
     if      (strncasecmp("periodic",Input.rte.mc.bcond_string,8)==0) Input.rte.mc.bcond = MCBCOND_PERIODIC;
     else if (strncasecmp("mirror",Input.rte.mc.bcond_string,6)==0)   Input.rte.mc.bcond = MCBCOND_MIRROR;
     else if ((strncasecmp("absorb",Input.rte.mc.bcond_string,6)==0) || (strncasecmp("absorp",Input.rte.mc.bcond_string,6)==0))   Input.rte.mc.bcond = MCBCOND_ABSORB;
     else {fprintf(stderr,"Unknown 'mc_bcond %s'\n", Input.rte.mc.bcond_string); ierror++;}
     free(Input.rte.mc.bcond_string);
  }

  /* if uvspec is told to be verbose, it cannot be quiet at the same time */
  if (Input.verbose)
    Input.quiet = 0;

  /*Logic behind the option output_format. Not sure if this is a good location. BR*/
  if (Input.output_format_string!=NULL) {
    if (strcmp(Input.output_format_string,"ascii") == 0)
      Input.output_format = OUTPUT_FORMAT_ASCII; /*BR*/
    else if (strcmp(Input.output_format_string,"flexstor") == 0)
      Input.output_format = OUTPUT_FORMAT_FLEXSTOR; /*BR*/
    else if (strcmp(Input.output_format_string,"netCDF") == 0){
      Input.output_format = OUTPUT_FORMAT_NETCDF; /*BR*/
      if (strlen(Input.filename[FN_OUTFILE]) == 0) {
        fprintf (stderr, "NetCDF-Filename not provided!\n");
        return -1;
      }
    }
    else if (strcmp(Input.output_format_string,"sat_picture") == 0) {
      Input.output_format = OUTPUT_FORMAT_SATPIC; /*BR*/
      if (strlen(Input.filename[FN_OUTFILE]) == 0) {
        fprintf (stderr, "NetCDF-Filename not provided!\n");
        return -1;
      }
    }
    else {
      fprintf (stderr, "Error, invalid output format %s!\n",Input.output_format_string);
      return -1;
    }
  }

  /* To calculate radiances, we want to increase nstr to at least 16 */
  if (Input.rte.numu > 0){
    if(Input.rte.solver==POLRADTRAN){ 
      if(!Input.quiet){  
             fprintf (stderr, "*** Warning, you want to calculate radiances with POLRADTRAN. \n");
	     fprintf (stderr, "*** POLRADTRAN uses only the first NSTR moments of the phase \n");
	     fprintf (stderr, "*** function which might result in bad accuracy, especially if \n");
	     fprintf (stderr, "*** the phase function has a strong forward scattering peak. \n");
	     fprintf (stderr, "*** If you are only interested in the scalar intensity, it is \n");
	     fprintf (stderr, "*** strongly recommended to use cdisort. For calculations with \n");
	     fprintf (stderr, "*** polarization, there is currently no other option, but please \n");
	     fprintf (stderr, "*** be aware, that the radiances might not be very accurate. \n\n");
	     fprintf (stderr, " ... setting polradtran parameters:\n");
	     fprintf (stderr, " ... polradtran_aziorder 4\n");
	     fprintf (stderr, " ... polradtran_quad_type E\n");
            } 
       Input.rte.pol_aziorder  = 4;
       Input.rte.pol_quad_type = "E";
       if (Input.rte.nstr<8) {
          if(!Input.quiet)
              fprintf (stderr, " ... setting nstr to 8 for radiance calculation!\n");
	  Input.rte.nstr=8;
        }
       } 
     else{
       if (Input.rte.nstr<16) {
          if(!Input.quiet)
             fprintf (stderr, " ... setting nstr to 16 for radiance calculation!\n");
	  Input.rte.nstr=16;
	  }
       }
    }
   
  /* nstr is implemented differently for polradtran, 2*nstr (polradtran)*/
  /* corresponds to nstr (disort) */
  if(Input.rte.solver==POLRADTRAN) 
     Input.rte.nstr*=2;
     
  /* MYSTIC with polarisation requires all 4 Stokes components */  
  if( Input.rte.mc.polarisation==1 )
      Input.rte.mc.nstokes = 4;

  /* if tica is specified, turn on ipa_3d (ulrike) */  
  if( Input.tipa>0 )
      Input.ipa3d = 1;

  if (Input.rte.numu>0 && Input.rte.solver==DISORT) {
    fprintf (stderr, "\n");
    fprintf (stderr, "*** Warning, you want to calculate radiances with disort 1.3.\n");
    fprintf (stderr, "*** It is strongly recommended to use cdisort in this case because\n");
    fprintf (stderr, "*** the older disort 1.3 version uses only the first NSTR moments of the\n");
    fprintf (stderr, "*** phase function which might result in really bad distortions.\n");
    fprintf (stderr, "*** cdisort is set up to consider at least the first 1024 moments.\n");
    fprintf (stderr, "\n");
  }


  /* defaut zout-level is 0 or -999 for MYSTIC (if no keyword is set) */
  if (Input.atm.nzout == NOT_DEFINED_INTEGER) {
    switch ( Input.atm.zout_source ) {
    case OUTLEVEL_ZOUT_ABOVE_SUR: 
    case OUTLEVEL_ZOUT_ABOVE_SEA:
    case OUTLEVEL_PRESS:
      Input.atm.nzout    = 1;
      Input.atm.zout_sur = (float *) calloc (Input.atm.nzout, sizeof(float)); 
      if (Input.rte.solver == MONTECARLO)
        Input.atm.zout_sur[0] = -999.0;
      else 
        Input.atm.zout_sur[0] = 0.0;
      break;
    case OUTLEVEL_ATM_LEVELS:
    case OUTLEVEL_ALL_LEVELS:
    case OUTLEVEL_MODEL_LEVELS:
    case OUTLEVEL_MODEL_LAYERS:
    case OUTLEVEL_MODEL_LEVELS_AND_LAYERS:
      break;
    default:
      fprintf (stderr, "Error, determining output levels zout.\n");
      fprintf (stderr, "       unknown option %d for zout_source! (line %d, function '%s' in '%s') \n", 
                               Input.atm.zout_source, __LINE__, __func__, __FILE__);
      return -1;
    }
  }

  if (Input.rte.solver== MONTECARLO && Input.rte.mc.surfaceparallel) {
    if (Input.atm.zout_sur[0] != -999.0) {
      fprintf (stderr, "*** Warning, mc_surfaceparallel does only make sense for radiation\n");
      fprintf (stderr, "*** calculations at the surface. Ignoring mc_surfaceparallel!\n");
      Input.rte.mc.surfaceparallel=0;
    }

    if (Input.atm.zout_sur[0] == -999.0 && Input.atm.nzout>1) {
      fprintf (stderr, "*** Warning, mc_surfaceparallel affects only radiation at the surface.\n");
      fprintf (stderr, "*** Switching off all higher output levels for safety reasons.\n");
      Input.atm.nzout=1;
    }
  }      

  if (Input.rte.solver== MONTECARLO && Input.alt.altitude != NOT_DEFINED_FLOAT) {
    fprintf (stderr, "Error, option altitude does not work with\n");
    fprintf (stderr, "       solver montecarlo! Use mc_elevation_file!\n");
    return -1;
  }
  
  if (Input.rte.solver == MONTECARLO && Input.source == SRC_THERMAL) {
       if (!Input.quiet)
            fprintf(stderr, " ... mc_surface_reflectalways disabled for thermal radiative transfer.\n");    
       Input.rte.mc.reflectalways = 0;
  }
  
  /* surface temperature, this is == NOT_DEFINED_FLOAT, if not specified in the input file */
  Output.surface_temperature = Input.surface_temperature;

  /* define default output as a user defined output */
  if (Input.n_output_user < 0) {
    if (Input.heating == NO_HEAT) {
      switch (Input.rte.solver) {
      case TWOSTRPP:
      case TWOSTR:
      case CTWOSTR:
      case RODENTS:
      case TWOSTREBE:
        Input.n_output_user = 5;
        Input.output_user   = calloc (Input.n_output_user, sizeof (int));
        Input.output_user[0] = OUTPUT_USER_WAVE;
        Input.output_user[1] = OUTPUT_USER_EDIR;
        Input.output_user[2] = OUTPUT_USER_EDN;
        Input.output_user[3] = OUTPUT_USER_EUP;
        Input.output_user[4] = OUTPUT_USER_U;
	break;
      case DISORT:
      case DISORT2:
      case CDISORT:
      case SDISORT:
      case SPSDISORT:
      case SUCORD:
      case MONTECARLO:
      case SSS:
      case TZS:
      case SSSI:
      case NULLSOLVER:
        Input.n_output_user = 7;
        Input.output_user = calloc (Input.n_output_user, sizeof (int));
        Input.output_user[0] = OUTPUT_USER_WAVE;
        Input.output_user[1] = OUTPUT_USER_EDIR;
        Input.output_user[2] = OUTPUT_USER_EDN;
        Input.output_user[3] = OUTPUT_USER_EUP;
        Input.output_user[4] = OUTPUT_USER_UDIR;
        Input.output_user[5] = OUTPUT_USER_UDN;
        Input.output_user[6] = OUTPUT_USER_UUP;
        break;
      case SSLIDAR:
        Input.n_output_user  = 3 + Input.sslidar_polarisation;
        Input.output_user    = calloc (Input.n_output_user, sizeof (int));
        Input.output_user[0] = OUTPUT_USER_ZOUT_SUR;
        Input.output_user[1] = OUTPUT_USER_SSLIDAR_NPHOT;
        Input.output_user[2] = OUTPUT_USER_SSLIDAR_RATIO;
        if (Input.sslidar_polarisation)
          Input.output_user[3] = OUTPUT_USER_SSLIDAR_NPHOT_Q;
	break;
      case POLRADTRAN:
        /* polradtran has its special output function, do nothing here */
        break;
      default:
        fprintf (stderr, "Error, unknown rte_solver %d (in uvspec_lex.l)\n", Input.rte.solver);
        return -2;
      }
    }
    else { 

      /* default output for spectral heating rates, as a 2D matrix */
      if (Input.processing != PROCESS_INT && Input.processing != PROCESS_SUM) {

        if( Input.output_format == OUTPUT_FORMAT_ASCII)
          Input.output_format = OUTPUT_FORMAT_MATRIX;

        Input.n_output_user = 3;
        Input.output_user = calloc (Input.n_output_user, sizeof (int));
        Input.output_user[0] = OUTPUT_USER_WAVE;
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
          Input.output_user[1] = OUTPUT_USER_ZOUT_SUR;
          break;
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
        case OUTLEVEL_MODEL_LEVELS:
	case OUTLEVEL_MODEL_LAYERS:
	case OUTLEVEL_MODEL_LEVELS_AND_LAYERS:
          Input.output_user[1] = OUTPUT_USER_ZOUT_SEA;
          break;
        case OUTLEVEL_PRESS:
          Input.output_user[1] = OUTPUT_USER_P;
          break;
        default:
          fprintf (stderr, "Error, unknown zout marker %d when defining default output (uvspec_lex.l)\n", Input.atm.zout_source);
          return -2;
        }
        Input.output_user[2] = OUTPUT_USER_HEAT;
      }
      else { /* integrated output */
        Input.n_output_user = 2;
        Input.output_user = calloc (Input.n_output_user, sizeof (int));
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
          Input.output_user[0] = OUTPUT_USER_ZOUT_SUR;
          break;
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
        case OUTLEVEL_MODEL_LEVELS:
	case OUTLEVEL_MODEL_LAYERS:
	case OUTLEVEL_MODEL_LEVELS_AND_LAYERS:
          Input.output_user[0] = OUTPUT_USER_ZOUT_SEA;
          break;
        case OUTLEVEL_PRESS:
          Input.output_user[0] = OUTPUT_USER_P;
          break;
        default:
          fprintf (stderr, "Error, unknown zout marker %d when defining default output (uvspec_lex.l)\n", Input.atm.zout_source);
          return -2;
        }
        Input.output_user[1] = OUTPUT_USER_HEAT;
      }
    }
  }

  /* if output_user requires heating_rate -> set heating_rate default calculation method */
  if (Input.heating == NO_HEAT) {
    for (io=0; io<Input.n_output_user; io++) {
      if (Input.output_user[io] == OUTPUT_USER_HEAT) { 
        Input.heating = HEAT_LAYER_CD;
        if (!Input.quiet)
          fprintf (stderr, " ... heating rates calculated with central differences\n");
      }
    }
  }

  /* if output_user any horizontal derivations -> atmosphere maps and wind file are mandatory */
  for (io=0; io<Input.n_output_user; io++) {
    if ( Input.output_user[io] == OUTPUT_USER_HEAT_AD_X || Input.output_user[io] == OUTPUT_USER_DTDX ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD_Y || Input.output_user[io] == OUTPUT_USER_DTDY ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD ) {
      if ( (strlen(Input.filename[FN_ECMWF]) == 0 && strlen(Input.filename[FN_ECHAM]) == 0) ) {
        fprintf (stderr, "Error. for output_user dTdx, dTdy, or any heat advection  \n");
        fprintf (stderr, "       An 3D atmosphere file in netCDF format are nessesary \n");
        return -1;
      }
    }
  }

  /* if output_user any horizontal derivations -> atmosphere maps and wind file are mandatory */
  for (io=0; io<Input.n_output_user; io++) {
    if ( Input.output_user[io] == OUTPUT_USER_HEAT_AD_X ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD_Y ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD ) {
      if ( strlen(Input.filename[FN_ECMWF_WIND_MAP]) == 0 ) {
        fprintf (stderr, "Error. for output_user heat_ad_x, heat_ad_y, or heat_ad  \n");
        fprintf (stderr, "       a wind file in netCDF format are nessesary \n");
        return -1;
      }
    }
  }

  /* activate z_interpolate for heating rate calculations */
  if (Input.heating != NO_HEAT && Input.atm.zout_interpolate == NO_ZOUT_INTERPOLATE) {
    Input.atm.zout_interpolate = ZOUT_INTERPOLATE;
    if (!Input.quiet)
      fprintf (stderr, " ... zout_interpolate is turned on by default for heating rates calculations\n");
  }

  /* activate zout_interpolate for calculations with rte_solver rodents (i.e. Robert Buras' twostream model;
                                                                         ulrike, June 2010) */
  
  /* tipa dir works only with rte_solver rodents (ulrike, June 2010) */
  if (Input.tipa == TIPA_DIR && Input.rte.solver != RODENTS) {
    fprintf (stderr, "Error: tipa dir works only with rte_solver rodents. Thus, choose rodents or mc_tipa dir!\n");
    return -1;
  }
  
  /* activate zout_interpolate for calculations with rte_solver twostrebe (i.e. Bernhard Mayers twostream model;
                                                           ulrike, June 2010) */
  if (Input.rte.solver == TWOSTREBE && Input.atm.zout_interpolate == NO_ZOUT_INTERPOLATE) {
    Input.atm.zout_interpolate = ZOUT_INTERPOLATE;
    if (!Input.quiet)
      fprintf (stderr, " ... zout_interpolate is turned on by default for calculations with rte_solver twostrebe\n");
  }

  /* turn on all zout levels to get range bins for sslidar        */
  /* this is a trick to allocate the memory needed for the output */
  if (Input.rte.solver == SSLIDAR)
    Input.atm.zout_source = OUTLEVEL_ALL_LEVELS;

  if (Input.sslidar_polarisation)
    Input.rte.pol_nstokes = 2;


  /* check, if only time or day_of_year are used */
  if (Input.UTC.tm_yday > 0 && Input.UTC.tm_mday > 0) {
    fprintf (stderr, "Error. Either 'day_of_year' or 'time' can be specified  \n");
    fprintf (stderr, "       in the input file, but not both. Please remove one of these lines. \n");
    return -1; 
  }

  /* copy date to uninitialised UTC structure, if LAT input */
  if (Input.time_zone == TIME_LAT) {
    Input.UTC.tm_year = Input.LAT.tm_year;
    Input.UTC.tm_mon  = Input.LAT.tm_mon;
    Input.UTC.tm_mday = Input.LAT.tm_mday;
  }

  /* calculate day of year */
  if ( Input.UTC.tm_mday > 0. && (Input.UTC.tm_mon + 1) > 0. ) { /* +1 == C-standard Jan=0, Feb=1, ...*/
    if ((Input.UTC.tm_yday = day_of_year(Input.UTC.tm_mday, (Input.UTC.tm_mon+1))) < 0)  { /* +1 == C-standard Jan=0, Feb=1, ...*/
      fprintf (stderr, "\n*** Error, calculating day_of_year, (in uvspec_lex.l)");
      fprintf (stderr, "\n    Date out of range! day = %d, month = %d\n", Input.UTC.tm_mday, Input.UTC.tm_mon+1);
      return -1;
    }
  }
  Input.LAT.tm_yday = Input.UTC.tm_yday;
  

  /* convert local apperent time to UTC, needs correct day_of_year */
  switch (Input.time_zone) {
  case TIME_UTC:
    /* do nothing already UTC */
    /* fprintf (stderr, " *** UTC! ***  year=%4d, month=%2d, day=%2d, hour=%2d, min=%2d, sec=%2d\n", */
    /*                  Input.UTC.tm_year+1900, Input.UTC.tm_mon+1, Input.UTC.tm_mday, Input.UTC.tm_hour, Input.UTC.tm_min, Input.UTC.tm_sec ); */
    break;
  case TIME_LAT:

    if (Input.verbose) {
      fprintf (stderr, " ... Convert local time to UTC\n"); 
      fprintf (stderr, "     local apparent time:        year=%4d, month=%2d, day=%2d, hour=%2d, min=%2d, sec=%2d, longitude = %7.2f \n",
                     Input.LAT.tm_year+1900, Input.LAT.tm_mon+1, Input.LAT.tm_mday, Input.LAT.tm_hour, Input.LAT.tm_min, Input.LAT.tm_sec, Input.longitude );
    }

    time_LAT_in_s = (int) ( 3600*Input.LAT.tm_hour + 60*Input.LAT.tm_min + Input.LAT.tm_sec );

    /* convert LAT to UTC, last argument standard longitude of UTC, ATTENTION: for function LAT() POSITIVE longitude is WEST */
    time_UTC_in_s = standard_time ( time_LAT_in_s, Input.LAT.tm_yday, -Input.longitude, 0.0); 

    /* correct negative UTCs and UTCs larger than 24:00 */
    /* to get human-readable output                     */ 
    while (time_UTC_in_s<0)
      time_UTC_in_s += 86400; 
    while (time_UTC_in_s>86400)
      time_UTC_in_s -= 86400;

    Input.UTC.tm_hour =  time_UTC_in_s / 3600;
    Input.UTC.tm_min  = (time_UTC_in_s - Input.UTC.tm_hour*3600) / 60;
    Input.UTC.tm_sec  = (time_UTC_in_s - Input.UTC.tm_hour*3600 - Input.UTC.tm_min*60);
           
    if (Input.verbose) {
      fprintf (stderr, "     universal time correlated:  year=%4d, month=%2d, day=%2d, hour=%2d, min=%2d, sec=%2d\n",
                     Input.UTC.tm_year+1900, Input.UTC.tm_mon+1, Input.UTC.tm_mday, Input.UTC.tm_hour, Input.UTC.tm_min, Input.UTC.tm_sec );
    }

    break;
  default:
    fprintf (stderr, "Error, unknown time_zone %d!\n", Input.time_zone);
    fprintf (stderr, "This is a program bug, please contact the programmers!\n");
    return -1;
  }

  /* check that both satellite_pixel and satellite_geometry are activated */
  if (Input.sat_pixel_x > 0 && strlen(Input.filename[FN_SATGEOMETRY]) == 0) {
    fprintf (stderr, "Error, satellite_pixel is specified, but no satellite geometry is given\n");
    return -1; 
  }
  if (Input.sat_pixel_x < 0 && strlen(Input.filename[FN_SATGEOMETRY]) > 0) {
    fprintf (stderr, "Error, satellite geometry is given, but no satellite_pixel is specified\n");
    return -1; 
  }


  /* set default SATELLITE GEOMETRY map for MSG */
  if (strncasecmp("MSG",Input.filename[FN_SATGEOMETRY],3)==0 && strlen(Input.filename[FN_SATGEOMETRY]) == 3) {
    strcpy (Input.filename[FN_SATGEOMETRY], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_SATGEOMETRY], "satellites/MSG_seviri/MSG_seviri_geometry.nc");
    if (Input.verbose)
      fprintf (stderr, " ... using MSG satellite geometry %s\n", Input.filename[FN_SATGEOMETRY] );
  }
  if (strlen(Input.filename[FN_SATGEOMETRY]) > 0 && Input.output_format == OUTPUT_FORMAT_NETCDF && !Input.quiet)
    fprintf (stderr, "*** Warning, using satellite geometry, but not 'output_format sat_picture'\n");

  /* initialise satellite geometry */
  if (strlen(Input.filename[FN_SATGEOMETRY]) > 0) {
    /* read satellite geometry: lat/lon/theta_sat/phi_sat */
    status = read_sat_geometry (Input.sat_pixel_x, Input.sat_pixel_y, Input.UTC, Input.filename[FN_SATGEOMETRY], 
                                &(Input.latitude), &(Input.longitude), 
                                &(Input.rte.numu),&(Input.rte.maxumu), &(Input.rte.umu),
                                &(Input.rte.nphi),&(Input.rte.maxphi), &(Input.rte.phi),
                                Input.rte.solver, Input.verbose, Input.quiet);    /* in ancillary.c */
    if (status!=0) {
      fprintf (stderr, "Error %d reading satellite geometry from %s in uvspec_lex.l\n", status, Input.filename[FN_SATGEOMETRY]);
      return status;
    }
    if (Input.latitude == 999) {
      /* if (!Input.quiet) */
      /*  fprintf (stderr, "Pixel outside Earth plate, skip this calculation\n"); */
      return 0;
      /* Input.rte.solver = NULLSOLVER; */
      /* Input.latitude  = 0.0; */
      /* Input.longitude = 0.0; */
    }
  }

  /* if no atmosphere file is given, check if we can find a appropriate atmosphere */
  if (Input.atmosphere_filename != NULL) { 
	strcpy (Input.filename[FN_ATMOSPHERE], Input.atmosphere_filename); 
	free( Input.atmosphere_filename); }
  if (strlen(Input.filename[FN_ATMOSPHERE]) == 0) {
    /* if lat, lon AND time is defined than search suitable atmosphere file */
    if ( ( fabs(Input.latitude  - NOT_DEFINED_FLOAT) > EPSILON ) &&
         ( fabs(Input.longitude - NOT_DEFINED_FLOAT) > EPSILON ) &&
         ( Input.UTC.tm_mday > 0 ) ) {

      if ( fabs (Input.latitude) < 23.0 )
        strcpy (Input.filename[FN_ATMOSPHERE], "tropics");
      else if ( fabs (Input.latitude) < 67.0 )
        strcpy (Input.filename[FN_ATMOSPHERE],"midlatitude");
      else 
        strcpy (Input.filename[FN_ATMOSPHERE],"subarctic");

      if ( fabs (Input.latitude) >= 23.0 ) { 
        if ( ( Input.latitude > 0 &&  (Input.UTC.tm_mon+1) >=  4 && (Input.UTC.tm_mon+1) <=  9  ) || 
             ( Input.latitude < 0 && ((Input.UTC.tm_mon+1) >= 10 || (Input.UTC.tm_mon+1) <=  3 )) )
          strcat (Input.filename[FN_ATMOSPHERE], "_summer");
        else 
          strcat (Input.filename[FN_ATMOSPHERE], "_winter");
      }
    }
    else /* not all nessesary information are given, choose standard atmosphere */
      strcpy (Input.filename[FN_ATMOSPHERE],"US-standard");

    if (Input.verbose)
      fprintf (stderr, " ... choosing atmosphere file %s\n",Input.filename[FN_ATMOSPHERE]);
  }
  


  /* replace keywords by atmosphere path + files */
  if      (strncasecmp("subarctic_winter",  Input.filename[FN_ATMOSPHERE],16)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglsw.dat");
  }
  else if (strncasecmp("subarctic_summer",  Input.filename[FN_ATMOSPHERE],16)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglss.dat");
  }
  else if (strncasecmp("midlatitude_summer",Input.filename[FN_ATMOSPHERE],18)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglms.dat");
  }
  else if (strncasecmp("midlatitude_winter",Input.filename[FN_ATMOSPHERE],18)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglmw.dat");
  }
  else if (strncasecmp("tropics",           Input.filename[FN_ATMOSPHERE], 7)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglt.dat");
  }
  else if (strncasecmp("US-standard",       Input.filename[FN_ATMOSPHERE],11)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglus.dat");
  }

  /* try to attach default path for solar_flux file, if file does not exit */
  if (Input.extraterrestrial_filename != NULL) {
	strcpy (Input.filename[FN_EXTRATERRESTRIAL], Input.extraterrestrial_filename); 
	free( Input.extraterrestrial_filename);
	}
  if (strlen(Input.filename[FN_EXTRATERRESTRIAL]) > 0) {
    if ((f = fopen(Input.filename[FN_EXTRATERRESTRIAL], "r")) == NULL) {

      /* file does not exit, try to add an default path */
      if (strlen(Input.filename[FN_PATH])>0)
        strcpy (tmp_filename,  Input.filename[FN_PATH]);
      else
        strcpy (tmp_filename,  "../data/");
      strcat (tmp_filename, "solar_flux/");
      strcat (tmp_filename, Input.filename[FN_EXTRATERRESTRIAL]);

      /* try to open file with default path */
      if ((f = fopen(tmp_filename, "r")) == NULL) {
        /* (default path + file) does not exits, also */
        /* final check if the solar spectrum is needed is done in */
        /* setup_extraterrestrial() (in extraterrestrial.c) */
      }
      else {
        /* (default path + file) does exit => copy filename */
        fclose(f);
        strcpy (Input.filename[FN_EXTRATERRESTRIAL],tmp_filename);
      }
    }
    else
      fclose(f);
  }

  /* CE: Generate aerosol default filenames and set aerosol types if standard OPAC aerosol is specified */
  /* May be this should moved somewhere else ??? */
  if(Input.aer.n_species==-1){

    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.aer.filename[FN_AER_SPECIES],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.aer.filename[FN_AER_SPECIES],  "../data/");

   if (strncasecmp(Input.aer.mixture_name,"continental_clean",17)==0){ 
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/continental_clean.dat");
      Input.aer.n_species=4;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";
      Input.aer.species_names[3]="suso";
    }
    else if (strncasecmp(Input.aer.mixture_name,"continental_average",19)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/continental_average.dat");
      Input.aer.n_species=4;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";
      Input.aer.species_names[3]="suso";
    }
    else if (strncasecmp(Input.aer.mixture_name,"continental_polluted",20)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/continental_polluted.dat");
      Input.aer.n_species=4;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";
      Input.aer.species_names[3]="suso";
    }  
    else if (strncasecmp(Input.aer.mixture_name,"maritime_clean",14)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/maritime_clean.dat");
      Input.aer.n_species=6;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";   
      Input.aer.species_names[3]="ssam";
      Input.aer.species_names[4]="sscm";
      Input.aer.species_names[5]="suso";
    }  
    else if (strncasecmp(Input.aer.mixture_name,"maritime_tropical",17)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/maritime_tropical.dat");
      Input.aer.n_species=6;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";   
      Input.aer.species_names[3]="ssam";
      Input.aer.species_names[4]="sscm";
      Input.aer.species_names[5]="suso";
    }  
    else if (strncasecmp(Input.aer.mixture_name,"maritime_polluted",17)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/maritime_polluted.dat");
      Input.aer.n_species=6;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";   
      Input.aer.species_names[3]="ssam";
      Input.aer.species_names[4]="sscm";
      Input.aer.species_names[5]="suso";
    }  
    else if (strncasecmp(Input.aer.mixture_name,"urban",5)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/urban.dat");
      Input.aer.n_species=4;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";
      Input.aer.species_names[3]="suso";
    }  
    else if (strncasecmp(Input.aer.mixture_name,"desert_spheroids",16)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/desert_spheroids.dat");
      Input.aer.n_species=7;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";
      Input.aer.species_names[3]="minm_spheroids";
      Input.aer.species_names[4]="miam_spheroids";
      Input.aer.species_names[5]="micm_spheroids";
      Input.aer.species_names[6]="suso";
    }  
    else if (strncasecmp(Input.aer.mixture_name,"desert",6)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/desert.dat");
      Input.aer.n_species=7;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";
      Input.aer.species_names[3]="minm";
      Input.aer.species_names[4]="miam";
      Input.aer.species_names[5]="micm";
      Input.aer.species_names[6]="suso";
    }  
    else if (strncasecmp(Input.aer.mixture_name,"antarctic",9)==0){
      strcat(Input.aer.filename[FN_AER_SPECIES],  "aerosol/OPAC/standard_aerosol_files/antarctic.dat");
      Input.aer.n_species=6;
      status = ASCII_calloc_char(&Input.aer.species_names,Input.aer.n_species,FILENAME_MAX+1);
      Input.aer.species_names[0]="inso";
      Input.aer.species_names[1]="waso";
      Input.aer.species_names[2]="soot";
      Input.aer.species_names[3]="ssam";
      Input.aer.species_names[4]="mitr";
      Input.aer.species_names[5]="suso";
    }
    else{
      fprintf(stderr, "Error, standard aerosol mixture %s not definded.\n", Input.aer.filename[FN_AER_SPECIES]);
      return -1; 
    }
  } 
  
  /* specify the individual interpolation methods for all gases*/
  /* !!! ATTENTION !!!: INTERP_METHOD_LINMIX not possible for press, temper, refind, rh, MOL_AIR !!! */ 
  switch (Input.atm.z_interpolate) {
  case Z_INTERPOLATE_LOGLIN:    
    if (Input.atm.interpol_method_press  < 0) 
        Input.atm.interpol_method_press  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_temper < 0)
        Input.atm.interpol_method_temper = INTERP_METHOD_LINEAR; 
    if (Input.atm.interpol_method_refind < 0)
        Input.atm.interpol_method_refind = INTERP_METHOD_LINEAR;

    if (Input.atm.interpol_method_gas[MOL_AIR]  < 0)
        Input.atm.interpol_method_gas[MOL_AIR]  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_gas[MOL_O3]   < 0)
        Input.atm.interpol_method_gas[MOL_O3]   = INTERP_METHOD_LINEAR; /* ozone is assumed to be not well-mixed which */
    if (Input.atm.interpol_method_gas[MOL_O2]   < 0)    
        Input.atm.interpol_method_gas[MOL_O2]   = INTERP_METHOD_LOG;    /* is certainly true for the stratosphere      */
    if (Input.atm.interpol_method_gas[MOL_H2O]  < 0)
        Input.atm.interpol_method_gas[MOL_H2O]  = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_CO2]  < 0)
        Input.atm.interpol_method_gas[MOL_CO2]  = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_NO2]  < 0)
        Input.atm.interpol_method_gas[MOL_NO2]  = INTERP_METHOD_LINEAR; /* NO2 is assumed to be not well-mixed which   */
    if (Input.atm.interpol_method_gas[MOL_BRO]  < 0)
        Input.atm.interpol_method_gas[MOL_BRO]  = INTERP_METHOD_LOG;    /* is certainly true for the stratosphere      */ 
    if (Input.atm.interpol_method_gas[MOL_OCLO] < 0)
        Input.atm.interpol_method_gas[MOL_OCLO] = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_HCHO] < 0)
        Input.atm.interpol_method_gas[MOL_HCHO] = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_O4] < 0)
        Input.atm.interpol_method_gas[MOL_O4]   = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_SO2]  < 0)
        Input.atm.interpol_method_gas[MOL_SO2]  = INTERP_METHOD_LINEAR; /* SO2 is assumed to be not well-mixed which   */
    break;
  case Z_INTERPOLATE_LINMIX:
    /* LINMIX means interpolation assuming linear mixing ratio for gases O3, ..., HCLO while       */
    /* air density is assumed to vary logarithically                                               */ 
    /* densities profiles interpolated             with LINMIX  */
    /* columns are summed up                       with LINMIX  */ 
    /* optical properties per layer are calculated with LINMIX  */
    /* in general: INTERP_METHOD_LINMIX not possible for press, temper, refind, rh, MOL_AIR        */  

    if (Input.atm.interpol_method_press  < 0) 
        Input.atm.interpol_method_press  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_temper < 0)
        Input.atm.interpol_method_temper = INTERP_METHOD_LINEAR; 
    if (Input.atm.interpol_method_refind < 0)
        Input.atm.interpol_method_refind = INTERP_METHOD_LINEAR;

    if (Input.atm.interpol_method_gas[MOL_AIR]  < 0)
        Input.atm.interpol_method_gas[MOL_AIR]  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_gas[MOL_O3]   < 0)
        Input.atm.interpol_method_gas[MOL_O3]   = INTERP_METHOD_LINMIX; 
    if (Input.atm.interpol_method_gas[MOL_O2]   < 0)    
        Input.atm.interpol_method_gas[MOL_O2]   = INTERP_METHOD_LINMIX; 
    if (Input.atm.interpol_method_gas[MOL_H2O]  < 0)
        Input.atm.interpol_method_gas[MOL_H2O]  = INTERP_METHOD_LINMIX;    
    if (Input.atm.interpol_method_gas[MOL_CO2]  < 0)
        Input.atm.interpol_method_gas[MOL_CO2]  = INTERP_METHOD_LINMIX;    
    if (Input.atm.interpol_method_gas[MOL_NO2]  < 0)
        Input.atm.interpol_method_gas[MOL_NO2]  = INTERP_METHOD_LINMIX; 
    if (Input.atm.interpol_method_gas[MOL_BRO]  < 0)
        Input.atm.interpol_method_gas[MOL_BRO]  = INTERP_METHOD_LINMIX;  
    if (Input.atm.interpol_method_gas[MOL_OCLO] < 0)
        Input.atm.interpol_method_gas[MOL_OCLO] = INTERP_METHOD_LINMIX;    
    if (Input.atm.interpol_method_gas[MOL_HCHO] < 0)
        Input.atm.interpol_method_gas[MOL_HCHO] = INTERP_METHOD_LINMIX;  
    if (Input.atm.interpol_method_gas[MOL_O4]   < 0)
        Input.atm.interpol_method_gas[MOL_O4]    = INTERP_METHOD_LINMIX;  
    if (Input.atm.interpol_method_gas[MOL_SO2]  < 0)
        Input.atm.interpol_method_gas[MOL_SO2]  = INTERP_METHOD_LINMIX; 
    break;
  case Z_INTERPOLATE_SPLINE:
    /*        CONTINUOUS FIRST DERIVATIVES, BUT COLUMNS AND OPTICAL PROPERTIES NOT CONSISTENT              */
    /* SPLINE means spline interpolation for additional levels zout and altitude levels as specified below */
    /* columns summed up  with LINMIX assumption (this might be improved, if nessesary)                    */   
    /* optical properties with LINMIX assumption (this might be improved, if nessesary)                    */
    /* in general: INTERP_METHOD_LINMIX not possible for press, temper, refind, rh, MOL_AIR                */

    if (Input.atm.interpol_method_press  < 0) 
        Input.atm.interpol_method_press  = INTERP_METHOD_LOG_SPLINE; /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_temper < 0)
        Input.atm.interpol_method_temper = INTERP_METHOD_SPLINE; 
    if (Input.atm.interpol_method_refind < 0)
        Input.atm.interpol_method_refind = INTERP_METHOD_SPLINE;

    if (Input.atm.interpol_method_gas[MOL_AIR]  < 0)
        Input.atm.interpol_method_gas[MOL_AIR]  = INTERP_METHOD_LOG_SPLINE;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_gas[MOL_O3]   < 0)
        Input.atm.interpol_method_gas[MOL_O3]   = INTERP_METHOD_SPLINE; 
    if (Input.atm.interpol_method_gas[MOL_O2]   < 0)
        Input.atm.interpol_method_gas[MOL_O2]   = INTERP_METHOD_LOG_SPLINE; 
    if (Input.atm.interpol_method_gas[MOL_H2O]  < 0)
        Input.atm.interpol_method_gas[MOL_H2O]  = INTERP_METHOD_LOG_SPLINE;    
    if (Input.atm.interpol_method_gas[MOL_CO2]  < 0)
        Input.atm.interpol_method_gas[MOL_CO2]  = INTERP_METHOD_LOG_SPLINE;    
    if (Input.atm.interpol_method_gas[MOL_NO2]  < 0)
        Input.atm.interpol_method_gas[MOL_NO2]  = INTERP_METHOD_SPLINE;
    if (Input.atm.interpol_method_gas[MOL_BRO]  < 0)
        Input.atm.interpol_method_gas[MOL_BRO]  = INTERP_METHOD_LOG_SPLINE;  
    if (Input.atm.interpol_method_gas[MOL_OCLO] < 0)
        Input.atm.interpol_method_gas[MOL_OCLO] = INTERP_METHOD_LOG_SPLINE;    
    if (Input.atm.interpol_method_gas[MOL_HCHO] < 0)
        Input.atm.interpol_method_gas[MOL_HCHO] = INTERP_METHOD_LOG_SPLINE;  
    if (Input.atm.interpol_method_gas[MOL_O4]   < 0)
        Input.atm.interpol_method_gas[MOL_O4]   = INTERP_METHOD_LOG_SPLINE;  
    if (Input.atm.interpol_method_gas[MOL_SO2]  < 0)
        Input.atm.interpol_method_gas[MOL_SO2]  = INTERP_METHOD_SPLINE;
    break;
  default:
    fprintf (stderr, "Error, unknown interpolation method z_interpolate %d!\n", Input.atm.z_interpolate);
    fprintf (stderr, "This is a program bug, please contact the programmers!\n");
    return -1;
    break;
  }

  if (Input.atm.interpol_method_temper != INTERP_METHOD_LINEAR &&
      Input.atm.interpol_method_temper != INTERP_METHOD_LOG && 
      Input.atm.interpol_method_temper != INTERP_METHOD_SPLINE) {
    fprintf (stderr, "Error, interpolation method %d not possible\n", Input.atm.interpol_method_temper);
    fprintf (stderr, "for vertical interpolation of temperature.\n");
    return -1;
  }

  /* check if EITHER one of o2_mixing_ratio OR dens_column[O2] is given in input file */
  if (Input.atm.column[MOL_O2] >= 0.0 && Input.mixing_ratio[MX_O2] >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column O2' or 'o2_mixing_ratio' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1;
  }

  /* check if EITHER one of precip_water OR dens_column[H2O] is given in input file */
  if (Input.atm.column[MOL_H2O] >= 0.0 && Input.precip_water >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column H2O' or 'h2o_precip' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1; 
  }

  /* check if EITHER one of h2o_mixing_ratio OR dens_column[HO2] is given in input file */
  if (Input.atm.column[MOL_H2O] >= 0.0 && Input.mixing_ratio[MX_H2O] >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column H2O' or 'h2o_mixing_ratio' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1;
  }

  /* check if EITHER one of co2_mixing_ratio OR dens_column[CO2] is given in input file */
  if (Input.atm.column[MOL_CO2] >= 0.0 && Input.mixing_ratio[MX_CO2] >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column CO2' or 'co2_mixing_ratio' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1; 
  }

  /* no user-defined transmittance grid if correlated-k */
  if ((Input.ck_scheme!=CK_NONE && Input.ck_scheme!=CK_LOWTRAN) && strlen(Input.filename[FN_WLTRANS])>0) {
    strcpy (Input.filename[FN_WLTRANS], "");
    fprintf (stderr, " ... option transmittance_wl_file ignored\n");
    fprintf (stderr, " ... (does not make sense with correlated-k)\n");
  }

  if (Input.alb.source == NOT_DEFINED_INTEGER) {
    /* default setting */
    Input.alb.source = ALBEDO_CONSTANT;
  }

  /* set default SURFACE TYPE map */
  if (strncasecmp("IGBP",Input.filename[FN_SURFACE_TYPE_MAP],4)==0) {
    strcpy (Input.filename[FN_SURFACE_TYPE_MAP], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_SURFACE_TYPE_MAP], "albedo/IGBP_map/SURFACE_TYPE_IGBP_10min.cdf");

    if (Input.alb.source != ALBEDO_IGBP_LIBRARY && !Input.quiet)
      fprintf (stderr, "*** Warning, using IGBP map, but not 'albedo_library IGBP'\n");
  }

  /* message to the user because of name change */
  if (strncasecmp("IGBP",Input.filename[FN_ALBEDO_MAP],4)==0) {
    fprintf (stderr, "\n*** ERROR, starting from libRadtran version 1.2 \n");
    fprintf (stderr,   "           'albedo_map' contains broadband albedo values \n");
    fprintf (stderr,   "           and not surface_type information any more. \n");
    fprintf (stderr,   "           Sorry for the inconvenience! \n");
    fprintf (stderr,   "           please use instead: \n");
    fprintf (stderr,   "           surface_type_map IGBP \n");
    return -1;
  }

  /* set default SURFACE TYPE netCDF variable name */
  if ( strlen(Input.alb.netCDF_surf_name) == 0 )
    strcpy (Input.alb.netCDF_surf_name, "surface_type");

  /* set default RPV Polder library path  */
  if (Input.rpv.source == RPV_IGBP_LIBRARY) {

    strcpy (Input.filename[FN_RPV_LIB_PATH], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_RPV_LIB_PATH], "albedo/IGBP_rpv_polder/");

  }

  /* set default ALTITUDE map */
  if (strncasecmp("GTOPO",Input.filename[FN_ALTITUDE_MAP],5)==0) {
    strcpy (Input.filename[FN_ALTITUDE_MAP], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_ALTITUDE_MAP], "altitude/ELEVATION_GTOPO_10min.cdf");
  }


  /* check format of the ECMWF file */
  if (strlen(Input.filename[FN_ECMWF])>0) {

#if HAVE_LIBNETCDF

    /* open netcdf file */
    status = nc_open (Input.filename[FN_ECMWF], NC_NOWRITE, &ncid);
    if (status!=NC_NOERR) {
      fprintf (stderr, "Error %d opening netCDF file %s\n", status, Input.filename[FN_ECMWF]);
      return status;
    }
    status = nc_inq_varid (ncid, "z", &id_var_test);
    if (status==NC_NOERR) {
      Input.atm.ECMWF_new_format=TRUE;
    }
    else {
      status = nc_inq_varid (ncid, "Z", &id_var_test);
      if (status==NC_NOERR) {
        Input.atm.ECMWF_new_format=FALSE;
      }
      else {
        fprintf (stderr, "Error, unknown format of the ECMWF atmosphere file %s\n", Input.filename[FN_ECMWF]);
        fprintf (stderr, "       neither 'z' nor 'Z' was found in data file\n");
        return status;
      }
    }
    nc_close (ncid);

#else
    fprintf (stderr, " ***********************************************************************\n");
    fprintf (stderr, " * You have built uvspec without libnetcdf and hence cannot            *\n");
    fprintf (stderr, " * use the ECMWF_atmosphere_file option. Please get netcdf and rebuild.*\n");
    fprintf (stderr, " ***********************************************************************\n");
    return -1;
#endif

  }
  

  /* set default ALTITUDE netCDF variable name */
  if ( strlen(Input.alt.netCDF_alt_name) == 0 ) {
    if ( Input.atm.ECMWF_new_format == TRUE && strlen(Input.filename[FN_ECMWF])>0 )
      strcpy (Input.alt.netCDF_alt_name, "z");
    else
      strcpy (Input.alt.netCDF_alt_name, "Z");
  }

  /* set default surface temperature netCDF variable name */
  if ( strlen(Input.filename[FN_SURFACE_TEMP_MAP]) > 0 ) {
#if HAVE_LIBNETCDF

    status = nc_open (Input.filename[FN_SURFACE_TEMP_MAP], NC_NOWRITE, &ncid);
    if (status!=NC_NOERR) {
      fprintf (stderr, "Error %d opening netCDF file %s\n", status, Input.filename[FN_SURFACE_TEMP_MAP]);
      return status;
    }
    
    /* if no name is given in the input file, try 'skt' and 'SKT' */
    if ( strlen(Input.netCDF_name_surf_T) == 0 ) {
      if      ( (status = nc_inq_varid (ncid, "skt", &id_var_test)) == NC_NOERR )
        strcpy (Input.netCDF_name_surf_T, "skt");
      else if ( (status = nc_inq_varid (ncid, "SKT", &id_var_test)) == NC_NOERR )
        strcpy (Input.netCDF_name_surf_T, "SKT");
      else {
        fprintf (stderr, "Error '%s' while getting id for surface temperature from '%s' \n", 
                                nc_strerror(status), Input.filename[FN_SURFACE_TEMP_MAP]);
        fprintf (stderr, "      (line %d, function %s in %s)\n", __LINE__, __func__, __FILE__);
        return -1;
      }
    }
    nc_close (ncid);

#else
    fprintf (stderr, " ***************************************************************************\n");
    fprintf (stderr, " * You have built uvspec without libnetcdf and hence cannot                *\n");
    fprintf (stderr, " * use the 'surface_temperature_map' option. Please get netcdf and rebuild.*\n");
    fprintf (stderr, " ***************************************************************************\n");
    return -1;
#endif
  }

  /* Default MYSTIC output filename */
  if (strlen(Input.rte.mc.filename[FN_MC_BASENAME]) == 0)
    strcpy (Input.rte.mc.filename[FN_MC_BASENAME], "mc");
  
  /* big caoth loop */
  for (isp=0; isp<Input.n_caoth;isp++) {
    if ( strncasecmp(Input.caoth[isp].name,"ic",2)==0 ) {
      if (Input.caoth[isp].prop_name!=NULL) {
        if      (strncasecmp("fu",Input.caoth[isp].prop_name,2)==0)          Input.caoth[isp].properties = PROP_IC_FU;
        else if (strncasecmp("echam4",Input.caoth[isp].prop_name,6)==0)      Input.caoth[isp].properties = PROP_IC_ECHAM4;
        else if (strncasecmp("mie",Input.caoth[isp].prop_name,3)==0)         Input.caoth[isp].properties = PROP_IC_MIE;
        else if (strncasecmp("yang",Input.caoth[isp].prop_name,4)==0)        Input.caoth[isp].properties = PROP_IC_YANG;
        else if (strncasecmp("key",Input.caoth[isp].prop_name,3)==0)         Input.caoth[isp].properties = PROP_IC_KEY;
        else if (strncasecmp("baum_hufit",Input.caoth[isp].prop_name,10)==0) Input.caoth[isp].properties = PROP_IC_BAUMHUFIT;
        else if (strncasecmp("baum",Input.caoth[isp].prop_name,4)==0)        Input.caoth[isp].properties = PROP_IC_BAUM;
        else if (strncasecmp("hey",Input.caoth[isp].prop_name,3)==0)         Input.caoth[isp].properties = PROP_IC_HEY;
        else { Input.caoth[isp].properties = PROP_FILE;
             strcpy (Input.caoth[isp].filename_prop, Input.caoth[isp].prop_name);
        }
      }
      switch(Input.caoth[isp].properties){
      case PROP_IC_KEY:
      case PROP_IC_YANG:
        Input.caoth[isp].habit = NOT_DEFINED_INTEGER;
        if  (strncasecmp("solid-column",Input.caoth[isp].ic_habit_name,12)==0)        Input.caoth[isp].habit = IC_HABIT_SOLIDCOLUMN;
        else if (strncasecmp("hollow-column",Input.caoth[isp].ic_habit_name,13)==0)   Input.caoth[isp].habit = IC_HABIT_HOLLOWCOLUMN;
        else if (strncasecmp("rough-aggregate",Input.caoth[isp].ic_habit_name,15)==0) Input.caoth[isp].habit = IC_HABIT_AGGREGATE;
        else if (strncasecmp("rosette-4",Input.caoth[isp].ic_habit_name,9)==0)        Input.caoth[isp].habit = IC_HABIT_ROSETTE4;
        else if (strncasecmp("rosette-6",Input.caoth[isp].ic_habit_name,9)==0)        Input.caoth[isp].habit = IC_HABIT_ROSETTE6;
        else if (strncasecmp("plate",Input.caoth[isp].ic_habit_name,5)==0)            Input.caoth[isp].habit = IC_HABIT_PLATE;
        else if (strncasecmp("droxtal",Input.caoth[isp].ic_habit_name,7)==0)          Input.caoth[isp].habit = IC_HABIT_DROXTAL;
        else if (strncasecmp("spheroid",Input.caoth[isp].ic_habit_name,8)==0)         Input.caoth[isp].habit = IC_HABIT_SPHEROID;
        else {fprintf(stderr,"Unknown 'ic_habit %s' on line %d\n", s, line_number+1); ierror++;}
	break;
      }

    }
    if ( strncasecmp(Input.caoth[isp].name,"wc",2)==0 ) {
      if (Input.caoth[isp].prop_name!=NULL) {
        if      (strncasecmp("hu",Input.caoth[isp].prop_name,2)==0)     Input.caoth[isp].properties = PROP_WC_HU;
        else if (strncasecmp("echam4",Input.caoth[isp].prop_name,6)==0) Input.caoth[isp].properties = PROP_WC_ECHAM4;
        else if (strncasecmp("mie",Input.caoth[isp].prop_name,3)==0)    Input.caoth[isp].properties = PROP_WC_MIE;
        else { Input.caoth[isp].properties = PROP_FILE;
             strcpy (Input.caoth[isp].filename_prop, Input.caoth[isp].prop_name);
        }
      }
    }

    /* caoth default values */
    if ( Input.caoth[isp].properties==PROP_NONE ) {
      if ( strncasecmp(Input.caoth[isp].name,"wc",2)==0 ) {
        Input.caoth[isp].properties = PROP_WC_HU;

	if (!Input.quiet)
          fprintf (stderr, " ... using default water cloud properties, Hu and Stamnes (1993)\n");
      }
      else if ( strncasecmp(Input.caoth[isp].name,"ic",2)==0 ) {
        Input.caoth[isp].properties = PROP_IC_FU;

	if (!Input.quiet)
          fprintf (stderr, " ... using default ice cloud properties, Fu et al. (1996/98)\n");
      }
      else {
        fprintf(stderr, "Error! You need to specify properties for %s!\n",
			Input.caoth[isp].fullname );
	return -1;
      }
    }

    if ( Input.caoth[isp].source == CAOTH_FROM_ECMWF && Input.caoth[isp].layer != 1 ) {
      Input.caoth[isp].layer=1;
      if (Input.verbose)
        fprintf (stderr, " ... switching on layer for %s, as ECMWF cloud data is representative for layers \n", Input.caoth[isp].fullname);
    }

    /* caoth warnings */
    switch (Input.caoth[isp].properties) {
    case PROP_WC_MIE:
    case PROP_FILE:
    case PROP_EXPLICIT:
    case PROP_IC_MIE:
    case PROP_IC_BAUM:
    case PROP_IC_BAUMHUFIT:
    case PROP_IC_HEY:
      if (Input.rte.solver==DISORT) {
        fprintf (stderr, "\n");
        fprintf (stderr, "*** Warning, you specified a tabulated phase function for %s.\n",Input.caoth[isp].fullname);
        fprintf (stderr, "*** It is strongly recommended to use cdisort in this case because\n");
        fprintf (stderr, "*** the older disort 1.3 version uses only the first NSTR moments of the\n");
        fprintf (stderr, "*** phase function which might result in really bad distortions.\n");
        fprintf (stderr, "*** cdisort consideres all moments correctly.\n");
        fprintf (stderr, "\n\n");
      }
      break;
    case PROP_IC_YANG:
#if HAVE_YANG
#else
      fprintf (stderr, "***********************************************************************\n");
      fprintf (stderr, "* Error, you have built uvspec without Yang/Key/Mayer support and     *\n");
      fprintf (stderr, "* hence cannot use 'ic_properties yang'.                              *\n");
      fprintf (stderr, "***********************************************************************\n");
      return -1;
#endif
    break;
    default:
      break;
    }

    /* something special: the new Mayer/Key/Yang parameterization is not yet */
    /* included in the distribution; if it is not found, Key et al is used   */
    /* instead.                                                              */
    #if HAVE_YANG
    #else
    if (Input.caoth[isp].properties == PROP_IC_YANG)
      Input.caoth[isp].properties = PROP_IC_KEY;
    #endif
  
    if(Input.caoth[isp].properties == PROP_IC_HEY){
      fprintf(stderr, "**************************************************************************\n");
      fprintf(stderr, "* Warning: ic_properties HEY is a new option and not yet well validated. *\n");
      fprintf(stderr, "*          Please check your results carefully.                          *\n");  
      fprintf(stderr, "**************************************************************************\n");
    }
 

    if (strlen(Input.caoth[isp].filename)>0 && Input.caoth[isp].properties==PROP_IC_FU) {
      if (Input.caoth[isp].scale_gg >= -1  || Input.caoth[isp].gg  >= -1 ||
  	Input.caoth[isp].scale_ssa >= 0  || Input.caoth[isp].ssa >= 0 ||
  	Input.caoth[isp].tau > 0         || Input.caoth[isp].tau550 > 0) {
        fprintf (stderr, "*** Warning, you define an ice cloud using the parameterization by Fu (1996/98) *\n");
        fprintf (stderr, "*** and explicitely modify one of the optical properties, optical thickness,    *\n"); 
        fprintf (stderr, "*** asymmetry parameter, or single scattering albedo. Please make sure that you *\n");
        fprintf (stderr, "*** understand the discussion of delta-scaling in context with the Fu (1996/98) *\n");
        fprintf (stderr, "*** parameterization in the libRadtran manual; see the description of           *\n");
        fprintf (stderr, "*** ic_properties, ic_fu_tau, and ic_fu_reff!                                   *\n");
      }
    }    

    /* set indices for wc and ic if present */
    if      (strncasecmp("wc",Input.caoth[isp].name, 2)==0)
      Input.i_wc=isp;
    if      (strncasecmp("ic",Input.caoth[isp].name, 2)==0)
      Input.i_ic=isp;

    /* quick fix for cloud_overlap introduced at SBCA */
    if (Input.cloud_overlap != CLOUD_OVERLAP_OFF && Input.cloud_overlap != CLOUD_OVERLAP_RAND)
      Input.caoth[isp].ipa=1;

    /* caoth properties */
    switch (Input.caoth[isp].properties) {
    case PROP_WC_MIE:
      /* Mie water cloud properties */ 

      switch (Input.ck_scheme) {
      case CK_KATO:
      case CK_KATO2:
      case CK_KATO2_96:

        if (Input.rte.disort_icm == DISORT_ICM_MOMENTS) {
          /* need to use old files, this will be soon be obsolete */
          if (Input.caoth[isp].interpolate)  {
            fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
            fprintf (stderr, "* wc_properties_interpolate for \"correlated_k KATO\" because\n");
            fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
            fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
            fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
            fprintf (stderr, "* Note that this only happens if you are using disort_icm moments.\n");
            fprintf (stderr, "* In such case, the old Mie files specifically create for the kato bands\n");
  	  fprintf (stderr, "* is being used. In case of using disort_icm phases (default), the\n");
  	  fprintf (stderr, "* new Mie files are being used, and interpolation is performed.\n");
            return -1;
          }
     
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/correlated_k/kato/");
          strcat (Input.caoth[isp].filename_prop, "wc.");
          if (Input.verbose)
            fprintf(stderr, "... using old mie files specifically created for kato bands\n");
        }
        else {
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/wc/mie/");
          strcat (Input.caoth[isp].filename_prop, "wc.");
          Input.caoth[isp].interpolate=1; 
          if (Input.verbose)
            fprintf(stderr, "... interpolate optical properties to central wavelengths of kato bands\n");
        }
  
        break;	    
  
      case CK_FU:
  
        if (Input.rte.disort_icm == DISORT_ICM_MOMENTS) {
          /* need to use old files, this will be soon be obsolete */
          if (Input.caoth[isp].interpolate)  {
            fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
            fprintf (stderr, "* wc_properties_interpolate for \"correlated_k FU\" because\n");
            fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
            fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
            fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
            fprintf (stderr, "* Note that this only happens if you are using disort_icm moments.\n");
            fprintf (stderr, "* In such case, the old Mie files specifically create for the Fu bands\n");
  	  fprintf (stderr, "* is being used. In case of using disort_icm phases (default), the\n");
  	  fprintf (stderr, "* new Mie files are being used, and interpolation is performed.\n");
            return -1;
          }
     
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/correlated_k/fu/");
          strcat (Input.caoth[isp].filename_prop, "wc.");
          if (Input.verbose)
            fprintf(stderr, "... using old mie files specifically created for Fu bands\n");
        }
        else {
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/wc/mie/");
          strcat (Input.caoth[isp].filename_prop, "wc.");
          Input.caoth[isp].interpolate=1; 
          if (Input.verbose)
            fprintf(stderr, "... interpolate optical properties to central wavelengths of Fu bands\n");
        }
  
        break;
        
      case CK_AVHRR_KRATZ:
  
        if (Input.rte.disort_icm == DISORT_ICM_MOMENTS) {
          /* need to use old files, this will be soon be obsolete */
          if (Input.caoth[isp].interpolate)  {
            fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
            fprintf (stderr, "* wc_properties_interpolate for \"correlated_k AVHRR_KRATZ\" because\n");
            fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
            fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
            fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
            fprintf (stderr, "* Note that this only happens if you are using disort_icm moments.\n");
            fprintf (stderr, "* In such case, the old Mie files specifically create for the kratz bands\n");
  	  fprintf (stderr, "* is being used. In case of using disort_icm phases (default), the\n");
  	  fprintf (stderr, "* new Mie files are being used, and interpolation is performed.\n");
            return -1;
          }
     
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/correlated_k/kratz/");
          strcat (Input.caoth[isp].filename_prop, "wc.");
          if (Input.verbose)
            fprintf(stderr, "... using old mie files specifically created for kratz bands\n");
        }
        else {
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/wc/mie/");
          strcat (Input.caoth[isp].filename_prop, "wc.");
          Input.caoth[isp].interpolate=1; 
          if (Input.verbose)
            fprintf(stderr, "... interpolate optical properties to central wavelengths of kratz bands\n");
        }
  
        break;
  
      case CK_NONE:
      case CK_RAMAN:
      case CK_LOWTRAN:
        if (!Input.caoth[isp].interpolate)  {
          fprintf (stderr, "* 'wc_properties mie' for a spectral calculation requires that the\n");
          fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
  	fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
  	fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
  	fprintf (stderr, "* option, please specify explicitely 'wc_properties_interpolate'\n");
  	fprintf (stderr, "* in the uvspec input!\n");
  	return -1;
        }
        else {
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/wc/mie/");
          strcat (Input.caoth[isp].filename_prop, "wc.");
        }
        break;
  
      case CK_GENERIC:
        fprintf (stderr, "Error, 'wc_properties mie' not implemented for a generic\n");
        fprintf (stderr, "correlated-k calculation\n");
        return -1;
  
      default:
        fprintf (stderr, "Error, unsupported correlated_k scheme %d\n", Input.ck_scheme);
        return -1;
      }
      break;

    case PROP_IC_MIE:
      /* Mie ice cloud properties */ 

      switch (Input.ck_scheme) {
      case CK_KATO:
      case CK_KATO2:
      case CK_KATO2_96:
        if (Input.rte.disort_icm == DISORT_ICM_PHASE) {
          fprintf(stderr, "* ic_properties mie for correlated_k kato are not implemented for the disort_icm phase.\n");
          fprintf(stderr, "* Please use disort_icm moments. \n");
          return -1;
         }
        if (Input.caoth[isp].interpolate)  {
          fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
          fprintf (stderr, "* ic_properties_interpolate for \"correlated_k KATO\" because\n");
  	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
  	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
  	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
  	return -1;
        }
  
        strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
        strcat (Input.caoth[isp].filename_prop, "/correlated_k/kato/");
        strcat (Input.caoth[isp].filename_prop, "ic.");
        break;	    
  		    
      case CK_FU:	 
        
        if (Input.rte.disort_icm == DISORT_ICM_PHASE) {
          fprintf(stderr, "* ic_properties mie for correlated_k fu are not implemented for the disort_icm phase.\n");
          fprintf(stderr, "* Please use disort_icm moments. \n");
          return -1;
         }
     
        if (Input.caoth[isp].interpolate)  {
          fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
          fprintf (stderr, "* ic_properties_interpolate for \"correlated_k FU\" because\n");
  	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
  	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
  	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
  	return -1;
        }
  
        strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
        strcat (Input.caoth[isp].filename_prop, "/correlated_k/fu/");
        strcat (Input.caoth[isp].filename_prop, "ic.");
        break;
        
      case CK_AVHRR_KRATZ:
  
        if (Input.rte.disort_icm == DISORT_ICM_PHASE) {
          fprintf(stderr, "* ic_properties mie are not implemented for correlated_k avhrr_kratz are not implemented for the disort_icm phase.\n");
          fprintf(stderr, "* Please use disort_icm moments. \n");
          return -1;
         }
  
        if (Input.caoth[isp].interpolate)  {
          fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
          fprintf (stderr, "* ic_properties_interpolate for \"correlated_k AVHRR_KRATZ\" because\n");
  	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
  	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
  	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
  	return -1;
        }
  
        strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
        strcat (Input.caoth[isp].filename_prop, "/correlated_k/kratz/");
        strcat (Input.caoth[isp].filename_prop, "ic.");
        break;
  
      case CK_NONE:
      case CK_RAMAN:
      case CK_LOWTRAN:
        if (!Input.caoth[isp].interpolate)  {
          fprintf (stderr, "* 'ic_properties mie' for a spectral calculation requires that the\n");
          fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
  	fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
  	fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
  	fprintf (stderr, "* option, please specify explicitely 'ic_properties_interpolate'\n");
  	fprintf (stderr, "* in the uvspec input!\n");
  	return -1;
        }
        else {
          strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
          strcat (Input.caoth[isp].filename_prop, "/ic/mie/");
          strcat (Input.caoth[isp].filename_prop, "ic.");
        }
        break;
  
      case CK_GENERIC:
        fprintf (stderr, "Error, 'ic_properties mie' not implemented for a generic\n");
        fprintf (stderr, "correlated-k calculation\n");
        return -1;
  
      default:
        fprintf (stderr, "Error, unsupported correlated_k scheme %d\n", Input.ck_scheme);
        return -1;
      }
  
      if (!Input.quiet)
         fprintf (stderr, " ... reading ice cloud optical properties from %s*\n", 
  	        Input.caoth[isp].filename_prop);
      break;
  
    case PROP_IC_BAUM:
    case PROP_IC_BAUMHUFIT:
      /* <<(CE:) */
      /* Baum ice cloud properties */ 
  
      if (!Input.caoth[isp].interpolate)  {
        fprintf (stderr, "* 'ic_properties baum' for a spectral calculation requires that the\n");
        fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
        fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
        fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
        fprintf (stderr, "* option, please specify explicitely 'ic_properties_interpolate'\n");
        fprintf (stderr, "* in the uvspec input!\n");
        
        return -1;
      }
      else if(Input.wl.start < 3076.92 &&  Input.wl.end > 2200){
        fprintf(stderr,"* 'ic_properties baum_hufit' is not available in the\n");
        fprintf(stderr,"* wavelength range from 2200 to 3076.92 nm. \n");
        return -1;
        }
      else {
        strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
  
        if (Input.caoth[isp].properties == PROP_IC_BAUM)
          strcat (Input.caoth[isp].filename_prop, "ic/baum/");
  
        if (Input.caoth[isp].properties == PROP_IC_BAUMHUFIT)
          strcat (Input.caoth[isp].filename_prop, "ic/baum_hufit/");
  
        strcat (Input.caoth[isp].filename_prop, "ic.");
      }
  
      if (!Input.quiet)
         fprintf (stderr, " ... reading ice cloud optical properties from %s*\n", 
  	        Input.caoth[isp].filename_prop);

      break;
    case PROP_IC_HEY:
      /* HEY ice cloud properties */ 
  
      if (!Input.caoth[isp].interpolate)  {
        fprintf (stderr, "* 'ic_properties hey' for a spectral calculation requires that the\n");
        fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
        fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
        fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
        fprintf (stderr, "* option, please specify explicitely 'ic_properties_interpolate'\n");
        fprintf (stderr, "* in the uvspec input!\n");
        
        return -1;
      }
      else if(Input.wl.start < 200 &&  Input.wl.end > 5000){
        fprintf(stderr,"* 'ic_properties hey' is only available in the\n");
        fprintf(stderr,"* wavelength range from 200 to 5000 nm. \n");
        return -1;
      }
      else {
        strcpy (Input.caoth[isp].filename_prop, Input.filename[FN_PATH]);
        strcat (Input.caoth[isp].filename_prop, "ic/hey/");
        strcat (Input.caoth[isp].filename_prop, "ic.");
      }
  
      switch(Input.caoth[isp].habit){
      case IC_HABIT_SOLIDCOLUMN:
        strcat(Input.caoth[isp].filename_prop, "col");
        break;
      case IC_HABIT_HOLLOWCOLUMN:
        strcat(Input.caoth[isp].filename_prop, "hol");
        break;
      case IC_HABIT_AGGREGATE:
        strcat(Input.caoth[isp].filename_prop, "agg");
        break;
      case IC_HABIT_ROSETTE6:
        strcat(Input.caoth[isp].filename_prop, "ros");
        break;	
      case IC_HABIT_PLATE:
        strcat(Input.caoth[isp].filename_prop, "pla");
        break;
      case IC_HABIT_DROXTAL:
        strcat(Input.caoth[isp].filename_prop, "dro");
        break;
      case IC_HABIT_ROSETTE4:
        fprintf (stderr, "Error, ic_habit rosette-4 is not implemented for \n"); 
        fprintf (stderr, "ic_properties hey.\n");
        return -1;
        break; 
      default: 
          fprintf (stderr, "Error, unknown ice particle habit. \n"); 
          return -1; 
      }
  
      if (!Input.quiet)
        fprintf (stderr, " ... reading ice cloud optical properties from %s*\n", 
			 Input.caoth[isp].filename_prop);
      break;
    default:
      break; 
    }
  } /* end loop isp */

  /* check IPA options */
  iscloudcover=0;
  isipafiles=0;
  for (isp=0; isp<Input.n_caoth;isp++) {
    if ( Input.caoth[isp].cloudcover > 0 )
      iscloudcover=1;
    if ( Input.caoth[isp].source == CAOTH_IPA_FILES )
      isipafiles=1;
  }

  if ( iscloudcover && isipafiles ) {
    fprintf (stderr, "Error, cloudcover does not work together with ipa_files.\n");
    fprintf (stderr, "Please choose either cloudcover OR ipa_files!\n");
    return -1;
  }

  /* aerosol properties */
  if (Input.aer.seasn>0 && Input.aer.vulcan>0 && Input.aer.haze>0 && Input.aer.visibility>0)
   Input.aer.standard = 1;


  /* BRDR stuff */
  Input.disort2_brdf = BRDF_NONE;
  brdftype=0;

  /******************************************/
  /* First, RPV (Rahman, Pinty, Verstraete) */
  /******************************************/

  /* homogeneous RPV */
  if (Input.rpv.k!=0 || Input.rpv.rho0!=0 || Input.rpv.theta!=0)
    brdftype++;


  /* homogeneous, wavelength-dependent RPV */
  if (strlen(Input.filename[FN_RPV])>0)
    brdftype++;

  if (Input.rpv.k!=0 || Input.rpv.rho0!=0 || Input.rpv.theta!=0 || strlen(Input.filename[FN_RPV])>0) {
    Input.disort2_brdf = BRDF_RPV;

    if (!Input.quiet)
      fprintf (stderr, " ... using RPV parameterization of the BRDF (cdisort, fdisort2 and MYSTIC only!)\n");
  }


  /* 2D RPV */
  if (strlen(Input.rte.mc.filename[FN_MC_RPV])>0)  {
    brdftype++;

    if (!Input.quiet)
      fprintf (stderr, " ... using 2D RPV parameterization of the BRDF (MYSTIC only!)\n");
  }


  /* check if both rpv_file and rpv_type were defined */
  if ((strlen(Input.rte.mc.filename[FN_MC_RPV])>0  && strlen(Input.rte.mc.filename[FN_MC_RPVTYPE])==0) ||
      (strlen(Input.rte.mc.filename[FN_MC_RPV])==0 && strlen(Input.rte.mc.filename[FN_MC_RPVTYPE])>0)) {
    fprintf (stderr, "Error, you need to define both mc_rpv_file and mc_rpv_type if you want 2D RPV\n");
    status--;
  }

  
  /******************************************/
  /* Second, Cox and Munk                   */
  /******************************************/

  if (Input.cm.u10 >= 0 || Input.cm.pcl >= 0 || Input.cm.sal >= 0 || Input.disort2_brdf == BRDF_CAM) {
    brdftype++;

    Input.disort2_brdf = BRDF_CAM;

    if (!Input.quiet)
      fprintf (stderr, " ... using Cox and Munk parameterization of the BRDF (cdisort, fdisort2 and MYSTIC only!)\n");
  }

  /* set default values for windspeed, pigment concentration, and salinity */
  if (Input.disort2_brdf == BRDF_CAM)  {

    if (Input.cm.u10<0)  {
      if (strlen(Input.filename[FN_U10_MAP])==0) {
        fprintf (stderr, "Error setting up Cox and Munk ocean BRDF: At least a wind speed cox_and_munk_u10\n");
	fprintf (stderr, "is required. Please define or switch off ocean BRDF!\n");
	return -1;
      }
      else {
        /* read wind speed from netCDF wind map */
        status = read_u10_from_map (Input.latitude, Input.longitude, Input.UTC, Input.atm.time_interpolate, 
                                    Input.filename[FN_U10_MAP],
                                    &Input.cm.u10, Input.verbose, Input.quiet);
        if (status!=0) {
          fprintf (stderr, "Error %d during read_u10_from_map in uvspec_lex.l \n", status);
          return status;
        }
      }
      if (Input.verbose)
        fprintf (stderr, " ... setting wind speed for Cox and Munk to %f m/s\n", Input.cm.u10);
    }

    if (Input.cm.pcl<0)  {

      if (strlen(Input.filename[FN_SALINITY_MAP])==0) {
        Input.cm.pcl = 0.01;
      }
      else {

        if ( strlen(Input.cm.pcl_netCDF_name) == 0 )
          strcpy (Input.cm.pcl_netCDF_name, "chlorophyll");

        /* read ocean water pigment concentration from map */
        status = get_number_from_netCDF_map (Input.latitude, Input.longitude, Input.UTC, Input.atm.time_interpolate,
                                             Input.filename[FN_PIGMENTS_MAP], &(Input.cm.pcl), TYPE_FLOAT, Input.cm.pcl_netCDF_name,
                                             Input.verbose, Input.quiet);
        if (status!=0) {
          if ( status == ERROR_READ_MISSING_VALUE )
            fprintf (stderr, "     Error '%s' reading '%s' from %s in %s (%s) \n", 
                                "read missing value", "chlorophyll", Input.filename[FN_PIGMENTS_MAP], function_name, file_name ); 
          else { 
#if HAVE_LIBNETCDF
            fprintf (stderr, "     Error '%s' reading '%s' from %s in %s (%s) \n", 
                                nc_strerror(status), "chlorophyll", Input.filename[FN_PIGMENTS_MAP], function_name, file_name );
#else
            fprintf (stderr, "     Error %d reading '%s' from %s in %s (%s) \n", 
                                            status , "chlorophyll", Input.filename[FN_PIGMENTS_MAP], function_name, file_name );
#endif
          }
          return status;
        }
      }

      /* take care of user scale value */
      Input.cm.pcl *= Input.cm.pcl_scale_factor;

      /* automatic check for plausible pigment concentration */
      if ( (Input.cm.pcl < 0.0 || 100.0 < Input.cm.pcl) && !Input.quiet )
        fprintf (stderr, "*** Warning: ocean pigment concentration is not plausible: %f mg/m3\n", Input.cm.pcl  );
      if ( Input.cm.pcl < 0.0 ) 
        return -1;

      if (Input.verbose)
        fprintf (stderr, " ... setting pigment concentration for Cox and Munk to %f mg/m3\n", Input.cm.pcl);
    }

    if (Input.cm.sal<0)  {

      if (strlen(Input.filename[FN_SALINITY_MAP])==0) {
        Input.cm.sal = 34.3;
      }
      else {

        if ( strlen(Input.cm.sal_netCDF_name) == 0 )
          strcpy (Input.cm.sal_netCDF_name, "salinity");

        /* read salinity from map */
        status = get_number_from_netCDF_map (Input.latitude, Input.longitude, Input.UTC, Input.atm.time_interpolate,
                                             Input.filename[FN_SALINITY_MAP], &(Input.cm.sal), TYPE_FLOAT, Input.cm.sal_netCDF_name,
                                             Input.verbose, Input.quiet);
        if ( status == ERROR_READ_MISSING_VALUE ) {
          Input.cm.sal = 34.3;
          if ( !Input.quiet ) {
            fprintf (stderr, "*** Warning: read missing value from map. \n" );
            fprintf (stderr, "***          set salinity to default value of %f \n", Input.cm.sal );
          }
          status=0;
        }

        if (status!=0) {
#if HAVE_LIBNETCDF
          fprintf (stderr, "     Error '%s' reading '%s' from %s in %s (%s) \n", 
                                nc_strerror(status), "SALT", Input.filename[FN_SALINITY_MAP], function_name, file_name );
#else
          fprintf (stderr, "     Error %d reading '%s' from %s in %s (%s) \n", 
                                            status , "SALT", Input.filename[FN_SALINITY_MAP], function_name, file_name );
#endif
          return status;
        }
      }

      /* take care of user scale value */
      Input.cm.sal *= Input.cm.sal_scale_factor;

      /* automatic check for plausible salinity range */
      if ( (Input.cm.sal < 0.0 || 55.0 < Input.cm.sal) && !Input.quiet )
        fprintf (stderr, "*** Warning: salinity value is not plausible: %f ppt\n", Input.cm.sal  );

      if ( Input.cm.sal < 0.0 ) 
        return -1;

      if (Input.verbose)
        fprintf (stderr, " ... setting salinity for Cox and Munk to %f ppm\n", Input.cm.sal);
    }
    
    /* we don't allow wind speeds smaller than 1 m/s because the  */
    /* BRDF is expressed as a series of nstr Legendre polynomials */
    /* which causes unwanted structures not only in the BRDF but  */
    /* also in the albedo                                         */
    /*     ??? Is this really required for MYSTIC - check ???     */
    if (Input.cm.u10<1)  {
      Input.cm.u10 = 1.0;
      fprintf (stderr, " ... increasing wind speed for Cox and Munk to %.2f m/s\n", Input.cm.u10);
    }
    
    /* we also don't want nstr < 16 for the same reason, even for */
    /* the calculation of fluxes                                  */

    if (Input.rte.solver!=MONTECARLO && Input.rte.nstr<16)  {
      fprintf (stderr, " ... increasing nstr to 16 because of Cox and Munk!\n");
      Input.rte.nstr=16;
    }
  }


  /******************************************/
  /* Third, AMBRALS                         */
  /******************************************/

  if (Input.ambrals.iso >= 0 || Input.ambrals.vol >= 0 || Input.ambrals.geo >= 0) {
    brdftype++;

    Input.disort2_brdf = BRDF_AMB;
    
    fprintf (stderr, " ... AMBRALS BRDF (cdisort, fdisort2 and MYSTIC only!)\n");
    fprintf (stderr, " ... iso = %f, vol = %f, geo = %f\n", Input.ambrals.iso, Input.ambrals.vol, Input.ambrals.geo);
  }

  
  /* Check if only one or more BRDF types have been defined */
  /* but allow 2D RPV in combination with Cox and Munk      */
  if (brdftype > 1 && !(Input.disort2_brdf==BRDF_CAM && strlen(Input.rte.mc.filename[FN_MC_RPV])))  {
    fprintf (stderr, "Error, it does not make sense to define more than one BRDF type!\n");
    return -1;
  }




  /**********************************************/
  /* Tsang BPDF                                 */
  /**********************************************/
  
  if (Input.bpdf.u10 >=0){
     Input.bpdf.type=BPDF_TSANG;
     
     if (!Input.quiet) 
        fprintf(stderr, " ... TSANG BPDF - newly implemented, please check results carefully !!!! \n");

     if(Input.rte.solver!=MONTECARLO){
       fprintf(stderr, "Error: Polarized reflectance matrix can currently only be specified with\n");
       fprintf(stderr, "rte_solver montecarlo \n");
       return -1;
     }
  }   
       
  
  /* Number of phase function moments equals the number of streams  */
  /* per default, but may be changed later in the code;             */
  /* in case of disort2 we increase nmom to at least 1024 to assure */
  /* correct representation of the phase function with only little  */
  /* computational cost                                             */
  Output.atm.nmom = Input.rte.nstr;      

  switch (Input.rte.solver) {
  case DISORT2:
  case CDISORT:
  case SSS:
  case SSSI:
    Output.atm.nmom = (Input.rte.nstr>1024 ? Input.rte.nstr : 1024);
    break;
  case SDISORT:
    if (Input.rte.nscat != 1 && Input.rte.nscat != 2)
       Input.rte.nscat = 2;
    break;
  case DISORT:
  case TWOSTR:
  case TWOSTRPP:
  case SUCORD:
  case MONTECARLO:
  case POLRADTRAN:
  case SPSDISORT:
  case TZS:
  case NULLSOLVER:
  case RODENTS:
  case TWOSTREBE:
  case CTWOSTR:
  case SSLIDAR:
    break;
  default:
    fprintf (stderr, "Error, unknown rte_solver %d (line %d, function '%s' in '%s')\n", Input.rte.solver, __LINE__, __func__, __FILE__);
    return -1;
  }

  Output.atm.molabs   = Input.molabs;
  Output.atm.rayleigh = Input.rayleigh;
  
  if (Input.rte.nphi <= 0 && Input.rte.ibcnd==0)  {
    switch (Input.rte.solver) {
    case SDISORT:
    case SPSDISORT:
    case DISORT:
    case DISORT2:
    case CDISORT:
      Input.rte.maxphi = 1;  /* Minimum number required by disort. */
      Input.rte.nphi   = 1;
      Input.rte.phi    = (float *) calloc (Input.rte.maxphi, sizeof(float));
      Output.print_phi = 0;  /* Do not print radiances if user does not want them */
      break;
    case POLRADTRAN:
      if (Input.rte.pol_aziorder == 0) 
	Input.rte.nphi = 0;
      break; 
    case TWOSTR:
    case TWOSTRPP:
    case SUCORD:
    case MONTECARLO:
    case TZS:
    case SSS:
    case SSSI:
    case NULLSOLVER:
    case RODENTS:
    case TWOSTREBE:
    case CTWOSTR:
    case SSLIDAR:
      break;
    default:
      fprintf (stderr, "Error, unknown rte_solver %d (line %d, function '%s' in '%s')\n", Input.rte.solver, __LINE__, __func__, __FILE__);
      return -1;
    }
  }

/*aky  if ( Input.rte.solver == SUCORD ) {
        fprintf (stderr, "Successive order of scattering code not yet in production.\n");
        fprintf (stderr, "Please change to another solver.\n");
	return -1;
  }aky*/
  if ( Input.flu.source >= 0 ) {
    Input.calibration = OUTCAL_TRANSMITTANCE; /* For fluorescence fbeam is its real value and not 1.0 */
                                              /* Hence, extraterrestrial is included in the rte*/
                                              /* solution and should not be multiplied with    */
                                              /* afterwards                                    */
  }
  if ( Input.raman ) {
    
    if (Input.rte.solver != CDISORT) {
      fprintf (stderr, "Only the 'rte_solver cdisort' is allowed with the 'raman' option.\n");
      return -1;
    }
    if (Input.ck_scheme != CK_RAMAN) {
      fprintf (stderr, "Only spectral calculations are allowed with the 'raman' option.\n");
      return -1;
    }

    if ( !Input.raman_original ) Input.raman_fast = 1;

    Input.n_raman_transitions_N2 =  48;           /* Same number as Spurr et al 2008 */
    Input.n_raman_transitions_O2 = 185;           /* Same number as Spurr et al 2008 */

    Output.crs.number_of_ramanshifts = Input.n_raman_transitions_N2 + Input.n_raman_transitions_O2;
    Output.crs.number_of_ramanwavelengths = Output.crs.number_of_ramanshifts + 1 ;

    Input.calibration = OUTCAL_TRANSMITTANCE; /* For Raman fbeam is its real value and not 1.0 */
                                              /* Hence, extraterrestrial is included in the rte*/
                                              /* solution and should not be multiplied with    */
                                              /* afterwards                                    */
    /* For Raman scattering only include wavelengths that the user asked. */
    /* Internally we have to include more wavelengths to account for      */
    /* Raman scattered radiation                                          */
    if ( Input.delta_wvl_raman <= 0 )  Output.wl.delta_wvl_raman = 8.0; 
    else                                  Output.wl.delta_wvl_raman = Input.delta_wvl_raman;
    
    /* Internally the lower and upper wavelengths for Raman scattering are different from */
    /* what the user specifies. This is setup in setup_wlgrid (ancillary.c).              */

    if ( Input.rte.numu > 0 ) {
      Input.rte.maxumu = Input.rte.nstr + Input.rte.numu;
    }
    else {
      Input.rte.maxumu = Input.rte.nstr;
    }

    Input.rte.cmuind = (int *) calloc (Input.rte.nstr, sizeof(int));
    Input.rte.umuind = (int *) calloc (Input.rte.numu, sizeof(int));
    Input.rte.umu    = add_cmu_to_umu(Input.rte.nstr, Input.rte.numu, Input.rte.maxumu, 
				      Input.rte.umu, Input.rte.cmuind, Input.rte.umuind);

    Input.rte.numu   = Input.rte.nstr + Input.rte.numu;
  }
  else if (Input.rte.numu <= 0) {
    Input.rte.maxumu = Input.rte.nstr;
    Input.rte.umu    = (float *) calloc (Input.rte.maxumu, sizeof(float));
    }
  
  /* Output processing: thermal source for brightness temperature */
  if (Input.calibration == OUTCAL_BRIGHTNESS && Input.source != SRC_THERMAL) {
    fprintf (stderr, "Error, brightness temperatures does only make sense for thermal sources!\n");
    return -1;
  }

  /* Output processing: no transmittance for thermal calculations */
  if (Input.calibration == OUTCAL_TRANSMITTANCE && Input.source == SRC_THERMAL) {
    fprintf (stderr, "Error, transmittance does not make sense for thermal sources!\n");
    return -1;
  }

  /* if thermal source, and no brightness temperature requested, use thermal calibration */
  if (Input.source == SRC_THERMAL && Input.calibration == OUTCAL_ABSOLUTE) {
    Input.calibration = OUTCAL_THERMAL;
  }


  if (Input.wl.start > Input.wl.end) {
    if (!Input.quiet) 
      fprintf(stderr," ... swap first and second wavelength.\n");
    tmpfloat = Input.wl.start;
    Input.wl.start = Input.wl.end;
    Input.wl.end = tmpfloat;
  }

  if (Input.wl.start <= 0 && Input.wl.start != NOT_DEFINED_INTEGER  ) {
    fprintf (stderr, "Error, start wavelength %f is negative.\n", Input.wl.start);
    return -1;
  }

  if (Input.wl.start_index > Input.wl.end_index) {
    if (!Input.quiet) 
      fprintf(stderr," ... swap first and second wavelength index.\n");
    tmpint = Input.wl.start_index;
    Input.wl.start_index = Input.wl.end_index;
    Input.wl.end_index = tmpint;
  }

  if (Input.wl.start_index <= 0 && Input.wl.start_index != -999) {
    fprintf (stderr, "Error, start wavelength_index %d is negative.\n", Input.wl.start_index);
    return -1;
  }

  /* check the wavelength range */
  if (Input.wl.end>850 && (Input.ck_scheme==CK_NONE && strlen(Input.filename[FN_MOLABS])==0))  {
    status=-1;
    fprintf (stderr, "Error %d, you want to do a spectral calculation for wavelengths larger than 850 nm. While uvspec\n", status);
    fprintf (stderr, "    treats ozone absorption correctly, molecular absorption is NOT considered in monochromatic\n");
    fprintf (stderr, "    uvspec calculations, as absorption cross-section are highly variable with wavelength.\n");
    fprintf (stderr, "    To consider molecular absorption other than ozone you have two choices \n");
    fprintf (stderr, "    with uvspec:\n");
    fprintf (stderr, "     (1) Do a line-by-line calculation using 'molecular_tau_file' to specify\n");
    fprintf (stderr, "         the wavelength-dependent absorption profile; to calculate the\n");
    fprintf (stderr, "         latter, you need something like David Edwards' genln2.\n");
    fprintf (stderr, "         ATTENTION: line-by-line calculations are very time-consuming!\n");
    fprintf (stderr, "     (2) Use the correlated-k approximation which is the most accurate\n");
    fprintf (stderr, "         solution after the line-by-line calculation; use either the\n");
    fprintf (stderr, "         pre-defined parameterization that come with libRadtran or provide\n");
    fprintf (stderr, "         your own; both options are selected with 'correlated_k ...'\n");
    fprintf (stderr, "\n");
    return -1;
  }

  if (Input.wl.end>500 && (Input.ck_scheme==CK_NONE && strlen(Input.filename[FN_MOLABS])==0))  {
    if (!Input.quiet)  {
      fprintf (stderr, "\n");
      fprintf (stderr, "*** WARNING, you want to do a spectral calculation for wavelengths larger\n");
      fprintf (stderr, "*** than 500 nm. Be warned that from about 500 nm upwards there are highly\n");
      fprintf (stderr, "*** structured molecular absorption bands which are NOT considered in the\n");
      fprintf (stderr, "*** spectral uvspec calculation. While uvspec treats ozone absorption\n");
      fprintf (stderr, "*** correctly, the water vapour absorption bands around 510nm, 545nm,\n");
      fprintf (stderr, "*** 595nm, 655nm, 725nm, 825nm, 935nm, and higher, as well as the\n");
      fprintf (stderr, "*** oxygen absorption at 630nm, 690nm, and 765nm are not implemented.\n");
      fprintf (stderr, "*** Below around 600nm these may usually be neglected; above 600nm it\n");
      fprintf (stderr, "*** starts getting more serious, and above 800nm it does not make any\n");
      fprintf (stderr, "*** sense to do the RTE calculation without considering molecular\n");
      fprintf (stderr, "*** absorption.\n");
      fprintf (stderr, "*** \n");
      fprintf (stderr, "*** To consider molecular absorption other than ozone you have two choices\n");
      fprintf (stderr, "*** with uvspec:\n");
      fprintf (stderr, "***  (1) Do a line-by-line calculation using 'molecular_tau_file' to specify\n");
      fprintf (stderr, "***      the wavelength-dependent absorption profile; to calculate the\n");
      fprintf (stderr, "***      latter, you need something like David Edwards' genln2.\n");
      fprintf (stderr, "***      ATTENTION: line-by-line calculations are very time-consuming!\n");
      fprintf (stderr, "***  (2) Use the correlated-k approximation which is the most accurate\n");
      fprintf (stderr, "***      solution after the line-by-line calculation; use either the\n");
      fprintf (stderr, "***      pre-defined parameterization that come with libRadtran or provide\n");
      fprintf (stderr, "***      your own; both options are selected with 'correlated_k ...'\n");
      fprintf (stderr, "\n");
    }
  }


  /**************************************/
  /* initialize random number generator */
  /**************************************/
  /* initialize random number generator */

  if (Input.rte.mc.readrandomseed) {
    /* read random seed for debugging purposes */
    if ((randomseedfile = fopen(randomseedfilename, "r")) != NULL) {
      fscanf(randomseedfile,"%d",&rseed);
      (void) fclose (randomseedfile);
    }
  }

  rseed = init_uvspec_random (&Input.rte.mc.randomseed, Input.rte.solver == MONTECARLO, Input.quiet);
  if (rseed<0) {
    fprintf (stderr, "Error %d returned from init_libRadtran_random\n", rseed);
    return rseed;
  }

  /* store random seed for debugging purposes */
  if ((randomseedfile = fopen(randomseedfilename, "w")) != NULL) {

    /* tmpnam generates nonexistent filename, but by the time the file is opened this may no longer be true */
    /* mkstemp should fix this, but does not work properly. instead, I create the file as quickly as possible */

    /* strcpy("/tmp/rand-XXXXXX", Input.filename[FN_RANDOMSTATUS]); */
    /* ensure exclusive access on buggy implementations of mkstemp */
    /* umask(0077); */
    /* int fd = mkstemp(Input.filename[FN_RANDOMSTATUS]); */
    /* close(fd); */

    fprintf (randomseedfile, "%d\n", rseed);

#ifdef WRITERANDOMSTATUS
    if (Input.rte.mc.readrandomstatus) {
      Input.filename[FN_RANDOMSTATUS] = "randomstatus";
    } else {
      tmpnam(Input.filename[FN_RANDOMSTATUS]);
      if ( ( randomstatusfile = fopen(Input.filename[FN_RANDOMSTATUS], "w")) != NULL)
        (void) fclose (randomstatusfile);
      else {
        fprintf (stderr, "Error during creation of uvspec randomstatusfile %s\n", Input.filename[FN_RANDOMSTATUS] );
        return -1;
      }
    }

    fprintf (randomseedfile, "Randomstatusfile %s\n", Input.filename[FN_RANDOMSTATUS]);
#endif

    (void) fclose (randomseedfile);
  }
    

  /********************************/
  /* Do the radiation calculation */
  /********************************/
  
  status = uvspec(Input,&Output);
  if (status!=0) {
    fprintf (stderr, "Error %d during execution of uvspec\n", status);
    return status;
  }

  
#ifndef UVSPEC_FUNCTION
  print_output(Input, &Output); 
#else
  print_output(Input, &Output, outfilename); 
#endif
  if (status!=0) {
    fprintf (stderr, "Error %d during print_output \n", status);
    return status;
  }

  free_output(Input, &Output);
  return status;

}

  /*************************/
  /* Output of the results */
  /*************************/

#ifndef UVSPEC_FUNCTION
static int print_output(input_struct Input, output_struct *output) 
#else 
static int print_output(input_struct Input, output_struct *output, char *outfilename) 
#endif
{
int status=0;

#ifndef UVSPEC_FUNCTION
  FILE *f;
  switch(Input.output_format) {
  case OUTPUT_FORMAT_FLEXSTOR:
  case OUTPUT_FORMAT_ASCII:
  case OUTPUT_FORMAT_MATRIX:
          /*If OUTPUT_FORMAT_ASCII is used with a filename then open this file so that the output*/
          /*goes there instead of stdout*/
          if (strlen(Input.filename[FN_OUTFILE]) != 0) {
                  f = fopen (Input.filename[FN_OUTFILE], "w");
          }
          else{
                  f = stdout;
          }
          break;
  default:
          f = stdout;
  }
#else
  FILE *f = fopen (outfilename, "w");
#endif


  switch(Input.output_format) {

  case OUTPUT_FORMAT_FLEXSTOR:
    write_flexstor_output(Input, output, f);
    break;
  case OUTPUT_FORMAT_ASCII:
    
    if (Input.header)
      status=write_header(Input, output, f);
      if (status) {
        fprintf (stderr, "Error, returned from write_header (line %d, function '%s' in '%s')\n", __LINE__, __func__, __FILE__);
	return -1;
      }

    if (Input.rte.solver != POLRADTRAN) {
      /* write user defined output to file with pointer f */
      status=write_user_defined_output(Input, output, f);
      if (status) {
        fprintf (stderr, "Error, returned from write_user_defined_output (line %d, function '%s' in '%s')\n", __LINE__, __func__, __FILE__);
	return -1;
      }
    }
    else { /* -> Input.rte.solver == POLRADTRAN) */
      /* write output in a special Polradtran format */
      write_polradtran_output(Input, output, f);
    }
    break;
  case OUTPUT_FORMAT_MATRIX:

    /* special output for spectral heating rates as 2D matrix */
    write_spectral_heat (Input, output, f);

    break; 
  case OUTPUT_FORMAT_NETCDF:
  case OUTPUT_FORMAT_SATPIC:

#if HAVE_LIBNETCDF

    status = write_netCDF_output(Input, output);
    if (status != NC_NOERR) {
      fprintf (stderr, "Error %d during write_netCDF_output() to %s (uvspec_lex.l) \n", status, Input.filename[FN_OUTFILE]);
      return -1;
    }
#else
    fprintf (stderr, " ***********************************************************************\n");
    fprintf (stderr, " * You have built uvspec without libnetcdf and hence cannot            *\n");
    fprintf (stderr, " * use the netCDF output option. Please get netcdf and rebuild.        *\n");
    fprintf (stderr, " ***********************************************************************\n");
    return -1;
#endif

    break;
  default:
    fprintf (stderr, "Error, unknown output format %d (line %d, function '%s' in '%s')\n", Input.output_format, __LINE__, __func__, __FILE__);
    return -1; 
  }


  fflush(f);

  if (f!=stdout)
    fclose(f);

  return 0;
}  


void write_flexstor_output(input_struct Input, output_struct *output, FILE *f)
{

  int lev=0;
  int iv=0;

  fprintf (f, "flexstor 1 0 suvdama 1 1\n");
  fprintf (f, "# This flexstor output has not been checked by flxcheck.\n");
  fprintf (f, "# Hence it may not follow flexstor/suvdama rules. This will\n");
  fprintf (f, "# be improved soon.\n");
  fprintf (f, "model_name() uvspec, version %s\n", THEVERSION);
  if (fabs(Input.aer.beta-0.0) > 0.0) {
    fprintf (f, "angstrom_alpha()  %f\n", Input.aer.alpha);
    fprintf (f, "angstrom_beta()  %f\n",  Input.aer.beta);
  }
  fprintf (f, "ozone_column(DU) %f\n", output->column[MOL_O3]);
  fprintf (f, "pressure(mb) %f\n", Input.pressure);
  fprintf (f, "self_day_of_year() %d\n", Input.UTC.tm_yday);
  
  /*
  fprintf (f, "self_zout(km)");
  for (lev=0;lev<output->atm.nzoutatm.nzout;lev++)
    fprintf (f, "  %f", Input.atm.zout[lev]);
  fprintf (f, "\n");
  */

  fprintf (f, "self_temper(K)");
  for (lev=0;lev<output->atm.nzout;lev++)
    fprintf (f, "  %f", output->atm.temper_out[lev]);
  
  fprintf (f, "\n");
  fprintf (f, "self_day_of_year() %d\n", Input.UTC.tm_yday);
  fprintf (f, "self_eccentricity_factor() %f\n", output->eccentricity_factor);
  fprintf (f, "begin table data\n");
  for (lev=0;lev<output->atm.nzout;lev++) {
    fprintf (f, "WL(nm) SZA(deg.dec) self_rfldir(mW m-2 nm-2) self_rfldn(mW m-2 nm-2) self_rflup(mW m-2 nm-2)");
    fprintf (f, " self_uavgso(mW m-2 nm-2) self_uavgdn(mW m-2 nm-2) self_uavgup(mW m-2 nm-2) ");
  }
  fprintf (f, "\n");
  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (lev=0;lev<output->atm.nzout;lev++) {
      if (Input.rte.solver == TWOSTR) {
        fprintf (f, "%7.3f %13.6e %13.6e %13.6e %13.6e ", 
        	 output->wl.lambda_h[iv], 
		 output->rfldir[lev][iv], 
	         output->rfldn[lev][iv], 
	         output->flup[lev][iv],
	         output->uavg[lev][iv]);
      }
      else {
        fprintf (f, "%7.3f %7.3f %8.5f %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e",
      	           output->wl.lambda_h[iv], 
		   output->sza_h[iv], 
		   output->atm.zout_sur[lev],
		   output->rfldir[lev][iv], 
		   output->rfldn[lev][iv], 
		   output->flup[lev][iv],
		   output->uavgso[lev][iv], 
		   output->uavgdn[lev][iv], 
		   output->uavgup[lev][iv]);
      }
      fprintf (f, "\n");
    }
  }
  fprintf (f, "end table data\n");
}


int write_header (input_struct Input, output_struct *output, FILE *f)
{
  int lev=0;

  fprintf (f, "#aerosol_haze                 : %16d\n",   Input.aer.haze);
  fprintf (f, "#aerosol_season               : %16d\n",   Input.aer.seasn);
  fprintf (f, "#aerosol_set_gg               : %16.3f\n", Input.aer.set_gg);
  fprintf (f, "#aerosol_set_ssa              : %16.3f\n", Input.aer.set_ssa);
  fprintf (f, "#aerosol_tau_file             : %s\n",     Input.aer.filename[FN_AER_TAU]);
  fprintf (f, "#aerosol_ssa_file             : %s\n",     Input.aer.filename[FN_AER_SSA]);
  fprintf (f, "#aerosol_gg_file              : %s\n",     Input.aer.filename[FN_AER_GG]);
  fprintf (f, "#aerosol_moments_file         : %s\n",     Input.aer.filename[FN_AER_MOM]);
  fprintf (f, "#aerosol_visibility           : %16.3f\n", Input.aer.visibility);
  fprintf (f, "#albedo                       : %16.3f\n", Input.alb.albedo);
  fprintf (f, "#angstrom                     : %16.3f  %16.3f\n", Input.aer.alpha, Input.aer.beta);
  fprintf (f, "#atmosphere_file              : %s\n",     Input.filename[FN_ATMOSPHERE]);
  fprintf (f, "#day_of_year                  : %16d\n",   Input.UTC.tm_yday);

  switch (Input.rte.deltam) {
  case 0:
    printf("#deltam                       : off\n");
    break;
  case 1:
    printf("#deltam                       : on\n");
    break;
  default:
    printf("#deltam                       : unknown\n");
    break;
  }

  switch (Input.ozone_spectrum) {
  case O3_BASSPAUR:
    printf("#o3_crs                       : Bass_and_Paur\n");
    break;
  case O3_MOLINA:
    printf("#o3_crs                       : Molina\n");
    break;
  case O3_DAUMONT:
    printf("#o3_crs                       : Daumont\n");
    break;
  default:
    printf("#o3_crs                       : unknown\n");
    break;
  }

  fprintf (f, "#ozone_column                 : %16.3f\n", Output.column[MOL_O3]);

  switch (Input.rte.solver) {
  case SDISORT:
    printf("#rte_solver                   : sdisort\n");
    break;
  case SPSDISORT:
    printf("#rte_solver                   : spsdisort\n");
    break;
  case DISORT:
    printf("#rte_solver                   : fdisort1\n");
    break;
  case DISORT2:
    printf("#rte_solver                   : fdisort2\n");
    break;
  case CDISORT:
    printf("#rte_solver                   : cdisort\n");
    break;
  case MONTECARLO:
    printf("#rte_solver                   : mystic\n");
    break;
  case POLRADTRAN:
    printf("#rte_solver                   : polradtran\n");
    break;
  case SUCORD:
    printf("#rte_solver                   : sucorcsca\n");
    break;
  case TZS: 
    printf("#rte_solver                   : tzs\n");
    break;
  case SSS: 
    printf("#rte_solver                   : sss\n");
    break;
  case SSSI: 
    printf("#rte_solver                   : sss\n");
    break;
  case TWOSTR: 
    printf("#rte_solver                   : twostr\n");
    break;
  case CTWOSTR: 
    printf("#rte_solver                   : ctwostr\n");
    break;
  case RODENTS:
    printf("#rte_solver                   : rodents\n");
    break;
  case TWOSTREBE:
    printf("#rte_solver                   : twostrebe\n");
    break;
  case SSLIDAR:
    printf("#rte_solver                   : sslidar\n");
    break;
  default:
    printf("#rte_solver                   : unknown (%d)\n",Input.rte.solver);
    return -1;
  }

  fprintf (f, "#slit_function_file           : %s\n", 
           Input.filename[FN_SLITFUNCTION]);
  fprintf (f, "#solar_file                   : %s\n", 
           Input.filename[FN_EXTRATERRESTRIAL]);
  fprintf (f, "#wavelength                          : %16.3f  %16.3f\n", 
           Input.wl.start, Input.wl.end);
  fprintf (f, "#zout                         : ");
  for (lev=0;lev<output->atm.nzout;lev++)
    fprintf (f, " %16.3f", Input.atm.zout_sur[lev]);
  fprintf (f, "\n");

  return 0;
}


void write_polradtran_output (input_struct Input, output_struct *output, FILE *f)
{

  int iv=0;
  int lev=0;
  int is=0;  /* counter for stokes element (1...4) */
  int iu=0;  /* counter for cos(SZA) */

  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (lev=0;lev<output->atm.nzout;lev++) {

      fprintf (f, "%7.3f ", output->wl.lambda_h[iv]);

      for (is=0;is<Input.rte.pol_nstokes;is++)
        fprintf (f, "%13.6e %13.6e ",
                 output->down_flux[lev][is][iv], 
	         output->up_flux[lev][is][iv]);

      fprintf (f, "\n");

      if (output->print_phi && Input.rte.nphi > 0) {
        fprintf (f, "                      ");
        for (j=0; j<Input.rte.nphi; j++)
          fprintf (f, " %13.3f ", Input.rte.phi[j]);
        fprintf (f, "\n");
        for (is=0; is<Input.rte.pol_nstokes; is++) {
          fprintf (f, "Stokes vector %c\n", Input.rte.pol_stokes[is]);
          for (iu=0; iu<Input.rte.numu; iu++) {
            /* printf("%7.4f ", output->mu_values[iu+Input.rte.nstr/2]);*/
	    printf("%7.4f ", Input.rte.umu[iu]);
	    printf("%13.6e ", 0.0);
	    for (j=0; j<Input.rte.nphi; j++) {
	      if (Input.rte.umu[iu] <= 0)
	        fprintf (f, " %13.6e ", output->down_rad[lev][j][iu][is][iv]);
              else
	        fprintf(f, " %13.6e ", output->up_rad[lev][j][iu][is][iv]);}
	    printf("\n");
	  }
        }
      }
    }
  }
}

void write_spectral_heat (input_struct Input, output_struct *output, FILE *f)
{
  int iv=0;
  int lev=0;
  int lev2=0;

  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (lev=0;lev<output->atm.nzout;lev++) {

      /* spectral output */
      if (iv == 0 && lev == 0) {
        fprintf (f, "%9.3e ", 0.0);
        for (lev2=0;lev2<output->atm.nzout;lev2++)
          fprintf (f, "%13.6f ", Input.atm.zout_sur[lev2]);
        fprintf (f, "\n");
      }
      if (lev == 0)
        fprintf (f, "%9.3f ", output->wl.lambda_h[iv]);
      fprintf (f, "%13.6e ", output->heat[lev][iv]);
      if (lev == output->atm.nzout-1)
        fprintf (f, "\n");
    }
  }
}


int write_user_defined_output(input_struct Input, output_struct *output, FILE *f)
{
  int io    = NOT_DEFINED_INTEGER;
  int i_gas = NOT_DEFINED_INTEGER;
  int iv    = NOT_DEFINED_INTEGER;
  int lev   = NOT_DEFINED_INTEGER;
  int llev  = NOT_DEFINED_INTEGER;
  int iu    = NOT_DEFINED_INTEGER;
  int iiu   = NOT_DEFINED_INTEGER;
  int numu  = NOT_DEFINED_INTEGER;
  int write_rad = TRUE; 

  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (llev=0;llev<output->atm.nzout_user;llev++) {
      if ( Input.raman ) lev = output->atm.zout_user_index[llev];
      else lev =llev;

      for (io=0; io<Input.n_output_user; io++) {
        switch (Input.output_user[io]) {
        case OUTPUT_USER_WAVE:
          fprintf (f, "%9.3f ", output->wl.lambda_h[iv]);
          break;
        case OUTPUT_USER_WAVE_MAX:
          fprintf (f, "%9.3f ", 1.0e7/output->wl.wvnmlo_h[iv]); 
          break;
        case OUTPUT_USER_WAVE_MIN:
          fprintf (f, "%9.3f ", 1.0e7/output->wl.wvnmhi_h[iv]); 
          break;
        case OUTPUT_USER_WAVENUMBER:
          fprintf (f, "%9.3f ", 1E7/output->wl.lambda_h[iv]);
          break;
        case OUTPUT_USER_WAVENUMBER_MAX:
          fprintf (f, "%9.3f ", output->wl.wvnmhi_h[iv]);        
          break;
        case OUTPUT_USER_WAVENUMBER_MIN:
          fprintf (f, "%9.3f ", output->wl.wvnmlo_h[iv]);        
          break;
        case OUTPUT_USER_ZOUT_SUR:
          fprintf (f, "%11.6f ", output->atm.zout_sur[lev]);
          break;
        case OUTPUT_USER_ZOUT_SEA:
          fprintf (f, "%11.6f ", output->atm.zout_sea[lev]);
          break;
        case OUTPUT_USER_Z_SUR:
          fprintf (f, "%11.6f ", output->alt.altitude);
          break;
        case OUTPUT_USER_EDIR:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]);
          break;
        case OUTPUT_USER_EGLO:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]+output->rfldn[lev][iv]);
          break;
        case OUTPUT_USER_EDN:
          fprintf (f, "%13.6e ", output->rfldn[lev][iv]);
          break;
        case OUTPUT_USER_EUP:
          fprintf (f, "%13.6e ", output->flup[lev][iv]);
          break;
        case OUTPUT_USER_ENET:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]+output->rfldn[lev][iv]-output->flup[lev][iv]);
          break;
        case OUTPUT_USER_ESUM:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]+output->rfldn[lev][iv]+output->flup[lev][iv]);
          break;
        case OUTPUT_USER_FDIR:
          fprintf (f, "%13.6e ", 4.0*PI*output->uavgso[lev][iv]);
          break;
        case OUTPUT_USER_FGLO:
          fprintf (f, "%13.6e ", 4.0*PI*(output->uavgso[lev][iv]+output->uavgdn[lev][iv]));
          break;
        case OUTPUT_USER_FDN:
          fprintf (f, "%13.6e ", 4.0*PI*output->uavgdn[lev][iv]);
          break;
        case OUTPUT_USER_FUP:
          fprintf (f, "%13.6e ", 4.0*PI*output->uavgup[lev][iv]);
          break;
        case OUTPUT_USER_SPHER_ALB:
	  fprintf (f, "%13.6e ", output->albmed[iv]);
	  break;
        case OUTPUT_USER_UU:
   	  if ( Input.raman ) numu = Input.rte.maxumu-Input.rte.nstr; 
	  else numu = Input.rte.numu;
          for (iu=0;iu<numu;iu++){
	    if ( Input.raman ) iiu = Input.rte.umuind[iu];
	    else iiu = iu;
            for (j=0;j<Input.rte.nphi;j++) {
              fprintf (f, " %16.9e ", output->uu[lev][j][iiu][iv]);
            }
          }
          write_rad = FALSE;
          break;
        case OUTPUT_USER_F:
          switch(Input.rte.solver) {
          case MONTECARLO:
            fprintf (f, "%13.6e ", 4.0*PI*(output->uavgso[lev][iv]+output->uavgdn[lev][iv]+output->uavgup[lev][iv]));
            break;
          default:
            fprintf (f, "%13.6e ", 4.0*PI*output->uavg[lev][iv]);
          }
	  break;
        case OUTPUT_USER_UDIR:
          fprintf (f, "%13.6e ", output->uavgso[lev][iv]);
          break;
        case OUTPUT_USER_UGLO:
          fprintf (f, "%13.6e ", output->uavgso[lev][iv]+output->uavgdn[lev][iv]);
          break;
        case OUTPUT_USER_UDN:
          fprintf (f, "%13.6e ", output->uavgdn[lev][iv]);
          break;
        case OUTPUT_USER_UUP:
          fprintf (f, "%13.6e ", output->uavgup[lev][iv]);
          break;
        case OUTPUT_USER_U:
          fprintf (f, "%13.6e ", output->uavg[lev][iv]);
          break;
        case OUTPUT_USER_ALB:
	  fprintf (f, "%13.6e ", output->flup[lev][iv]/(output->rfldir[lev][iv]+output->rfldn[lev][iv]));
          break;
        case OUTPUT_USER_SZA:
          fprintf (f, "%9.3f ", output->sza_h[iv]);
          break;
        case OUTPUT_USER_C_P:
          fprintf (f, "%13.6e ", output->atm.microphys.c_p[lev]);
          break;
        case OUTPUT_USER_HEAT:
          fprintf (f, "%13.6e ", output->heat[lev][iv]);
          break;
        case OUTPUT_USER_EMIS:
          fprintf (f, "%13.6e ", output->emis[lev][iv]);
          break;
        case OUTPUT_USER_ABS:
          fprintf (f, "%13.6e ", output->heat[lev][iv] - output->emis[lev][iv]);
          break;
        case OUTPUT_USER_W_RAD:
          fprintf (f, "%13.6e ", output->w_zout[lev][iv]/3600.0); /* m/day -> m/s */
          break;
        case OUTPUT_USER_M_RAD:
          fprintf (f, "%13.6e ", output->w_zout[lev][iv] * output->atm.microphys.dens_zout[MOL_AIR][lev] 
                                 * 1.e+6 * 1.e-3 * Input.atm.mol_mass[MOL_AIR] / avogadro );   /* kg/ (m2 day) */
          break;
        case OUTPUT_USER_P:
          fprintf (f, "%14.9f ", output->atm.microphys.press_zout[lev]);
          break;
        case OUTPUT_USER_T:
          fprintf (f, "%11.5f ", output->atm.microphys.temper_zout[lev]);
          break;
        case OUTPUT_USER_T_D:
          fprintf (f, "%11.5f ", output->atm.microphys.temper_d_zout[lev]);
          break;
        case OUTPUT_USER_T_SUR:
          fprintf (f, "%11.5f ", output->surface_temperature);
          break;
        case OUTPUT_USER_THETA:
          fprintf (f, "%11.5f ", output->atm.microphys.theta_zout[lev]);
          break;
        case OUTPUT_USER_THETA_E:
          fprintf (f, "%11.5f ", output->atm.microphys.theta_e_zout[lev]);
          break;
        case OUTPUT_USER_CLWC: /* (cwd[g/m3]/1000.)/rho_air[kg/m3] == cwc[kg/kg] */
          if (Input.i_wc!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_wc].microphys.lwc_zout[lev] * 1.e-3 / 
			(output->atm.microphys.dens_zout[MOL_AIR][lev]*1.e+6*1.e-3*Input.atm.mol_mass[MOL_AIR]/avogadro) );
          else
            fprintf (f, "%13.6e ", 0.0 );
          break;
        case OUTPUT_USER_CLWD:
          if (Input.i_wc!=-1)
	    fprintf (f, "%13.6e ", output->caoth[Input.i_wc].microphys.lwc_zout[lev]);
	  else
	    fprintf (f, "%13.6e ", 0.0 );
          break;
        case OUTPUT_USER_TCLW:
          if (Input.i_wc!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_wc].microphys.tcw);
	  else
	    fprintf (f, "%13.6e ", 0.0 );
	  break;
        case OUTPUT_USER_REFF_WAT:
          if (Input.i_wc!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_wc].microphys.effr_zout[lev]);
	  else
	    fprintf (f, "%13.6e ", 0.0 );
          break;
        case OUTPUT_USER_CIWC: /* (cwd[g/m3]/1000.)/rho_air[kg/m3] == cwc[kg/kg] */
          if (Input.i_ic!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_ic].microphys.lwc_zout[lev] * 1.e-3 / 
			(output->atm.microphys.dens_zout[MOL_AIR][lev]*1.e+6*1.e-3*Input.atm.mol_mass[MOL_AIR]/avogadro) );
	  else
	    fprintf (f, "%13.6e ", 0.0 );
          break;
        case OUTPUT_USER_CIWD:
          if (Input.i_ic!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_ic].microphys.lwc_zout[lev]);
	  else
	    fprintf (f, "%13.6e ", 0.0 );
          break;
        case OUTPUT_USER_TCIW:
          if (Input.i_ic!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_ic].microphys.tcw);
	  else
	    fprintf (f, "%13.6e ", 0.0 );
	  break;
        case OUTPUT_USER_REFF_ICE:
          if (Input.i_ic!=-1) {
            if (Input.caoth[isp].properties==PROP_IC_FU && Input.caoth[isp].fu2yang) 
              fprintf (f, "%13.6e ", output->caoth[Input.i_ic].microphys.effr_zout[lev]*(3.0*sqrt(3.0)/4.0));
            else
              fprintf (f, "%13.6e ", output->caoth[Input.i_ic].microphys.effr_zout[lev]);
	  }
	  else
	    fprintf (f, "%13.6e ", 0.0 );
          break;
        case OUTPUT_USER_TCW:
          if (Input.i_wc!=-1 && Input.i_ic!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_wc].microphys.tcw + output->caoth[Input.i_ic].microphys.tcw);
	  else if (Input.i_wc!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_wc].microphys.tcw);
	  else if (Input.i_ic!=-1)
            fprintf (f, "%13.6e ", output->caoth[Input.i_ic].microphys.tcw);
          else
	    fprintf (f, "%13.6e ", 0.0 );
	  break;
        case OUTPUT_USER_CC:
          fprintf (f, "%13.6e ", output->cf.cf_zout[lev]);
          break;
        case OUTPUT_USER_TCC:
          fprintf (f, "%13.6e ", output->cf.cf_total);
          break;
        case OUTPUT_USER_CLOUDS:
          if (Input.i_wc!=-1 && Input.i_ic!=-1)
            fprintf (f, "%13d ", 3);
	  else if (Input.i_wc!=-1)
            fprintf (f, "%13d ", 2); 
	  else if (Input.i_ic!=-1)
            fprintf (f, "%13d ", 1);
          else
	    fprintf (f, "%13d ", 0 );
          break;
        case OUTPUT_USER_N:
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[i_gas][lev]);
          break;
        case OUTPUT_USER_RHO:   /* 1.e+6: convert from cm-3 to m-3; 1.e-3: convert g -> kg */
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[i_gas][lev] 
                                 * 1.e+6 * 1.e-3 * Input.atm.mol_mass[i_gas] / avogadro);                   
          break;
        case OUTPUT_USER_VMR:
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[i_gas][lev]/
                                 output->atm.microphys.dens_zout[MOL_AIR][lev]);
          break;
        case OUTPUT_USER_MMR:
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", (output->atm.microphys.dens_zout[i_gas][lev]*
                                  Input.atm.mol_mass[i_gas]) /
                                  (output->atm.microphys.dens_zout[MOL_AIR][lev]*
                                  Input.atm.mol_mass[MOL_AIR]));
          break;
        case OUTPUT_USER_RH:
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[MOL_H2O][lev]/
                                 vapor_pressure(output->atm.microphys.temper_zout[lev]) * 100.0 );
          break;
        case OUTPUT_USER_RH_ICE:
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[MOL_H2O][lev]/
                                 vapor_pressure_over_ice(output->atm.microphys.temper_zout[lev]) * 100.0 );
          break;
        case OUTPUT_USER_WIND_U:
          fprintf (f, "%13.6e ", output->wind.u_zout[lev] );  /* in m/s */
          break;
        case OUTPUT_USER_WIND_V:
          fprintf (f, "%13.6e ", output->wind.v_zout[lev] );  /* in m/s */
          break;
        case OUTPUT_USER_WIND_W:
          fprintf (f, "%13.6e ", output->wind.w_zout[lev] );  /* in Pa/s !!! in m/s */
          break;
        case OUTPUT_USER_DTDX:
          fprintf (f, "%13.6e ", output->atm.microphys.dtheta_dx[lev] );  /* in K/m */
          break;
        case OUTPUT_USER_DTDY:
          fprintf (f, "%13.6e ", output->atm.microphys.dtheta_dy[lev] );  /* in K/m */
          break;
        case OUTPUT_USER_DTDZ:
          fprintf (f, "%13.6e ", output->atm.microphys.dtheta_dz[lev] );  /* in K/m */
          break;
        case OUTPUT_USER_HEAT_AD_X:
          fprintf (f, "%13.6e ", output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] *24*60*60 );     /* in K/day */
          break;
        case OUTPUT_USER_HEAT_AD_Y:
          fprintf (f, "%13.6e ", output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] *24*60*60 );     /* in K/day */
          break;
        case OUTPUT_USER_HEAT_AD_Z:
          fprintf (f, "%13.6e ", output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev] *24*60*60 );     /* in K/day */
          break;
        case OUTPUT_USER_HEAT_AD:
          fprintf (f, "%13.6e ", (output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] + 
                                  output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] +
                                  output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev]  ) *24*60*60 ); /* in K/day */
          break;
	case OUTPUT_USER_SSLIDAR_NPHOT:
          fprintf (f, "%13.6e ", output->sslidar_nphot[lev][iv]);
          break;
	case OUTPUT_USER_SSLIDAR_NPHOT_Q:
          fprintf (f, "%13.6e ", output->sslidar_nphot_q[lev][iv]);
          break;
	case OUTPUT_USER_SSLIDAR_RATIO:
          fprintf (f, "%13.6e ", output->sslidar_ratio[lev][iv]);
          break;
        default:
          fprintf (stderr, "Error, unknown user output %d\n", Input.output_user[io]);
	  return -1;
        }
      }
      if (Input.n_output_user>0)
        fprintf (f, "\n");


      /* radiance output */
      if ( Input.output_user_flag == 0) {  /* User has not specified any output. We supply what we think the 
                                              user wants. At least this worked for us as users for many years. 
                                              AK20091116  */ 
      
         if (Input.rte.numu > 0 && !output->print_phi) {
	    if ( Input.raman ) {
	      numu = Input.rte.maxumu-Input.rte.nstr; 
   	    }
	    else {
	      numu = Input.rte.numu;
	    }
            for (iu=0;iu<numu;iu++) {
	      if ( Input.raman ) iiu = Input.rte.umuind[iu];
	      else iiu = iu;
              fprintf (f, "%7.4f %16.6e \n", Input.rte.umu[iiu], output->u0u[lev][iiu][iv]);
	    }
         }
         if (output->print_phi && Input.rte.numu> 0 && write_rad == TRUE) {
	   switch (Input.rte.solver) {
	   case DISORT:
           case DISORT2:
           case CDISORT:
           case SDISORT:
	   case SPSDISORT:
	   case SSS:
	   case SSSI:
	   case TZS:
	     if ( Input.raman ) {
	       numu = Input.rte.maxumu-Input.rte.nstr; 
  	     }
  	     else {
  	       numu = Input.rte.numu;
  	     }
             fprintf (f, "                      ");
             for (j=0;j<Input.rte.nphi;j++)
               fprintf (f, " %16.3f ", Input.rte.phi[j]);
             fprintf (f, "\n");
             for (iu=0;iu<numu;iu++) {
  	     if ( Input.raman ) iiu = Input.rte.umuind[iu];
  	     else iiu = iu;
               fprintf (f, "%7.4f %16.9e ", Input.rte.umu[iiu], output->u0u[lev][iiu][iv]);
               for (j=0;j<Input.rte.nphi;j++)
                  fprintf (f, " %16.9e ", output->uu[lev][j][iiu][iv]);
               fprintf (f, "\n");
             }
	     break;
	   case TWOSTR:
	   case TWOSTRPP:
	   case SUCORD:
	   case MONTECARLO:
	   case POLRADTRAN:
	   case NULLSOLVER:
	   case RODENTS:
	   case TWOSTREBE:
	   case CTWOSTR:
	   case SSLIDAR:
	     break;
	   default:
	     fprintf (stderr, "Error, unknown rte_solver %d (line %d, function '%s' in '%s')\n", Input.rte.solver, __LINE__, __func__, __FILE__);
	     return -1;
	   }
         }
       } /* radiance output */

    } /* for (lev=0;lev<output->atm.nzout;lev++) */
  } /* for (iv=0;iv<output->wl.nlambda_h;iv++) */ 

  return 0;
}



int write_netCDF_output (input_struct Input, output_struct *output)
{

#if HAVE_LIBNETCDF

  int status=0;
  int io=0, iv=0, lev=0, i_gas=0;

  int ncid=0;

  int *id_dim  = NULL;
  int id_nx      = 0;
  int id_ny      = 0;
  int id_ntime   = 0;

  char dim_x_name[10];
  char dim_y_name[10];
  char x_name[10];
  char y_name[10];

  int periodic_x = NOT_DEFINED_INTEGER;
  int periodic_y = NOT_DEFINED_INTEGER;

  double x = NOT_DEFINED_FLOAT;
  double y = NOT_DEFINED_FLOAT;

  int id_nzout   = 0;
  int id_nlambda = 0;

  int *id_data  = NULL;
  int id_levels = 0;
  int id_lambda = 0;

  float *model_levels=NULL;

  int id_phi  = 0;
  int id_nphi = 0;
  int id_umu  = 0;
  int id_numu = 0;

  size_t nx   = NOT_DEFINED_INTEGER;
  size_t ny   = NOT_DEFINED_INTEGER;
  float dt = 0.0;

  double *x_grid         = NULL;
  double *y_grid         = NULL;

  int itime1=0, itime2=0, nt=0;

  int ix  = NOT_DEFINED_INTEGER;
  int iy  = NOT_DEFINED_INTEGER;
  int itime = NOT_DEFINED_INTEGER;

  size_t index_3D[3] = {0,0,0};
  size_t index_4D[4] = {0,0,0,0};
  size_t index_5D[5] = {0,0,0,0,0};
  size_t index_6D[6] = {0,0,0,0,0,0};
  size_t index_7D[7] = {0,0,0,0,0,0,0};

  size_t n_4D[4] = {0,0,0,0};
  size_t n_5D[5] = {0,0,0,0,0};
  size_t n_6D[6] = {0,0,0,0,0,0};
  size_t n_7D[7] = {0,0,0,0,0,0,0};

  int id_dim_array3[3];
  int id_dim_array4[4]; 
  int id_dim_array5[5];
  int id_dim_array6[6];
  int id_dim_array7[7];
  const int dim3=3, dim4=4, dim5=5;

  int *valid_result=NULL;    /* marker, if the RTE solver can calculate the disired quantity */
  int simulation_done=TRUE;  /* marker for valid (not NaN and not Infinity) results          */

  float tmp_float = NOT_DEFINED_FLOAT;
  float  *tmp_data_1D = NULL;
  float **tmp_data_3D = NULL;

  char *gas      = NULL;
  char var_name[20];
  char long_name[40];
  char irradiance_units[40];
  char radiance_units[40];

  char function_name[]="write_netCDF_output";
  char file_name[]="uvspec_lex.l";

  if (Input.verbose)
    fprintf (stderr, " ... start writing result to netCDF file %s\n", Input.filename[FN_OUTFILE]);

  /* open netcdf file */
  status = nc_open (Input.filename[FN_OUTFILE], NC_WRITE, &ncid); /* read and write access */
  if (status != NC_NOERR) {
    fprintf (stderr, "Error %d opening netCDF file %s\n", status, Input.filename[FN_OUTFILE]);
    return status;
  }

  /* ----------------------------------------------------------------------- */

  /* get identification number and read data */

  /* get id for dimension "time" (the dimension, not the variable) */
  if      (Input.time_zone==TIME_UTC)
    status = nc_inq_dimid (ncid, "time", &(id_ntime));
  else if (Input.time_zone==TIME_LAT)
    status = nc_inq_dimid (ncid, "local_apparent_time", &(id_ntime)); 
  else {
    fprintf (stderr, "Error, unknown time_zone %d (line %d, function '%s' in '%s')\n", Input.time_zone, __LINE__, __func__, __FILE__);
    return status;
  }
  if (status!=NC_NOERR) {
    fprintf (stderr, "Error %d locating 'ntime' in %s \n", status, Input.filename[FN_OUTFILE]);
    return status;
  }


  switch (Input.output_format)  {
  case OUTPUT_FORMAT_NETCDF:
    strcpy (dim_x_name, "lon");
    strcpy (dim_y_name, "lat");
    strcpy (x_name, "lon");
    strcpy (y_name, "lat");
    x = Input.longitude;
    y = Input.latitude;
    periodic_x = FALSE;
    periodic_y = TRUE;
    break;
  case OUTPUT_FORMAT_SATPIC: 
    strcpy (dim_x_name, "pixel_x");
    strcpy (dim_y_name, "pixel_y");
    strcpy (x_name, "pixel_x");
    strcpy (y_name, "pixel_y");
    x = Input.sat_pixel_x;
    y = Input.sat_pixel_y;
    periodic_x = FALSE;
    periodic_y = FALSE;
    break;
  default:
    fprintf (stderr, "Error, unknown output_format %d (line %d, function '%s' in '%s')\n", Input.output_format, __LINE__, __func__, __FILE__);
    return (-1);
  }

  /* get id for dimension "lat/pixel_x" (the dimension, not the variable) */
  status = nc_inq_dimid (ncid, dim_x_name, &(id_nx));
  if (status!=NC_NOERR) {
    fprintf (stderr, "Error %d locating '%s' in %s \n", status, dim_x_name, Input.filename[FN_OUTFILE]);
    return status;
  }

  /* read latitude array (must already be stored in the file) */
  status = alloc_and_read_netCDF_1D_double(ncid, dim_x_name, &nx, x_name, &(x_grid));
  if (status != 0) {
    fprintf (stderr, "Error %d reading %s from %s\n", status, x_name, Input.filename[FN_OUTFILE]);
    return status;
  }

  /* get id for dimension "lon/pixel_y" (the dimension, not the variable) */
  status = nc_inq_dimid (ncid, dim_y_name, &(id_ny));
  if (status!=NC_NOERR) {
    fprintf (stderr, "Error %d locating '%s' in %s \n", status, dim_y_name, Input.filename[FN_OUTFILE]);
    return status;
  }

  /* read longitude array (must already be stored in the file) */
  status = alloc_and_read_netCDF_1D_double( ncid, dim_y_name, &ny, y_name, &(y_grid) );
  if (status != 0) {
    fprintf (stderr, "Error %d reading %s from %s\n", status, y_name, Input.filename[FN_OUTFILE]);
    return status;
  }


  /* ----------------------------------------------------------------------- */

  /* search index */

  if (Input.time_zone==TIME_UTC)
    status = get_time_index (ncid, Input.UTC, TIME_NEAREST_DATE, 
                             &(nt), &(itime1), &(itime2), &(dt),
                             Input.verbose, Input.quiet);
  else if (Input.time_zone==TIME_LAT)
    status = get_local_apparent_time_index (ncid, Input.LAT, TIME_NEAREST_DATE, 
                                            &(nt), &(itime1), &(itime2), &(dt),
                                            Input.verbose, Input.quiet);
  else {
    fprintf (stderr, "Error, unknown time_zone %d (line %d, function '%s' in '%s')\n", Input.time_zone, __LINE__, __func__, __FILE__);
    return status;
  }


  if (status != 0) {
    fprintf (stderr, "Error %d, during get_time_index in write_netCDF_output (uvspec_lex.l)\n", status);
    return status;
  }
  itime = itime1;


  /* get lat/pixel_x-index in the result grid, where to write data */
  ix = get_grid_index(x, x_grid, nx, periodic_x);
  if (ix < 0) {
    fprintf (stderr, "Error %d, while searching index in %s_grid \n", ix, x_name);
    return ix;
  }

  /* get lon/pixel_y-index in the result grid, where to write data */
  iy = get_grid_index(y, y_grid, ny, periodic_y);
  if (iy < 0) {
    fprintf (stderr, "Error %d, while searching index in %s_grid \n", iy, y_name);
    return iy;
  }



  /* ----------------------------------------------------------------------- */

  /* check if lambda exists, (write lambda if not), and get dimension id for nlambda */
  write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                         "lambda", &id_lambda, "wavelength", "nm", output->wl.lambda_h, 
                         "lambda", &id_nlambda, output->wl.nlambda_h);

  /* check if zout exists, (write zout if not), and get dimension id for zout */
  switch (Input.atm.zout_source) {
  case OUTLEVEL_ZOUT_ABOVE_SUR:
    write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                           "zout_sur", &id_levels, "altitude above surface", "km", output->atm.zout_sur_org, 
                           "zout_sur", &id_nzout, output->atm.nzout_org );
    break;
  case OUTLEVEL_ZOUT_ABOVE_SEA:
  case OUTLEVEL_ATM_LEVELS:
  case OUTLEVEL_ALL_LEVELS:
    write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                           "zout_sea", &id_levels, "altitude above sea level", "km", output->atm.zout_sea_org, 
                           "zout_sea", &id_nzout, output->atm.nzout_org );
    break;
  case OUTLEVEL_PRESS:
    write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                           "p", &id_levels, "pressure", "hPa", Input.atm.press_zout, 
                           "p", &id_nzout, Input.atm.nzout);
    break;
  case OUTLEVEL_MODEL_LEVELS:
  case OUTLEVEL_MODEL_LAYERS:
  case OUTLEVEL_MODEL_LEVELS_AND_LAYERS:

    /* write simple the numbers from 1 to mlev, instead of p or z */
    if ((model_levels = calloc (output->atm.nzout_org, sizeof(float) )) == NULL) {
      fprintf (stderr,"Error, allocating memory for 'model_levels' in %s (in %s)\n", function_name, file_name);
      return -1;
    }

    for (lev=0;lev<output->atm.nzout;lev++)
      model_levels[lev]=lev+1;

    write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                           "mlev", &id_levels, "model_levels", "", model_levels, 
                           "mlev", &id_nzout, output->atm.nzout_org);

    break;
  default:
    fprintf (stderr, "Error, unknown zout_source %d (line %d, function '%s' in '%s')\n", Input.atm.zout_source, __LINE__, __func__, __FILE__);
    return -2;
  }

  /* ----------------------------------------------------------------------- */


  /* initialisation of the index, where to write */
  id_dim_array3[POSITION_TIME_3D] = id_ntime;  
  id_dim_array3[POSITION_LAT_3D ] = id_ny;     /* one before last dimension is default y-axis in ncview */
  id_dim_array3[POSITION_LON_3D ] = id_nx;     /*            last dimension is default x-axis in ncview */

  /* initialisation of the index, where to write */
  index_3D[POSITION_TIME_3D] = itime;
  index_3D[POSITION_LAT_3D]  = iy;               /* latitude or x-pixel */
  index_3D[POSITION_LON_3D]  = ix;               /* longitude or y-pixel */
 
  /* order of dimensions */
  id_dim_array4[POSITION_TIME_4D] = id_ntime;
  id_dim_array4[POSITION_ZOUT_4D] = id_nzout;
  id_dim_array4[POSITION_LAT_4D ] = id_ny;     /* one before last dimension is default y-axis in ncview */
  id_dim_array4[POSITION_LON_4D ] = id_nx;     /*            last dimension is default x-axis in ncview */

  /* initialisation of the index, where to write */
  index_4D[POSITION_TIME_4D] = itime;
  n_4D[POSITION_ZOUT_4D]     = output->atm.nzout; /* loop over zout */
  index_4D[POSITION_LAT_4D]  = iy;                /* latitude or x-pixel */
  index_4D[POSITION_LON_4D]  = ix;                /* longitude or y-pixel */

  /* order of dimensions */
  id_dim_array5[POSITION_TIME_5D  ] = id_ntime;
  id_dim_array5[POSITION_LAMBDA_5D] = id_nlambda;
  id_dim_array5[POSITION_ZOUT_5D  ] = id_nzout;
  id_dim_array5[POSITION_LAT_5D   ] = id_ny;  /* one before last dimension is default y-axis in ncview */
  id_dim_array5[POSITION_LON_5D   ] = id_nx;  /*            last dimension is default x-axis in ncview */

  /* initialisation of the index, where to write */
  index_5D[POSITION_TIME_5D] = itime;
  n_5D[POSITION_LAMBDA_5D]   = output->wl.nlambda_h; /* loop over lambda */    
  n_5D[POSITION_ZOUT_5D]     = output->atm.nzout;    /* loop over zout */ 
  index_5D[POSITION_LAT_5D]  = iy;                   /* latitude or x-pixel */
  index_5D[POSITION_LON_5D]  = ix;                   /* longitude or y-pixel */

  /* ----------------------------------------------------------------------- */

  /* allocate memory for id_dimensions */
  if ((id_dim  = (int *) calloc (Input.n_output_user, sizeof (int *))) == NULL) {
    fprintf (stderr,"Error, allocating memory for id_dim  in write_netCDF_output (in uvspec_lex.l)\n");
    return -1;
  }

  /* allocate memory for id_variables */
  if ((id_data = (int *) calloc (Input.n_output_user, sizeof (int *))) == NULL) {
    fprintf (stderr,"Error, allocating memory for id_data in write_netCDF_output (in uvspec_lex.l)\n");
    return -2;
  }

  /* allocate memory for marker, if result can be calculated by the solver */
  if ((valid_result = (int *) calloc (Input.n_output_user, sizeof (int *))) == NULL) {
    fprintf (stderr,"Error, allocating memory for valid_result in write_netCDF_output (in uvspec_lex.l)\n");
    return -2;
  }

  /* check if result exist for the specific solver */
  for (io=0; io<Input.n_output_user; io++)
    status += check_if_result_exists(Input.output_user, Input.n_output_user, Input.rte.solver, 
              &(valid_result), Input.quiet);
  if (status != 0) {
    fprintf (stderr,"Error, during check_if_result_exists in write_netCDF_output (in uvspec_lex.l)\n");
    return -2;
  }

  /* ----------------------------------------------------------------------- */

  switch(Input.processing) {
  case PROCESS_NONE:
    switch(Input.output_unit) {
    case UNIT_NOT_DEFINED:
      strcpy (irradiance_units, "unknown units");
      strcpy (radiance_units,  "unknown units");
      break;
    case UNIT_PER_NM:
      strcpy (irradiance_units, "(W or mW)/(m^2 nm)");
      strcpy (radiance_units,  "(W or mW)/(m^2 nm sr)");
      break;
    case UNIT_PER_CMm1:
      strcpy (irradiance_units, "(W or mW)/(m^2 cm^-1)");
      strcpy (radiance_units,  "(W or mW)/(m^2 cm^-1 sr)");
      break;
    case UNIT_PER_CK_BAND:
      strcpy (irradiance_units, "(W or mW)/(m^2 ck-band)");
      strcpy (radiance_units,  "(W or mW)/(m^2 ck-band sr)");
      break;
      break;
    default:
      fprintf (stderr, "Error, unknown output units %d (line %d, function '%s' in '%s')\n", Input.output_unit, __LINE__, __func__, __FILE__);
      return -1;
    }
    break;
  case PROCESS_INT:
  case PROCESS_SUM:
    strcpy (irradiance_units, "(W or mW)/m^2");
    strcpy (radiance_units,  "(W or mW)/(m^2 sr)");
    break;

  case PROCESS_RGBNORM:
    strcpy (irradiance_units, "(red, green, blue) (normalised)");
    strcpy (radiance_units,   "(red, green, blue) (normalised)");
    break;

  case PROCESS_RGB:
    strcpy (irradiance_units, "(red, green, blue)");
    strcpy (radiance_units,   "(red, green, blue)");
    break;

  default:
    fprintf (stderr, "Error, unknown output processing %d (line %d, function '%s' in '%s')\n", Input.processing, __LINE__, __func__, __FILE__);
    return -1;
  }


  /* ----------------------------------------------------------------------- */
  /* write the results into the netCDF file                                  */

  for (io=0; io<Input.n_output_user; io++) {

    if (valid_result[io] == TRUE) {

      switch (Input.output_user[io]) {

      case OUTPUT_USER_WAVE:
        /* is done in each case, already done above, do nothing here */
      break;

      case OUTPUT_USER_ZOUT_SUR:
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
          /* is done in each case, already done above, do nothing here  */

          /* if cold point tropopause is output level, than we like to save the height in the output */
          if (output->atm.microphys.z_cpt_sea != NOT_DEFINED_INTEGER) {
            tmp_float = output->atm.microphys.z_cpt_sea - output->alt.altitude;
            status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                            "z_cpt_sur", "cold point tropopause altitude above surface", "km", tmp_float, 
                                            id_dim_array3, index_3D, dim3);
            if (status!=0) {
              fprintf (stderr, " *** Error, while writing 'z_cpt_sur' into %s, %s (%s)\n", 
                               Input.filename[FN_OUTFILE], function_name, file_name );
            }
          }
          break;
        case OUTLEVEL_PRESS:
        case OUTLEVEL_MODEL_LEVELS:
	case OUTLEVEL_MODEL_LAYERS:
	case OUTLEVEL_MODEL_LEVELS_AND_LAYERS:
          status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "zout_sur", "altitude above surface", "km", output->atm.zout_sur, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'zout_sur' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
          } 
          break;
        default:
	  fprintf (stderr, "Error, unknown zout_source %d (line %d, function '%s' in '%s')\n", Input.atm.zout_source, __LINE__, __func__, __FILE__);
          return -2;
        }
        break;

      case OUTPUT_USER_ZOUT_SEA:
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
          /* is done in each case, already done above, do nothing here  */

          /* if cold point tropopause is output level, than we like to save the height in the output */
          if (output->atm.microphys.z_cpt_sea != NOT_DEFINED_INTEGER) {
            status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                            "z_cpt_sea", "cold point tropopause altitude above sealevel", "km", output->atm.microphys.z_cpt_sea, 
                                            id_dim_array3, index_3D, dim3);
            if (status!=0) {
              fprintf (stderr, " *** Error, while writing 'z_cpt_sea' into %s, %s (%s)\n", 
                               Input.filename[FN_OUTFILE], function_name, file_name );
            }
          }
          break;
        case OUTLEVEL_PRESS:
        case OUTLEVEL_MODEL_LEVELS:
	case OUTLEVEL_MODEL_LAYERS:
	case OUTLEVEL_MODEL_LEVELS_AND_LAYERS:
          status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "zout_sea", "altitude above sealevel", "km", output->atm.zout_sea, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'zout_sea' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
          } 
          break;
        default:
	  fprintf (stderr, "Error, unknown zout_source %d (line %d, function '%s' in '%s')\n", Input.atm.zout_source, __LINE__, __func__, __FILE__);
          return -2;
        }
        break;

      case OUTPUT_USER_Z_SUR:
        status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "z_sur", "surface altitude", "km", output->alt.altitude, 
                                        id_dim_array3, index_3D, dim3);
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'z_sur' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        } 
        break;

      case OUTPUT_USER_WAVENUMBER:
       /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->wl.nlambda_h, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          tmp_data_1D[iv] = 1.0E7/output->wl.lambda_h[iv];  /* convert from nm to cm-1 */

        status = write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "wavenumber", &(id_data[io]), "wavenumber", "cm-1", output->wl.lambda_h, 
                                        "wavenumber", &(id_dim[io]),  output->wl.nlambda_h);
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'wavenumber' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }  
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_EDIR:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "edir", "direct irradiance", irradiance_units, output->rfldir, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'edir' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }  
        break;

      case OUTPUT_USER_EGLO:
        if ((status = ASCII_calloc_float (&tmp_data_3D, Input.n_output_user, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++)
            tmp_data_3D[lev][iv] = output->rfldir[lev][iv]+output->rfldn[lev][iv];
  
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "eglo", "global ( = direct + diffuse downward ) irradiance", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'eglo' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }  

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;
  
      case OUTPUT_USER_EDN:

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "edn", "diffuse downward irradiance", irradiance_units, output->rfldn, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'edn' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;
  
      case OUTPUT_USER_EUP:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "eup", "diffuse upward irradiance", irradiance_units, output->flup, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'eup' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_ENET:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->rfldir[lev][iv]+output->rfldn[lev][iv]-output->flup[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "enet", "net ( = global - upward ) irradiance", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'enet' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_ESUM:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->rfldir[lev][iv]+output->rfldn[lev][iv]+output->flup[lev][iv];
  
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "esum", "total ( = global + upward ) irradiance", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'esum' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FDIR:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavgso[lev][iv];
  
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "fdir", "direct actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'fdir' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FGLO:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*(output->uavgso[lev][iv]+output->uavgdn[lev][iv]);

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "fglo", "global ( = direct + diffuse downward ) actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'fglo' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FDN:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavgdn[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "fdn", "diffuse downward actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'fdn' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FUP:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavgup[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "fup", "diffuse upward actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'fup' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_F:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavg[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "f", "total actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'f' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_UDIR:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "uavgdir", "direct mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavgso, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'uavgdir' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_UGLO:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->uavgso[lev][iv]+output->uavgdn[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "uavgglo", "global mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'uavgglo' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_UDN:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "uavgdn", "diffusive downward mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavgdn, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'uavgdn' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_UUP:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "uavgup", "diffusive upward mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavgup, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'uavgup' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_U:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "uavg", "diffusive upward mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavg,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'uavg' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_ALB:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->flup[lev][iv]/(output->rfldir[lev][iv]+output->rfldn[lev][iv]);

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "albedo", "albedo", " ", tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'albedo' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_SZA:
        status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "SZA", "solar zenith angle", "degrees", output->sza_h[0], 
                                        id_dim_array3, index_3D, dim3);
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'SZA' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_C_P:

       /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "c_p", "c_p", "J/(kg K)", output->atm.microphys.c_p, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing '%s' into %s, %s (%s)\n", 
                           var_name, Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_HEAT:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "heat", "heating rate", "K/day", output->heat,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'heat' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_EMIS:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "emis", "emission rate", "K/day", output->emis,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'emis' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_ABS:

        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for tmp_data in %s (%s)\n", function_name, file_name);
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->heat[lev][iv] - output->emis[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "abs", "absorption rate", "K/day", tmp_data_3D,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'abs' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_W_RAD:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for tmp_data in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->w_zout[lev][iv]/3600.0; /* m/day -> m/s */

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "w_rad", "radiative vertical wind", "m/s", tmp_data_3D,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'w_rad' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_M_RAD:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for tmp_data in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->w_zout[lev][iv] * output->atm.microphys.dens_zout[MOL_AIR][lev] 
                                   * 1.0e+6 * 1.0e-3 * Input.atm.mol_mass[MOL_AIR] / avogadro;   /* kg/ (m2 day) */

        status = write_netCDF_5D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "m_rad", "radiative vertical mass flux", "kg/(m2 day)", tmp_data_3D,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'm_rad' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_P:
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
        case OUTLEVEL_MODEL_LEVELS:
	case OUTLEVEL_MODEL_LAYERS:
	case OUTLEVEL_MODEL_LEVELS_AND_LAYERS:
          status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "p", "pressure", "hPa", output->atm.microphys.press_zout, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'p' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
          }
          break;
        case OUTLEVEL_PRESS:
          /* is done in each case, already done above in 1D, do nothing here  */
          /* if cold point tropopause is output level, than we like to save the height in the output */
          if (output->atm.microphys.press_cpt != NOT_DEFINED_INTEGER) {
            status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                            "p_cpt", "cold point tropopause pressure", "hPa", output->atm.microphys.press_cpt, 
                                            id_dim_array3, index_3D, dim3);
            if (status!=0) {
              fprintf (stderr, " *** Error, while writing 'p_cpt' into %s, %s (%s)\n", 
                               Input.filename[FN_OUTFILE], function_name, file_name );
            }
          }
          break;
        default:
	  fprintf (stderr, "Error, unknown zout_source %d (line %d, function '%s' in '%s')\n", Input.atm.zout_source, __LINE__, __func__, __FILE__);
          return -2;
        }
        break;

      case OUTPUT_USER_T:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "T", "temperature", "K", output->atm.microphys.temper_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'T' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
       break;

      case OUTPUT_USER_T_D:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "T_d", "dew point temperature", "K", output->atm.microphys.temper_d_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'T_d' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_T_SUR:
        status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "T_sur", "surface skin temperature", "K", output->surface_temperature, 
                                        id_dim_array3, index_3D, dim3);
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'T_sur' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_THETA:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "theta", "potential temperature", "K", 
                                        output->atm.microphys.theta_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'theta' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_THETA_E:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "theta_e", "equivalent potential temperature", "K", 
                                        output->atm.microphys.theta_e_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'theta_e' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_CLWC: /* lwc[g/m3]*1000./rho_air[kg/m3] == clwc[kg/kg] */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in %s (%s)\n", var_name, function_name, file_name);
          return -1;
        }

        /* calculate data */  /* 1.e-3 == convert g -> kg, devided by rho air */
        for (lev=0;lev<output->atm.nzout;lev++) 
          if (Input.i_wc!=-1)
            tmp_data_1D[lev] = output->caoth[Input.i_wc].microphys.lwc_zout[lev] * 1.e-3 / 
                             (output->atm.microphys.dens_zout[MOL_AIR][lev]*1.e+6*1.e-3*Input.atm.mol_mass[MOL_AIR]/avogadro) ;
	  else
	    tmp_data_1D[lev] = 0.0;

        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "CLWC", "cloud liquid water content", "kg/kg", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing '%s' into %s, %s (%s)\n", 
                           var_name, Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_CLWD:
        if (Input.i_wc!=-1) {
          status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "CLWD", "cloud ice water density",    "g/m3", output->caoth[Input.i_wc].microphys.lwc_zout, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'CLWD' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
	  }
        }
        break;

      case OUTPUT_USER_TCLW:
        if (Input.i_wc!=-1) {
          status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "TCLW", "Total Column Liquid Water", "kg/m2", output->caoth[Input.i_wc].microphys.tcw, 
                                          id_dim_array3, index_3D, dim3);
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'TCLW' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
          }
	}
        break;

      case OUTPUT_USER_REFF_WAT:
        if (Input.i_wc!=-1) {
          status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "reff_water", "effective radius liquid water cloud", "micro m", output->caoth[Input.i_wc].microphys.effr_zout, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'reff_wat' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
          }
	}
        break;

      case OUTPUT_USER_CIWC: /* lwc[g/m3]*1000./rho_air[kg/m3] == clwc[kg/kg] */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */  /* 1.e-3 == convert g -> kg, devided by rho air */
        for (lev=0;lev<output->atm.nzout;lev++) 
          if (Input.i_ic!=-1)
            tmp_data_1D[lev] = output->caoth[Input.i_ic].microphys.lwc_zout[lev] * 1.e-3 / 
	      (output->atm.microphys.dens_zout[MOL_AIR][lev]*1.e+6*1.e-3*Input.atm.mol_mass[MOL_AIR]/avogadro) ;
          else
            tmp_data_1D[lev] = 0.0;
        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "CIWC", "cloud ice water content", "kg/kg", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing '%s' into %s, %s (%s)\n", 
                           var_name, Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_CIWD:
        if (Input.i_ic!=-1) {
          status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "CIWD", "cloud ice water density",    "g/m3", output->caoth[Input.i_ic].microphys.lwc_zout, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'CIWD' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
          }
	}
        break;

      case OUTPUT_USER_TCIW:
        if (Input.i_ic!=-1) {
          status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                          "TCIW", "Total Column Ice Water", "kg/m2", output->caoth[Input.i_ic].microphys.tcw, 
                                          id_dim_array3, index_3D, dim3);
          if (status!=0) {
            fprintf (stderr, " *** Error, while writing 'TCIW' into %s, %s (%s)\n", 
                             Input.filename[FN_OUTFILE], function_name, file_name );
          }
	}
        break;

      case OUTPUT_USER_REFF_ICE:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        if (Input.i_ic!=-1) {
          if (Input.caoth[isp].properties==PROP_IC_FU && Input.caoth[isp].fu2yang) 
	    for (lev=0; lev<output->atm.nzout; lev++)
	      tmp_data_1D[lev] = output->caoth[Input.i_ic].microphys.effr_zout[lev] * (3.0*sqrt(3.0)/4.0);
          else
            for (lev=0; lev<output->atm.nzout; lev++)
	      tmp_data_1D[lev] = output->caoth[Input.i_ic].microphys.effr_zout[lev];
        }
	else
	  for (lev=0; lev<output->atm.nzout; lev++)
	    tmp_data_1D[lev] = 0.0;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "reff_ice", "effective radius ice water cloud",    "micro m", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'reff_ice' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        /* free memory */
        free(tmp_data_1D);

        break;

      case OUTPUT_USER_TCW:
        if (Input.i_wc!=-1 && Input.i_ic!=-1)
          status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE],
                                          "TCW", "Total Column Water", "kg/m2", output->caoth[Input.i_wc].microphys.tcw + output->caoth[Input.i_ic].microphys.tcw, 
                                          id_dim_array3, index_3D, dim3);
	else if (Input.i_wc!=-1)
          status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE],
                                          "TCW", "Total Column Water", "kg/m2", output->caoth[Input.i_wc].microphys.tcw,
                                          id_dim_array3, index_3D, dim3);
	else if (Input.i_ic!=-1)
          status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE],
                                          "TCW", "Total Column Water", "kg/m2", output->caoth[Input.i_ic].microphys.tcw, 
                                          id_dim_array3, index_3D, dim3);

        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'TCW' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_CC:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "CC", "cloud cover",    "0-1", output->cf.cf_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'CC' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_TCC:
        status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "TCC", "Total Cloud Cover", "0-1", output->cf.cf_total, 
                                        id_dim_array3, index_3D, dim3);
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'TCC' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_CLOUDS:
        iscr=0;
	if (Input.i_wc!=-1)
	  iscr+=2;
	if (Input.i_ic!=-1)
	  iscr+=1;
	  
        status = write_netCDF_3D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "clouds", "Clouds: IC=1, WC=2, IC+WC=3", "0-3", iscr, 
                                        id_dim_array3, index_3D, dim3);
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'clouds' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;

      case OUTPUT_USER_N:
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "n_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces before and behind text */

        strcpy (long_name, "number density of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        var_name, long_name, "1/cm^3", output->atm.microphys.dens_zout[i_gas], 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing '%s' into %s, %s (%s)\n", 
                           var_name, Input.filename[FN_OUTFILE], function_name, file_name );
        }
        break;
  
      case OUTPUT_USER_RHO:   
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "rho_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces before and behind text */

        strcpy (long_name, "mass density of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */  /* 1.e+6: convert from cm-3 to m-3; 1.e-3: convert g -> kg */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[i_gas][lev] 
                               * 1.e+6 * 1.e-3 * Input.atm.mol_mass[i_gas] / avogadro ;  

        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        var_name, long_name, "kg/m^3", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing '%s' into %s, %s (%s)\n", 
                           var_name, Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_VMR:
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "vmr_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces before and behind text */

        strcpy (long_name, "volume mixing ratio of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[i_gas]  [lev]/
                             output->atm.microphys.dens_zout[MOL_AIR][lev];  

        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        var_name, long_name, " ", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing '%s' into %s, %s (%s)\n", 
                           var_name, Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_MMR:
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "mmr_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces */

        strcpy (long_name, "mass mixing ratio of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = (output->atm.microphys.dens_zout[i_gas]  [lev] * Input.atm.mol_mass[i_gas]  ) /
                             (output->atm.microphys.dens_zout[MOL_AIR][lev] * Input.atm.mol_mass[MOL_AIR]);

        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        var_name, long_name, "kg/kg", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing '%s' into %s, %s (%s)\n", 
                           var_name, Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_RH:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[MOL_H2O][lev]/
                             vapor_pressure(output->atm.microphys.temper_zout[lev]) * 100.0;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "rh", "relative humidity over water", "per cent", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'rh' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_RH_ICE:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[MOL_H2O][lev]/
                             vapor_pressure_over_ice(output->atm.microphys.temper_zout[lev]) * 100.0;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "rh_ice", "relative humidity over ice", "per cent", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'rh_ice' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_WIND_U:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "u", "zonal wind", "m/s", output->wind.u_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'u' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
       break;

      case OUTPUT_USER_WIND_V:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "v", "meridional wind", "m/s", output->wind.v_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'v' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
       break;

      case OUTPUT_USER_WIND_W:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "w", "vertical wind", "Pa/s", output->wind.w_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'w' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
       break;

      case OUTPUT_USER_DTDX:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "dtheta_dx", "zonal gradiant of potential temperature", "K/m",    
                                        output->atm.microphys.dtheta_dx, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'dtheta_dx' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
       break;

      case OUTPUT_USER_DTDY:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "dtheta_dy", "merional gradiant of potential temperature", "K/m", 
                                        output->atm.microphys.dtheta_dy, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'dtheta_dy' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
       break;

      case OUTPUT_USER_DTDZ:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                       "dtheta_dz", "vertical gradiant of potential temperature", "K/m", 
                                        output->atm.microphys.dtheta_dz, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'dtheta_dz' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }
       break;

      case OUTPUT_USER_HEAT_AD_X:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] *24*60*60; /* K/day */

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "heat_ad_x", "zonal heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'heat_ad_x' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);

       break;

     case OUTPUT_USER_HEAT_AD_Y:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] *24*60*60; /* K/day */

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "heat_ad_y", "meridional heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'heat_ad_y' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);

       break;

     case OUTPUT_USER_HEAT_AD_Z:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev] *24*60*60;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "heat_ad_z", "vertical heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'heat_ad_z' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);

       break;

     case OUTPUT_USER_HEAT_AD:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = (output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] + 
                              output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] +
                              output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev]  ) *24*60*60;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_OUTFILE], 
                                        "heat_ad", "heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index, &(simulation_done));
        if (status!=0) {
          fprintf (stderr, " *** Error, while writing 'heat_ad' into %s, %s (%s)\n", 
                           Input.filename[FN_OUTFILE], function_name, file_name );
        }

        /* free memory */
        free(tmp_data_1D);

       break;

      default:
        fprintf (stderr, "Error, unknown user output %d (line %d, function '%s' in '%s')\n", 
                         Input.output_user[io], __LINE__, __func__, __FILE__);
        return -1;
      }
    }
  }



  if (Input.rte.numu > 0 && Input.rte.nphi > 0) {
    /* write phi array */
    status = write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                           "phi", &id_phi, "radiance azimuth angle", "degrees", Input.rte.phi, 
                           "phi", &id_nphi, Input.rte.nphi );
    if (status!=0) {
      fprintf (stderr, " *** Error, while writing 'phi' into %s, %s (%s)\n", 
                       Input.filename[FN_OUTFILE], function_name, file_name );
    }

    /* write umu array */
    status = write_netCDF_1D_float (ncid, Input.filename[FN_OUTFILE], 
                           "umu", &id_umu, "radiance zenith angle", "degrees", Input.rte.umu, 
                           "umu", &id_numu, Input.rte.numu );
    if (status!=0) {
      fprintf (stderr, " *** Error, while writing 'umu' into %s, %s (%s)\n", 
                       Input.filename[FN_OUTFILE], function_name, file_name );
    }

    /* order of dimensions */
    id_dim_array6[POSITION_TIME_6D  ] = id_ntime;
    id_dim_array6[POSITION_LAMBDA_6D] = id_nlambda;
    id_dim_array6[POSITION_ZOUT_6D  ] = id_nzout;
    id_dim_array6[POSITION_UMU_6D   ] = id_numu;
    id_dim_array6[POSITION_LAT_6D   ] = id_ny;  /* one before last dimension is default y-axis in ncview */
    id_dim_array6[POSITION_LON_6D   ] = id_nx;  /*            last dimension is default x-axis in ncview */

    /* initialisation of the index, where to write */
    index_6D[POSITION_TIME_6D]   = itime;
    n_6D    [POSITION_LAMBDA_6D] = output->wl.nlambda_h; /* loop over lambda */    
    n_6D    [POSITION_ZOUT_6D]   = output->atm.nzout;      /* loop over zout   */ 
    n_6D    [POSITION_UMU_6D]    = Input.rte.numu;       /* loop over umu    */ 
    index_6D[POSITION_LAT_6D]    = iy;                   /* latitude or x-pixel */
    index_6D[POSITION_LON_6D]    = ix;                   /* longitude or y-pixel */

    /* order of dimensions */
    id_dim_array7[POSITION_TIME_7D  ] = id_ntime;
    id_dim_array7[POSITION_LAMBDA_7D] = id_nlambda;
    id_dim_array7[POSITION_ZOUT_7D  ] = id_nzout;
    id_dim_array7[POSITION_UMU_7D   ] = id_nphi;
    id_dim_array7[POSITION_PHI_7D   ] = id_numu;
    id_dim_array7[POSITION_LAT_7D   ] = id_ny;  /* one before last dimension is default y-axis in ncview */
    id_dim_array7[POSITION_LON_7D   ] = id_nx;  /*            last dimension is default x-axis in ncview */

    /* initialisation of the index, where to write */
    index_7D[POSITION_TIME_7D]   = itime;
    n_7D    [POSITION_LAMBDA_7D] = output->wl.nlambda_h; /* loop over lambda */    
    n_7D    [POSITION_ZOUT_7D]   = output->atm.nzout;      /* loop over zout   */ 
    n_7D    [POSITION_UMU_7D]    = Input.rte.numu;       /* loop over umu    */ 
    n_7D    [POSITION_PHI_7D]    = Input.rte.nphi;       /* loop over umu    */ 
    index_7D[POSITION_LAT_7D]    = iy;                   /* latitude  or x-pixel */
    index_7D[POSITION_LON_7D]    = ix;                   /* longitude or y-pixel */

    /* write horizontal averaged radiance u0u */
    status = write_netCDF_6D_float (ncid, Input.filename[FN_OUTFILE], 
                                    "u0u", "horizontally averaged radiance", radiance_units, output->u0u, 
                                    id_dim_array6, index_6D, 6, n_6D, output->atm.zout_index, &(simulation_done));
    if (status!=0) {
      fprintf (stderr, " *** Error, while writing 'u0u' into %s, %s (%s)\n", 
                       Input.filename[FN_OUTFILE], function_name, file_name );
    }

    /* write radiance */
    status = write_netCDF_7D_float (ncid, Input.filename[FN_OUTFILE], 
                                    "uu", "radiance", radiance_units, output->uu, 
                                    id_dim_array7, index_7D, 7, n_7D, output->atm.zout_index, &(simulation_done));
    if (status!=0) {
      fprintf (stderr, " *** Error, while writing 'uu' into %s, %s (%s)\n", 
                       Input.filename[FN_OUTFILE], function_name, file_name );
    }
  }

  if (Input.verbose)
    fprintf (stderr, " ... write simulation_done = %d (0=not_done,1=done) in netCDF file %s\n", simulation_done, Input.filename[FN_OUTFILE]);

  status += write_netCDF_3D_byte (ncid, Input.filename[FN_OUTFILE], 
                                 "done", "0=not jet calculated, 1=calculation done", " ", simulation_done, 
                                 id_dim_array3, index_3D, dim3);
  if (status!=0) {
    fprintf (stderr, " *** Error, while writing 'done' into %s, %s (%s)\n", 
                     Input.filename[FN_OUTFILE], function_name, file_name );
  }

  nc_close (ncid);
  return status;

#else
    fprintf (stderr, " ***********************************************************************\n");
    fprintf (stderr, " * You have built uvspec without libnetcdf and hence cannot            *\n");
    fprintf (stderr, " * use the netCDF output option. Please get netcdf and rebuild.        *\n");
    fprintf (stderr, " ***********************************************************************\n");
    return -1;
#endif

}


int check_if_result_exists (int *output_user, int n_output_user, int solver, int **valid_result, int quiet)
{
  int status=0;
  int io=0;

  for ( io=0; io< n_output_user; io++ ) {
    switch (solver) {
    case TWOSTRPP:
    case CTWOSTR:
    case TWOSTR:
      /* no dir, dn up parts of the actinic flux and radiance */

      switch (Input.output_user[io]) {
      case OUTPUT_USER_FDIR:
      case OUTPUT_USER_FGLO:
      case OUTPUT_USER_FDN:
      case OUTPUT_USER_FUP:
      case OUTPUT_USER_UDIR:
      case OUTPUT_USER_UGLO:
      case OUTPUT_USER_UDN:
        (*valid_result)[io] = FALSE;
        if (!quiet) {
          fprintf (stderr, "*** Warning: twostream solver does not separate between upward and \n");
          fprintf (stderr, "***          downward actinic flux or radiances. THIS OUTPUT IS SKIPPED. \n");
        }
        break;
      case OUTPUT_USER_WAVE:
      case OUTPUT_USER_WAVENUMBER:
      case OUTPUT_USER_ZOUT_SUR:
      case OUTPUT_USER_ZOUT_SEA:
      case OUTPUT_USER_Z_SUR:
      case OUTPUT_USER_EDIR:
      case OUTPUT_USER_EGLO:
      case OUTPUT_USER_EDN:
      case OUTPUT_USER_EUP:
      case OUTPUT_USER_ENET:
      case OUTPUT_USER_ESUM:
      case OUTPUT_USER_F:
      case OUTPUT_USER_U:
      case OUTPUT_USER_ALB:
      case OUTPUT_USER_SZA:
      case OUTPUT_USER_C_P:
      case OUTPUT_USER_HEAT:
      case OUTPUT_USER_W_RAD:
      case OUTPUT_USER_M_RAD:
      case OUTPUT_USER_P:
      case OUTPUT_USER_T:
      case OUTPUT_USER_T_D:
      case OUTPUT_USER_THETA:
      case OUTPUT_USER_THETA_E:
      case OUTPUT_USER_N:
      case OUTPUT_USER_RHO:   
      case OUTPUT_USER_VMR:
      case OUTPUT_USER_MMR:
      case OUTPUT_USER_CLWC:
      case OUTPUT_USER_CLWD:
      case OUTPUT_USER_TCLW:
      case OUTPUT_USER_REFF_WAT: 
      case OUTPUT_USER_CIWC:
      case OUTPUT_USER_CIWD:
      case OUTPUT_USER_TCIW:
      case OUTPUT_USER_REFF_ICE:
      case OUTPUT_USER_TCW:
      case OUTPUT_USER_CC:
      case OUTPUT_USER_TCC:
      case OUTPUT_USER_CLOUDS:
      case OUTPUT_USER_RH:
      case OUTPUT_USER_RH_ICE:
      case OUTPUT_USER_WIND_U:
      case OUTPUT_USER_WIND_V:
      case OUTPUT_USER_WIND_W:
      case OUTPUT_USER_HEAT_AD_X:
      case OUTPUT_USER_HEAT_AD_Y:
      case OUTPUT_USER_HEAT_AD_Z:
      case OUTPUT_USER_HEAT_AD:
        (*valid_result)[io] = TRUE;
        break;
      default:
        fprintf (stderr, "Error, unknown user output %d (line %d, function '%s' in '%s')\n", 
                         Input.output_user[io], __LINE__, __func__, __FILE__);
        return -1;
      }
      break;      

    case DISORT:
    case DISORT2:
    case CDISORT:
    case SDISORT:
    case SPSDISORT:
    case MONTECARLO:
    case NULLSOLVER:
    case SUCORD:
      /* everything possible */
      (*valid_result)[io] = TRUE;
      break;

    case POLRADTRAN:
      fprintf (stderr, "Error, netCDF output for POLRADTRAN not yet implemented (line %d, function '%s' in '%s')\n",
                        __LINE__, __func__, __FILE__);
      return -1;
      break;

    case SSS:
    case SSSI:
    case TZS:
      /* only radiance in a direction (no fluxes, no actinic fluxes, no averaged fluxes) */
      fprintf (stderr, "Error, netCDF for 'solar single scattering' and 'thermal_zero_scattering'\n");
      fprintf (stderr, "       not yet implemented (in uvspec_lex.l)\n");
      return -2;
      break;

    default:
      fprintf (stderr, "Error, unknown rte_solver %d (line %d, function '%s' in '%s')\n", solver, __LINE__, __func__, __FILE__);
      return -3;
    }
  }

  return status;
}




static void free_output(input_struct Input, output_struct *output) 
{

  if (output->down_flux != NULL)
    ASCII_free_float_3D (output->down_flux, 
	                 output->atm.nzout, Input.rte.pol_nstokes);

  if (output->down_flux_t != NULL)
    ASCII_free_float_3D (output->down_flux_t, 
			 output->atm.nzout, Input.rte.pol_nstokes);
  
  if (output->up_flux != NULL)
    ASCII_free_float_3D (output->up_flux, 
			 output->atm.nzout, Input.rte.pol_nstokes);
  
  if (output->up_flux_t != NULL)
    ASCII_free_float_3D (output->up_flux_t, 
			 output->atm.nzout, Input.rte.pol_nstokes);
  
  if (output->down_rad != NULL)
    ASCII_free_float_5D (output->down_rad, 
			 output->atm.nzout, Input.rte.nphi, 
			 Input.rte.numu, Input.rte.pol_nstokes);
  
  if (output->down_rad_t != NULL)
    ASCII_free_float_5D (output->down_rad_t, 
			 output->atm.nzout, Input.rte.nphi,
			 Input.rte.numu, Input.rte.pol_nstokes);
  
  if (output->up_rad != NULL)
    ASCII_free_float_5D (output->up_rad, 
			 output->atm.nzout, Input.rte.nphi,
			 Input.rte.numu, Input.rte.pol_nstokes);
  
  if (output->up_rad_t != NULL)
    ASCII_free_float_5D (output->up_rad_t, 
			 output->atm.nzout, Input.rte.nphi,
			 Input.rte.numu, Input.rte.pol_nstokes);

  for (isp=0; isp<Input.n_caoth; isp++) {
    if (output->mc.dt != NULL) {
      if (output->mc.dt[isp] != NULL)
        free(output->mc.dt[isp]);
      if (output->mc.om[isp] != NULL)
        free(output->mc.om[isp]);
      if (output->mc.g1[isp] != NULL)
        free(output->mc.g1[isp]);
      if (output->mc.g2[isp] != NULL)
        free(output->mc.g2[isp]);
      if (output->mc.ff[isp] != NULL)
        free(output->mc.ff[isp]);
      if (output->mc.ds[isp] != NULL)
        free(output->mc.ds[isp]);
      if (output->mc.re[isp] != NULL)
        free(output->mc.re[isp]);
    }
  }
  if (output->mc.dt != NULL)
    free(output->mc.dt);
  if (output->mc.om != NULL)
    free(output->mc.om);
  if (output->mc.g1 != NULL)
    free(output->mc.g1);
  if (output->mc.g2 != NULL)
    free(output->mc.g2);
  if (output->mc.ff != NULL)
    free(output->mc.ff);
  if (output->mc.ds != NULL)
    free(output->mc.ds);
  if (output->mc.re != NULL)
    free(output->mc.re);

  if (output->dtauc     != NULL) free(output->dtauc);
  if (output->dtauc_md  != NULL) free(output->dtauc_md);
  if (output->sza_h     != NULL) free(output->sza_h);
  if (output->sza_s     != NULL) free(output->sza_s);
  if (output->mu_values != NULL) free(output->mu_values);

}


static float yytext2float(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=UVSPEC_TOKEN;
  char *p=NULL, *s=NULL;
  int count=0;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);
  free(tmpstring);
  return (float) dnumber;
}


static double yytext2double(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=UVSPEC_TOKEN;
  char *p=NULL, *s=NULL;
  int count=0;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);
  
  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);

  free(tmpstring);
  return dnumber;
}


static float* yytext2floats(char* yytext, int nary) {
  int i=0;
  char token[]=UVSPEC_TOKEN;
  char *p=NULL;
  char *tmpstring=NULL;
  
  /* Fill up ary */
  float *ary = (float *) calloc (nary, sizeof(float));
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atof(p);

  free(tmpstring);
  return ary;
}


static int* yytext2integers(char* yytext, int nary) {
  int i=0;
  char token[]=UVSPEC_TOKEN;
  char *p=NULL;
  char *tmpstring=NULL;
  
  /* Fill up ary */
  int *ary    = (int *) calloc (nary, sizeof(int));

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atoi(p);

  free(tmpstring);
  return ary;
}


static int yytext2ntokens(char* yytext) {
  int nary=0;
  char token[]=UVSPEC_TOKEN;
  char *p=NULL;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  /* Get number of ary elements */
  nary = 0;
  p = strtok(tmpstring,token);

  while ((p = strtok(NULL,token)) != NULL) 
    nary++;

  free(tmpstring);
  return nary;
}


static int yytext2int(char* yytext, int which_token, int max_token) {
  int inumber=0;
  char token[]=UVSPEC_TOKEN;
  char *p=NULL, *s=NULL;
  int count=0;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  inumber = atoi(s);

  free(tmpstring);
  return inumber;
}


static char* yytext2string(char* yytext, int which_token, int max_token) {
  char token[]=UVSPEC_TOKEN;
  char *p=NULL, *s=NULL, *string=NULL, *tmpstring=NULL;
  int count=0;

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  if (which_token == 1) {
    s = p;
  }
  else { 
    while ((p = strtok(NULL,token)) != NULL) {
      count++;
      if (count == which_token)
        s = p;
    }
  }

  if ((count != which_token) && (count > max_token))
    fprintf(stderr,"Wrong number of arguments on line %d (yytext2string) \n", line_number+1);

  string = (char *) calloc (strlen(s)+1, sizeof (char));
  strcpy (string, s);

  free(tmpstring);

  return string;
}


/* like yytext2string, but string is copied to a pre-allocated string dest */
static int yytextCstring(char *dest, char* yytext, int which_token, int max_token) {
  char token[]=UVSPEC_TOKEN;
  char *p=NULL, *s=NULL, *tmpstring=NULL;
  int count=0;

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  strcpy (dest, s);

  free(tmpstring);
  return 0;
}

int compare_ascending_float (void *ap, void *bp) { /* Sort such that umu is in ascending order */
  float *a = (float *) ap;
  float *b = (float *) bp;
  
  if (*a == *b) 
    return 0;

  if (*a > *b) 
    return 1;
  else 
    return -1;
}

static float* add_cmu_to_umu(int nstr, int numu, int maxumu, float* oldumu, int* cmuind, int* umuind) {
  
  int nn, i, k=0;
  float *cmu = (float *) calloc (nstr/2, sizeof(float));
  float *cwt = (float *) calloc (nstr/2, sizeof(float));
  float *umu = NULL;
  int not = 0;

  umu = (float *) calloc (nstr+numu, sizeof(float));

  nn = nstr/2;

  F77_FUNC (qgausn, QGAUSN)(&nn, cmu, cwt); 

  for (i=0;i<nn;i++){
    umu[i]=-cmu[i];
    umu[i+nn]=cmu[i];
  }
  for (i=0;i<numu;i++)
    umu[nstr+i]=oldumu[i];

  free(cwt);
  free(cmu);

  /* Sort umu in increasing order */
  qsort ( umu, maxumu, sizeof(float),  QSORT_CAST compare_ascending_float);

  k = 0;
  for (j=0;j<numu;j++) {
    for (i=0;i<maxumu;i++) {
      if (fabs(umu[i]-oldumu[j]) < FLT_MIN ) {
	umuind[k++] = i;   /* These are the indices for the user angles */
      }
    }
  }

  free(oldumu);

  /* Get the indics for the computational angles */
  k=0;
  i=0;
  for (j=0;j<maxumu;j++) {
    not = 0;
    for (i=0;i<numu;i++) {
      if ( j == umuind[i] ) {	not = 1; }
      

    }
    if ( ! not ) 
      cmuind[k++] = j; 
  }
    
  return umu;
}


